@@ -1,7 +1,7 @@
 /* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This application is open source and may be redistributed and/or modified   
- * freely and without restriction, both in commericial and non commericial applications,
+ * freely and without restriction, both in commercial and non commercial applications,
  * as long as this copyright notice is maintained.
  * 
  * This application is distributed in the hope that it will be useful,
@@ -29,7 +29,7 @@ int main( int argc, char **argv )
     
     // set up the usage document, in case we need to print out how to use this program.
     arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is an application for collecting a set of seperate files into a single archive file that can be later read in OSG applications..");
+    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is an application for collecting a set of separate files into a single archive file that can be later read in OSG applications..");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
         
     // if user request help write it out to cout.
@@ -79,7 +79,12 @@ int main( int argc, char **argv )
                 else if (fileType==osgDB::DIRECTORY)
                 {
                     osgDB::DirectoryContents directory = osgDB::getDirectoryContents(arguments[pos]);
-                    files.insert(files.end(),directory.begin(),directory.end());
+                    osgDB::DirectoryContents::iterator it = directory.begin();
+                    while( it != directory.end())
+                    {
+                        files.push_back(filePath + "/" + (*it));
+                        ++it;
+                    }
                 }
             }
             else
@@ -134,7 +139,15 @@ int main( int argc, char **argv )
                 if (obj.valid())
                 {
                     std::cout<<"  write to archive "<<*itr<<std::endl;
-                    archive->writeObject(*obj, *itr);
+                    osg::Image* image = dynamic_cast<osg::Image*>(obj.get());
+                    osg::HeightField* hf = dynamic_cast<osg::HeightField*>(obj.get());
+                    osg::Node* node = dynamic_cast<osg::Node*>(obj.get());
+                    osg::Shader* shader = dynamic_cast<osg::Shader*>(obj.get());
+                    if (image) archive->writeImage(*image, *itr);
+                    else if (hf) archive->writeHeightField(*hf, *itr);
+                    else if (node) archive->writeNode(*node, *itr);
+                    else if (shader) archive->writeShader(*shader, *itr);
+                    else archive->writeObject(*obj, *itr);
                 }
             }
         }
