@@ -122,6 +122,7 @@ const char* invalidNames[] =
     "LineStipple",
     "Logos",
     "LongIDRecord",
+    "InputIterator",
     "Make",
     "Makefile",
     "Material",
@@ -247,7 +248,20 @@ const char* invalidNames[] =
     "UpdateSkeleton",
     "Vec3",
     "Plugin",
-    "Get"
+    "Get",
+    "LightSource",
+    "SlideEventHandler",
+    "GraphicsContext*",
+    "Apple",
+    "GeometryNew",
+    "FrameBufferObject",
+    "Lua",
+    "VolumeTile",
+    "PushStackValueVisitor",
+    "RayIntersector",
+    "OpenSceneGraph-Data",
+    "Node",
+    "AlphaFunc"
 };
 
 
@@ -259,6 +273,7 @@ struct TypoCorrection
 
 TypoCorrection typoCorrections[] =
 {
+    {"Cheaev",         "Chebaev"},
     {"Alaxandre",      "Alexandre"},
     {"Aderian",        "Adrian"},
     {"Adndre",         "Andre"},
@@ -268,6 +283,7 @@ TypoCorrection typoCorrections[] =
     {"André",         "Andr"},
     {"Antione",        "Antoine"},
     {"Antonoine",      "Antoine"},
+    {"Aurelein",       "Aurelien"},
     {"Atr",            "Art"},
     {"Baverage",       "Beverage"},
     {"Bistroviae",     "Bistrovic"},
@@ -286,6 +302,7 @@ TypoCorrection typoCorrections[] =
     {"Fabian",         "Fabien"},
     {"Fautre",         "Fautr"},
     {"Frashid",        "Farshid"},
+    {"Farshild",       "Farshid"},
     {"Fred",           "Frederic"},
     {"Fredrick",       "Frederic"},
     {"Fredric",        "Frederic"},
@@ -311,6 +328,8 @@ TypoCorrection typoCorrections[] =
     {"Happalahti",     "Haapalahti"},
     {"Hertlien",       "Hertlein"},
     {"Hatwig",         "Hartwig"},
+    {"Hauknes",        "Haukness"},
+    {"Hunber",        "Huber"},
     {"Hi",             "He"},
     {"Hooper",         "Hopper"},
     {"Inverson",       "Iverson"},
@@ -335,6 +354,7 @@ TypoCorrection typoCorrections[] =
     {"Krulthof",       "Kruithof"},
     {"Lagrade",        "Lagarde"},
     {"Largade",        "Lagarde"},
+    {"Largarde",       "Lagarde"},
     {"Larshkari",      "Lashkari"},
     {"Lashakari",      "Lashkari"},
     {"Lashari",        "Lashkari"},
@@ -380,6 +400,7 @@ TypoCorrection typoCorrections[] =
     {"Oritz",          "Ortiz"},
     {"Osfeld",         "Osfield"},
     {"Osfied",         "Osfield"},
+    {"Paulk",          "Paul"},
     {"Pail",           "Paul"},
     {"Perciva",        "Peciva"},
     {"Pecvia",         "Peciva"},
@@ -440,7 +461,30 @@ TypoCorrection typoCorrections[] =
     {"Jahannes","Johannes"},
     {"Eskland","Ekstrand"},
     {"Baeuerele","Baeuerle"},
-    {"Bauerle","Baeuerle"}
+    {"Bauerle","Baeuerle"},
+    {"Baeurele","Baeuerle"},
+    {"Nillson","Nilsson"},
+    {"Bjorn","Bjrn"},
+    {"Björn","Bjrn"},
+    {"Stepan","Stephan"},
+    {"Kristoger","Kristofer"},
+    {"Blessing","Blissing"},
+    {"Dannahuer","Dannhauer"},
+    {"Chebeav", "Chebaev"},
+    {"Messershmidt","Messerschmidt"},
+    {"Auelien","Aurelien"},
+    {"Aurélien","Aurlien"},
+    {"McDonnel","Mc Donnell"},
+    {"McDonnell","Mc Donnell"},
+    {"Delallée","Delalle"},
+    {"Gjøl","Gjl"},
+    {"Ravšelj","Ravelj"},
+    {"Ravsel",  "Ravelj"},
+    {"Ravselj", "Ravelj"},
+    {"Janik", "Jannik"},
+    {"Viganò", "Vigan"},
+    {"Vigano", "Vigan"},
+    {"Frashud", "Farshid"}
 };
 
 
@@ -454,18 +498,28 @@ struct NameCorrection
 
 NameCorrection nameCorrections[] =
 {
+    {"FrancoisTigeot","",
+     "Francois", "Tigeot"},
+    {"Juan","Manuel",
+     "Juan", "Manuel Alvarez"},
+    {"Jaap","Gas",
+     "Jaap", "Glas"},
     {"Philip","Lamp",
      "Philip", "Lamb"},
     {"Dimi","Christop",
      "Dimi", "Christopoulos"},
     {"Jorge","Ciges",
-     "Jorge", "Izquierdo"},
+     "Jorge", "Izquierdo Ciges"},
+    {"Jorge","Izquierdo",
+     "Jorge", "Izquierdo Ciges"},
     {"Rafa","Gata",
      "Rafa", "Gaitan"},
     {"Sukender","I",
      "Sukender", ""},
     {"Sukender","Here",
      "Sukender", ""},
+    {"Sukender","Fix",
+     "Sukender", ""},
     {"Ewe","Woessner",
      "Uwe", "Woessner"},
     {"Martin","von",
@@ -480,6 +534,8 @@ NameCorrection nameCorrections[] =
      "Thomas",         "Hogarth"},
     {"Marin",          "Lavery",
      "Martin",         "Lavery"},
+    {"Michael",        "Bach",
+     "Michael",        "Bach Jensen"},
     {"Nguyen",         "Van",
      "Nguyen",         "Van Truong"},
     {"Thom",           "Carlo",
@@ -536,6 +592,8 @@ NameCorrection nameCorrections[] =
      "Melchior",       "Franz"},
     {"Glen",           "Waldon",
      "Glenn",          "Waldron"},
+    {"Glen",           "Waldron",
+     "Glenn",          "Waldron"},
     {"Ralf",           "Karn",
      "Ralf",           "Kern"},
     {"Donny",          "Cipperly",
@@ -568,8 +626,12 @@ NameCorrection nameCorrections[] =
      "Romano",         "Jos Magacho da Silva"},
     {"Rommano",        "Silva",
      "Romano",         "Jos Magacho da Silva"},
+    {"Romano",         "Magacho",
+     "Romano",         "Jos Magacho da Silva"},
     {"Leandro",        "Motta",
      "Leandro",        "Motta Barros"},
+    {"Leandro",        "Motto",
+     "Leandro",        "Motta Barros"},
     {"A",              "Botorabi",
      "Ali",            "Botorabi"},
     {"Waltice",        "",
@@ -586,10 +648,32 @@ NameCorrection nameCorrections[] =
      "Luc",            "Frauciel"},
     {"Alberto",        "Lucas",
      "Alberto",        "Luaces"},
+    {"Alberto",        "Luacus",
+     "Alberto",        "Luaces"},
     {"Tyge",           "",
      "Tyge",           "Lvset"},
     {"Ricard",         "Schmidt",
-     "Richard",        "Schmidt"}
+     "Richard",        "Schmidt"},
+    {"Matthias",       "Helsing",
+     "Mattias",        "Helsing"},
+    {"Clement",        "Boesch",
+     "Clment",        "Bsch"},
+    {"Lauren",         "Voerman",
+     "Laurens",        "Voerman"},
+    {"Pjotr",          "Sventachov",
+     "Pjotr",          "Svetachov"},
+     {"Bradley",       "Baker",
+      "Bradley",       "Baker Searles"},
+     {"PawelKsiezopolski", "",
+      "Pawel",          "Ksiezopolski"},
+     {"Albert", "Luaces",
+      "Alberto","Luaces"},
+     {"KOS", "",
+      "Konstantin","Matveyev"},
+     {"WeSee", "",
+       "Alois", "Wismer"},
+     {"We", "See",
+       "Alois", "Wismer"}
 };
 
 
@@ -921,7 +1005,9 @@ void printContributors(const std::string& changeLog, bool printNumEntries)
         cout << "-------------------------" << endl;
         for (SortedNameMap::reverse_iterator sitr = sortedNames.rbegin(); sitr != sortedNames.rend(); ++sitr)
         {
-            cout << sitr->first << "\t" << sitr->second.first << " " << sitr->second.second << endl;
+            cout << sitr->first << "\t" << sitr->second.first;
+            if (!sitr->second.second.empty()) cout << " " << sitr->second.second;
+            cout << endl;
         }
     }
     else
@@ -930,7 +1016,9 @@ void printContributors(const std::string& changeLog, bool printNumEntries)
         cout << "-----------------" << endl;
         for (SortedNameMap::reverse_iterator sitr = sortedNames.rbegin(); sitr != sortedNames.rend(); ++sitr)
         {
-            cout << sitr->second.first << " " << sitr->second.second << endl;
+            cout << sitr->second.first;
+            if (!sitr->second.second.empty()) cout << " " << sitr->second.second;
+            cout << endl;
         }
     }
 }
