@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgcopy.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/MatrixTransform>
 #include <osg/Billboard>
 #include <osg/Geode>
@@ -9,11 +27,11 @@
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osgUtil/Optimizer>
 
-using namespace osg;
+#include <iostream>
 
 // Customize the CopyOp so that we add our own verbose 
 // output of what's being copied.
@@ -127,46 +145,63 @@ class MyCopyOp : public osg::CopyOp
         mutable int _step;
 };
 
+// this CopyOp class will preserve the multi-parent structure of the copied 
+// object, instead of expanding it into a tree. Works with the 
+// DEEP_COPY_NODES flag.
+class GraphCopyOp : public osg::CopyOp
+{
+    public:
+    
+        inline GraphCopyOp(CopyFlags flags=SHALLOW_COPY):
+            osg::CopyOp(flags) { _nodeCopyMap.clear();}
+            
+        virtual osg::Node* operator() (const osg::Node* node) const
+        {
+            if (node && _flags&DEEP_COPY_NODES)
+            {
+                if ( node->getNumParents() > 1 )
+                {
+                    if ( _nodeCopyMap.find(node) != _nodeCopyMap.end() )
+                    {
+                        std::cout<<"Copy of node "<<node<<" "<<node->getName()<<", "
+                            << _nodeCopyMap[node]<<", will be reused"<<std::endl;
+                        return (osg::Node*)(_nodeCopyMap[node]);
+                    }
+                    else
+                    {
+                        osg::Node* newNode = dynamic_cast<osg::Node*>( node->clone(*this) );
+                        _nodeCopyMap[node] = newNode;
+                        return newNode;
+                    }
+                }
+                else
+                    return dynamic_cast<osg::Node*>( node->clone(*this) );
+            }
+            else
+                return const_cast<osg::Node*>(node);
+        }
+         
+    protected:
+    
+        // must be mutable since CopyOp is passed around as const to
+        // the various clone/copy constructors.
+        mutable std::map<const osg::Node*,osg::Node*> _nodeCopyMap;
+
+};
+
 int main( int argc, char **argv )
 {
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
 
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates use of deep vs shallow vs custom copying of scene graphs.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-   
     // initialize the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-    
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
+    osgViewer::Viewer viewer;
 
     // load the nodes from the commandline arguments.
     osg::Node* rootnode = osgDB::readNodeFiles(arguments);
     if (!rootnode)
     {
+        osg::notify(osg::NOTICE)<<"Please specify a model filename on the command line."<<std::endl;
         return 1;
     }
     
@@ -178,29 +213,38 @@ int main( int argc, char **argv )
     
     // do a deep copy, using MyCopyOp to reveal whats going on under the good,
     // in your own code you'd typically just use the basic osg::CopyOp something like
-    // osg::Node* mycopy = dynamic_cast<osg::Node*>(rootnode->clone(osg::CopyOp::DEEP_COPY_ALL));
+    osg::ref_ptr<osg::Node> mycopy = dynamic_cast<osg::Node*>(rootnode->clone(osg::CopyOp::DEEP_COPY_ALL));
     std::cout << "Doing a deep copy of scene graph"<<std::endl;
+
     // note, we need the dyanmic_cast because MS Visual Studio can't handle covarient
     // return types, so that clone has return just Object*.  bahh hum bug
-    osg::Node* deep_copy = dynamic_cast<osg::Node*>(rootnode->clone(MyCopyOp(osg::CopyOp::DEEP_COPY_ALL)));
+    osg::ref_ptr<osg::Node> deep_copy = dynamic_cast<osg::Node*>(rootnode->clone(MyCopyOp(osg::CopyOp::DEEP_COPY_ALL)));
     
     std::cout << "----------------------------------------------------------------"<<std::endl;
 
+    // do a graph preserving deep copy.
+    std::cout << "Doing a graph preserving deep copy of scene graph nodes"<<std::endl;
+    osg::ref_ptr<osg::Node> graph_copy = dynamic_cast<osg::Node*>(rootnode->clone(GraphCopyOp(osg::CopyOp::DEEP_COPY_NODES)));
+
+
     // do a shallow copy.
     std::cout << "Doing a shallow copy of scene graph"<<std::endl;
-    osg::Node* shallow_copy = dynamic_cast<osg::Node*>(rootnode->clone(MyCopyOp(osg::CopyOp::SHALLOW_COPY)));
+    osg::ref_ptr<osg::Node> shallow_copy = dynamic_cast<osg::Node*>(rootnode->clone(MyCopyOp(osg::CopyOp::SHALLOW_COPY)));
 
 
     // write out the various scene graphs so that they can be browsed, either
     // in an editor or using a graphics diff tool gdiff/xdiff/xxdiff.
-    std::cout << std::endl << "Writing out the original scene graph as 'original.osg'"<<std::endl;
-    osgDB::writeNodeFile(*rootnode,"original.osg");
+    std::cout << std::endl << "Writing out the original scene graph as 'original.osgt'"<<std::endl;
+    osgDB::writeNodeFile(*rootnode,"original.osgt");
 
-    std::cout << "Writing out the deep copied scene graph as 'deep_copy.osg'"<<std::endl;
-    osgDB::writeNodeFile(*deep_copy,"deep_copy.osg");
+    std::cout << std::endl << "Writing out the graph preserving scene graph as 'graph_copy.osgt'"<<std::endl;
+    osgDB::writeNodeFile(*graph_copy,"graph_copy.osgt");
 
-    std::cout << "Writing out the shallow copied scene graph as 'shallow_copy.osg'"<<std::endl;
-    osgDB::writeNodeFile(*shallow_copy,"shallow_copy.osg");
+    std::cout << "Writing out the deep copied scene graph as 'deep_copy.osgt'"<<std::endl;
+    osgDB::writeNodeFile(*deep_copy,"deep_copy.osgt");
+
+    std::cout << "Writing out the shallow copied scene graph as 'shallow_copy.osgt'"<<std::endl;
+    osgDB::writeNodeFile(*shallow_copy,"shallow_copy.osgt");
 
 
     // You can use a bit mask to control which parts of the scene graph are shallow copied
@@ -231,31 +275,7 @@ int main( int argc, char **argv )
     // set the scene to render
     viewer.setSceneData(rootnode);
 
-    // create the windows and run the threads.
-    viewer.realize();
+    // viewer.setThreadingModel(osgViewer::Viewer::SingleThreaded);
 
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
