@@ -31,7 +31,11 @@
 #include <osg/StateSet>
 #include <osg/Switch>
 #include <osg/Texture2D>
+#include <osg/TextureBuffer>
+#include <osg/Image>
 #include <osg/TexEnv>
+#include <osg/VertexProgram>
+#include <osg/FragmentProgram>
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
@@ -48,6 +52,7 @@
 #include <osgGA/StateSetManipulator>
 
 #include <iostream>
+#include <sstream>
 
 // for the grid data..
 #include "../osghangglide/terrain_coords.h"
@@ -82,9 +87,9 @@ public:
         float           _height;
         unsigned int    _type;
     };
-    
+
     typedef std::vector< osg::ref_ptr<Tree> > TreeList;
-    
+
     class Cell : public osg::Referenced
     {
     public:
@@ -92,21 +97,21 @@ public:
 
         Cell():_parent(0) {}
         Cell(osg::BoundingBox& bb):_parent(0), _bb(bb) {}
-        
+
         void addCell(Cell* cell) { cell->_parent=this; _cells.push_back(cell); }
 
         void addTree(Tree* tree) { _trees.push_back(tree); }
-        
+
         void addTrees(const TreeList& trees) { _trees.insert(_trees.end(),trees.begin(),trees.end()); }
-        
+
         void computeBound();
-        
+
         bool contains(const osg::Vec3& position) const { return _bb.contains(position); }
-        
+
         bool divide(unsigned int maxNumTreesPerCell=10);
-        
+
         bool divide(bool xAxis, bool yAxis, bool zAxis);
-        
+
         void bin();
 
 
@@ -114,7 +119,7 @@ public:
         osg::BoundingBox    _bb;
         CellList            _cells;
         TreeList            _trees;
-        
+
     };
 
     float random(float min,float max) { return min + (max-min)*(float)rand()/(float)RAND_MAX; }
@@ -136,11 +141,17 @@ public:
     osg::Node* createTransformGraph(Cell* cell,osg::StateSet* stateset);
 
     osg::Node* createShaderGraph(Cell* cell,osg::StateSet* stateset);
-    
+
+    osg::Node* createGeometryShaderGraph(Cell* cell, osg::StateSet* stateset);
+
+    osg::Node* createTextureBufferGraph(Cell* cell, osg::Geometry* templateGeometry);
+
+    void CollectTreePositions(Cell* cell, std::vector< osg::Vec3 >& positions);
+
     osg::Node* createHUDWithText(const std::string& text);
 
-    osg::Node* createScene(unsigned int numTreesToCreates);
-    
+    osg::Node* createScene(unsigned int numTreesToCreates, unsigned int maxNumTreesPerCell);
+
     void advanceToNextTechnique(int delta=1)
     {
         if (_techniqueSwitch.valid())
@@ -153,10 +164,10 @@ public:
             _techniqueSwitch->setSingleChildOn(_currentTechnique);
         }
     }
-    
+
     osg::ref_ptr<osg::Switch>   _techniqueSwitch;
     int                         _currentTechnique;
-    
+
 
 };
 
@@ -166,22 +177,22 @@ class TechniqueEventHandler : public osgGA::GUIEventHandler
 public:
 
     TechniqueEventHandler(ForestTechniqueManager* ttm=0) { _ForestTechniqueManager = ttm; }
-    
+
     META_Object(osgforestApp,TechniqueEventHandler);
 
     virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&, osg::Object*, osg::NodeVisitor*);
-    
+
     virtual void getUsage(osg::ApplicationUsage& usage) const;
 
 protected:
 
     ~TechniqueEventHandler() {}
-    
+
     TechniqueEventHandler(const TechniqueEventHandler&,const osg::CopyOp&) {}
-    
+
     osg::ref_ptr<ForestTechniqueManager> _ForestTechniqueManager;
 
-        
+
 };
 
 bool TechniqueEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&, osg::Object*, osg::NodeVisitor*)
@@ -191,14 +202,14 @@ bool TechniqueEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAc
         case(osgGA::GUIEventAdapter::KEYDOWN):
         {
             if (ea.getKey()=='n' ||
-                ea.getKey()==osgGA::GUIEventAdapter::KEY_Right || 
+                ea.getKey()==osgGA::GUIEventAdapter::KEY_Right ||
                 ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Right)
             {
                 _ForestTechniqueManager->advanceToNextTechnique(1);
                 return true;
             }
             else if (ea.getKey()=='p' ||
-                     ea.getKey()==osgGA::GUIEventAdapter::KEY_Left || 
+                     ea.getKey()==osgGA::GUIEventAdapter::KEY_Left ||
                      ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Left)
             {
                 _ForestTechniqueManager->advanceToNextTechnique(-1);
@@ -326,8 +337,8 @@ bool ForestTechniqueManager::Cell::divide(bool xAxis, bool yAxis, bool zAxis)
 }
 
 void ForestTechniqueManager::Cell::bin()
-{   
-    // put trees in apprpriate cells.
+{
+    // put trees in appropriate cells.
     TreeList treesNotAssigned;
     for(TreeList::iterator titr=_trees.begin();
         titr!=_trees.end();
@@ -384,9 +395,9 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
         texture->setImage(image);
         stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
     }
-    
+
     geode->setStateSet( stateset );
-    
+
     unsigned int numColumns = 38;
     unsigned int numRows = 39;
     unsigned int r;
@@ -403,7 +414,7 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
             max_z = osg::maximum(max_z,vertex[r+c*numRows][2]);
         }
     }
-    
+
     float scale_z = size.z()/(max_z-min_z);
 
 
@@ -424,22 +435,22 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
                 grid->setHeight(c,r,(vertex[r+c*numRows][2]-min_z)*scale_z);
             }
         }
-        
+
         geode->addDrawable(new osg::ShapeDrawable(grid));
     }
     else
     {
         osg::Geometry* geometry = new osg::Geometry;
-        
+
         osg::Vec3Array& v = *(new osg::Vec3Array(numColumns*numRows));
         osg::Vec2Array& t = *(new osg::Vec2Array(numColumns*numRows));
         osg::Vec4ubArray& color = *(new osg::Vec4ubArray(1));
-        
+
         color[0].set(255,255,255,255);
 
         float rowCoordDelta = size.y()/(float)(numRows-1);
         float columnCoordDelta = size.x()/(float)(numColumns-1);
-        
+
         float rowTexDelta = 1.0f/(float)(numRows-1);
         float columnTexDelta = 1.0f/(float)(numColumns-1);
 
@@ -461,12 +472,11 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
             pos.y() += rowCoordDelta;
             tex.y() += rowTexDelta;
         }
-        
+
         geometry->setVertexArray(&v);
-        geometry->setColorArray(&color);
-        geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
+        geometry->setColorArray(&color, osg::Array::BIND_OVERALL);
         geometry->setTexCoordArray(0,&t);
-        
+
         for(r=0;r<numRows-1;++r)
         {
             osg::DrawElementsUShort& drawElements = *(new osg::DrawElementsUShort(GL_QUAD_STRIP,2*numColumns));
@@ -478,13 +488,13 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
                 drawElements[ei++] = (r)*numColumns+c;
             }
         }
-        
+
         geode->addDrawable(geometry);
-        
+
         osgUtil::SmoothingVisitor sv;
         sv.smooth(*geometry);
     }
-    
+
     return geode;
 }
 
@@ -493,7 +503,7 @@ void ForestTechniqueManager::createTreeList(osg::Node* terrain,const osg::Vec3&
 
     float max_TreeHeight = sqrtf(size.length2()/(float)numTreesToCreate);
     float max_TreeWidth = max_TreeHeight*0.5f;
-    
+
     float min_TreeHeight = max_TreeHeight*0.3f;
     float min_TreeWidth = min_TreeHeight*0.5f;
 
@@ -508,14 +518,14 @@ void ForestTechniqueManager::createTreeList(osg::Node* terrain,const osg::Vec3&
         tree->_width = random(min_TreeWidth,max_TreeWidth);
         tree->_height = random(min_TreeHeight,max_TreeHeight);
         tree->_type = 0;
-        
+
         if (terrain)
         {
-            osg::ref_ptr<osgUtil::LineSegmentIntersector> intersector = 
+            osg::ref_ptr<osgUtil::LineSegmentIntersector> intersector =
                 new osgUtil::LineSegmentIntersector(tree->_position,tree->_position+osg::Vec3(0.0f,0.0f,size.z()));
 
             osgUtil::IntersectionVisitor iv(intersector.get());
-            
+
             terrain->accept(iv);
 
             if (intersector->containsIntersections())
@@ -530,7 +540,7 @@ void ForestTechniqueManager::createTreeList(osg::Node* terrain,const osg::Vec3&
                 }
             }
         }
-        
+
         trees.push_back(tree);
     }
 }
@@ -560,8 +570,7 @@ osg::Geometry* ForestTechniqueManager::createSprite( float w, float h, osg::Vec4
 
     geom->setTexCoordArray( 0, &t );
 
-    geom->setColorArray( &c );
-    geom->setColorBinding( osg::Geometry::BIND_OVERALL );
+    geom->setColorArray( &c, osg::Array::BIND_OVERALL );
 
     geom->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,4) );
 
@@ -574,7 +583,7 @@ osg::Geometry* ForestTechniqueManager::createOrthogonalQuads( const osg::Vec3& p
     osg::Vec3Array& v = *(new osg::Vec3Array(8));
     osg::Vec2Array& t = *(new osg::Vec2Array(8));
     osg::Vec4ubArray& c = *(new osg::Vec4ubArray(1));
-    
+
     float rotation = random(0.0f,osg::PI/2.0f);
     float sw = sinf(rotation)*w*0.5f;
     float cw = cosf(rotation)*w*0.5f;
@@ -607,8 +616,7 @@ osg::Geometry* ForestTechniqueManager::createOrthogonalQuads( const osg::Vec3& p
 
     geom->setTexCoordArray( 0, &t );
 
-    geom->setColorArray( &c );
-    geom->setColorBinding( osg::Geometry::BIND_OVERALL );
+    geom->setColorArray( &c, osg::Array::BIND_OVERALL );
 
     geom->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,8) );
 
@@ -619,10 +627,10 @@ osg::Node* ForestTechniqueManager::createBillboardGraph(Cell* cell,osg::StateSet
 {
     bool needGroup = !(cell->_cells.empty());
     bool needBillboard = !(cell->_trees.empty());
-    
+
     osg::Billboard* billboard = 0;
     osg::Group* group = 0;
-    
+
     if (needBillboard)
     {
         billboard = new osg::Billboard;
@@ -632,10 +640,10 @@ osg::Node* ForestTechniqueManager::createBillboardGraph(Cell* cell,osg::StateSet
             ++itr)
         {
             Tree& tree = **itr;
-            billboard->addDrawable(createSprite(tree._width,tree._height,tree._color),tree._position);    
+            billboard->addDrawable(createSprite(tree._width,tree._height,tree._color),tree._position);
         }
     }
-    
+
     if (needGroup)
     {
         group = new osg::Group;
@@ -645,9 +653,9 @@ osg::Node* ForestTechniqueManager::createBillboardGraph(Cell* cell,osg::StateSet
         {
             group->addChild(createBillboardGraph(itr->get(),stateset));
         }
-        
+
         if (billboard) group->addChild(billboard);
-        
+
     }
     if (group) return group;
     else return billboard;
@@ -657,15 +665,15 @@ osg::Node* ForestTechniqueManager::createXGraph(Cell* cell,osg::StateSet* states
 {
     bool needGroup = !(cell->_cells.empty());
     bool needTrees = !(cell->_trees.empty());
-    
+
     osg::Geode* geode = 0;
     osg::Group* group = 0;
-    
+
     if (needTrees)
     {
         geode = new osg::Geode;
         geode->setStateSet(stateset);
-        
+
         for(TreeList::iterator itr=cell->_trees.begin();
             itr!=cell->_trees.end();
             ++itr)
@@ -674,7 +682,7 @@ osg::Node* ForestTechniqueManager::createXGraph(Cell* cell,osg::StateSet* states
             geode->addDrawable(createOrthogonalQuads(tree._position,tree._width,tree._height,tree._color));
         }
     }
-    
+
     if (needGroup)
     {
         group = new osg::Group;
@@ -684,9 +692,9 @@ osg::Node* ForestTechniqueManager::createXGraph(Cell* cell,osg::StateSet* states
         {
             group->addChild(createXGraph(itr->get(),stateset));
         }
-        
+
         if (geode) group->addChild(geode);
-        
+
     }
     if (group) return group;
     else return geode;
@@ -696,16 +704,16 @@ osg::Node* ForestTechniqueManager::createTransformGraph(Cell* cell,osg::StateSet
 {
     bool needGroup = !(cell->_cells.empty());
     bool needTrees = !(cell->_trees.empty());
-    
+
     osg::Group* transform_group = 0;
     osg::Group* group = 0;
-    
+
     if (needTrees)
     {
         transform_group = new osg::Group;
-        
+
         osg::Geometry* geometry = createOrthogonalQuads(osg::Vec3(0.0f,0.0f,0.0f),1.0f,1.0f,osg::Vec4ub(255,255,255,255));
-        
+
         for(TreeList::iterator itr=cell->_trees.begin();
             itr!=cell->_trees.end();
             ++itr)
@@ -713,7 +721,7 @@ osg::Node* ForestTechniqueManager::createTransformGraph(Cell* cell,osg::StateSet
             Tree& tree = **itr;
             osg::MatrixTransform* transform = new osg::MatrixTransform;
             transform->setMatrix(osg::Matrix::scale(tree._width,tree._width,tree._height)*osg::Matrix::translate(tree._position));
-    
+
             osg::Geode* geode = new osg::Geode;
             geode->setStateSet(stateset);
             geode->addDrawable(geometry);
@@ -721,7 +729,7 @@ osg::Node* ForestTechniqueManager::createTransformGraph(Cell* cell,osg::StateSet
             transform_group->addChild(transform);
         }
     }
-    
+
     if (needGroup)
     {
         group = new osg::Group;
@@ -731,9 +739,9 @@ osg::Node* ForestTechniqueManager::createTransformGraph(Cell* cell,osg::StateSet
         {
             group->addChild(createTransformGraph(itr->get(),stateset));
         }
-        
+
         if (transform_group) group->addChild(transform_group);
-        
+
     }
     if (group) return group;
     else return transform_group;
@@ -744,7 +752,7 @@ osg::Geometry* ForestTechniqueManager::createOrthogonalQuadsNoColor( const osg::
     // set up the coords
     osg::Vec3Array& v = *(new osg::Vec3Array(8));
     osg::Vec2Array& t = *(new osg::Vec2Array(8));
-    
+
     float rotation = random(0.0f,osg::PI/2.0f);
     float sw = sinf(rotation)*w*0.5f;
     float cw = cosf(rotation)*w*0.5f;
@@ -792,14 +800,14 @@ class ShaderGeometry : public osg::Drawable
         META_Object(osg,ShaderGeometry)
 
         typedef std::vector<osg::Vec4> PositionSizeList;
-        
+
         virtual void drawImplementation(osg::RenderInfo& renderInfo) const
         {
             for(PositionSizeList::const_iterator itr = _trees.begin();
                 itr != _trees.end();
                 ++itr)
             {
-                glColor4fv(itr->ptr());
+                renderInfo.getState()->Color((*itr)[0],(*itr)[1],(*itr)[2],(*itr)[3]);
                 _geometry->draw(renderInfo);
             }
         }
@@ -819,29 +827,241 @@ class ShaderGeometry : public osg::Drawable
             }
             return bb;
         }
-        
+
         void setGeometry(osg::Geometry* geometry)
         {
             _geometry = geometry;
         }
-        
+
         void addTree(ForestTechniqueManager::Tree& tree)
         {
             _trees.push_back(osg::Vec4(tree._position.x(), tree._position.y(), tree._position.z(), tree._height));
         }
-        
+
         osg::ref_ptr<osg::Geometry> _geometry;
 
         PositionSizeList _trees;
 
     protected:
-    
+
         virtual ~ShaderGeometry() {}
-        
+
 };
 
 osg::Geometry* shared_geometry = 0;
 
+osg::Program* createGeometryShader()
+{
+    static const char* vertSource = {
+    "#version 120\n"
+    "#extension GL_EXT_geometry_shader4 : enable\n"
+    "varying vec2 texcoord;\n"
+    "void main(void)\n"
+    "{\n"
+    "    gl_Position = gl_Vertex;\n"
+    "    texcoord = gl_MultiTexCoord0.st;\n"
+    "}\n"
+    };
+
+    static const char* geomSource = {
+    "#version 120\n"
+    "#extension GL_EXT_geometry_shader4 : enable\n"
+    "varying vec2 texcoord;\n"
+    "varying float intensity; \n"
+    "varying float red_intensity; \n"
+    "void main(void)\n"
+    "{\n"
+    "    vec4 v = gl_PositionIn[0];\n"
+    "    vec4 info = gl_PositionIn[1];\n"
+    "    intensity = info.y;\n"
+    "    red_intensity = info.z;\n"
+    "\n"
+    "    float h = info.x;\n"
+    "    float w = h*0.35;\n"
+    "    vec4 e;\n"
+    "    e = v + vec4(-w,0.0,0.0,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e; texcoord = vec2(0.0,0.0); EmitVertex();\n"
+    "    e = v + vec4(w,0.0,0.0,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e;  texcoord = vec2(1.0,0.0); EmitVertex();\n"
+    "    e = v + vec4(-w,0.0,h,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e;  texcoord = vec2(0.0,1.0); EmitVertex();\n"
+    "    e = v + vec4(w,0.0,h,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e;  texcoord = vec2(1.0,1.0); EmitVertex();\n"
+    "    EndPrimitive();\n"
+    "    e = v + vec4(0.0,-w,0.0,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e; texcoord = vec2(0.0,0.0); EmitVertex();\n"
+    "    e = v + vec4(0.0,w,0.0,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e;  texcoord = vec2(1.0,0.0); EmitVertex();\n"
+    "    e = v + vec4(0.0,-w,h,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e;  texcoord = vec2(0.0,1.0); EmitVertex();\n"
+    "    e = v + vec4(0.0,w,h,0.0);  gl_Position = gl_ModelViewProjectionMatrix * e;  texcoord = vec2(1.0,1.0); EmitVertex();\n"
+    "    EndPrimitive();\n"
+    "}\n"
+    };
+
+
+    static const char* fragSource = {
+        "uniform sampler2D baseTexture; \n"
+        "varying vec2 texcoord; \n"
+        "varying float intensity; \n"
+        "varying float red_intensity; \n"
+        "\n"
+        "void main(void) \n"
+        "{ \n"
+        "   vec4 finalColor = texture2D( baseTexture, texcoord); \n"
+        "   vec4 color = finalColor * intensity;\n"
+        "   color.w = finalColor.w;\n"
+        "   color.x *= red_intensity;\n"
+        "   gl_FragColor = color;\n"
+        "}\n"
+    };
+
+
+    osg::Program* pgm = new osg::Program;
+    pgm->setName( "osgshader2 demo" );
+
+    pgm->addShader( new osg::Shader( osg::Shader::VERTEX,   vertSource ) );
+    pgm->addShader( new osg::Shader( osg::Shader::FRAGMENT, fragSource ) );
+
+    pgm->addShader( new osg::Shader( osg::Shader::GEOMETRY, geomSource ) );
+    pgm->setParameter( GL_GEOMETRY_VERTICES_OUT_EXT, 8 );
+    pgm->setParameter( GL_GEOMETRY_INPUT_TYPE_EXT, GL_LINES );
+    pgm->setParameter( GL_GEOMETRY_OUTPUT_TYPE_EXT, GL_TRIANGLE_STRIP);
+
+    return pgm;
+}
+
+void ForestTechniqueManager::CollectTreePositions(Cell* cell, std::vector< osg::Vec3 >& positions)
+{
+    bool needGroup = !(cell->_cells.empty());
+    bool needTrees = !(cell->_trees.empty());
+
+    if (needTrees)
+    {
+        for(TreeList::iterator itr=cell->_trees.begin();
+            itr!=cell->_trees.end();
+            ++itr)
+        {
+            Tree& tree = **itr;
+            positions.push_back(tree._position);
+        }
+    }
+
+    if (needGroup)
+    {
+        for(Cell::CellList::iterator itr=cell->_cells.begin();
+            itr!=cell->_cells.end();
+            ++itr)
+        {
+            CollectTreePositions(itr->get(),positions);
+        }
+
+    }
+}
+
+osg::Node* ForestTechniqueManager::createGeometryShaderGraph(Cell* cell, osg::StateSet* dstate)
+{
+    bool needGroup = !(cell->_cells.empty());
+    bool needTrees = !(cell->_trees.empty());
+
+    osg::Geode* geode = 0;
+    osg::Group* group = 0;
+
+    if (needTrees)
+    {
+        geode = new osg::Geode;
+        geode->setStateSet(dstate);
+
+        osg::Geometry* geometry = new osg::Geometry;
+        geode->addDrawable(geometry);
+
+        osg::Vec3Array* v = new osg::Vec3Array;
+
+        for(TreeList::iterator itr=cell->_trees.begin();
+            itr!=cell->_trees.end();
+            ++itr)
+        {
+            Tree& tree = **itr;
+            v->push_back(tree._position);
+            v->push_back(osg::Vec3(/*tree._height*/30.0,(double)random(0.75f,1.15f),(double)random(1.0f,1.250f)));
+        }
+        geometry->setVertexArray( v );
+        geometry->addPrimitiveSet( new osg::DrawArrays( GL_LINES, 0, v->size() ) );
+
+        osg::StateSet* sset = geode->getOrCreateStateSet();
+        sset->setMode( GL_LIGHTING, osg::StateAttribute::OFF );
+        sset->setAttribute( createGeometryShader() );
+
+        osg::Uniform* baseTextureSampler = new osg::Uniform("baseTexture",0);
+        sset->addUniform(baseTextureSampler);
+
+    }
+
+    if (needGroup)
+    {
+        group = new osg::Group;
+        for(Cell::CellList::iterator itr=cell->_cells.begin();
+            itr!=cell->_cells.end();
+            ++itr)
+        {
+            group->addChild(createGeometryShaderGraph(itr->get(),dstate));
+        }
+
+        if (geode) group->addChild(geode);
+
+    }
+    if (group) return group;
+    else return geode;
+}
+
+osg::Node* ForestTechniqueManager::createTextureBufferGraph(Cell* cell, osg::Geometry* templateGeometry)
+{
+    bool needGroup = !(cell->_cells.empty());
+    bool needTrees = !(cell->_trees.empty());
+
+    osg::Geode* geode = 0;
+    osg::Group* group = 0;
+
+    if (needTrees)
+    {
+        osg::Geometry* geometry = (osg::Geometry*)templateGeometry->clone( osg::CopyOp::DEEP_COPY_PRIMITIVES );
+        osg::DrawArrays* primSet = dynamic_cast<osg::DrawArrays*>( geometry->getPrimitiveSet(0) );
+        primSet->setNumInstances( cell->_trees.size() );
+        geode = new osg::Geode;
+        geode->addDrawable(geometry);
+
+        osg::ref_ptr<osg::Image> treeParamsImage = new osg::Image;
+        treeParamsImage->allocateImage( 3*cell->_trees.size(), 1, 1, GL_RGBA, GL_FLOAT );
+
+        unsigned int i=0;
+        for(TreeList::iterator itr=cell->_trees.begin();
+            itr!=cell->_trees.end();
+            ++itr,++i)
+        {
+            osg::Vec4f* ptr = (osg::Vec4f*)treeParamsImage->data(3*i);
+            Tree& tree = **itr;
+            ptr[0] = osg::Vec4f(tree._position.x(),tree._position.y(),tree._position.z(),1.0);
+            ptr[1] = osg::Vec4f((float)tree._color.r()/255.0f,(float)tree._color.g()/255.0f, (float)tree._color.b()/255.0f, 1.0);
+            ptr[2] = osg::Vec4f(tree._width, tree._height, 1.0, 1.0);
+        }
+        osg::ref_ptr<osg::TextureBuffer> tbo = new osg::TextureBuffer;
+        tbo->setImage( treeParamsImage.get() );
+        tbo->setInternalFormat(GL_RGBA32F_ARB);
+        geometry->getOrCreateStateSet()->setTextureAttribute(1, tbo.get());
+        geometry->setInitialBound( cell->_bb );
+    }
+
+    if (needGroup)
+    {
+        group = new osg::Group;
+        for(Cell::CellList::iterator itr=cell->_cells.begin();
+            itr!=cell->_cells.end();
+            ++itr)
+        {
+            group->addChild(createTextureBufferGraph(itr->get(),templateGeometry));
+        }
+
+        if (geode) group->addChild(geode);
+
+    }
+    if (group) return group;
+    else return geode;
+}
+
+
 osg::Node* ForestTechniqueManager::createShaderGraph(Cell* cell,osg::StateSet* stateset)
 {
     if (shared_geometry==0)
@@ -853,18 +1073,18 @@ osg::Node* ForestTechniqueManager::createShaderGraph(Cell* cell,osg::StateSet* s
 
     bool needGroup = !(cell->_cells.empty());
     bool needTrees = !(cell->_trees.empty());
-    
+
     osg::Geode* geode = 0;
     osg::Group* group = 0;
-    
+
     if (needTrees)
     {
         geode = new osg::Geode;
-        
+
         ShaderGeometry* shader_geometry = new ShaderGeometry;
         shader_geometry->setGeometry(shared_geometry);
-        
-        
+
+
         for(TreeList::iterator itr=cell->_trees.begin();
             itr!=cell->_trees.end();
             ++itr)
@@ -877,7 +1097,7 @@ osg::Node* ForestTechniqueManager::createShaderGraph(Cell* cell,osg::StateSet* s
         geode->setStateSet(stateset);
         geode->addDrawable(shader_geometry);
     }
-    
+
     if (needGroup)
     {
         group = new osg::Group;
@@ -887,9 +1107,9 @@ osg::Node* ForestTechniqueManager::createShaderGraph(Cell* cell,osg::StateSet* s
         {
             group->addChild(createShaderGraph(itr->get(),stateset));
         }
-        
+
         if (geode) group->addChild(geode);
-        
+
     }
     if (group) return group;
     else return geode;
@@ -898,14 +1118,14 @@ osg::Node* ForestTechniqueManager::createShaderGraph(Cell* cell,osg::StateSet* s
 osg::Node* ForestTechniqueManager::createHUDWithText(const std::string& str)
 {
     osg::Geode* geode = new osg::Geode();
-    
+
     std::string timesFont("fonts/arial.ttf");
 
     // turn lighting off for the text and disable depth test to ensure its always ontop.
     osg::StateSet* stateset = geode->getOrCreateStateSet();
     stateset->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
 
-    // or disable depth test, and make sure that the hud is drawn after everything 
+    // or disable depth test, and make sure that the hud is drawn after everything
     // else so that it always appears ontop.
     stateset->setMode(GL_DEPTH_TEST,osg::StateAttribute::OFF);
     stateset->setRenderBinDetails(11,"RenderBin");
@@ -920,11 +1140,11 @@ osg::Node* ForestTechniqueManager::createHUDWithText(const std::string& str)
         text->setFont(timesFont);
         text->setPosition(position);
         text->setText(str);
-        
-        position += delta;
-    }    
 
-   
+        position += delta;
+    }
+
+
     // create the hud.
     osg::MatrixTransform* modelview_abs = new osg::MatrixTransform;
     modelview_abs->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
@@ -938,7 +1158,7 @@ osg::Node* ForestTechniqueManager::createHUDWithText(const std::string& str)
     return projection;
 }
 
-osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
+osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates, unsigned int maxNumTreesPerCell)
 {
     osg::Vec3 origin(0.0f,0.0f,0.0f);
     osg::Vec3 size(1000.0f,1000.0f,200.0f);
@@ -946,27 +1166,28 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
     std::cout<<"Creating terrain...";
     osg::ref_ptr<osg::Node> terrain = createTerrain(origin,size);
     std::cout<<"done."<<std::endl;
-    
+
     std::cout<<"Creating tree locations...";std::cout.flush();
     TreeList trees;
     createTreeList(terrain.get(),origin,size,numTreesToCreates,trees);
     std::cout<<"done."<<std::endl;
-    
+
     std::cout<<"Creating cell subdivision...";
     osg::ref_ptr<Cell> cell = new Cell;
     cell->addTrees(trees);
-    cell->divide();
-     std::cout<<"done."<<std::endl;
-   
-    
+    cell->divide(maxNumTreesPerCell);
+    std::cout<<"done."<<std::endl;
+
+
     osg::Texture2D *tex = new osg::Texture2D;
     tex->setWrap( osg::Texture2D::WRAP_S, osg::Texture2D::CLAMP );
     tex->setWrap( osg::Texture2D::WRAP_T, osg::Texture2D::CLAMP );
     tex->setImage(osgDB::readImageFile("Images/tree0.rgba"));
 
     osg::StateSet *dstate = new osg::StateSet;
-    {    
+    {
         dstate->setTextureAttributeAndModes(0, tex, osg::StateAttribute::ON );
+
         dstate->setTextureAttribute(0, new osg::TexEnv );
 
         dstate->setAttributeAndModes( new osg::BlendFunc, osg::StateAttribute::ON );
@@ -979,7 +1200,7 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
 
         dstate->setRenderingHint( osg::StateSet::TRANSPARENT_BIN );
     }
-    
+
 
     _techniqueSwitch = new osg::Switch;
 
@@ -987,11 +1208,11 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
         std::cout<<"Creating osg::Billboard based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createBillboardGraph(cell.get(),dstate));
-        group->addChild(createHUDWithText("Using osg::Billboard's to create a forest\n\nPress left cursor key to select OpenGL shader based forest\nPress right cursor key to select double quad based forest"));
+        group->addChild(createHUDWithText("Using osg::Billboard's to create a forest\n\nPress left cursor key to select geometry instancing with Texture Buffer Object\nPress right cursor key to select double quad based forest"));
         _techniqueSwitch->addChild(group);
         std::cout<<"done."<<std::endl;
     }
-    
+
     {
         std::cout<<"Creating double quad based forest...";
         osg::Group* group = new osg::Group;
@@ -1005,19 +1226,77 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
         std::cout<<"Creating osg::MatrixTransform based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createTransformGraph(cell.get(),dstate));
-        group->addChild(createHUDWithText("Using osg::MatrixTransform's to create a forest\n\nPress left cursor key to select double quad based forest\nPress right cursor key to select OpenGL shader based forest"));
+        group->addChild(createHUDWithText("Using osg::MatrixTransform's to create a forest\n\nPress left cursor key to select double quad based forest\nPress right cursor key to select osg::Vertex/FragmentProgram based forest"));
         _techniqueSwitch->addChild(group);
         std::cout<<"done."<<std::endl;
     }
 
     {
+        std::cout<<"Creating osg::Vertex/FragmentProgram based forest...";
         osg::Group* group = new osg::Group;
-        
 
-        osg::StateSet* stateset = new osg::StateSet;
+        osg::StateSet* stateset = new osg::StateSet(*dstate, osg::CopyOp::DEEP_COPY_ALL);
 
-        stateset->setTextureAttributeAndModes(0, tex, osg::StateAttribute::ON );
-        stateset->setRenderingHint( osg::StateSet::TRANSPARENT_BIN );
+        {
+            // vertex program
+            std::ostringstream vp_oss;
+            vp_oss <<
+                "!!ARBvp1.0\n"
+
+                "ATTRIB vpos = vertex.position;\n"
+                "ATTRIB vcol = vertex.color;\n"
+                "ATTRIB tc = vertex.texcoord[" << 0 << "];"
+
+                "PARAM mvp[4] = { state.matrix.mvp };\n"
+                "PARAM one = { 1.0, 1.0, 1.0, 1.0 };"
+
+                "TEMP position;\n"
+
+                // vec3 position = gl_Vertex.xyz * gl_Color.w + gl_Color.xyz;
+                "MAD position, vpos, vcol.w, vcol;\n"
+
+                // gl_Position     = gl_ModelViewProjectionMatrix * vec4(position,1.0);
+                "MOV position.w, one;\n"
+                "DP4 result.position.x, mvp[0], position;\n"
+                "DP4 result.position.y, mvp[1], position;\n"
+                "DP4 result.position.z, mvp[2], position;\n"
+                "DP4 result.position.w, mvp[3], position;\n"
+
+                // gl_FrontColor = vec4(1.0,1.0,1.0,1.0);
+                "MOV result.color.front.primary, one;\n"
+
+                // texcoord = gl_MultiTexCoord0.st;
+                "MOV result.texcoord, tc;\n"
+                "END\n";
+
+
+            // fragment program
+            std::ostringstream fp_oss;
+            fp_oss <<
+                "!!ARBfp1.0\n"
+                "TEX result.color, fragment.texcoord[" << 0 << "], texture[" << 0 << "], 2D;"
+                "END\n";
+
+            osg::ref_ptr<osg::VertexProgram> vp = new osg::VertexProgram;
+            vp->setVertexProgram(vp_oss.str());
+            stateset->setAttributeAndModes(vp.get(), osg::StateAttribute::OVERRIDE|osg::StateAttribute::ON);
+
+            osg::ref_ptr<osg::FragmentProgram> fp = new osg::FragmentProgram;
+            fp->setFragmentProgram(fp_oss.str());
+            stateset->setAttributeAndModes(fp.get(), osg::StateAttribute::OVERRIDE|osg::StateAttribute::ON);
+        }
+
+        group->addChild(createShaderGraph(cell.get(),stateset));
+        group->addChild(createHUDWithText("Using osg::Vertex/FragmentProgram to create a forest\n\nPress left cursor key to select osg::MatrixTransform's based forest\nPress right cursor key to select OpenGL shader based forest"));
+        _techniqueSwitch->addChild(group);
+        std::cout<<"done."<<std::endl;
+    }
+
+    {
+        std::cout<<"Creating OpenGL shader based forest...";
+        osg::Group* group = new osg::Group;
+
+        osg::StateSet* stateset = new osg::StateSet(*dstate, osg::CopyOp::DEEP_COPY_ALL);
 
         {
             osg::Program* program = new osg::Program;
@@ -1025,10 +1304,10 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
 
 #if 1
             // use inline shaders
-            
+
             ///////////////////////////////////////////////////////////////////
             // vertex shader using just Vec4 coefficients
-            char vertexShaderSource[] = 
+            char vertexShaderSource[] =
                 "varying vec2 texcoord;\n"
                 "\n"
                 "void main(void)\n"
@@ -1042,7 +1321,7 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
             //////////////////////////////////////////////////////////////////
             // fragment shader
             //
-            char fragmentShaderSource[] = 
+            char fragmentShaderSource[] =
                 "uniform sampler2D baseTexture; \n"
                 "varying vec2 texcoord; \n"
                 "\n"
@@ -1056,7 +1335,7 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
 
             osg::Shader* fragment_shader = new osg::Shader(osg::Shader::FRAGMENT, fragmentShaderSource);
             program->addShader(fragment_shader);
-            
+
 #else
 
             // get shaders from source
@@ -1069,19 +1348,100 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
             stateset->addUniform(baseTextureSampler);
         }
 
-        std::cout<<"Creating OpenGL shader based forest...";
         group->addChild(createShaderGraph(cell.get(),stateset));
-        group->addChild(createHUDWithText("Using OpenGL Shader to create a forest\n\nPress left cursor key to select osg::MatrixTransform based forest\nPress right cursor key to select osg::Billboard based forest"));
+        group->addChild(createHUDWithText("Using OpenGL Shader to create a forest\n\nPress left cursor key to select osg::Vertex/FragmentProgram based forest\nPress right cursor key to select osg::Vertex/Geometry/FragmentProgram based forest"));
         _techniqueSwitch->addChild(group);
         std::cout<<"done."<<std::endl;
     }
 
+    {
+        std::cout<<"Creating Geometry Shader based forest...";
+
+        osg::StateSet* stateset = new osg::StateSet(*dstate, osg::CopyOp::DEEP_COPY_ALL);
+
+        osg::Group* group = new osg::Group;
+        group->addChild(createGeometryShaderGraph(cell.get(), stateset));
+        group->addChild(createHUDWithText("Using osg::Vertex/Geometry/FragmentProgram to create a forest\n\nPress left cursor key to select OpenGL Shader based forest\nPress right cursor key to select geometry instancing with Texture Buffer Object"));
+
+        _techniqueSwitch->addChild(group);
+        std::cout<<"done."<<std::endl;
+    }
+
+    {
+        std::cout<<"Creating forest using geometry instancing and texture buffer objects ...";
+
+        osg::StateSet* stateset = new osg::StateSet(*dstate, osg::CopyOp::DEEP_COPY_ALL);
+        {
+            osg::Program* program = new osg::Program;
+            stateset->setAttribute(program);
+
+            char vertexShaderSource[] =
+                "#version 420 compatibility\n"
+                "uniform samplerBuffer dataBuffer;\n"
+                "layout(location = 0) in vec3 VertexPosition;\n"
+                "layout(location = 8) in vec3 VertexTexCoord;\n"
+                "out vec2 TexCoord;\n"
+                "out vec4 Color;\n"
+                "void main()\n"
+                "{\n"
+                "   int instanceAddress = gl_InstanceID * 3;\n"
+                "   vec3 position = texelFetch(dataBuffer, instanceAddress).xyz;\n"
+                "   Color         = texelFetch(dataBuffer, instanceAddress + 1);\n"
+                "   vec2 size     = texelFetch(dataBuffer, instanceAddress + 2).xy;\n"
+                "   mat4 mvpMatrix = gl_ModelViewProjectionMatrix *\n"
+                "        mat4( size.x, 0.0, 0.0, 0.0,\n"
+                "              0.0, size.x, 0.0, 0.0,\n"
+                "              0.0, 0.0, size.y, 0.0,\n"
+                "              position.x, position.y, position.z, 1.0);\n"
+                "   gl_Position = mvpMatrix * vec4(VertexPosition,1.0) ;\n"
+                "   TexCoord = VertexTexCoord.xy;\n"
+                "}\n";
+
+            char fragmentShaderSource[] =
+                "#version 420 core\n"
+                "uniform sampler2D baseTexture; \n"
+                "in vec2 TexCoord;\n"
+                "in vec4 Color;\n"
+                "layout(location = 0, index = 0) out vec4 FragData0;\n"
+                "void main(void) \n"
+                "{\n"
+                "    FragData0 = Color*texture(baseTexture, TexCoord);\n"
+                "}\n";
+
+            osg::Shader* vertex_shader = new osg::Shader(osg::Shader::VERTEX, vertexShaderSource);
+            program->addShader(vertex_shader);
+
+            osg::Shader* fragment_shader = new osg::Shader(osg::Shader::FRAGMENT, fragmentShaderSource);
+            program->addShader(fragment_shader);
+
+            osg::Uniform* baseTextureSampler = new osg::Uniform("baseTexture",0);
+            stateset->addUniform(baseTextureSampler);
+
+            osg::Uniform* dataBufferSampler = new osg::Uniform("dataBuffer",1);
+            stateset->addUniform(dataBufferSampler);
+        }
+
+        osg::ref_ptr<osg::Geometry> templateGeometry = createOrthogonalQuadsNoColor(osg::Vec3(0.0f,0.0f,0.0f),1.0f,1.0f);
+        templateGeometry->setUseVertexBufferObjects(true);
+        templateGeometry->setUseDisplayList(false);
+        osg::Node* textureBufferGraph = createTextureBufferGraph(cell.get(), templateGeometry.get());
+        textureBufferGraph->setStateSet( stateset );
+        osg::Group* group = new osg::Group;
+        group->addChild(textureBufferGraph);
+        group->addChild(createHUDWithText("Using geometry instancing to create a forest\n\nPress left cursor key to select osg::Vertex/Geometry/FragmentProgram based forest\nPress right cursor key to select osg::Billboard based forest"));
+
+        _techniqueSwitch->addChild(group);
+
+        std::cout<<"done."<<std::endl;
+    }
+
+
     _currentTechnique = 0;
     _techniqueSwitch->setSingleChildOn(_currentTechnique);
-    
+
 
     osg::Group* scene = new osg::Group;
-    
+
     scene->addChild(terrain.get());
     scene->addChild(_techniqueSwitch.get());
 
@@ -1093,15 +1453,19 @@ int main( int argc, char **argv )
 
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
-   
+
     // construct the viewer.
     osgViewer::Viewer viewer(arguments);
 
-    float numTreesToCreates = 10000;
-    arguments.read("--trees",numTreesToCreates);
-    
+    unsigned int numTreesToCreate = 10000;
+    arguments.read("--trees",numTreesToCreate);
+
+    unsigned int maxNumTreesPerCell = sqrtf(static_cast<float>(numTreesToCreate));
+
+    arguments.read("--trees-per-cell",maxNumTreesPerCell);
+
     osg::ref_ptr<ForestTechniqueManager> ttm = new ForestTechniqueManager;
-    
+
     // add the stats handler
     viewer.addEventHandler(new osgViewer::StatsHandler);
 
@@ -1109,7 +1473,7 @@ int main( int argc, char **argv )
     viewer.addEventHandler(new osgGA::StateSetManipulator(viewer.getCamera()->getOrCreateStateSet()));
 
     // add model to viewer.
-    viewer.setSceneData( ttm->createScene((unsigned int)numTreesToCreates) );
+    viewer.setSceneData( ttm->createScene(numTreesToCreate, maxNumTreesPerCell) );
 
 
     return viewer.run();
