@@ -94,8 +94,8 @@ public:
                     return true;
                 }
                 if (ea.getKey() == 'x') {
-                    osgDB::writeNodeFile(*ep_->getRoot(), "osgfx_model.osg");
-                    std::cout << "written nodes to \"osgfx_model.osg\"\n";
+                    osgDB::writeNodeFile(*ep_->getRoot(), "osgfx_model.osgt");
+                    std::cout << "written nodes to \"osgfx_model.osgt\"\n";
                     return true;
                 }
                 if (ea.getKey() == 'r') {
@@ -320,10 +320,16 @@ int main(int argc, char *argv[])
 
     viewer.setThreadingModel(threading);
 
+    // setup stencil buffer for Outline f/x.
+    osg::DisplaySettings::instance()->setMinimumNumStencilBits(1);
+    unsigned int clearMask = viewer.getCamera()->getClearMask();
+    viewer.getCamera()->setClearMask(clearMask | GL_STENCIL_BUFFER_BIT);
+    viewer.getCamera()->setClearStencil(0);
+ 
     // any option left unread are converted into errors to write out later.
     arguments.reportRemainingOptionsAsUnrecognized();
 
-    // report any errors if they have occured when parsing the program aguments.
+    // report any errors if they have occurred when parsing the program arguments.
     if (arguments.errors()) {
         arguments.writeErrorMessages(std::cout);
         return 1;
@@ -333,7 +339,7 @@ int main(int argc, char *argv[])
     osg::ref_ptr<osg::Node> loadedModel = osgDB::readNodeFiles(arguments);
 
     // if not loaded assume no arguments passed in, try use default mode instead.
-    if (!loadedModel) loadedModel = osgDB::readNodeFile("dumptruck.osg");
+    if (!loadedModel) loadedModel = osgDB::readNodeFile("dumptruck.osgt");
   
     if (!loadedModel)
     {
@@ -341,7 +347,7 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    // optimize the scene graph, remove rendundent nodes and state etc.
+    // optimize the scene graph, remove redundant nodes and state etc.
     osgUtil::Optimizer optimizer;
     optimizer.optimize(loadedModel.get());
 
