@@ -21,7 +21,6 @@
 #include <iostream>
 #include <list>
 #include <string>
-#include <fstream>
 #include <sstream>
 
 #include <osg/Geode>
@@ -48,6 +47,7 @@
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 
 int runApp(std::string xapp);
 
@@ -63,7 +63,7 @@ public:
     
     bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& us);
 
-    std::string pick(float x, float y);
+    std::string pick(const osgGA::GUIEventAdapter& event);
     
     void highlight(const std::string& name)
     {
@@ -83,15 +83,15 @@ bool PickHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapte
     case(osgGA::GUIEventAdapter::FRAME):
     case(osgGA::GUIEventAdapter::MOVE):
     {
-        //osg::notify(osg::NOTICE)<<"MOVE "<<ea.getX()<<ea.getY()<<std::endl;
-        std::string picked_name = pick(ea.getX(),ea.getY());
+        // osg::notify(osg::NOTICE)<<"MOVE "<<ea.getX()<<", "<<ea.getY()<<std::endl;
+        std::string picked_name = pick(ea);
         highlight(picked_name);
         return false;
     }
     case(osgGA::GUIEventAdapter::PUSH):
     {
-        //osg::notify(osg::NOTICE)<<"PUSH "<<ea.getX()<<ea.getY()<<std::endl;
-        std::string picked_name = pick(ea.getX(),ea.getY());
+        // osg::notify(osg::NOTICE)<<"PUSH "<<ea.getX()<<", "<<ea.getY()<<std::endl;
+        std::string picked_name = pick(ea);
         if (!picked_name.empty())
         {
             runApp(picked_name);
@@ -108,10 +108,10 @@ bool PickHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapte
 }
 
 
-std::string PickHandler::pick(float x, float y)
+std::string PickHandler::pick(const osgGA::GUIEventAdapter& event)
 {
     osgUtil::LineSegmentIntersector::Intersections intersections;
-    if (_viewer->computeIntersections(x, y, intersections))
+    if (_viewer->computeIntersections(event, intersections))
     {
         for(osgUtil::LineSegmentIntersector::Intersections::iterator hitr = intersections.begin();
             hitr != intersections.end();
@@ -194,6 +194,8 @@ class Xample
     {
         return app;
     }
+  private:
+      Xample() {}
 }; // end class Xample
 
 
@@ -228,12 +230,11 @@ int runApp(std::string xapp)
             
             osg::notify(osg::INFO) << "char* = " << cxapp <<std::endl;
             
-            system(cxapp);
-            return 1;
+            return system(cxapp);
         }
     }
     osg::notify(osg::INFO) << "app not found!" << std::endl;
-    return 0;
+    return 1;
 } // end printList()
 
 
@@ -249,7 +250,7 @@ void readConfFile(const char* confFile)
     }
     
 
-    std::ifstream in(fileName.c_str());
+    osgDB::ifstream in(fileName.c_str());
     if (!in)
     {
         osg::notify(osg::INFO) << "File " << fileName << " can not be opened!" << std::endl;
