@@ -47,7 +47,7 @@ int main( int argc, char **argv )
     osg::Node* rootnode = osgDB::readNodeFiles(arguments);
     
     // if not loaded assume no arguments passed in, try use default mode instead.
-    if (!rootnode) rootnode = osgDB::readNodeFile("cessnafire.osg");
+    if (!rootnode) rootnode = osgDB::readNodeFile("cessnafire.osgt");
     
     if (!rootnode)
     {
