@@ -1,13 +1,19 @@
-// -*-c++-*-
-
-/*
- * This application is open source and may be redistributed and/or modified   
- * freely and without restriction, both in commericial and non commericial
- * applications,as long as this copyright notice is maintained.
- * 
- * This application is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+/* OpenSceneGraph example, osgsequence.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
 #include <osgText/Text>
@@ -19,8 +25,9 @@
 
 #include <osgDB/ReadFile>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
+#include <iostream>
 
 // create text drawable at 'pos'
 osg::Geode* createText(const std::string& str, const osg::Vec3& pos)
@@ -114,15 +121,37 @@ osg::Sequence* createSequence(osg::ArgumentParser& arguments)
 {
     // assumes any remaining parameters are models
     osg::Sequence* seq = new osg::Sequence;
-    for (int i = 1; i < arguments.argc(); ++i)
+
+    typedef std::vector<std::string> Filenames;
+    Filenames filenames;
+    
+    if (arguments.argc() > 1)
+    {
+        for (int i = 1; i < arguments.argc(); ++i)
+        {
+            filenames.push_back(arguments[i]);
+        }
+    }
+    else
+    {
+        filenames.push_back("cow.osgt");
+        filenames.push_back("dumptruck.osgt");
+        filenames.push_back("cessna.osgt");
+        filenames.push_back("glider.osgt");
+    }
+    
+    for(Filenames::iterator itr = filenames.begin();
+        itr != filenames.end();
+        ++itr)        
     {
         // load model
-        osg::Node* node = osgDB::readNodeFile(arguments[i]);
-        if (!node) {
-            continue;
+        osg::Node* node = osgDB::readNodeFile(*itr);
+
+        if (node)
+        {
+            seq->addChild(createScaledNode(node, 100.0f));
+            seq->setTime(seq->getNumChildren()-1, 1.0f);
         }
-        seq->addChild(createScaledNode(node, 100.0f));
-        seq->setTime(seq->getNumChildren()-1, 1.0f);
     }
 
     // loop through all children
@@ -131,6 +160,8 @@ osg::Sequence* createSequence(osg::ArgumentParser& arguments)
     // real-time playback, repeat indefinitively
     seq->setDuration(1.0f, -1);
 
+    seq->setMode(osg::Sequence::START);
+
     return seq;
 }
 
@@ -149,7 +180,7 @@ public:
     {
         if (ea.getEventType() == osgGA::GUIEventAdapter::KEYDOWN) {
             switch (ea.getKey()) {
-            case 'S':
+            case 's':
                 {
                     osg::Sequence::SequenceMode mode = _seq->getMode();
                     if (mode == osg::Sequence::STOP) {
@@ -167,7 +198,7 @@ public:
                     _seq->setMode(mode);
                 }
                 break;
-            case 'L':
+            case 'l':
                 {
                     osg::Sequence::LoopMode mode;
                     int begin, end;
@@ -191,9 +222,6 @@ public:
         return false;
     }
 
-    // accept visits
-    virtual void accept(osgGA::GUIEventHandlerVisitor&) {}
-
 private:
     osg::ref_ptr<osg::Sequence> _seq;
 };
@@ -203,44 +231,9 @@ int main( int argc, char **argv )
 {
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
-
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates use of osg::Sequence.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
    
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-    
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
-    
+    osgViewer::Viewer viewer;
     // root
     osg::Group* rootNode = new osg::Group;
 
@@ -252,8 +245,8 @@ int main( int argc, char **argv )
         "- assigns a display duration to each child",
         "- can loop or swing through an interval of it's children",
         "- can repeat the interval a number of times or indefinitively",
-        "- press 'Shift-S' to start/pause/resume",
-        "- press 'Shift-L' to toggle loop/swing mode",
+        "- press 's' to start/pause/resume",
+        "- press 'l' to toggle loop/swing mode",
         NULL
     };
     rootNode->addChild(createHUD(createTextGroup(text)));
@@ -266,33 +259,7 @@ int main( int argc, char **argv )
     viewer.setSceneData(rootNode);
 
     // add event handler to control sequence
-    viewer.getEventHandlerList().push_front(new SequenceEventHandler(seq));
+    viewer.addEventHandler(new SequenceEventHandler(seq));
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
