@@ -17,10 +17,12 @@
 */
 
 #include <osgViewer/Viewer>
+#include <osgDB/fstream>
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
 #include <osgUtil/Optimizer>
 
+#include <osg/ImageStream>
 #include <osg/Geode>
 #include <osg/Notify>
 #include <osg/MatrixTransform>
@@ -32,15 +34,53 @@
 
 typedef std::vector<std::string> FileList;
 
+#include <osg/Program>
+#include <osg/Shader>
+
+osg::StateSet* createColorToGreyscaleStateSet()
+{
+    osg::StateSet* stateset = new osg::StateSet;
+
+    osg::Program* program = new osg::Program;
+    stateset->setAttribute(program);
+
+    const char* fragSource =
+    {
+        "uniform sampler2D baseTexture;\n"
+        "uniform mat4 colorMatrix;\n"
+        "void main(void)\n"
+        "{\n"
+        "    vec4 color = texture2D( baseTexture, gl_TexCoord[0].st );\n"
+        "    gl_FragColor = colorMatrix * color;\n"
+        "}\n"
+    };
+    program->addShader(new osg::Shader(osg::Shader::FRAGMENT, fragSource));
+
+    stateset->addUniform(new osg::Uniform("baseTexture",0));
+
+    osg::Matrixf colorMatrix(
+        0.3f, 0.3f, 0.3f, 0.0f,
+        0.59f, 0.59f, 0.59f, 0.0f,
+        0.11f, 0.11f, 0.11f, 0.0f,
+        0.0f, 0.0f, 0.0f, 1.0f
+    );
+
+    stateset->addUniform(new osg::Uniform("colorMatrix",colorMatrix));
+
+    return stateset;
+}
+
+
 osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s,float t, float radius, float height, float length)
 {
+    bool flip = image->getOrigin()==osg::Image::TOP_LEFT;
 
     int numSegments = 20;
     float Theta = length/radius;
     float dTheta = Theta/(float)(numSegments-1);
-    
+
     float ThetaZero = height*s/(t*radius);
-    
+
     // set up the texture.
     osg::Texture2D* texture = new osg::Texture2D;
     texture->setFilter(osg::Texture2D::MIN_FILTER,osg::Texture2D::LINEAR);
@@ -63,7 +103,7 @@ osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s
 
     osg::Vec3Array* coords = new osg::Vec3Array();
     osg::Vec2Array* tcoords = new osg::Vec2Array();
-   
+
     int i;
     float angle = -Theta/2.0f;
     for(i=0;
@@ -72,9 +112,9 @@ osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s
     {
         coords->push_back(osg::Vec3(sinf(angle)*radius,cosf(angle)*radius,height*0.5f)); // top
         coords->push_back(osg::Vec3(sinf(angle)*radius,cosf(angle)*radius,-height*0.5f)); // bottom.
-        
-        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f,1.0f)); // top
-        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f,0.0f)); // bottom.
+
+        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f, flip ? 0.0f : 1.0f)); // top
+        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f, flip ? 1.0f : 0.0f)); // bottom.
 
     }
 
@@ -83,12 +123,11 @@ osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s
 
     osg::DrawArrays* elements = new osg::DrawArrays(osg::PrimitiveSet::QUAD_STRIP,0,coords->size());
 
-    
+
 
     geom->setVertexArray(coords);
     geom->setTexCoordArray(0,tcoords);
-    geom->setColorArray(colors);
-    geom->setColorBinding(osg::Geometry::BIND_OVERALL);
+    geom->setColorArray(colors, osg::Array::BIND_OVERALL);
 
     geom->addPrimitiveSet(elements);
 
@@ -100,33 +139,41 @@ osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s
 
 }
 
-osg::Group * loadImages(std::string image1, std::string image2, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length) {
-        osg::ref_ptr<osg::Image> imageLeft = osgDB::readImageFile(image1);
-        osg::ref_ptr<osg::Image> imageRight = osgDB::readImageFile(image2);
-        if (imageLeft.valid() && imageRight.valid())
-        {
-            float average_s = (imageLeft->s()+imageRight->s())*0.5f;
-            float average_t = (imageLeft->t()+imageRight->t())*0.5f;
-            osg::Geode* geodeLeft = createSectorForImage(imageLeft.get(),texmatLeft,average_s,average_t, radius, height, length);
-            geodeLeft->setNodeMask(0x01);
+osg::Group * loadImages(std::string image1, std::string image2, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length)
+{
+    osg::ref_ptr<osg::Image> imageLeft = osgDB::readImageFile(image1);
+    osg::ref_ptr<osg::Image> imageRight = osgDB::readImageFile(image2);
+    if (imageLeft.valid() && imageRight.valid())
+    {
+	osg::ImageStream* streamLeft = dynamic_cast<osg::ImageStream*>(imageLeft.get());
+	if (streamLeft) streamLeft->play();
 
-            osg::Geode* geodeRight = createSectorForImage(imageRight.get(),texmatRight,average_s,average_t, radius, height, length);
-            geodeRight->setNodeMask(0x02);
+	osg::ImageStream* streamRight = dynamic_cast<osg::ImageStream*>(imageRight.get());
+	if (streamRight) streamRight->play();
 
-            osg::Group * imageGroup = new osg::Group;
-            
-            imageGroup->addChild(geodeLeft);
-            imageGroup->addChild(geodeRight);
-            return imageGroup;
-        }
-        else
-        {
-            std::cout << "Warning: Unable to load both image files, '"<<image1<<"' & '"<<image2<<"', required for stereo imaging."<<std::endl;
-            return 0;
-        }
+
+	float average_s = (imageLeft->s()+imageRight->s())*0.5f;
+	float average_t = (imageLeft->t()+imageRight->t())*0.5f;
+	osg::Geode* geodeLeft = createSectorForImage(imageLeft.get(),texmatLeft,average_s,average_t, radius, height, length);
+	geodeLeft->setNodeMask(0x01);
+
+	osg::Geode* geodeRight = createSectorForImage(imageRight.get(),texmatRight,average_s,average_t, radius, height, length);
+	geodeRight->setNodeMask(0x02);
+
+	osg::Group * imageGroup = new osg::Group;
+
+	imageGroup->addChild(geodeLeft);
+	imageGroup->addChild(geodeRight);
+	return imageGroup;
+    }
+    else
+    {
+	std::cout << "Warning: Unable to load both image files, '"<<image1<<"' & '"<<image2<<"', required for stereo imaging."<<std::endl;
+	return 0;
+    }
 }
 
-// create a switch containing a set of child each containing a 
+// create a switch containing a set of child each containing a
 // stereo image pair.
 osg::Switch* createScene(FileList fileList, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length)
 {
@@ -154,7 +201,7 @@ class SlideEventHandler : public osgGA::GUIEventHandler
 public:
 
     SlideEventHandler();
-    
+
     META_Object(osgStereImageApp,SlideEventHandler);
 
 
@@ -164,17 +211,17 @@ public:
 
 
     virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&);
-    
+
     virtual void getUsage(osg::ApplicationUsage& usage) const;
 
     virtual void operator()(osg::Node* node, osg::NodeVisitor* nv);
-    
+
     void nextSlide();
-    
+
     void previousSlide();
-    
+
     void scaleImage(float s);
-    
+
     void offsetImage(float ds,float dt);
 
     void rotateImage(float rx,float ry);
@@ -202,7 +249,7 @@ protected:
     float                       _initSeperationY;
     float                       _currentSeperationY;
     FileList                     _fileList;
-        
+
 };
 
 SlideEventHandler::SlideEventHandler():
@@ -226,8 +273,8 @@ void SlideEventHandler::set(osg::Switch* sw, float offsetX, float offsetY, osg::
     _texmatRight = texmatRight;
 
     _timePerSlide = timePerSlide;
-    _autoSteppingActive = autoSteppingActive;    
-    
+    _autoSteppingActive = autoSteppingActive;
+
     _initSeperationX = offsetX;
     _currentSeperationX = _initSeperationX;
 
@@ -255,8 +302,8 @@ void SlideEventHandler::set(FileList fileList, osg::Switch* sw, float offsetX, f
     _length=length;
 
     _timePerSlide = timePerSlide;
-    _autoSteppingActive = autoSteppingActive;    
-    
+    _autoSteppingActive = autoSteppingActive;
+
     _initSeperationX = offsetX;
     _currentSeperationX = _initSeperationX;
 
@@ -279,22 +326,22 @@ bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
                 _previousTime = ea.getTime();
                 return true;
             }
-            else if ((ea.getKey()=='n') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_Right)) 
+            else if ((ea.getKey()=='n') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_Right))
             {
                 nextSlide();
                 return true;
             }
-            else if ((ea.getKey()=='p') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_Left)) 
+            else if ((ea.getKey()=='p') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_Left))
             {
                 previousSlide();
                 return true;
             }
-            else if ((ea.getKey()=='w') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Add)) 
+            else if ((ea.getKey()=='w') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Add))
             {
                 scaleImage(0.99f);
                 return true;
             }
-            else if ((ea.getKey()=='s') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Subtract)) 
+            else if ((ea.getKey()=='s') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Subtract))
             {
                 scaleImage(1.01f);
                 return true;
@@ -331,15 +378,15 @@ bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
         {
             static float px = ea.getXnormalized();
             static float py = ea.getYnormalized();
-            
+
             float dx = ea.getXnormalized()-px;
             float dy = ea.getYnormalized()-py;
-            
+
             px = ea.getXnormalized();
             py = ea.getYnormalized();
-            
+
             rotateImage(dx,dy);
-            
+
             return true;
         }
 
@@ -367,7 +414,7 @@ void SlideEventHandler::operator()(osg::Node* node, osg::NodeVisitor* nv)
     if (_autoSteppingActive && nv->getFrameStamp())
     {
         double time = nv->getFrameStamp()->getSimulationTime();
-        
+
         if (_firstTraversal)
         {
             _firstTraversal = false;
@@ -376,10 +423,10 @@ void SlideEventHandler::operator()(osg::Node* node, osg::NodeVisitor* nv)
         else if (time-_previousTime>_timePerSlide)
         {
             _previousTime = time;
-            
+
             nextSlide();
         }
-        
+
     }
 
     traverse(node,nv);
@@ -413,7 +460,7 @@ void SlideEventHandler::previousSlide()
         else --_activeSlide;
         osg::ref_ptr<osg::Group> images = loadImages(_fileList[2*_activeSlide],_fileList[2*_activeSlide+1],_texmatLeft.get(),_texmatRight.get(),_radius,_height,_length);
         if (images.valid()) _switch->replaceChild(_switch->getChild(0),images.get());
-    } else {    
+    } else {
         if (_activeSlide==0) _activeSlide = _switch->getNumChildren()-1;
         else --_activeSlide;
 
@@ -455,7 +502,7 @@ int main( int argc, char **argv )
 {
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
-    
+
     // set up the usage document, in case we need to print out how to use this program.
     arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates use node masks to create stereo images.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] image_file_left_eye image_file_right_eye");
@@ -470,10 +517,10 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("--CullDrawThreadPerContext","Select CullDrawThreadPerContext threading model for viewer.");
     arguments.getApplicationUsage()->addCommandLineOption("--DrawThreadPerContext","Select DrawThreadPerContext threading model for viewer.");
     arguments.getApplicationUsage()->addCommandLineOption("--CullThreadPerCameraDrawThreadPerContext","Select CullThreadPerCameraDrawThreadPerContext threading model for viewer.");
-    
+
 
     // construct the viewer.
-    osgViewer::Viewer viewer;
+    osgViewer::Viewer viewer(arguments);
 
     // register the handler to add keyboard and mouse handling.
     SlideEventHandler* seh = new SlideEventHandler();
@@ -499,13 +546,13 @@ int main( int argc, char **argv )
     FileList fileList;
     // extract the filenames from the a file, one filename per line.
     while (arguments.read("-files",filename)) {
-        std::ifstream is(filename.c_str());
+        osgDB::ifstream is(filename.c_str());
         if (is) {
                 std::string line;
                 while (std::getline(is,line,'\n')) fileList.push_back(line);
                 is.close();
             }
-    
+
     }
 
     // if user request help write it out to cout.
@@ -532,7 +579,7 @@ int main( int argc, char **argv )
         arguments.writeErrorMessages(std::cout);
         return 1;
     }
-    
+
     // extract the filenames from the arguments list.
     for(int pos=1;pos<arguments.argc();++pos)
     {
@@ -568,7 +615,8 @@ int main( int argc, char **argv )
     if (!onDisk)  rootNode = createScene(fileList,texmatLeft,texmatRight,radius,height,length);
     else rootNode=new osg::Switch();
 
-    //osgDB::writeNodeFile(*rootNode,"test.osg");
+    //osgDB::writeNodeFile(*rootNode,"test.osgt");
+
 
 
     viewer.getCamera()->setCullMask(0xffffffff);
@@ -578,6 +626,12 @@ int main( int argc, char **argv )
     // set up the use of stereo by default.
     osg::DisplaySettings::instance()->setStereo(true);
 
+    if (osg::DisplaySettings::instance()->getStereoMode()==osg::DisplaySettings::ANAGLYPHIC)
+    {
+        rootNode->setStateSet(createColorToGreyscaleStateSet());
+    }
+
+
     // set the scene to render
     viewer.setSceneData(rootNode.get());
 
@@ -601,19 +655,19 @@ int main( int argc, char **argv )
     // set up the SlideEventHandler.
     if (onDisk) seh->set(fileList,rootNode.get(),offsetX,offsetY,texmatLeft,texmatRight,radius,height,length,timeDelayBetweenSlides,autoSteppingActive);
     else seh->set(rootNode.get(),offsetX,offsetY,texmatLeft,texmatRight,timeDelayBetweenSlides,autoSteppingActive);
-    
+
     osg::Matrix homePosition;
     homePosition.makeLookAt(osg::Vec3(0.0f,0.0f,0.0f),osg::Vec3(0.0f,1.0f,0.0f),osg::Vec3(0.0f,0.0f,1.0f));
-        
+
     while( !viewer.done() )
     {
         viewer.getCamera()->setViewMatrix(homePosition);
 
         // fire off the cull and draw traversals of the scene.
         viewer.frame();
-        
+
     }
-    
+
     return 0;
 }
 
