@@ -64,9 +64,6 @@ Win32ThreadPrivateData::~Win32ThreadPrivateData()
 {
 }
 
-const std::string OPENTHREAD_VERSION_STRING = "OpenThread v1.2preAlpha, WindowThreads (Public Implementation)";
-
-
 //-----------------------------------------------------------------------------
 // Initialize thread master priority level
 //
@@ -204,12 +201,15 @@ namespace OpenThreads {
             return status!=0;
         };
     };
-};
+}
 
 Thread* Thread::CurrentThread()
 {
-    return (Thread* )TlsGetValue(Win32ThreadPrivateData::TLS.getId());
-};
+    DWORD ID = Win32ThreadPrivateData::TLS.getId();
+    if (ID == TLS_OUT_OF_INDEXES)
+        return 0;
+    return (Thread* )TlsGetValue(ID);
+}
 
 //----------------------------------------------------------------------------
 //
@@ -219,7 +219,7 @@ Thread* Thread::CurrentThread()
 //
 int Thread::SetConcurrency(int) {
     return -1;
-};
+}
 
 //----------------------------------------------------------------------------
 //
@@ -229,7 +229,20 @@ int Thread::SetConcurrency(int) {
 //
 int Thread::GetConcurrency() {
     return -1;
-};
+}
+
+Win32ThreadPrivateData::Win32ThreadPrivateData()
+{
+    stackSize = 0;
+    isRunning = false;
+    cancelMode = 0;
+    uniqueId = 0;
+    threadPriority = Thread::THREAD_PRIORITY_DEFAULT;
+    threadPolicy = Thread::THREAD_SCHEDULE_DEFAULT;
+    detached = false;
+    cancelEvent.set(CreateEvent(NULL,TRUE,FALSE,NULL));
+    cpunum = -1;
+}
 
 //----------------------------------------------------------------------------
 //
@@ -244,16 +257,6 @@ Thread::Thread() {
 
     Win32ThreadPrivateData *pd = new Win32ThreadPrivateData();
 
-    pd->stackSize = 0;
-    pd->isRunning = false;
-    pd->cancelMode = 0;
-    pd->uniqueId = 0;
-    pd->threadPriority = Thread::THREAD_PRIORITY_DEFAULT;
-    pd->threadPolicy = Thread::THREAD_SCHEDULE_DEFAULT;
-    pd->detached = false;
-    pd->cancelEvent.set(CreateEvent(NULL,TRUE,FALSE,NULL));
-    pd->cpunum = -1;
-
     _prvData = static_cast<void *>(pd);
 }
 
@@ -341,7 +344,7 @@ int Thread::start() {
     
     pd->threadStartedBlock.reset();
 
-    pd->tid.set( (void*)_beginthreadex(NULL,pd->stackSize,ThreadPrivateActions::StartThread,static_cast<void *>(this),0,&ID));
+    pd->tid.set( (void*)_beginthreadex(NULL,static_cast<unsigned>(pd->stackSize),ThreadPrivateActions::StartThread,static_cast<void *>(this),0,&ID));
 
     pd->uniqueId = (int)ID;
 
@@ -678,8 +681,6 @@ int OpenThreads::GetNumberOfProcessors()
 
 int OpenThreads::SetProcessorAffinityOfCurrentThread(unsigned int cpunum)
 {
-    if (cpunum<0) return -1;
-    
     Thread::Init();
 
     Thread* thread = Thread::CurrentThread();
