@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 #include <osg/ClipNode>
@@ -17,15 +17,18 @@
 using namespace osg;
 
 ClipNode::ClipNode():
-    _value(StateAttribute::ON)
+    _value(StateAttribute::ON),
+    _referenceFrame(RELATIVE_RF)
 {
-    _stateset = new StateSet;
+    setStateSet(new StateSet);
 }
 
 ClipNode::ClipNode(const ClipNode& cn, const CopyOp& copyop):
     Group(cn,copyop),
-    _value(cn._value)
+    _value(cn._value),
+    _referenceFrame(cn._referenceFrame)
 {
+    setStateSet(new StateSet);
     for(ClipPlaneList::const_iterator itr=cn._planes.begin();
         itr!=cn._planes.end();
         ++itr)
@@ -42,6 +45,12 @@ ClipNode::~ClipNode()
 {
 }
 
+
+void ClipNode::setReferenceFrame(ReferenceFrame rf)
+{
+    _referenceFrame = rf;
+}
+
 // Create a 6 clip planes to create a clip box.
 void ClipNode::createClipBox(const BoundingBox& bb,unsigned int clipPlaneNumberBase)
 {
@@ -64,7 +73,7 @@ void ClipNode::createClipBox(const BoundingBox& bb,unsigned int clipPlaneNumberB
     _stateset->setAssociatedModes(_planes.back().get(), _value);
 }
 
-// Add a ClipPlane to a ClipNode. Return true if plane is added, 
+// Add a ClipPlane to a ClipNode. Return true if plane is added,
 // return false if plane already exists in ClipNode, or clipplane is false.
 bool ClipNode::addClipPlane(ClipPlane* clipplane)
 {
@@ -84,7 +93,7 @@ bool ClipNode::addClipPlane(ClipPlane* clipplane)
     }
 }
 
-// Remove ClipPlane from a ClipNode. Return true if plane is removed, 
+// Remove ClipPlane from a ClipNode. Return true if plane is removed,
 // return false if plane does not exists in ClipNode.
 bool ClipNode::removeClipPlane(ClipPlane* clipplane)
 {
@@ -104,7 +113,7 @@ bool ClipNode::removeClipPlane(ClipPlane* clipplane)
     }
 }
 
-// Remove ClipPlane, at specified index, from a ClipNode. Return true if plane is removed, 
+// Remove ClipPlane, at specified index, from a ClipNode. Return true if plane is removed,
 // return false if plane does not exists in ClipNode.
 bool ClipNode::removeClipPlane(unsigned int pos)
 {
@@ -136,7 +145,8 @@ void ClipNode::setStateSetModes(StateSet& stateset,StateAttribute::GLModeValue v
 void ClipNode::setLocalStateSetModes(StateAttribute::GLModeValue value)
 {
     _value = value;
-    if (!_stateset) _stateset = new StateSet;
+    if (!_stateset) setStateSet(new StateSet);
+
     setStateSetModes(*_stateset,value);
 }
 
