@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 #include <osg/Notify>
@@ -23,7 +23,7 @@
 
 using namespace osg;
 
-// static cache of deleted fragment programs which can only 
+// static cache of deleted fragment programs which can only
 // by completely deleted once the appropriate OpenGL context
 // is set.
 typedef std::list<GLuint> FragmentProgramObjectList;
@@ -69,7 +69,7 @@ void FragmentProgram::flushDeletedFragmentProgramObjects(unsigned int contextID,
             elapsedTime = timer.delta_s(start_tick,timer.tick());
         }
     }
-        
+
     availableTime -= elapsedTime;
 }
 
@@ -91,13 +91,13 @@ FragmentProgram::FragmentProgram(const FragmentProgram& vp,const CopyOp& copyop)
 {
     _fragmentProgram = vp._fragmentProgram;
 
-    for( LocalParamList::const_iterator itr = vp._programLocalParameters.begin(); 
+    for( LocalParamList::const_iterator itr = vp._programLocalParameters.begin();
         itr != vp._programLocalParameters.end(); ++itr )
     {
         _programLocalParameters[itr->first] = itr->second;
     }
 
-    for( MatrixList::const_iterator mitr = vp._matrixList.begin(); 
+    for( MatrixList::const_iterator mitr = vp._matrixList.begin();
         mitr != vp._matrixList.end(); ++mitr )
     {
         _matrixList[mitr->first] = mitr->second;
@@ -125,6 +125,8 @@ void FragmentProgram::dirtyFragmentProgramObject()
 
 void FragmentProgram::apply(State& state) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
+
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);
 
@@ -148,19 +150,19 @@ void FragmentProgram::apply(State& state) const
                                      _fragmentProgram.length(), _fragmentProgram.c_str());
 
         // Check for errors
-        GLint errorposition;
+        GLint errorposition = 0;
         glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorposition);
         if (errorposition != -1)
         {
-            notify(osg::FATAL) << "FragmentProgram: " << glGetString(GL_PROGRAM_ERROR_STRING_ARB) << std::endl;
+            OSG_FATAL << "FragmentProgram: " << glGetString(GL_PROGRAM_ERROR_STRING_ARB) << std::endl;
 
             std::string::size_type start = _fragmentProgram.rfind('\n', errorposition);
             std::string::size_type stop  = _fragmentProgram.find('\n', errorposition);
             if (start!=std::string::npos && stop!=std::string::npos)
             {
-                notify(osg::FATAL) << "             : " << _fragmentProgram.substr(start+1, stop-start-2) << std::endl;
+                OSG_FATAL << "             : " << _fragmentProgram.substr(start+1, stop-start-2) << std::endl;
                 std::string pointAtproblem(errorposition-(start+1), ' ');
-                notify(osg::FATAL) << "             : " << pointAtproblem << '^' << std::endl;
+                OSG_FATAL << "             : " << pointAtproblem << '^' << std::endl;
             }
             return;
         }
@@ -188,6 +190,9 @@ void FragmentProgram::apply(State& state) const
         }
         glMatrixMode(GL_MODELVIEW); // restore matrix mode
     }
+#else
+    OSG_NOTICE<<"Warning: FragmentProgram::apply(State&) - not supported."<<std::endl;
+#endif
 }
 
 void FragmentProgram::resizeGLObjectBuffers(unsigned int maxSize)
@@ -272,8 +277,8 @@ void FragmentProgram::Extensions::glBindProgram(GLenum target, GLuint id) const
     }
     else
     {
-        notify(WARN)<<"Error: glBindProgram not supported by OpenGL driver"<<std::endl;
-    }    
+        OSG_WARN<<"Error: glBindProgram not supported by OpenGL driver"<<std::endl;
+    }
 }
 
 void FragmentProgram::Extensions::glGenPrograms(GLsizei n, GLuint *programs) const
@@ -284,7 +289,7 @@ void FragmentProgram::Extensions::glGenPrograms(GLsizei n, GLuint *programs) con
     }
     else
     {
-        notify(WARN)<<"Error: glGenPrograms not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glGenPrograms not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -296,7 +301,7 @@ void FragmentProgram::Extensions::glDeletePrograms(GLsizei n, GLuint *programs)
     }
     else
     {
-        notify(WARN)<<"Error: glDeletePrograms not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glDeletePrograms not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -304,11 +309,11 @@ void FragmentProgram::Extensions::glProgramString(GLenum target, GLenum format,
 {
     if (_glProgramString)
     {
-        _glProgramString(target,format, len, string); 
+        _glProgramString(target,format, len, string);
     }
     else
     {
-        notify(WARN)<<"Error: glProgramString not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glProgramString not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -320,6 +325,6 @@ void FragmentProgram::Extensions::glProgramLocalParameter4fv(GLenum target, GLui
     }
     else
     {
-        notify(WARN)<<"Error: glProgramLocalParameter4fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glProgramLocalParameter4fv not supported by OpenGL driver"<<std::endl;
     }
 }
