@@ -40,7 +40,7 @@ using namespace osg;
 
 // Use a static reference pointer to hold the window system interface.
 // Wrap this within a function, in order to control the order in which
-// the static pointer's constructor is executed. 
+// the static pointer's constructor is executed.
 
 static ref_ptr<GraphicsContext::WindowingSystemInterface> &windowingSystemInterfaceRef()
 {
@@ -72,11 +72,11 @@ GraphicsContext* GraphicsContext::createGraphicsContext(Traits* traits)
     {
         // catch any undefined values.
         if (traits) traits->setUndefinedScreenDetailsToDefaultScreen();
-        
+
         return wsref->createGraphicsContext(traits);
     }
     else
-        return 0;    
+        return 0;
 }
 
 GraphicsContext::ScreenIdentifier::ScreenIdentifier():
@@ -112,9 +112,12 @@ void GraphicsContext::ScreenIdentifier::setScreenIdentifier(const std::string& d
 {
     std::string::size_type colon = displayName.find_last_of(':');
     std::string::size_type point = displayName.find_last_of('.');
-    
-    if (point!=std::string::npos && 
-        colon==std::string::npos && 
+
+    // handle the case where the host name is supplied with '.' such as 127.0.0.1:0  with only DisplayNum provided
+    // here the point to picks up on the .1 from the host name, rather then demarking the DisplayNum/ScreenNum as
+    // no ScreenNum is provided, hence no . in the rhs of the :
+    if (point!=std::string::npos &&
+        colon!=std::string::npos &&
         point < colon) point = std::string::npos;
 
     if (colon==std::string::npos)
@@ -125,7 +128,7 @@ void GraphicsContext::ScreenIdentifier::setScreenIdentifier(const std::string& d
     {
         hostName = displayName.substr(0,colon);
     }
-    
+
     std::string::size_type startOfDisplayNum = (colon==std::string::npos) ? 0 : colon+1;
     std::string::size_type endOfDisplayNum = (point==std::string::npos) ?  displayName.size() : point;
 
@@ -147,7 +150,7 @@ void GraphicsContext::ScreenIdentifier::setScreenIdentifier(const std::string& d
         screenNum = -1;
     }
 
-#if 0    
+#if 0
     OSG_NOTICE<<"   hostName ["<<hostName<<"]"<<std::endl;
     OSG_NOTICE<<"   displayNum "<<displayNum<<std::endl;
     OSG_NOTICE<<"   screenNum "<<screenNum<<std::endl;
@@ -195,7 +198,7 @@ GraphicsContext::Traits::Traits(DisplaySettings* ds):
     {
         alpha = ds->getMinimumNumAlphaBits();
         stencil = ds->getMinimumNumStencilBits();
-        sampleBuffers = ds->getMultiSamples();
+        if (ds->getMultiSamples()!=0) sampleBuffers = 1;
         samples = ds->getNumMultiSamples();
         if (ds->getStereo())
         {
@@ -208,7 +211,7 @@ GraphicsContext::Traits::Traits(DisplaySettings* ds):
                 default: break;
             }
         }
-        
+
         glContextVersion = ds->getGLContextVersion();
         glContextFlags = ds->getGLContextFlags();
         glContextProfileMask = ds->getGLContextProfileMask();
@@ -217,6 +220,18 @@ GraphicsContext::Traits::Traits(DisplaySettings* ds):
     }
 }
 
+bool GraphicsContext::Traits::getContextVersion(unsigned int& major, unsigned int& minor) const
+{
+    if (glContextVersion.empty()) return false;
+
+    std::istringstream istr( glContextVersion );
+    unsigned char dot;
+    istr >> major >> dot >> minor;
+
+    return true;
+}
+
+
 class ContextData
 {
 public:
@@ -225,7 +240,7 @@ public:
         _numContexts(0) {}
 
     unsigned int _numContexts;
-    
+
     void incrementUsageCount() {  ++_numContexts; }
 
     void decrementUsageCount()
@@ -237,11 +252,11 @@ public:
         if (_numContexts <= 1 && _compileContext.valid())
         {
             OSG_INFO<<"resetting compileContext "<<_compileContext.get()<<" refCount "<<_compileContext->referenceCount()<<std::endl;
-            
+
             _compileContext = 0;
         }
     }
-    
+
     osg::ref_ptr<osg::GraphicsContext> _compileContext;
 
 };
@@ -255,7 +270,7 @@ static GraphicsContext::GraphicsContexts s_registeredContexts;
 unsigned int GraphicsContext::createNewContextID()
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    
+
     // first check to see if we can reuse contextID;
     for(ContextIDMap::iterator itr = s_contextIDMap.begin();
         itr != s_contextIDMap.end();
@@ -275,15 +290,15 @@ unsigned int GraphicsContext::createNewContextID()
 
     unsigned int contextID = s_contextIDMap.size();
     s_contextIDMap[contextID]._numContexts = 1;
-    
+
     OSG_INFO<<"GraphicsContext::createNewContextID() creating contextID="<<contextID<<std::endl;
     OSG_INFO<<"Updating the MaxNumberOfGraphicsContexts to "<<contextID+1<<std::endl;
 
-    // update the the maximum number of graphics contexts, 
+    // update the the maximum number of graphics contexts,
     // to ensure that texture objects and display buffers are configured to the correct size.
     osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts( contextID + 1 );
 
-    return contextID;    
+    return contextID;
 }
 
 unsigned int GraphicsContext::getMaxContextID()
@@ -303,10 +318,10 @@ unsigned int GraphicsContext::getMaxContextID()
 void GraphicsContext::incrementContextIDUsageCount(unsigned int contextID)
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    
-    OSG_INFO<<"GraphicsContext::incrementContextIDUsageCount("<<contextID<<") to "<<s_contextIDMap[contextID]._numContexts<<std::endl;
 
     s_contextIDMap[contextID].incrementUsageCount();
+
+    OSG_INFO<<"GraphicsContext::incrementContextIDUsageCount("<<contextID<<") to "<<s_contextIDMap[contextID]._numContexts<<std::endl;
 }
 
 void GraphicsContext::decrementContextIDUsageCount(unsigned int contextID)
@@ -321,7 +336,7 @@ void GraphicsContext::decrementContextIDUsageCount(unsigned int contextID)
     else
     {
         OSG_NOTICE<<"Warning: decrementContextIDUsageCount("<<contextID<<") called on expired contextID."<<std::endl;
-    } 
+    }
 
     OSG_INFO<<"GraphicsContext::decrementContextIDUsageCount("<<contextID<<") to "<<s_contextIDMap[contextID]._numContexts<<std::endl;
 
@@ -375,7 +390,7 @@ GraphicsContext::GraphicsContexts GraphicsContext::getRegisteredGraphicsContexts
     }
 
     OSG_INFO<<"GraphicsContext::getRegisteredGraphicsContexts "<<contextID<<" contexts.size()="<<contexts.size()<<std::endl;
-    
+
     return contexts;
 }
 
@@ -383,14 +398,14 @@ GraphicsContext* GraphicsContext::getOrCreateCompileContext(unsigned int context
 {
     OSG_NOTICE<<"GraphicsContext::createCompileContext."<<std::endl;
 
-    {    
+    {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
         if (s_contextIDMap[contextID]._compileContext.valid()) return s_contextIDMap[contextID]._compileContext.get();
     }
-    
+
     GraphicsContext::GraphicsContexts contexts = GraphicsContext::getRegisteredGraphicsContexts(contextID);
     if (contexts.empty()) return 0;
-    
+
     GraphicsContext* src_gc = contexts.front();
     const osg::GraphicsContext::Traits* src_traits = src_gc->getTraits();
 
@@ -410,7 +425,7 @@ GraphicsContext* GraphicsContext::getOrCreateCompileContext(unsigned int context
 
     osg::ref_ptr<osg::GraphicsContext> gc = osg::GraphicsContext::createGraphicsContext(traits);
     if (gc.valid() && gc->realize())
-    {    
+    {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
         s_contextIDMap[contextID]._compileContext = gc;
         OSG_NOTICE<<"   succeeded GraphicsContext::createCompileContext."<<std::endl;
@@ -420,7 +435,7 @@ GraphicsContext* GraphicsContext::getOrCreateCompileContext(unsigned int context
     {
         return 0;
     }
-    
+
 }
 
 void GraphicsContext::setCompileContext(unsigned int contextID, GraphicsContext* gc)
@@ -497,7 +512,7 @@ bool GraphicsContext::realize()
         return true;
     }
     else
-    {   
+    {
         return false;
     }
 }
@@ -508,17 +523,17 @@ void GraphicsContext::close(bool callCloseImplementation)
 
     // switch off the graphics thread...
     setGraphicsThread(0);
-    
-    
+
+
     bool sharedContextExists = false;
-    
+
     if (_state.valid())
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
         if (s_contextIDMap[_state->getContextID()]._numContexts>1) sharedContextExists = true;
     }
 
-    // release all the OpenGL objects in the scene graphs associated with this 
+    // release all the OpenGL objects in the scene graphs associated with this
     for(Cameras::iterator itr = _cameras.begin();
         itr != _cameras.end();
         ++itr)
@@ -532,20 +547,39 @@ void GraphicsContext::close(bool callCloseImplementation)
     }
 
 
+    bool contextIDValid = _state.valid();
+    unsigned int contextID = _state.valid() ? _state->getContextID() : ~0U;
+
     if (callCloseImplementation && _state.valid() && isRealized())
     {
         OSG_INFO<<"Closing still viable window "<<sharedContextExists<<" _state->getContextID()="<<_state->getContextID()<<std::endl;
 
         if (makeCurrent())
         {
-        
-            OSG_INFO<<"Doing delete of GL objects"<<std::endl;
+            if ( !sharedContextExists )
+            {
+                OSG_INFO<<"Doing delete of GL objects"<<std::endl;
 
-            osg::deleteAllGLObjects(_state->getContextID());
+                _state->reset();
 
-            OSG_INFO<<"Done delete of GL objects"<<std::endl;
+                // Kill the State now so all the (directly or indirectly) referenced GLObjects can make it into delete cache in time.
+                // That way osg::deleteAllGLObjects won't miss them and left them to be deleted afterwards in a new GraphicsContext 
+                // that happens to have the same contextID (see SceneView::draw() and _requiresFlush flag).
+                _state = 0; 
 
-            _state->reset();
+                osg::deleteAllGLObjects(contextID);
+
+                OSG_INFO<<"Done delete of GL objects"<<std::endl;
+            }
+            else
+            {
+                // If the GL objects are shared with other contexts then only flush those
+                // which have already been deleted
+
+                osg::flushAllDeletedGLObjects(_state->getContextID());
+
+                _state->reset();
+            }
 
             releaseContext();
         }
@@ -554,25 +588,25 @@ void GraphicsContext::close(bool callCloseImplementation)
             OSG_INFO<<"makeCurrent did not succeed, could not do flush/deletion of OpenGL objects."<<std::endl;
         }
     }
-    
+
     if (callCloseImplementation) closeImplementation();
 
 
-    // now discard any deleted deleted OpenGL objects that the are still hanging around - such as due to 
+    // now discard any deleted deleted OpenGL objects that the are still hanging around - such as due to
     // the the flushDelete*() methods not being invoked, such as when using GraphicContextEmbedded where makeCurrent
     // does not work.
-    if (_state.valid())
+    if ( !sharedContextExists && contextIDValid)
     {
         OSG_INFO<<"Doing discard of deleted OpenGL objects."<<std::endl;
 
-        osg::discardAllGLObjects(_state->getContextID());
+        osg::discardAllGLObjects(contextID);
     }
 
-    if (_state.valid())
+    if (contextIDValid)
     {
-        decrementContextIDUsageCount(_state->getContextID());
-        
-        _state = 0;
+        decrementContextIDUsageCount(contextID);
+
+        if (_state.valid()) _state = 0;
     }
 }
 
@@ -582,14 +616,14 @@ bool GraphicsContext::makeCurrent()
     _threadOfLastMakeCurrent = OpenThreads::Thread::CurrentThread();
 
     bool result = makeCurrentImplementation();
-    
+
     if (result)
     {
         // initialize extension process, not only initializes on first
-        // call, will be a non-op on subsequent calls.        
+        // call, will be a non-op on subsequent calls.
         getState()->initializeExtensionProcs();
     }
-    
+
     return result;
 }
 
@@ -602,19 +636,19 @@ bool GraphicsContext::makeContextCurrent(GraphicsContext* readContext)
         _threadOfLastMakeCurrent = OpenThreads::Thread::CurrentThread();
 
         // initialize extension process, not only initializes on first
-        // call, will be a non-op on subsequent calls.        
+        // call, will be a non-op on subsequent calls.
         getState()->initializeExtensionProcs();
     }
-    
+
     return result;
 }
 
 bool GraphicsContext::releaseContext()
 {
     bool result = releaseContextImplementation();
-    
+
     _threadOfLastMakeCurrent = (OpenThreads::Thread*)(-1);
-    
+
     return result;
 }
 
@@ -625,7 +659,7 @@ void GraphicsContext::swapBuffers()
         swapBuffersCallbackOrImplemenation();
         clear();
     }
-    else if (_graphicsThread.valid() && 
+    else if (_graphicsThread.valid() &&
              _threadOfLastMakeCurrent == _graphicsThread.get())
     {
         _graphicsThread->add(new SwapBuffersOperation);
@@ -648,9 +682,9 @@ void GraphicsContext::createGraphicsThread()
 
 void GraphicsContext::setGraphicsThread(GraphicsThread* gt)
 {
-    if (_graphicsThread==gt) return; 
+    if (_graphicsThread==gt) return;
 
-    if (_graphicsThread.valid()) 
+    if (_graphicsThread.valid())
     {
         // need to kill the thread in some way...
         _graphicsThread->cancel();
@@ -658,8 +692,8 @@ void GraphicsContext::setGraphicsThread(GraphicsThread* gt)
     }
 
     _graphicsThread = gt;
-    
-    if (_graphicsThread.valid()) 
+
+    if (_graphicsThread.valid())
     {
         _graphicsThread->setParent(this);
     }
@@ -701,7 +735,7 @@ void GraphicsContext::remove(Operation* operation)
 void GraphicsContext::remove(const std::string& name)
 {
     OSG_INFO<<"Doing remove named operation"<<std::endl;
-    
+
     // acquire the lock on the operations queue to prevent anyone else for modifying it at the same time
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_operationsMutex);
 
@@ -728,18 +762,6 @@ void GraphicsContext::removeAllOperations()
     _operationsBlock->set(false);
 }
 
-
-struct CameraRenderOrderSortOp
-{
-    inline bool operator() (const Camera* lhs,const Camera* rhs) const
-    {
-        if (lhs->getRenderOrder()<rhs->getRenderOrder()) return true;
-        if (rhs->getRenderOrder()<lhs->getRenderOrder()) return false;
-        return lhs->getRenderOrderNum()<rhs->getRenderOrderNum();
-    }
-};
-
-
 void GraphicsContext::runOperations()
 {
     // sort the cameras into order
@@ -747,7 +769,7 @@ void GraphicsContext::runOperations()
     CameraVector camerasCopy;
     std::copy(_cameras.begin(), _cameras.end(), std::back_inserter(camerasCopy));
     std::sort(camerasCopy.begin(), camerasCopy.end(), CameraRenderOrderSortOp());
-    
+
     for(CameraVector::iterator itr = camerasCopy.begin();
         itr != camerasCopy.end();
         ++itr)
@@ -778,7 +800,7 @@ void GraphicsContext::runOperations()
                 ++itr;
             }
         }
-                
+
         if (_currentOperation.valid())
         {
             // OSG_INFO<<"Doing op "<<_currentOperation->getName()<<" "<<this<<std::endl;
@@ -786,7 +808,7 @@ void GraphicsContext::runOperations()
             // call the graphics operation.
             (*_currentOperation)(this);
 
-            {            
+            {
                 OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_operationsMutex);
                 _currentOperation = 0;
             }
@@ -810,9 +832,9 @@ void GraphicsContext::removeCamera(osg::Camera* camera)
         NodeSet nodes;
         for(unsigned int i=0; i<camera->getNumChildren(); ++i)
         {
-            nodes.insert(camera->getChild(i));            
+            nodes.insert(camera->getChild(i));
         }
-        
+
         for(Cameras::iterator citr = _cameras.begin();
             citr != _cameras.end();
             ++citr)
@@ -825,9 +847,9 @@ void GraphicsContext::removeCamera(osg::Camera* camera)
                     NodeSet::iterator nitr = nodes.find(otherCamera->getChild(i));
                     if (nitr != nodes.end()) nodes.erase(nitr);
                 }
-            }            
+            }
         }
-        
+
         // now release the GLobjects associated with these non shared nodes
         for(NodeSet::iterator nitr = nodes.begin();
             nitr != nodes.end();
@@ -852,21 +874,21 @@ void GraphicsContext::resizedImplementation(int x, int y, int width, int height)
     std::set<osg::Viewport*> processedViewports;
 
     if (!_traits) return;
-    
+
     double widthChangeRatio = double(width) / double(_traits->width);
     double heigtChangeRatio = double(height) / double(_traits->height);
-    double aspectRatioChange = widthChangeRatio / heigtChangeRatio; 
-    
+    double aspectRatioChange = widthChangeRatio / heigtChangeRatio;
+
 
     for(Cameras::iterator itr = _cameras.begin();
         itr != _cameras.end();
         ++itr)
     {
         Camera* camera = (*itr);
-        
+
         // resize doesn't affect Cameras set up with FBO's.
         if (camera->getRenderTargetImplementation()==osg::Camera::FRAME_BUFFER_OBJECT) continue;
-        
+
         Viewport* viewport = camera->getViewport();
         if (viewport)
         {
