@@ -1,13 +1,14 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  * Copyright (C) 2003-2005 3Dlabs Inc. Ltd.
  * Copyright (C) 2004-2005 Nathan Cournia
  * Copyright (C) 2008 Zebra Imaging
  * Copyright (C) 2010 VIRES Simulationstechnologie GmbH
+ * Copyright (C) 2012 David Callu
  *
- * This application is open source and may be redistributed and/or modified   
+ * This application is open source and may be redistributed and/or modified
  * freely and without restriction, both in commercial and non commercial
  * applications, as long as this copyright notice is maintained.
- * 
+ *
  * This application is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -34,6 +35,8 @@
 #include <OpenThreads/ScopedLock>
 #include <OpenThreads/Mutex>
 
+#include <string.h>
+
 using namespace osg;
 
 ///////////////////////////////////////////////////////////////////////////
@@ -102,7 +105,8 @@ Program::ProgramBinary::ProgramBinary() : _format(0)
 {
 }
 
-Program::ProgramBinary::ProgramBinary(const ProgramBinary& rhs, const osg::CopyOp&) :
+Program::ProgramBinary::ProgramBinary(const ProgramBinary& rhs, const osg::CopyOp& copyop) :
+    osg::Object(rhs, copyop),
     _data(rhs._data), _format(rhs._format)
 {
 }
@@ -133,7 +137,7 @@ void Program::ProgramBinary::assign(unsigned int size, const unsigned char* data
 Program::Program() :
     _geometryVerticesOut(1), _geometryInputType(GL_TRIANGLES),
     _geometryOutputType(GL_TRIANGLE_STRIP),
-    _patchVertices(3)
+    _numGroupsX(0), _numGroupsY(0), _numGroupsZ(0)
 {
 }
 
@@ -162,7 +166,9 @@ Program::Program(const Program& rhs, const osg::CopyOp& copyop):
     _geometryInputType = rhs._geometryInputType;
     _geometryOutputType = rhs._geometryOutputType;
 
-    _patchVertices = rhs._patchVertices;
+    _numGroupsX = rhs._numGroupsX;
+    _numGroupsY = rhs._numGroupsY;
+    _numGroupsZ = rhs._numGroupsZ;
 }
 
 
@@ -181,7 +187,7 @@ int Program::compare(const osg::StateAttribute& sa) const
     // check the types are equal and then create the rhs variable
     // used by the COMPARE_StateAttribute_Parameter macros below.
     COMPARE_StateAttribute_Types(Program,sa)
-    
+
     if( _shaderList.size() < rhs._shaderList.size() ) return -1;
     if( rhs._shaderList.size() < _shaderList.size() ) return 1;
 
@@ -197,8 +203,14 @@ int Program::compare(const osg::StateAttribute& sa) const
     if( _geometryOutputType < rhs._geometryOutputType ) return -1;
     if( rhs._geometryOutputType < _geometryOutputType ) return 1;
 
-    if( _patchVertices < rhs._patchVertices ) return -1;
-    if( rhs._patchVertices < _patchVertices ) return 1;
+    if( _numGroupsX < rhs._numGroupsX ) return -1;
+    if( rhs._numGroupsX < _numGroupsX ) return 1;
+
+    if( _numGroupsY < rhs._numGroupsY ) return -1;
+    if( rhs._numGroupsY < _numGroupsY ) return 1;
+
+    if( _numGroupsZ < rhs._numGroupsZ ) return -1;
+    if( rhs._numGroupsZ < _numGroupsZ ) return 1;
 
     ShaderList::const_iterator litr=_shaderList.begin();
     ShaderList::const_iterator ritr=rhs._shaderList.begin();
@@ -270,7 +282,7 @@ void Program::releaseGLObjects(osg::State* state) const
     {
         unsigned int contextID = state->getContextID();
         _pcpList[contextID] = 0;
-    }   
+    }
 }
 
 bool Program::addShader( Shader* shader )
@@ -342,44 +354,14 @@ void Program::setParameter( GLenum pname, GLint value )
             //dirtyProgram();    // needed?
             break;
         case GL_PATCH_VERTICES:
-            _patchVertices = value;
-            dirtyProgram();
+            OSG_WARN << "Program::setParameter invalid param " << GL_PATCH_VERTICES << ", use osg::PatchParameter when setting GL_PATCH_VERTICES."<<std::endl;
             break;
         default:
-            OSG_WARN << "setParameter invalid param " << pname << std::endl;
+            OSG_WARN << "Program::setParameter invalid param " << pname << std::endl;
             break;
     }
 }
 
-void Program::setParameterfv( GLenum pname, const GLfloat* value )
-{
-    switch( pname )
-    {
-      // todo tessellation default level
-        case GL_PATCH_DEFAULT_INNER_LEVEL:
-            break;
-        case GL_PATCH_DEFAULT_OUTER_LEVEL:
-            break;
-        default:
-            OSG_WARN << "setParameter invalid param " << pname << std::endl;
-            break;
-    }
-}
-
-const GLfloat* Program::getParameterfv( GLenum pname ) const
-{
-    /*switch( pname )
-    {
-      ;
-      // todo tessellation default level
-      //        case GL_PATCH_DEFAULT_INNER_LEVEL: return _patchDefaultInnerLevel;
-      //        case GL_PATCH_DEFAULT_OUTER_LEVEL: return _patchDefaultOuterLevel;
-
-    }*/
-    OSG_WARN << "getParameter invalid param " << pname << std::endl;
-    return 0;
-}
-
 GLint Program::getParameter( GLenum pname ) const
 {
     switch( pname )
@@ -387,12 +369,24 @@ GLint Program::getParameter( GLenum pname ) const
         case GL_GEOMETRY_VERTICES_OUT_EXT: return _geometryVerticesOut;
         case GL_GEOMETRY_INPUT_TYPE_EXT:   return _geometryInputType;
         case GL_GEOMETRY_OUTPUT_TYPE_EXT:  return _geometryOutputType;
-        case GL_PATCH_VERTICES:            return _patchVertices; 
     }
     OSG_WARN << "getParameter invalid param " << pname << std::endl;
     return 0;
 }
 
+void Program::setComputeGroups( GLint numGroupsX, GLint numGroupsY, GLint numGroupsZ )
+{
+    _numGroupsX = numGroupsX;
+    _numGroupsY = numGroupsY;
+    _numGroupsZ = numGroupsZ;
+}
+
+void Program::getComputeGroups( GLint& numGroupsX, GLint& numGroupsY, GLint& numGroupsZ ) const
+{
+    numGroupsX = _numGroupsX;
+    numGroupsY = _numGroupsY;
+    numGroupsZ = _numGroupsZ;
+}
 
 void Program::addBindAttribLocation( const std::string& name, GLuint index )
 {
@@ -509,25 +503,39 @@ const Program::ActiveVarInfoMap& Program::getActiveAttribs(unsigned int contextI
     return getPCP( contextID )->getActiveAttribs();
 }
 
+const Program::UniformBlockMap& Program::getUniformBlocks(unsigned contextID) const
+{
+    return getPCP( contextID )->getUniformBlocks();
+}
+
 ///////////////////////////////////////////////////////////////////////////
 // osg::Program::PerContextProgram
 // PCP is an OSG abstraction of the per-context glProgram
 ///////////////////////////////////////////////////////////////////////////
 
-Program::PerContextProgram::PerContextProgram(const Program* program, unsigned int contextID ) :
+Program::PerContextProgram::PerContextProgram(const Program* program, unsigned int contextID, GLuint programHandle ) :
         osg::Referenced(),
+        _glProgramHandle(programHandle),
         _loadedBinary(false),
-        _contextID( contextID )
+        _contextID( contextID ),
+        _ownsProgramHandle(false)
 {
     _program = program;
     _extensions = GL2Extensions::Get( _contextID, true );
-    _glProgramHandle = _extensions->glCreateProgram();
+    if (_glProgramHandle == 0)
+    {
+        _glProgramHandle = _extensions->glCreateProgram();
+        _ownsProgramHandle = true;
+    }
     requestLink();
 }
 
 Program::PerContextProgram::~PerContextProgram()
 {
-    Program::deleteGlProgram( _contextID, _glProgramHandle );
+    if (_ownsProgramHandle)
+    {
+        Program::deleteGlProgram( _contextID, _glProgramHandle );
+    }
 }
 
 
@@ -569,12 +577,6 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
             _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_OUTPUT_TYPE_EXT, _program->_geometryOutputType );
         }
 
-        if (_extensions->areTessellationShadersSupported() )
-        {
-            _extensions->glPatchParameteri( GL_PATCH_VERTICES, _program->_patchVertices );
-            // todo: add default tessellation level
-        }
-
         // Detach removed shaders
         for( unsigned int i=0; i < _shadersToDetach.size(); ++i )
         {
@@ -649,10 +651,10 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
         std::string infoLog;
         if( getInfoLog(infoLog) )
         {
-            OSG_WARN << "Program \""<< _program->getName() << "\" " 
+            OSG_WARN << "Program \""<< _program->getName() << "\" "
                                       "infolog:\n" << infoLog << std::endl;
         }
-        
+
         return;
     }
     else
@@ -672,7 +674,7 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
         _extensions->glGetProgramiv(_glProgramHandle, GL_ACTIVE_UNIFORM_BLOCKS,
                                     reinterpret_cast<GLint*>(&activeUniformBlocks));
         _extensions->glGetProgramiv(_glProgramHandle,
-                                    GL_ACTIVE_UNIFORM_MAX_LENGTH,
+                                    GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH,
                                     &maxBlockNameLen);
         if (maxBlockNameLen > 0)
         {
@@ -712,11 +714,12 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
             {
                 OSG_WARN << "uniform block " << blockName << " has no binding.\n";
             }
-
         }
-
     }
 
+    typedef std::map<GLuint, std::string> AtomicCounterMap;
+    AtomicCounterMap atomicCounterMap;
+
     // build _uniformInfoMap
     GLint numUniforms = 0;
     GLsizei maxLen = 0;
@@ -733,8 +736,22 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
             _extensions->glGetActiveUniform( _glProgramHandle,
                     i, maxLen, 0, &size, &type, name );
 
+            int pos = strlen(name);
+            if (pos>0 && name[pos-1]==']')
+            {
+                // need to trim [..] from end of name as some drivers append this causing problems with look up.
+                --pos;
+                while(pos>0 && name[pos]!='[') { --pos; }
+                name[pos] = 0;
+            }
+
+            if (type == GL_UNSIGNED_INT_ATOMIC_COUNTER)
+            {
+                atomicCounterMap[i] = name;
+            }
+
             GLint loc = _extensions->glGetUniformLocation( _glProgramHandle, name );
-            
+
             if( loc != -1 )
             {
                 _uniformInfoMap[Uniform::getNameID(reinterpret_cast<const char*>(name))] = ActiveVarInfo(loc,type,size);
@@ -749,6 +766,84 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
         delete [] name;
     }
 
+    // print atomic counter
+
+    if (_extensions->isShaderAtomicCounterSupported() && !atomicCounterMap.empty())
+    {
+        std::vector<GLint> bufferIndex( atomicCounterMap.size(), 0 );
+        std::vector<GLuint> uniformIndex;
+        for (AtomicCounterMap::iterator it = atomicCounterMap.begin(), end = atomicCounterMap.end();
+             it != end; ++it)
+        {
+            uniformIndex.push_back(it->first);
+        }
+
+        _extensions->glGetActiveUniformsiv( _glProgramHandle, uniformIndex.size(),
+                                            &(uniformIndex[0]), GL_UNIFORM_ATOMIC_COUNTER_BUFFER_INDEX,
+                                            &(bufferIndex[0]) );
+
+        for (unsigned int j = 0; j < uniformIndex.size(); ++j)
+        {
+            OSG_INFO << "\tUniform atomic counter \""<<atomicCounterMap[ uniformIndex[j] ] <<"\""
+                     <<" buffer bind= " << bufferIndex[j] << ".\n";
+        }
+
+        std::map<int, std::vector<int> > bufferIndexToUniformIndices;
+        for (unsigned int i=0; i<bufferIndex.size(); ++i)
+        {
+            bufferIndexToUniformIndices[ bufferIndex[i] ].push_back( uniformIndex[i] );
+        }
+
+        GLuint activeAtomicCounterBuffers = 0;
+        _extensions->glGetProgramiv(_glProgramHandle, GL_ACTIVE_ATOMIC_COUNTER_BUFFERS,
+                                    reinterpret_cast<GLint*>(&activeAtomicCounterBuffers));
+        if (activeAtomicCounterBuffers > 0)
+        {
+            for (GLuint i = 0; i < activeAtomicCounterBuffers; ++i)
+            {
+                GLint bindID = 0;
+                _extensions->glGetActiveAtomicCounterBufferiv(_glProgramHandle, i,
+                                                              GL_ATOMIC_COUNTER_BUFFER_BINDING,
+                                                              &bindID);
+
+                GLsizei num = 0;
+                _extensions->glGetActiveAtomicCounterBufferiv(_glProgramHandle, i,
+                                                              GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTERS,
+                                                              &num);
+                GLsizei minSize = 0;
+                _extensions->glGetActiveAtomicCounterBufferiv(_glProgramHandle, i,
+                                                              GL_ATOMIC_COUNTER_BUFFER_DATA_SIZE,
+                                                              &minSize);
+
+
+                OSG_INFO << "\tUniform atomic counter buffer bind \"" << bindID << "\""
+                         << " num active atomic counter= "<< num
+                         << " min size= " << minSize << "\n";
+
+                if (num)
+                {
+                    std::vector<GLint> indices(num);
+                    _extensions->glGetActiveAtomicCounterBufferiv(_glProgramHandle, i,
+                                                                  GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTER_INDICES,
+                                                                  &(indices[0]));
+                    OSG_INFO << "\t\tindices used= ";
+                    for (GLint j = 0; j < num; ++j)
+                    {
+                        OSG_INFO << indices[j];
+                        if (j < (num-1))
+                        {
+                            OSG_INFO <<  ", ";
+                        }
+                        else
+                        {
+                            OSG_INFO <<  ".\n";
+                        }
+                    }
+                }
+            }
+        }
+    }
+
     // build _attribInfoMap
     GLint numAttrib = 0;
     _extensions->glGetProgramiv( _glProgramHandle, GL_ACTIVE_ATTRIBUTES, &numAttrib );
@@ -765,7 +860,7 @@ void Program::PerContextProgram::linkProgram(osg::State& state)
                     i, maxLen, 0, &size, &type, name );
 
             GLint loc = _extensions->glGetAttribLocation( _glProgramHandle, name );
-            
+
             if( loc != -1 )
             {
                 _attribInfoMap[reinterpret_cast<char*>(name)] = ActiveVarInfo(loc,type,size);
@@ -799,7 +894,7 @@ bool Program::PerContextProgram::validateProgram()
         OSG_WARN << "infolog:\n" << infoLog << std::endl;
 
     OSG_WARN << std::endl;
-    
+
     return false;
 }
 
@@ -828,4 +923,8 @@ Program::ProgramBinary* Program::PerContextProgram::compileProgramBinary(osg::St
 void Program::PerContextProgram::useProgram() const
 {
     _extensions->glUseProgram( _glProgramHandle  );
+    if ( _program->_numGroupsX>0 && _program->_numGroupsY>0 && _program->_numGroupsZ>0 )
+    {
+        _extensions->glDispatchCompute( _program->_numGroupsX, _program->_numGroupsY, _program->_numGroupsZ );
+    }
 }
