@@ -1,19 +1,18 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 #include <osg/GLExtensions>
 #include <osg/Texture2DArray>
 #include <osg/State>
-#include <osg/ImageSequence>
 #include <osg/Notify>
 
 #include <string.h>
@@ -33,26 +32,31 @@ Texture2DArray::Texture2DArray(const Texture2DArray& text,const CopyOp& copyop):
             Texture(text,copyop),
             _textureWidth(text._textureWidth),
             _textureHeight(text._textureHeight),
-            _textureDepth(text._textureDepth),
+            _textureDepth(0),
             _numMipmapLevels(text._numMipmapLevels),
             _subloadCallback(text._subloadCallback)
 {
+    setTextureDepth(text._textureDepth);
+
     // copy all images by iterating through all of them
     for (int i=0; i < text._textureDepth; i++)
     {
-        _images.push_back(copyop(text._images[i].get()));
-        _modifiedCount.push_back(ImageModifiedCount());
+        setImage(i, copyop(text._images[i].get()));
     }
 }
 
 Texture2DArray::~Texture2DArray()
 {
+    for (int i=0; i<_textureDepth; ++i)
+    {
+        setImage(i, NULL);
+    }
 }
 
 int Texture2DArray::compare(const StateAttribute& sa) const
 {
     // check the types are equal and then create the rhs variable
-    // used by the COMPARE_StateAttribute_Parameter macro's below.
+    // used by the COMPARE_StateAttribute_Parameter macros below.
     COMPARE_StateAttribute_Types(Texture2DArray,sa)
 
     bool noImages = true;
@@ -60,7 +64,7 @@ int Texture2DArray::compare(const StateAttribute& sa) const
     {
         if (noImages && _images[n].valid()) noImages = false;
         if (noImages && rhs._images[n].valid()) noImages = false;
-    
+
         if (_images[n]!=rhs._images[n]) // smart pointer comparison.
         {
             if (_images[n].valid())
@@ -72,17 +76,17 @@ int Texture2DArray::compare(const StateAttribute& sa) const
                 }
                 else
                 {
-                    return 1; // valid lhs._image is greater than null. 
+                    return 1; // valid lhs._image is greater than null.
                 }
             }
-            else if (rhs._images[n].valid()) 
+            else if (rhs._images[n].valid())
             {
-                return -1; // valid rhs._image is greater than null. 
+                return -1; // valid rhs._image is greater than null.
             }
         }
     }
 
-    
+
     if (noImages)
     {
         int result = compareTextureObjects(rhs);
@@ -98,7 +102,7 @@ int Texture2DArray::compare(const StateAttribute& sa) const
     COMPARE_StateAttribute_Parameter(_textureDepth)
     COMPARE_StateAttribute_Parameter(_subloadCallback)
 
-    return 0; // passed all the above comparison macro's, must be equal.
+    return 0; // passed all the above comparison macros, must be equal.
 }
 
 void Texture2DArray::setImage(unsigned int layer, Image* image)
@@ -107,46 +111,54 @@ void Texture2DArray::setImage(unsigned int layer, Image* image)
     if (static_cast<int>(layer) >= _textureDepth)
     {
         // print warning and do nothing
-        notify(WARN)<<"Warning: Texture2DArray::setImage(..) failed, the given layer number is bigger then the size of the texture array."<<std::endl;
+        OSG_WARN<<"Warning: Texture2DArray::setImage(..) failed, the given layer number is bigger then the size of the texture array."<<std::endl;
         return;
     }
-    
+
     if (_images[layer] == image) return;
 
-    unsigned numImageSequencesBefore = 0;
+    unsigned numImageRequireUpdateBefore = 0;
     for (unsigned int i=0; i<getNumImages(); ++i)
     {
-        osg::ImageSequence* is = dynamic_cast<osg::ImageSequence*>(_images[i].get());
-        if (is) ++numImageSequencesBefore;
+        if (_images[i].valid() && _images[i]->requiresUpdateCall()) ++numImageRequireUpdateBefore;
+    }
+
+    if (_images[layer].valid())
+    {
+        _images[layer]->removeClient(this);
     }
 
     // set image
     _images[layer] = image;
-   _modifiedCount[layer].setAllElementsTo(0);
+    _modifiedCount[layer].setAllElementsTo(0);
 
-    // find out if we need to reset the update callback to handle the animation of ImageSequence
-    unsigned numImageSequencesAfter = 0;
-    for (unsigned int i=0; i<getNumImages(); ++i)
+    if (_images[layer].valid())
     {
-        osg::ImageSequence* is = dynamic_cast<osg::ImageSequence*>(_images[i].get());
-        if (is) ++numImageSequencesAfter;
+        _images[layer]->addClient(this);
     }
 
-    if (numImageSequencesBefore>0)
+    // find out if we need to reset the update callback to handle the animation of image
+    unsigned numImageRequireUpdateAfter = 0;
+    for (unsigned int i=0; i<getNumImages(); ++i)
     {
-        if (numImageSequencesAfter==0)
+        if (_images[i].valid() && _images[i]->requiresUpdateCall()) ++numImageRequireUpdateAfter;
+    }
+
+    if (numImageRequireUpdateBefore>0)
+    {
+        if (numImageRequireUpdateAfter==0)
         {
             setUpdateCallback(0);
             setDataVariance(osg::Object::STATIC);
         }
     }
-    else if (numImageSequencesAfter>0)
+    else if (numImageRequireUpdateAfter>0)
     {
-        setUpdateCallback(new ImageSequence::UpdateCallback());
+        setUpdateCallback(new Image::UpdateCallback());
         setDataVariance(osg::Object::DYNAMIC);
     }
 }
- 
+
 void Texture2DArray::setTextureSize(int width, int height, int depth)
 {
     // set dimensions
@@ -156,21 +168,21 @@ void Texture2DArray::setTextureSize(int width, int height, int depth)
 }
 
 void Texture2DArray::setTextureDepth(int depth)
-{    
+{
     // if we decrease the number of layers, then delete non-used
     if (depth < _textureDepth)
     {
         _images.resize(depth);
         _modifiedCount.resize(depth);
     }
-    
+
     // if we increase the array, then add new empty elements
     if (depth > _textureDepth)
     {
         _images.resize(depth, ref_ptr<Image>(0));
         _modifiedCount.resize(depth, ImageModifiedCount());
     }
-        
+
     // resize the texture array
     _textureDepth = depth;
 }
@@ -198,30 +210,57 @@ bool Texture2DArray::imagesValid() const
 
 void Texture2DArray::computeInternalFormat() const
 {
-    if (imagesValid()) computeInternalFormatWithImage(*_images[0]); 
+    if (imagesValid()) computeInternalFormatWithImage(*_images[0]);
     else computeInternalFormatType();
 }
 
 
 void Texture2DArray::apply(State& state) const
 {
-    // get the contextID (user defined ID of 0 upwards) for the 
+    // get the contextID (user defined ID of 0 upwards) for the
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
+
+    Texture::TextureObjectManager* tom = Texture::getTextureObjectManager(contextID).get();
+    ElapsedTime elapsedTime(&(tom->getApplyTime()));
+    tom->getNumberApplied()++;
+
     const Extensions* extensions = getExtensions(contextID,true);
 
     // if not supported, then return
     if (!extensions->isTexture2DArraySupported() || !extensions->isTexture3DSupported())
     {
-        notify(WARN)<<"Warning: Texture2DArray::apply(..) failed, 2D texture arrays are not support by OpenGL driver."<<std::endl;
+        OSG_WARN<<"Warning: Texture2DArray::apply(..) failed, 2D texture arrays are not support by OpenGL driver."<<std::endl;
         return;
     }
-    
+
     // get the texture object for the current contextID.
     TextureObject* textureObject = getTextureObject(contextID);
 
-    // if we already have an texture object, then 
-    if (textureObject != 0)
+    if (textureObject && _textureDepth>0)
+    {
+        const osg::Image* image = _images[0].get();
+        if (image && getModifiedCount(0, contextID) != image->getModifiedCount())
+        {
+            // compute the internal texture format, this set the _internalFormat to an appropriate value.
+            computeInternalFormat();
+
+            GLsizei new_width, new_height, new_numMipmapLevels;
+
+            // compute the dimensions of the texture.
+            computeRequiredTextureDimensions(state, *image, new_width, new_height, new_numMipmapLevels);
+
+            if (!textureObject->match(GL_TEXTURE_2D_ARRAY_EXT, new_numMipmapLevels, _internalFormat, new_width, new_height, _textureDepth, _borderWidth))
+            {
+                Texture::releaseTextureObject(contextID, _textureObjectBuffer[contextID].get());
+                _textureObjectBuffer[contextID] = 0;
+                textureObject = 0;
+            }
+        }
+    }
+
+    // if we already have an texture object, then
+    if (textureObject)
     {
         // bind texture object
         textureObject->bind();
@@ -240,7 +279,7 @@ void Texture2DArray::apply(State& state) const
             for (GLsizei n=0; n < _textureDepth; n++)
             {
                 osg::Image* image = _images[n].get();
-                
+
                 // if image content is modified, then upload it to the GPU memory
                 if (image && getModifiedCount(n,contextID) != image->getModifiedCount())
                 {
@@ -251,17 +290,17 @@ void Texture2DArray::apply(State& state) const
         }
 
     }
-    
+
     // there is no texture object, but exists a subload callback, so use it to upload images
     else if (_subloadCallback.valid())
     {
         // generate texture (i.e. glGenTexture) and apply parameters
-        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(contextID, GL_TEXTURE_2D_ARRAY_EXT);
+        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(this, contextID, GL_TEXTURE_2D_ARRAY_EXT);
         textureObject->bind();
         applyTexParameters(GL_TEXTURE_2D_ARRAY_EXT, state);
         _subloadCallback->load(*this,state);
     }
-    
+
     // nothing before, but we have valid images, so do manual upload and create texture object manually
     // TODO: we assume _images[0] is valid, however this may not be always the case
     //       some kind of checking for the first valid image is required (Art, may 2008)
@@ -275,19 +314,45 @@ void Texture2DArray::apply(State& state) const
 
         // create texture object
         textureObject = generateTextureObject(
-                contextID,GL_TEXTURE_2D_ARRAY_EXT,_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
-        
+                this, contextID,GL_TEXTURE_2D_ARRAY_EXT,_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
+
         // bind texture
         textureObject->bind();
         applyTexParameters(GL_TEXTURE_2D_ARRAY_EXT, state);
 
-        // first we need to allocate the texture memory
-        extensions->glTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, 0, _internalFormat,
-                     _textureWidth, _textureHeight, _textureDepth,
-                     _borderWidth,
-                     _sourceFormat ? _sourceFormat : _internalFormat,
-                     _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
-                     0); 
+        _textureObjectBuffer[contextID] = textureObject;
+
+        // First we need to allocate the texture memory
+        int sourceFormat = _sourceFormat ? _sourceFormat : _internalFormat;
+
+        if( isCompressedInternalFormat( sourceFormat ) &&
+            sourceFormat == _internalFormat &&
+            extensions->isCompressedTexImage3DSupported() )
+        {
+            extensions->glCompressedTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, 0, _internalFormat,
+                     _textureWidth, _textureHeight, _textureDepth, _borderWidth,
+                     _images[0]->getImageSizeInBytes() * _textureDepth,
+                     0);
+        }
+        else
+        {
+            // Override compressed source format with safe GL_RGBA value which not generate error
+            // We can safely do this as source format is not important when source data is NULL
+            if( isCompressedInternalFormat( sourceFormat ) )
+                sourceFormat = GL_RGBA;
+
+            extensions->glTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, 0, _internalFormat,
+                     _textureWidth, _textureHeight, _textureDepth, _borderWidth,
+                     sourceFormat, _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
+                     0);
+        }
+
+       // For certain we have to manually allocate memory for mipmaps if images are compressed
+       // if not allocated OpenGL will produce errors on mipmap upload.
+       // I have not tested if this is neccessary for plain texture formats but
+       // common sense suggests its required as well.
+       if( _min_filter != LINEAR && _min_filter != NEAREST && _images[0]->isMipmap() )
+           allocateMipmap( state );
 
         // now for each layer we upload it into the memory
         for (GLsizei n=0; n<_textureDepth; n++)
@@ -301,44 +366,52 @@ void Texture2DArray::apply(State& state) const
                 getModifiedCount(n,contextID) = image->getModifiedCount();
             }
         }
+
+        const Texture::Extensions* texExtensions = Texture::getExtensions(contextID,true);
+        // source images have no mipmamps but we could generate them...
+        if( _min_filter != LINEAR && _min_filter != NEAREST && !_images[0]->isMipmap() &&
+            _useHardwareMipMapGeneration && texExtensions->isGenerateMipMapSupported() )
+            {
+                _numMipmapLevels = Image::computeNumberOfMipmapLevels( _textureWidth, _textureHeight );
+                generateMipmap( state );
+            }
+
         textureObject->setAllocated(_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
 
-        _textureObjectBuffer[contextID] = textureObject;
-        
-        // no idea what this for ;-)
-        if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded())
+        // unref image data?
+        if (isSafeToUnrefImageData(state))
         {
             Texture2DArray* non_const_this = const_cast<Texture2DArray*>(this);
             for (int n=0; n<_textureDepth; n++)
-            {                
+            {
                 if (_images[n].valid() && _images[n]->getDataVariance()==STATIC)
                 {
-                    non_const_this->_images[n] = 0;
+                    non_const_this->_images[n] = NULL;
                 }
             }
         }
-        
+
     }
-    
+
     // No images present, but dimensions are set. So create empty texture
     else if ( (_textureWidth > 0) && (_textureHeight > 0) && (_textureDepth > 0) && (_internalFormat!=0) )
     {
-        // generate texture 
+        // generate texture
         _textureObjectBuffer[contextID] = textureObject = generateTextureObject(
-                contextID, GL_TEXTURE_2D_ARRAY_EXT,_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
-        
+                this, contextID, GL_TEXTURE_2D_ARRAY_EXT,_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
+
         textureObject->bind();
         applyTexParameters(GL_TEXTURE_2D_ARRAY_EXT,state);
-        
+
         extensions->glTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, 0, _internalFormat,
                      _textureWidth, _textureHeight, _textureDepth,
                      _borderWidth,
                      _sourceFormat ? _sourceFormat : _internalFormat,
                      _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
-                     0); 
-        
+                     0);
+
     }
-    
+
     // nothing before, so just unbind the texture target
     else
     {
@@ -352,19 +425,20 @@ void Texture2DArray::apply(State& state) const
     }
 }
 
+
 void Texture2DArray::applyTexImage2DArray_subload(State& state, Image* image, GLsizei inwidth, GLsizei inheight, GLsizei indepth, GLint inInternalFormat, GLsizei& numMipmapLevels) const
 {
     // if we don't have a valid image we can't create a texture!
     if (!imagesValid())
         return;
 
-    // get the contextID (user defined ID of 0 upwards) for the 
+    // get the contextID (user defined ID of 0 upwards) for the
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
-    const Extensions* extensions = getExtensions(contextID,true);    
+    const Extensions* extensions = getExtensions(contextID,true);
     const Texture::Extensions* texExtensions = Texture::getExtensions(contextID,true);
     GLenum target = GL_TEXTURE_2D_ARRAY_EXT;
-    
+
     // compute the internal texture format, this set the _internalFormat to an appropriate value.
     computeInternalFormat();
 
@@ -376,8 +450,8 @@ void Texture2DArray::applyTexImage2DArray_subload(State& state, Image* image, GL
     if (indepth > extensions->maxLayerCount())
     {
         // we give a warning and do nothing
-        notify(WARN)<<"Warning: Texture2DArray::applyTexImage2DArray_subload(..) the given layer number exceeds the maximum number of supported layers."<<std::endl;
-        return;        
+        OSG_WARN<<"Warning: Texture2DArray::applyTexImage2DArray_subload(..) the given layer number exceeds the maximum number of supported layers."<<std::endl;
+        return;
     }
 
     //Rescale if resize hint is set or NPOT not supported or dimensions exceed max size
@@ -387,23 +461,32 @@ void Texture2DArray::applyTexImage2DArray_subload(State& state, Image* image, GL
         image->ensureValidSizeForTexturing(extensions->max2DSize());
 
     // image size or format has changed, this is not allowed, hence return
-    if (image->s()!=inwidth || 
-        image->t()!=inheight || 
-        image->getInternalTextureFormat()!=inInternalFormat ) 
+    if (image->s()!=inwidth ||
+        image->t()!=inheight ||
+        image->getInternalTextureFormat()!=inInternalFormat )
     {
-        notify(WARN)<<"Warning: Texture2DArray::applyTexImage2DArray_subload(..) given image do have wrong dimension or internal format."<<std::endl;
-        return;        
-    }    
-    
+        OSG_WARN<<"Warning: Texture2DArray::applyTexImage2DArray_subload(..) given image do have wrong dimension or internal format."<<std::endl;
+        return;
+    }
+
     glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
+#if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+    glPixelStorei(GL_UNPACK_ROW_LENGTH,image->getRowLength());
+#endif
+
+    bool useHardwareMipmapGeneration =
+        !image->isMipmap() && _useHardwareMipMapGeneration && texExtensions->isGenerateMipMapSupported();
 
     // if no special mipmapping is required, then
-    if( _min_filter == LINEAR || _min_filter == NEAREST )
+    if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipmapGeneration )
     {
-        numMipmapLevels = 1;
+        if( _min_filter == LINEAR || _min_filter == NEAREST )
+            numMipmapLevels = 1;
+        else //Hardware Mipmap Generation
+            numMipmapLevels = image->getNumMipmapLevels();
 
         // upload non-compressed image
-        if (!compressed_image)
+        if ( !compressed_image )
         {
             extensions->glTexSubImage3D( target, 0,
                                       0, 0, indepth,
@@ -412,21 +495,20 @@ void Texture2DArray::applyTexImage2DArray_subload(State& state, Image* image, GL
                                       (GLenum)image->getDataType(),
                                       image->data() );
         }
-        
+
         // if we support compression and image is compressed, then
         else if (extensions->isCompressedTexImage3DSupported())
         {
-            // notify(WARN)<<"glCompressedTexImage3D "<<inwidth<<", "<<inheight<<", "<<indepth<<std::endl;
-            numMipmapLevels = 1;
+            // OSG_WARN<<"glCompressedTexImage3D "<<inwidth<<", "<<inheight<<", "<<indepth<<std::endl;
 
             GLint blockSize, size;
-            getCompressedSize(_internalFormat, inwidth, inheight, indepth, blockSize,size);
+            getCompressedSize(_internalFormat, inwidth, inheight, 1, blockSize,size);
 
             extensions->glCompressedTexSubImage3D(target, 0,
-                0, 0, indepth,  
-                inwidth, inheight, 1, 
+                0, 0, indepth,
+                inwidth, inheight, 1,
                 (GLenum)image->getPixelFormat(),
-                size, 
+                size,
                 image->data());
         }
 
@@ -434,34 +516,64 @@ void Texture2DArray::applyTexImage2DArray_subload(State& state, Image* image, GL
     }else
     {
         // image does not provide mipmaps, so we have to create them
-        if(!image->isMipmap())
+        if( !image->isMipmap() )
         {
-            notify(WARN)<<"Warning: Texture2DArray::applyTexImage2DArray_subload(..) automagic mipmap generation is currently not implemented. Check texture's min/mag filters."<<std::endl;
+            numMipmapLevels = 1;
+            OSG_WARN<<"Warning: Texture2DArray::applyTexImage2DArray_subload(..) mipmap layer not passed, and auto mipmap generation turned off or not available. Check texture's min/mag filters & hardware mipmap generation."<<std::endl;
 
-        // the image object does provide mipmaps, so upload the in the certain levels of a layer
+            // the image object does provide mipmaps, so upload the in the certain levels of a layer
         }else
         {
             numMipmapLevels = image->getNumMipmapLevels();
 
             int width  = image->s();
             int height = image->t();
-            
-            for( GLsizei k = 0 ; k < numMipmapLevels  && (width || height ) ;k++)
+
+            if( !compressed_image )
             {
 
-                if (width == 0)
-                    width = 1;
-                if (height == 0)
-                    height = 1;
+                for( GLsizei k = 0 ; k < numMipmapLevels  && (width || height ) ;k++)
+                {
+                    if (width == 0)
+                       width = 1;
+                    if (height == 0)
+                       height = 1;
 
-                extensions->glTexSubImage3D( target, k, 0, 0, indepth,
-                                          width, height, 1, 
-                                          (GLenum)image->getPixelFormat(),
-                                          (GLenum)image->getDataType(),
-                                          image->getMipmapData(k));
+                    extensions->glTexSubImage3D( target, k, 0, 0, indepth,
+                                              width, height, 1,
+                                              (GLenum)image->getPixelFormat(),
+                                              (GLenum)image->getDataType(),
+                                               image->getMipmapData(k));
 
-                width >>= 1;
-                height >>= 1;
+                    width >>= 1;
+                    height >>= 1;
+                }
+            }
+            else if (extensions->isCompressedTexImage3DSupported())
+            {
+                GLint blockSize,size;
+                for( GLsizei k = 0 ; k < numMipmapLevels  && (width || height) ;k++)
+                {
+                    if (width == 0)
+                        width = 1;
+                    if (height == 0)
+                        height = 1;
+
+                    getCompressedSize(image->getInternalTextureFormat(), width, height, 1, blockSize,size);
+
+//                    state.checkGLErrors("before extensions->glCompressedTexSubImage3D(");
+
+                    extensions->glCompressedTexSubImage3D(target, k, 0, 0, indepth,
+                                                       width, height, 1,
+                                                       (GLenum)image->getPixelFormat(),
+                                                       size,
+                                                       image->getMipmapData(k));
+
+//                    state.checkGLErrors("after extensions->glCompressedTexSubImage3D(");
+
+                    width >>= 1;
+                    height >>= 1;
+                }
             }
         }
 
@@ -491,7 +603,7 @@ void Texture2DArray::copyTexSubImage2DArray(State& state, int xoffset, int yoffs
     }
     else
     {
-        notify(WARN)<<"Warning: Texture2DArray::copyTexSubImage2DArray(..) failed, cannot not copy to a non existant texture."<<std::endl;
+        OSG_WARN<<"Warning: Texture2DArray::copyTexSubImage2DArray(..) failed, cannot not copy to a non existant texture."<<std::endl;
     }
 }
 
@@ -501,12 +613,21 @@ void Texture2DArray::allocateMipmap(State& state) const
 
     // get the texture object for the current contextID.
     TextureObject* textureObject = getTextureObject(contextID);
-    
+
     if (textureObject && _textureWidth != 0 && _textureHeight != 0 && _textureDepth != 0)
     {
         const Extensions* extensions = getExtensions(contextID,true);
-        // const Texture::Extensions* texExtensions = Texture::getExtensions(contextID,true);
-        
+
+        int safeSourceFormat = _sourceFormat ? _sourceFormat : _internalFormat;
+
+        // Make sure source format does not contain compressed formats value (like DXT3)
+        // they are invalid when passed to glTexImage3D source format parameter
+        if( isCompressedInternalFormat( safeSourceFormat ) )
+        {
+            if( safeSourceFormat != _internalFormat || !extensions->isCompressedTexImage3DSupported() )
+                safeSourceFormat = GL_RGBA;
+        }
+
         // bind texture
         textureObject->bind();
 
@@ -518,7 +639,7 @@ void Texture2DArray::allocateMipmap(State& state) const
         // we do not reallocate the level 0, since it was already allocated
         width >>= 1;
         height >>= 1;
-        
+
         for( GLsizei k = 1; k < numMipmapLevels  && (width || height); k++)
         {
             if (width == 0)
@@ -526,17 +647,31 @@ void Texture2DArray::allocateMipmap(State& state) const
             if (height == 0)
                 height = 1;
 
-            extensions->glTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, k, _internalFormat,
+            if( isCompressedInternalFormat(safeSourceFormat) )
+            {
+                int size = 0, blockSize = 0;
+
+                getCompressedSize( _internalFormat, width, height, _textureDepth, blockSize, size);
+
+                extensions->glCompressedTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, k, _internalFormat,
+                                                    width, height, _textureDepth, _borderWidth,
+                                                    size,
+                                                    NULL);
+            }
+            else
+            {
+                extensions->glTexImage3D( GL_TEXTURE_2D_ARRAY_EXT, k, _internalFormat,
                      width, height, _textureDepth, _borderWidth,
-                     _sourceFormat ? _sourceFormat : _internalFormat,
-                     _sourceType ? _sourceType : GL_UNSIGNED_BYTE, NULL);
+                     safeSourceFormat, _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
+                     NULL);
+            }
 
             width >>= 1;
             height >>= 1;
         }
-                
+
         // inform state that this texture is the current one bound.
-        state.haveAppliedTextureAttribute(state.getActiveTextureUnit(), this);        
+        state.haveAppliedTextureAttribute(state.getActiveTextureUnit(), this);
     }
 }
 
@@ -564,10 +699,10 @@ Texture2DArray::Extensions::Extensions(const Extensions& rhs):
 {
     _isTexture3DSupported = rhs._isTexture3DSupported;
     _isTexture2DArraySupported = rhs._isTexture2DArraySupported;
-    
+
     _max2DSize = rhs._max2DSize;
     _maxLayerCount = rhs._maxLayerCount;
-    
+
     _glTexImage3D = rhs._glTexImage3D;
     _glTexSubImage3D = rhs._glTexSubImage3D;
     _glCopyTexSubImage3D = rhs._glCopyTexSubImage3D;
@@ -591,10 +726,15 @@ void Texture2DArray::Extensions::lowestCommonDenominator(const Extensions& rhs)
 
 void Texture2DArray::Extensions::setupGLExtensions(unsigned int contextID)
 {
-    _isTexture3DSupported = isGLExtensionSupported(contextID,"GL_EXT_texture3D");
-    _isTexture2DArraySupported = isGLExtensionSupported(contextID,"GL_EXT_texture_array");
+    _isTexture3DSupported = OSG_GL3_FEATURES ||
+                            isGLExtensionSupported(contextID,"GL_EXT_texture3D") ||
+                            strncmp((const char*)glGetString(GL_VERSION),"1.2",3)>=0;
 
+    _isTexture2DArraySupported = OSG_GL3_FEATURES || isGLExtensionSupported(contextID,"GL_EXT_texture_array");
+
+    _max2DSize = 0;
     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &_max2DSize);
+    _maxLayerCount = 0;
     glGetIntegerv(GL_MAX_ARRAY_TEXTURE_LAYERS_EXT, &_maxLayerCount);
 
     setGLExtensionFuncPtr(_glTexImage3D, "glTexImage3D","glTexImage3DEXT");
@@ -612,7 +752,7 @@ void Texture2DArray::Extensions::glTexImage3D( GLenum target, GLint level, GLenu
     }
     else
     {
-        notify(WARN)<<"Error: glTexImage3D not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glTexImage3D not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -624,7 +764,7 @@ void Texture2DArray::Extensions::glTexSubImage3D( GLenum target, GLint level, GL
     }
     else
     {
-        notify(WARN)<<"Error: glTexSubImage3D not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glTexSubImage3D not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -636,7 +776,7 @@ void Texture2DArray::Extensions::glCompressedTexImage3D(GLenum target, GLint lev
     }
     else
     {
-        notify(WARN)<<"Error: glCompressedTexImage3D not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glCompressedTexImage3D not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -648,7 +788,7 @@ void Texture2DArray::Extensions::glCompressedTexSubImage3D( GLenum target, GLint
     }
     else
     {
-        notify(WARN)<<"Error: glCompressedTexImage2D not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glCompressedTexImage2D not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -660,7 +800,7 @@ void Texture2DArray::Extensions::glCopyTexSubImage3D( GLenum target, GLint level
     }
     else
     {
-        notify(WARN)<<"Error: glCopyTexSubImage3D not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glCopyTexSubImage3D not supported by OpenGL driver"<<std::endl;
     }
 }
 
