@@ -1,5 +1,5 @@
 // dxtctool.cpp: implementation of DXTC Tools functions.
-// 
+//
 // Copyright (C) 2002 Tanguy Fautr.
 // For conditions of distribution and use,
 // see copyright notice in dxtctool.h
@@ -17,6 +17,7 @@ namespace dxtc_tool {
     const size_t dxtc_pixels::BSIZE_ALPHA_DXT3 = 8;
     const size_t dxtc_pixels::BSIZE_ALPHA_DXT5 = 8;
 
+
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
@@ -60,7 +61,7 @@ bool dxtc_pixels::VFlip() const
     // Nothing to do if Height == 1
     if (m_Height == 1)
         return true;
-        
+
     if (DXT1())
         VFlip_DXT1();
     else if (DXT3())
@@ -120,7 +121,7 @@ void dxtc_pixels::VFlip_DXT3() const
             for (size_t j = 0; j < (m_Width + 3) / 4; ++j) {
                 const size_t TargetRow = ((m_Height + 3) / 4) - (i + 1);
                 BVF_Alpha_DXT3(GetBlock(i, j, BSIZE_DXT3), GetBlock(TargetRow, j, BSIZE_DXT3));
-                BVF_Color(((dxtc_int8 * ) GetBlock(i, j, BSIZE_DXT3)) + BSIZE_ALPHA_DXT3, 
+                BVF_Color(((dxtc_int8 * ) GetBlock(i, j, BSIZE_DXT3)) + BSIZE_ALPHA_DXT3,
                           ((dxtc_int8 * ) GetBlock(TargetRow, j, BSIZE_DXT3)) + BSIZE_ALPHA_DXT3);
             }
 }
@@ -149,12 +150,80 @@ void dxtc_pixels::VFlip_DXT5() const
             for (size_t j = 0; j < (m_Width + 3) / 4; ++j) {
                 const size_t TargetRow = ((m_Height + 3) / 4) - (i + 1);
                 BVF_Alpha_DXT5(GetBlock(i, j, BSIZE_DXT5), GetBlock(TargetRow, j, BSIZE_DXT5));
-                BVF_Color(((dxtc_int8 * ) GetBlock(i, j, BSIZE_DXT5)) + BSIZE_ALPHA_DXT5, 
+                BVF_Color(((dxtc_int8 * ) GetBlock(i, j, BSIZE_DXT5)) + BSIZE_ALPHA_DXT5,
                           ((dxtc_int8 * ) GetBlock(TargetRow, j, BSIZE_DXT5)) + BSIZE_ALPHA_DXT5);
             }
 }
 
+//
+// Structure of a DXT-1 compressed texture block
+// see page "Opaque and 1-Bit Alpha Textures (Direct3D 9)" on http://msdn.microsoft.com
+// url at time of writing http://msdn.microsoft.com/en-us/library/bb147243(v=VS.85).aspx
+//
+struct DXT1TexelsBlock
+{
+    unsigned short color_0;     // colors at their
+    unsigned short color_1;     // extreme
+    unsigned int   texels4x4;   // interpolated colors (2 bits per texel)
+};
 
+bool CompressedImageTranslucent(size_t width, size_t height, GLenum format, void * imageData)
+{
+    // OSG_NOTICE<<"CompressedImageTranslucent("<<width<<", "<<height<<", "<<format<<", "<<imageData<<")"<<std::endl;
 
+    switch(format)
+    {
+        case(GL_COMPRESSED_RGB_S3TC_DXT1_EXT):
+            return false;
+
+        case(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT):
+        {
+            const DXT1TexelsBlock *texelsBlock = reinterpret_cast<const DXT1TexelsBlock*>(imageData);
+
+            // Only do the check on the first mipmap level, and stop when we
+            // see the first alpha texel
+            int i = (width*height)/16;
+            bool foundAlpha = false;
+            while ((!foundAlpha) && (i>0))
+            {
+                // See if this block might contain transparent texels
+                if (texelsBlock->color_0<=texelsBlock->color_1)
+                {
+                    // Scan the texels block for the '11' bit pattern that
+                    // indicates a transparent texel
+                    int j = 0;
+                    while ((!foundAlpha) && (j < 32))
+                    {
+                        // Check for the '11' bit pattern on this texel
+                        if ( ((texelsBlock->texels4x4 >> j) & 0x03) == 0x03)
+                        {
+                            // Texture is using the 1-bit alpha encoding, so we
+                            return true;
+                        }
+
+                        // Next texel
+                        j += 2;
+                    }
+                }
+
+                // Next block
+                --i;
+                ++texelsBlock;
+            }
+            return false;
+        }
+
+        case(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT):
+            return true;
+
+        case(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT):
+            return true;
+
+        default:
+            break;
+    }
+
+    return false;
+}
 
 } // namespace dxtc_tool
