@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -42,11 +42,12 @@ bool FileCache::isFileAppropriateForFileCache(const std::string& originalFileNam
 
 std::string FileCache::createCacheFileName(const std::string& originalFileName) const
 {
-    std::string cacheFileName = _fileCachePath + "/" + 
-                                osgDB::getServerAddress(originalFileName) + "/" + 
+    std::string serverAddress = osgDB::getServerAddress(originalFileName);
+    std::string cacheFileName = _fileCachePath + "/" +
+                                serverAddress + (serverAddress.empty()?"":"/") +
                                 osgDB::getServerFileName(originalFileName);
 
-    OSG_INFO<<"FileCache::createCacheFileName("<<originalFileName<<") = "<<cacheFileName<<std::endl;
+    OSG_DEBUG<<"FileCache::createCacheFileName("<<originalFileName<<") = "<<cacheFileName<<std::endl;
 
     return cacheFileName;
 }
