@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -32,7 +32,7 @@ typedef char TCHAR;
     #include <sys/types.h>
     #include <sys/stat.h>
     #include <direct.h> // for _mkdir
-    
+
     #define mkdir(x,y) _mkdir((x))
     #define stat64 _stati64
 
@@ -51,13 +51,13 @@ typedef char TCHAR;
     //>OSG_IOS
     //IOS includes
     #include "TargetConditionals.h"
-    
-    #if (TARGET_OS_IPHONE) 
+
+    #if (TARGET_OS_IPHONE)
         #include <Availability.h>
         // workaround a bug which appears when compiling for SDK < 4.0 and for the simulator
-        #ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+        #if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
             #define stat64 stat
-        #else 
+        #else
             #if !TARGET_IPHONE_SIMULATOR
                 #define stat64 stat
             #endif
@@ -68,15 +68,16 @@ typedef char TCHAR;
     // 10.5 defines stat64 so we can't use this #define
     // By default, MAC_OS_X_VERSION_MAX_ALLOWED is set to the latest
     // system the headers know about. So I will use this as the control
-    // variable. (MIN_ALLOWED is set low by default so it is 
-    // unhelpful in this case.) 
+    // variable. (MIN_ALLOWED is set low by default so it is
+    // unhelpful in this case.)
     // Unfortunately, we can't use the label MAC_OS_X_VERSION_10_4
     // for older OS's like Jaguar, Panther since they are not defined,
     // so I am going to hardcode the number.
     #if (MAC_OS_X_VERSION_MAX_ALLOWED <= 1040)
         #define stat64 stat
     #endif
-#elif defined(__CYGWIN__) || defined(__FreeBSD__) || (defined(__hpux) && !defined(_LARGEFILE64_SOURCE))
+#elif defined(__CYGWIN__) || defined(__FreeBSD__) || defined(__DragonFly__) || \
+      (defined(__hpux) && !defined(_LARGEFILE64_SOURCE))
     #define stat64 stat
 #endif
 
@@ -144,7 +145,7 @@ bool osgDB::makeDirectory( const std::string &path )
         OSG_DEBUG << "osgDB::makeDirectory(): cannot create an empty directory" << std::endl;
         return false;
     }
-    
+
     struct stat64 stbuf;
 #ifdef OSG_USE_UTF8_FILENAME
     if( _wstat64( OSGDB_STRING_TO_FILENAME(path).c_str(), &stbuf ) == 0 )
@@ -168,7 +169,7 @@ bool osgDB::makeDirectory( const std::string &path )
     {
         if( dir.empty() )
             break;
- 
+
 #ifdef OSG_USE_UTF8_FILENAME
         if( _wstat64( OSGDB_STRING_TO_FILENAME(dir).c_str(), &stbuf ) < 0 )
 #else
@@ -181,7 +182,7 @@ bool osgDB::makeDirectory( const std::string &path )
                 case ENOTDIR:
                     paths.push( dir );
                     break;
- 
+
                 default:
                     OSG_DEBUG << "osgDB::makeDirectory(): "  << strerror(errno) << std::endl;
                     return false;
@@ -193,7 +194,7 @@ bool osgDB::makeDirectory( const std::string &path )
     while( !paths.empty() )
     {
         std::string dir = paths.top();
- 
+
         #if defined(WIN32)
             //catch drive name
             if (dir.size() == 2 && dir.c_str()[1] == ':') {
@@ -208,9 +209,14 @@ bool osgDB::makeDirectory( const std::string &path )
         if( mkdir( dir.c_str(), 0755 )< 0 )
 #endif
         {
-            OSG_DEBUG << "osgDB::makeDirectory(): "  << strerror(errno) << std::endl;
-            return false;
-        } 
+            // Only return an error if the directory actually doesn't exist.  It's possible that the directory was created
+            // by another thread or process
+            if (!osgDB::fileExists(dir))
+            {
+                OSG_DEBUG << "osgDB::makeDirectory(): "  << strerror(errno) << std::endl;
+                return false;
+            }
+        }
         paths.pop();
     }
     return true;
@@ -245,7 +251,7 @@ bool osgDB::setCurrentWorkingDirectory( const std::string &newCurrentWorkingDire
         OSG_DEBUG << "osgDB::setCurrentWorkingDirectory(): called with empty string." << std::endl;
         return false;
     }
-    
+
 #ifdef OSG_USE_UTF8_FILENAME
     return _wchdir( OSGDB_STRING_TO_FILENAME(newCurrentWorkingDirectory).c_str()) == 0;
 #else
@@ -279,7 +285,7 @@ void osgDB::convertStringPathIntoFilePathList(const std::string& paths,FilePathL
         if (!lastPath.empty())
             filepath.push_back(lastPath);
     }
- 
+
 }
 
 bool osgDB::fileExists(const std::string& filename)
@@ -313,10 +319,10 @@ osgDB::FileType osgDB::fileType(const std::string& filename)
 
 std::string osgDB::findFileInPath(const std::string& filename, const FilePathList& filepath,CaseSensitivity caseSensitivity)
 {
-    if (filename.empty()) 
+    if (filename.empty())
         return filename;
 
-    if (!isFileNameNativeStyle(filename)) 
+    if (!isFileNameNativeStyle(filename))
         return findFileInPath(convertFileNameToNativeStyle(filename), filepath, caseSensitivity);
 
 
@@ -326,24 +332,24 @@ std::string osgDB::findFileInPath(const std::string& filename, const FilePathLis
     {
         OSG_DEBUG << "itr='" <<*itr<< "'\n";
         std::string path = itr->empty() ? filename : concatPaths(*itr, filename);
-        
+
         path = getRealPath(path);
 
         OSG_DEBUG << "FindFileInPath() : trying " << path << " ...\n";
-        if(fileExists(path)) 
+        if(fileExists(path))
         {
             OSG_DEBUG << "FindFileInPath() : USING " << path << "\n";
             return path;
         }
-#ifndef WIN32 
+#ifndef WIN32
 // windows already case insensitive so no need to retry..
-        else if (caseSensitivity==CASE_INSENSITIVE) 
+        else if (caseSensitivity==CASE_INSENSITIVE)
         {
             std::string foundfile = findFileInDirectory(filename,*itr,CASE_INSENSITIVE);
             if (!foundfile.empty()) return foundfile;
         }
 #endif
-            
+
     }
 
     return std::string();
@@ -380,7 +386,7 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
     #else
         bool win32 = false;
     #endif
-    
+
     // If the fileName contains extra path information, make that part of the
     // directory name instead
     if (fileName != getSimpleFileName(fileName))
@@ -458,7 +464,7 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
                 // in the parent path
                 realDirName = findFileInDirectory(lastElement, parentPath,
                                                   CASE_INSENSITIVE);
-      
+
                 dc = osgDB::getDirectoryContents(realDirName);
                 char lastChar = realDirName[realDirName.size()-1];
                 if (lastChar=='/') needFollowingBackslash = false;
@@ -499,7 +505,7 @@ static void appendInstallationLibraryFilePaths(osgDB::FilePathList& filepath)
 #ifdef OSG_DEFAULT_LIBRARY_PATH
 
     // Append the install prefix path to the library search path if configured
-    filepath.push_back(ADDQUOTES(OSG_DEFAULT_LIBRARY_PATH));
+    filepath.push_back(OSG_DEFAULT_LIBRARY_PATH);
 #endif
 }
 
@@ -549,6 +555,13 @@ static void appendInstallationLibraryFilePaths(osgDB::FilePathList& filepath)
 
 #endif // unix getDirectoryContexts
 
+osgDB::DirectoryContents osgDB::getSortedDirectoryContents(const std::string& dirName)
+{
+    osgDB::DirectoryContents filenames = osgDB::getDirectoryContents(dirName);
+    std::sort(filenames.begin(), filenames.end(), osgDB::FileNameComparator());
+    return filenames;
+}
+
 
 osgDB::DirectoryContents osgDB::expandWildcardsInFilename(const std::string& filename)
 {
@@ -652,6 +665,19 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 }
 
 
+bool osgDB::containsCurrentWorkingDirectoryReference(const FilePathList& paths)
+{
+    const std::string cwd(".");
+    for(FilePathList::const_iterator itr = paths.begin();
+        itr != paths.end();
+        ++itr)
+    {
+        if (itr->empty()) return true;
+        if (*itr==cwd) return true;
+    }
+    return false;
+}
+
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 // Implementation of appendPlatformSpecificLibraryFilePaths(..)
@@ -710,18 +736,18 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 
         convertStringPathIntoFilePathList("/usr/bin/:/usr/local/bin/",filepath);
     }
-    
+
 #elif defined(WIN32)
 
     void osgDB::appendPlatformSpecificLibraryFilePaths(FilePathList& filepath)
     {
         // See http://msdn2.microsoft.com/en-us/library/ms682586.aspx
 
-        // Safe DLL search mode changes the DLL search order to search for 
+        // Safe DLL search mode changes the DLL search order to search for
         // DLLs in the current directory after the system directories, instead
-        // of right after the application's directory. According to the article 
-        // linked above, on Windows XP and Windows 2000, Safe DLL search mode 
-        // is disabled by default. However, it is a good idea to enable it. We 
+        // of right after the application's directory. According to the article
+        // linked above, on Windows XP and Windows 2000, Safe DLL search mode
+        // is disabled by default. However, it is a good idea to enable it. We
         // will search as if it was enabled.
 
         // So if SafeDllSearchMode is enabled, the search order is as follows:
@@ -734,7 +760,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         if (retval != 0 && retval < size)
         {
             filenamestring pathstr(path);
-            filenamestring executableDir(pathstr, 0, 
+            filenamestring executableDir(pathstr, 0,
                                       pathstr.find_last_of(OSGDB_FILENAME_TEXT("\\/")));
             convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(executableDir), filepath);
         }
@@ -783,35 +809,35 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
             }
         #endif
 
-        //   3. The system directory. Use the GetSystemDirectory function to 
+        //   3. The system directory. Use the GetSystemDirectory function to
         //      get the path of this directory.
         filenamechar systemDir[(UINT)size];
         retval = OSGDB_WINDOWS_FUNCT(GetSystemDirectory)(systemDir, (UINT)size);
 
         if (retval != 0 && retval < size)
         {
-            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(systemDir), 
+            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(systemDir),
                                               filepath);
         }
         else
         {
             OSG_WARN << "Could not get system directory using "
                 "Win32 API, using default directory." << std::endl;
-            convertStringPathIntoFilePathList("C:\\Windows\\System32", 
+            convertStringPathIntoFilePathList("C:\\Windows\\System32",
                                               filepath);
         }
 
-        //   4. The 16-bit system directory. There is no function that obtains 
+        //   4. The 16-bit system directory. There is no function that obtains
         //      the path of this directory, but it is searched.
-        //   5. The Windows directory. Use the GetWindowsDirectory function to 
+        //   5. The Windows directory. Use the GetWindowsDirectory function to
         //      get the path of this directory.
         filenamechar windowsDir[(UINT)size];
         retval = OSGDB_WINDOWS_FUNCT(GetWindowsDirectory)(windowsDir, (UINT)size);
         if (retval != 0 && retval < size)
         {
-            convertStringPathIntoFilePathList(std::string(OSGDB_FILENAME_TO_STRING(windowsDir)) + 
+            convertStringPathIntoFilePathList(std::string(OSGDB_FILENAME_TO_STRING(windowsDir)) +
                                               "\\System", filepath);
-            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(windowsDir), 
+            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(windowsDir),
                                               filepath);
         }
         else
@@ -826,8 +852,8 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         //   6. The current directory.
         convertStringPathIntoFilePathList(".", filepath);
 
-        //   7. The directories that are listed in the PATH environment 
-        //      variable. Note that this does not include the per-application 
+        //   7. The directories that are listed in the PATH environment
+        //      variable. Note that this does not include the per-application
         //      path specified by the App Paths registry key.
         filenamechar* ptr;
 #ifdef OSG_USE_UTF8_FILENAME
@@ -845,7 +871,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 
         appendInstallationLibraryFilePaths(filepath);
     }
-    
+
 #elif defined(__APPLE__)
 #if (TARGET_OS_IPHONE)
     #define COMPILE_COCOA_VERSION
@@ -862,7 +888,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         #include <Carbon/Carbon.h>
     #endif
     #include <iostream>
-    
+
     // These functions are local to FileUtils.cpp and not exposed to the API
     // returns the path string except for numToShorten directories stripped off the end
     std::string GetShortenedPath(std::string path, int numToShorten)
@@ -899,7 +925,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
             CFRelease(urlRef); // docs say we are responsible for releasing CFURLRef
         }
         return path;
-        
+
     }
 
     std::string GetApplicationParentPath(CFBundleRef mainBundle)
@@ -917,7 +943,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
             CFRelease(urlRef);
         }
         return path;
-        
+
     }
 
     std::string GetApplicationResourcesPath(CFBundleRef mainBundle)
@@ -934,17 +960,17 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 
     // The Cocoa version is about 10 lines of code.
     // The Carbon version is noticably longer.
-    // Unfortunately, the Cocoa version requires -lobjc to be 
-    // linked in when creating an executable. 
+    // Unfortunately, the Cocoa version requires -lobjc to be
+    // linked in when creating an executable.
     // Rumor is that this will be done autmatically in gcc 3.5/Tiger,
     // but for now, this will cause a lot of headaches for people
-    // who aren't familiar with this concept, so the Carbon version 
+    // who aren't familiar with this concept, so the Carbon version
     // is preferable.
     // But for the curious, both implementations are here.
-    // Note that if the Cocoa version is used, the file should be 
+    // Note that if the Cocoa version is used, the file should be
     // renamed to use the .mm extension to denote Objective-C++.
     // And of course, you will need to link against Cocoa
-    // Update: There is a bug in the Cocoa version. Advanced users can remap 
+    // Update: There is a bug in the Cocoa version. Advanced users can remap
     // their systems so these paths go somewhere else. The Carbon calls
     // will catch this, but the hardcoded Cocoa code below will not.
 
@@ -955,26 +981,26 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
     // ~/Library/Application Support/OpenSceneGraph/PlugIns
     // /Library/Application Support/OpenSceneGraph/PlugIns
     // /Network/Library/Application Support/OpenSceneGraph/PlugIns
-    // 
-    // As a side effect of this function, if the application is not a 
+    //
+    // As a side effect of this function, if the application is not a
     // bundle, the first place searched becomes
     // YourProgram/PlugIns
     //
     // In principle, these other directories should be searched:
     // ~/Library/Application Support/YourProgram/PlugIns
     // /Library/Application Support/YourProgram/PlugIns
-    // /Network/Library/Application Support/TheProgram/PlugIns 
-    // But I'm not going to worry about it for now because the 
-    // bundle's PlugIns directory is supposed to be the preferred 
+    // /Network/Library/Application Support/TheProgram/PlugIns
+    // But I'm not going to worry about it for now because the
+    // bundle's PlugIns directory is supposed to be the preferred
     // place for this anyway.
     //
     // Another directory that might be worth considering is
     // the directory the program resides in,
     // but I'm worried about multiplatform distribution.
-    // Because .so is used by other platforms like Linux, we 
+    // Because .so is used by other platforms like Linux, we
     // could end up loading the wrong binary.
     // I'm not sure how robust the current code is for this case.
-    // Assuming the program doesn't crash, will OSG move on to the 
+    // Assuming the program doesn't crash, will OSG move on to the
     // next search directory, or just give up?
     void osgDB::appendPlatformSpecificLibraryFilePaths(FilePathList& filepath)
     {
@@ -988,7 +1014,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 
         // Since this is currently the only Objective-C code in the
         // library, we need an autoreleasepool for obj-c memory management.
-        // If more Obj-C is added, we might move this pool to another 
+        // If more Obj-C is added, we might move this pool to another
         // location so it can be shared. Pools seem to be stackable,
         // so I don't think there will be a problem if multiple pools
         // exist at a time.
@@ -1004,7 +1030,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 
         // Now setup the other search paths
         // Cocoa has a nice method for tilde expansion.
-        // There's probably a better way of getting this directory, but I 
+        // There's probably a better way of getting this directory, but I
         // can't find the call.
         userSupportDir = [@"~/Library/Application Support/OpenSceneGraph/PlugIns" stringByExpandingTildeInPath];
 
@@ -1035,18 +1061,18 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
     // In principle, these other directories should be searched:
     // ~/Library/Application Support/YourProgram/PlugIns
     // /Library/Application Support/YourProgram/PlugIns
-    // /Network/Library/Application Support/TheProgram/PlugIns 
-    // But I'm not going to worry about it for now because the 
-    // bundle's PlugIns directory is supposed to be the preferred 
+    // /Network/Library/Application Support/TheProgram/PlugIns
+    // But I'm not going to worry about it for now because the
+    // bundle's PlugIns directory is supposed to be the preferred
     // place for this anyway.
     //
     // Another directory that might be worth considering is
     // the directory the program resides in,
     // but I'm worried about multiplatform distribution.
-    // Because .so is used by other platforms like Linux, we 
+    // Because .so is used by other platforms like Linux, we
     // could end up loading the wrong binary.
     // I'm not sure how robust the current code is for this case.
-    // Assuming the program doesn't crash, will OSG move on to the 
+    // Assuming the program doesn't crash, will OSG move on to the
     // next search directory, or just give up?
     void osgDB::appendPlatformSpecificLibraryFilePaths(FilePathList& filepath)
     {
@@ -1069,10 +1095,10 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         // Get the main bundle first. No need to retain or release it since
         //  we are not keeping a reference
         myBundle = CFBundleGetMainBundle();
-        
+
         if(myBundle != NULL)
         {
-            // CFBundleGetMainBundle will return a bundle ref even if 
+            // CFBundleGetMainBundle will return a bundle ref even if
             //  the application isn't part of a bundle, so we need to check
             //  if the path to the bundle ends in ".app" to see if it is a
             //  proper application bundle. If it is, the plugins path is added
@@ -1112,7 +1138,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         {
             // Get the URL
             url = CFURLCreateFromFSRef( 0, &f );
-            
+
             if(url)
             {
                 filepath.push_back(GetPathFromCFURLRef(url) + OSG_PLUGIN_PATH);
@@ -1136,7 +1162,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         {
             // Get the URL
             url = CFURLCreateFromFSRef( 0, &f );
-            
+
             if(url)
             {
                 filepath.push_back(GetPathFromCFURLRef(url) + OSG_PLUGIN_PATH);
@@ -1165,8 +1191,8 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
         appendInstallationLibraryFilePaths(filepath);
     }
     #endif
-    
-#else   
+
+#else
 
     void osgDB::appendPlatformSpecificLibraryFilePaths(FilePathList& filepath)
     {
@@ -1197,18 +1223,18 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
     {
         // Get the main application bundle
         CFBundleRef mainBundle = CFBundleGetMainBundle();
-        
+
         if (mainBundle != NULL) {
             // Get the parent directory and the resources directory
             std::string bundlePath = GetApplicationBundlePath(mainBundle);
             std::string resourcesPath = GetApplicationResourcesPath(mainBundle);
-            
+
             // check if application is really part of a .app bundle
             if(bundlePath.substr(bundlePath.length() - 4, 4) == std::string(".app"))
             {
                 if(resourcesPath != std::string(""))
                     filepath.push_back( resourcesPath );
-                
+
                 std::string parentPath = GetShortenedPath(bundlePath, 1);
                 if(parentPath != std::string(""))
                     filepath.push_back( parentPath );
@@ -1226,7 +1252,3 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
 #endif
 
 
-
-
-
-
