@@ -24,14 +24,30 @@ using namespace osgDB;
 static std::string s_lastSchema;
 
 InputStream::InputStream( const osgDB::Options* options )
-    :   _fileVersion(0), _byteSwap(0), _useSchemaData(false), _forceReadingImage(false), _dataDecompress(0)
+    :   _fileVersion(0), _useSchemaData(false), _forceReadingImage(false), _dataDecompress(0)
 {
+    BEGIN_BRACKET.set( "{", +INDENT_VALUE );
+    END_BRACKET.set( "}", -INDENT_VALUE );
+
     if ( !options ) return;
     _options = options;
-    
-    std::string schema;
+
     if ( options->getPluginStringData("ForceReadingImage")=="true" )
         _forceReadingImage = true;
+
+    if ( !options->getPluginStringData("CustomDomains").empty() )
+    {
+        StringList domains, keyAndValue;
+        split( options->getPluginStringData("CustomDomains"), domains, ';' );
+        for ( unsigned int i=0; i<domains.size(); ++i )
+        {
+            split( domains[i], keyAndValue, ':' );
+            if ( keyAndValue.size()>1 )
+                _domainVersionMap[keyAndValue.front()] = atoi(keyAndValue.back().c_str());
+        }
+    }
+
+    std::string schema;
     if ( !options->getPluginStringData("SchemaFile").empty() )
     {
         schema = options->getPluginStringData("SchemaFile");
@@ -43,7 +59,7 @@ InputStream::InputStream( const osgDB::Options* options )
             s_lastSchema = schema;
         }
     }
-    
+
     if ( schema.empty() )
     {
         resetSchema();
@@ -57,6 +73,13 @@ InputStream::~InputStream()
         delete _dataDecompress;
 }
 
+int InputStream::getFileVersion( const std::string& d ) const
+{
+    if ( d.empty() ) return _fileVersion;
+    VersionMap::const_iterator itr = _domainVersionMap.find(d);
+    return itr==_domainVersionMap.end() ? 0 : itr->second;
+}
+
 InputStream& InputStream::operator>>( osg::Vec2b& v )
 {
     char x, y; *this >> x >> y;
@@ -78,9 +101,23 @@ InputStream& InputStream::operator>>( osg::Vec4b& v )
     return *this;
 }
 
+InputStream& InputStream::operator>>( osg::Vec2ub& v )
+{
+    unsigned char x, y; *this >> x >> y;
+    v.set( x, y );
+    return *this;
+}
+
+InputStream& InputStream::operator>>( osg::Vec3ub& v )
+{
+    unsigned char x, y, z; *this >> x >> y >> z;
+    v.set( x, y, z );
+    return *this;
+}
+
 InputStream& InputStream::operator>>( osg::Vec4ub& v )
 {
-    char r, g, b, a; *this >> r >> g >> b >> a;
+    unsigned char r, g, b, a; *this >> r >> g >> b >> a;
     v.set( r, g, b, a );
     return *this;
 }
@@ -94,6 +131,36 @@ InputStream& InputStream::operator>>( osg::Vec3s& v )
 InputStream& InputStream::operator>>( osg::Vec4s& v )
 { *this >> v.x() >> v.y() >> v.z() >> v.w(); return *this; }
 
+InputStream& InputStream::operator>>( osg::Vec2us& v )
+{ *this >> v.x() >> v.y(); return *this; }
+
+InputStream& InputStream::operator>>( osg::Vec3us& v )
+{ *this >> v.x() >> v.y() >> v.z(); return *this; }
+
+InputStream& InputStream::operator>>( osg::Vec4us& v )
+{ *this >> v.x() >> v.y() >> v.z() >> v.w(); return *this; }
+
+
+InputStream& InputStream::operator>>( osg::Vec2i& v )
+{ *this >> v.x() >> v.y(); return *this; }
+
+InputStream& InputStream::operator>>( osg::Vec3i& v )
+{ *this >> v.x() >> v.y() >> v.z(); return *this; }
+
+InputStream& InputStream::operator>>( osg::Vec4i& v )
+{ *this >> v.x() >> v.y() >> v.z() >> v.w(); return *this; }
+
+
+InputStream& InputStream::operator>>( osg::Vec2ui& v )
+{ *this >> v.x() >> v.y(); return *this; }
+
+InputStream& InputStream::operator>>( osg::Vec3ui& v )
+{ *this >> v.x() >> v.y() >> v.z(); return *this; }
+
+InputStream& InputStream::operator>>( osg::Vec4ui& v )
+{ *this >> v.x() >> v.y() >> v.z() >> v.w(); return *this; }
+
+
 InputStream& InputStream::operator>>( osg::Vec2f& v )
 { *this >> v.x() >> v.y(); return *this; }
 
@@ -103,6 +170,7 @@ InputStream& InputStream::operator>>( osg::Vec3f& v )
 InputStream& InputStream::operator>>( osg::Vec4f& v )
 { *this >> v.x() >> v.y() >> v.z() >> v.w(); return *this; }
 
+
 InputStream& InputStream::operator>>( osg::Vec2d& v )
 { *this >> v.x() >> v.y(); return *this; }
 
@@ -112,6 +180,7 @@ InputStream& InputStream::operator>>( osg::Vec3d& v )
 InputStream& InputStream::operator>>( osg::Vec4d& v )
 { *this >> v.x() >> v.y() >> v.z() >> v.w(); return *this; }
 
+
 InputStream& InputStream::operator>>( osg::Quat& q )
 { *this >> q.x() >> q.y() >> q.z() >> q.w(); return *this; }
 
@@ -223,16 +292,16 @@ InputStream& InputStream::operator>>( osg::Matrixd& mat )
 osg::Array* InputStream::readArray()
 {
     osg::ref_ptr<osg::Array> array = NULL;
-    
+
     unsigned int id = 0;
     *this >> PROPERTY("ArrayID") >> id;
-    
+
     ArrayMap::iterator itr = _arrayMap.find( id );
     if ( itr!=_arrayMap.end() )
     {
         return itr->second.get();
     }
-    
+
     DEF_MAPPEE(ArrayType, type);
     *this >> type;
     switch ( type.get() )
@@ -240,154 +309,189 @@ osg::Array* InputStream::readArray()
     case ID_BYTE_ARRAY:
         {
             osg::ByteArray* ba = new osg::ByteArray;
-            readArrayImplementation( ba, CHAR_SIZE, true );
+            readArrayImplementation( ba, 1, CHAR_SIZE);
             array = ba;
         }
         break;
     case ID_UBYTE_ARRAY:
         {
             osg::UByteArray* uba = new osg::UByteArray;
-            readArrayImplementation( uba, CHAR_SIZE, true );
+            readArrayImplementation( uba, 1, CHAR_SIZE );
             array = uba;
         }
         break;
     case ID_SHORT_ARRAY:
         {
             osg::ShortArray* sa = new osg::ShortArray;
-            readArrayImplementation( sa, SHORT_SIZE, true );
+            readArrayImplementation( sa, 1, SHORT_SIZE );
             array = sa;
         }
         break;
     case ID_USHORT_ARRAY:
         {
             osg::UShortArray* usa = new osg::UShortArray;
-            readArrayImplementation( usa, SHORT_SIZE, true );
+            readArrayImplementation( usa, 1, SHORT_SIZE );
             array = usa;
         }
         break;
     case ID_INT_ARRAY:
         {
             osg::IntArray* ia = new osg::IntArray;
-            readArrayImplementation( ia, INT_SIZE, true );
+            readArrayImplementation( ia, 1, INT_SIZE );
             array = ia;
         }
         break;
     case ID_UINT_ARRAY:
         {
             osg::UIntArray* uia = new osg::UIntArray;
-            readArrayImplementation( uia, INT_SIZE, true );
+            readArrayImplementation( uia, 1, INT_SIZE );
             array = uia;
         }
         break;
     case ID_FLOAT_ARRAY:
         {
             osg::FloatArray* fa = new osg::FloatArray;
-            readArrayImplementation( fa, FLOAT_SIZE, true );
+            readArrayImplementation( fa, 1, FLOAT_SIZE );
             array = fa;
         }
         break;
     case ID_DOUBLE_ARRAY:
         {
             osg::DoubleArray* da = new osg::DoubleArray;
-            readArrayImplementation( da, DOUBLE_SIZE, true );
+            readArrayImplementation( da, 1, DOUBLE_SIZE );
             array = da;
         }
         break;
     case ID_VEC2B_ARRAY:
         {
             osg::Vec2bArray* va = new osg::Vec2bArray;
-            readArrayImplementation( va, 2*CHAR_SIZE );
+            readArrayImplementation( va, 2, CHAR_SIZE );
             array = va;
         }
         break;
     case ID_VEC3B_ARRAY:
         {
             osg::Vec3bArray* va = new osg::Vec3bArray;
-            readArrayImplementation( va, 3*CHAR_SIZE );
+            readArrayImplementation( va, 3, CHAR_SIZE );
             array = va;
         }
         break;
     case ID_VEC4B_ARRAY:
         {
             osg::Vec4bArray* va = new osg::Vec4bArray;
-            readArrayImplementation( va, 4*CHAR_SIZE );
+            readArrayImplementation( va, 4, CHAR_SIZE );
+            array = va;
+        }
+        break;
+    case ID_VEC2UB_ARRAY:
+        {
+            osg::Vec2ubArray* va = new osg::Vec2ubArray;
+            readArrayImplementation( va, 2, CHAR_SIZE );
+            array = va;
+        }
+        break;
+    case ID_VEC3UB_ARRAY:
+        {
+            osg::Vec3ubArray* va = new osg::Vec3ubArray;
+            readArrayImplementation( va, 3, CHAR_SIZE );
             array = va;
         }
         break;
     case ID_VEC4UB_ARRAY:
         {
             osg::Vec4ubArray* va = new osg::Vec4ubArray;
-            readArrayImplementation( va, 4*CHAR_SIZE );
+            readArrayImplementation( va, 4, CHAR_SIZE );
             array = va;
         }
         break;
     case ID_VEC2S_ARRAY:
         {
             osg::Vec2sArray* va = new osg::Vec2sArray;
-            readArrayImplementation( va, 2*SHORT_SIZE );
+            readArrayImplementation( va, 2, SHORT_SIZE );
             array = va;
         }
         break;
     case ID_VEC3S_ARRAY:
         {
             osg::Vec3sArray* va = new osg::Vec3sArray;
-            readArrayImplementation( va, 3*SHORT_SIZE );
+            readArrayImplementation( va, 3, SHORT_SIZE );
             array = va;
         }
         break;
     case ID_VEC4S_ARRAY:
         {
             osg::Vec4sArray* va = new osg::Vec4sArray;
-            readArrayImplementation( va, 4*SHORT_SIZE );
+            readArrayImplementation( va, 4, SHORT_SIZE );
+            array = va;
+        }
+        break;
+    case ID_VEC2US_ARRAY:
+        {
+            osg::Vec2usArray* va = new osg::Vec2usArray;
+            readArrayImplementation( va, 2, SHORT_SIZE );
+            array = va;
+        }
+        break;
+    case ID_VEC3US_ARRAY:
+        {
+            osg::Vec3usArray* va = new osg::Vec3usArray;
+            readArrayImplementation( va, 3, SHORT_SIZE );
+            array = va;
+        }
+        break;
+    case ID_VEC4US_ARRAY:
+        {
+            osg::Vec4usArray* va = new osg::Vec4usArray;
+            readArrayImplementation( va, 4, SHORT_SIZE );
             array = va;
         }
         break;
     case ID_VEC2_ARRAY:
         {
             osg::Vec2Array* va = new osg::Vec2Array;
-            readArrayImplementation( va, 2*FLOAT_SIZE );
+            readArrayImplementation( va, 2, FLOAT_SIZE );
             array = va;
         }
         break;
     case ID_VEC3_ARRAY:
         {
             osg::Vec3Array* va = new osg::Vec3Array;
-            readArrayImplementation( va, 3*FLOAT_SIZE );
+            readArrayImplementation( va, 3, FLOAT_SIZE );
             array = va;
         }
         break;
     case ID_VEC4_ARRAY:
         {
             osg::Vec4Array* va = new osg::Vec4Array;
-            readArrayImplementation( va, 4*FLOAT_SIZE );
+            readArrayImplementation( va, 4, FLOAT_SIZE );
             array = va;
         }
         break;
     case ID_VEC2D_ARRAY:
         {
             osg::Vec2dArray* va = new osg::Vec2dArray;
-            readArrayImplementation( va, 2*DOUBLE_SIZE );
+            readArrayImplementation( va, 2, DOUBLE_SIZE );
             array = va;
         }
         break;
     case ID_VEC3D_ARRAY:
         {
             osg::Vec3dArray* va = new osg::Vec3dArray;
-            readArrayImplementation( va, 3*DOUBLE_SIZE );
+            readArrayImplementation( va, 3, DOUBLE_SIZE );
             array = va;
         }
         break;
     case ID_VEC4D_ARRAY:
         {
             osg::Vec4dArray* va = new osg::Vec4dArray;
-            readArrayImplementation( va, 4*DOUBLE_SIZE );
+            readArrayImplementation( va, 4, DOUBLE_SIZE );
             array = va;
         }
         break;
     default:
         throwException( "InputStream::readArray(): Unsupported array type." );
     }
-    
+
     if ( getException() ) return NULL;
     _arrayMap[id] = array;
 
@@ -397,11 +501,16 @@ osg::Array* InputStream::readArray()
 osg::PrimitiveSet* InputStream::readPrimitiveSet()
 {
     osg::ref_ptr<osg::PrimitiveSet> primitive = NULL;
-    
+
     DEF_MAPPEE(PrimitiveType, type);
     DEF_MAPPEE(PrimitiveType, mode);
+    unsigned int numInstances = 0u;
     *this >> type >> mode;
-    
+    if ( _fileVersion>96 )
+    {
+        *this >> numInstances;
+    }
+
     switch ( type.get() )
     {
     case ID_DRAWARRAYS:
@@ -410,6 +519,7 @@ osg::PrimitiveSet* InputStream::readPrimitiveSet()
             *this >> first >> count;
             osg::DrawArrays* da = new osg::DrawArrays( mode.get(), first, count );
             primitive = da;
+            primitive->setNumInstances( numInstances );
         }
         break;
     case ID_DRAWARRAY_LENGTH:
@@ -424,6 +534,7 @@ osg::PrimitiveSet* InputStream::readPrimitiveSet()
             }
             *this >> END_BRACKET;
             primitive = dl;
+            primitive->setNumInstances( numInstances );
         }
         break;
     case ID_DRAWELEMENTS_UBYTE:
@@ -438,6 +549,7 @@ osg::PrimitiveSet* InputStream::readPrimitiveSet()
             }
             *this >> END_BRACKET;
             primitive = de;
+            primitive->setNumInstances( numInstances );
         }
         break;
     case ID_DRAWELEMENTS_USHORT:
@@ -452,6 +564,7 @@ osg::PrimitiveSet* InputStream::readPrimitiveSet()
             }
             *this >> END_BRACKET;
             primitive = de;
+            primitive->setNumInstances( numInstances );
         }
         break;
     case ID_DRAWELEMENTS_UINT:
@@ -466,22 +579,24 @@ osg::PrimitiveSet* InputStream::readPrimitiveSet()
             }
             *this >> END_BRACKET;
             primitive = de;
+            primitive->setNumInstances( numInstances );
         }
         break;
     default:
         throwException( "InputStream::readPrimitiveSet(): Unsupported array type." );
     }
-    
+
     if ( getException() ) return NULL;
     return primitive.release();
 }
 
-osg::Image* InputStream::readImage(bool readFromExternal) 
+osg::Image* InputStream::readImage(bool readFromExternal)
 {
+    std::string className = "osg::Image";
+    if ( _fileVersion>94 )  // ClassName property is only supported in 3.1.4 and higher
+        *this >> PROPERTY("ClassName") >> className;
 
-    std::string className="osg::Image";
     unsigned int id = 0;
-
     *this >> PROPERTY("UniqueID") >> id;
     if ( getException() ) return NULL;
 
@@ -498,22 +613,19 @@ osg::Image* InputStream::readImage(bool readFromExternal)
     if ( getException() ) return NULL;
 
     osg::ref_ptr<osg::Image> image = NULL;
-    
     switch ( decision )
     {
     case IMAGE_INLINE_DATA:
         if ( isBinary() )
         {
-            image = new osg::Image;
-            
             // _origin, _s & _t & _r, _internalTextureFormat
             int origin, s, t, r, internalFormat;
             *this >> origin >> s >> t >> r >> internalFormat;
-            
+
             // _pixelFormat, _dataType, _packing, _allocationMode
             int pixelFormat, dataType, packing, mode;
             *this >> pixelFormat >> dataType >> packing >> mode;
-            
+
             // _data
             unsigned int size = 0; *this >> size;
             if ( size )
@@ -522,11 +634,12 @@ osg::Image* InputStream::readImage(bool readFromExternal)
                 if ( !data )
                     throwException( "InputStream::readImage() Out of memory." );
                 if ( getException() ) return NULL;
-                
+
                 readCharArray( data, size );
+                image = new osg::Image;
                 image->setOrigin( (osg::Image::Origin)origin );
                 image->setImage( s, t, r, internalFormat, pixelFormat, dataType,
-                    (unsigned char*)data, (osg::Image::AllocationMode)mode, packing );
+                    (unsigned char*)data, osg::Image::USE_NEW_DELETE, packing );
             }
 
             // _mipmapData
@@ -536,7 +649,7 @@ osg::Image* InputStream::readImage(bool readFromExternal)
             {
                 *this >> levels[i];
             }
-            if ( levelSize>0 )
+            if ( image && levelSize>0 )
                 image->setMipmapLevels( levels );
             readFromExternal = false;
         }
@@ -554,7 +667,7 @@ osg::Image* InputStream::readImage(bool readFromExternal)
                     if ( getException() ) return NULL;
                 }
                 readCharArray( data, size );
-                
+
                 std::string ext = osgDB::getFileExtension( name );
                 osgDB::ReaderWriter* reader =
                     osgDB::Registry::instance()->getReaderWriterForExtension( ext );
@@ -562,7 +675,7 @@ osg::Image* InputStream::readImage(bool readFromExternal)
                 {
                     std::stringstream inputStream;
                     inputStream.write( data, size );
-                    
+
                     osgDB::ReaderWriter::ReadResult rr = reader->readImage( inputStream );
                     if ( rr.validImage() )
                         image = rr.takeImage();
@@ -587,23 +700,20 @@ osg::Image* InputStream::readImage(bool readFromExternal)
     default:
         break;
     }
-    
-    if ( readFromExternal )
+
+    if ( readFromExternal && !name.empty() )
     {
         image = osgDB::readImageFile( name, getOptions() );
         if ( !image && _forceReadingImage ) image = new osg::Image;
     }
+
+    image = static_cast<osg::Image*>( readObjectFields(className, id, image.get()) );
     if ( image.valid() )
     {
         image->setFileName( name );
         image->setWriteHint( (osg::Image::WriteHint)writeHint );
     }
-
-    image = static_cast<osg::Image*>( readObjectFields(className, image.get()) );
-
-   _identifierMap[id] = image;
-
-   return image.release();
+    return image.release();
 }
 
 osg::Object* InputStream::readObject( osg::Object* existingObj )
@@ -612,7 +722,7 @@ osg::Object* InputStream::readObject( osg::Object* existingObj )
     unsigned int id = 0;
     *this >> className >> BEGIN_BRACKET >> PROPERTY("UniqueID") >> id;
     if ( getException() ) return NULL;
-    
+
     IdentifierMap::iterator itr = _identifierMap.find( id );
     if ( itr!=_identifierMap.end() )
     {
@@ -620,16 +730,14 @@ osg::Object* InputStream::readObject( osg::Object* existingObj )
         return itr->second.get();
     }
 
-    osg::ref_ptr<osg::Object> obj = readObjectFields( className, existingObj );
-
-    _identifierMap[id] = obj;
+    osg::ref_ptr<osg::Object> obj = readObjectFields( className, id, existingObj );
 
     advanceToCurrentEndBracket();
 
     return obj.release();
 }
 
-osg::Object* InputStream::readObjectFields( const std::string& className, osg::Object* existingObj )
+osg::Object* InputStream::readObjectFields( const std::string& className, unsigned int id, osg::Object* existingObj )
 {
     ObjectWrapper* wrapper = Registry::instance()->getObjectWrapperManager()->findWrapper( className );
     if ( !wrapper )
@@ -638,9 +746,9 @@ osg::Object* InputStream::readObjectFields( const std::string& className, osg::O
                                << className << std::endl;
         return NULL;
     }
-    _fields.push_back( className );
-    
+
     osg::ref_ptr<osg::Object> obj = existingObj ? existingObj : wrapper->getProto()->cloneType();
+    _identifierMap[id] = obj;
     if ( obj.valid() )
     {
         const StringList& associates = wrapper->getAssociates();
@@ -654,14 +762,12 @@ osg::Object* InputStream::readObjectFields( const std::string& className, osg::O
                 continue;
             }
             _fields.push_back( assocWrapper->getName() );
-            
             assocWrapper->read( *this, *obj );
             if ( getException() ) return NULL;
-            
+
             _fields.pop_back();
         }
     }
-    _fields.pop_back();
     return obj.release();
 }
 
@@ -672,11 +778,11 @@ void InputStream::readSchema( std::istream& fin )
     while ( std::getline(fin, line) )
     {
         if ( line[0]=='#' ) continue;  // Comment
-        
+
         StringList keyAndValue;
         split( line, keyAndValue, '=' );
         if ( keyAndValue.size()<2 ) continue;
-        
+
         setWrapperSchema( osgDB::trimEnclosingSpaces(keyAndValue[0]),
                           osgDB::trimEnclosingSpaces(keyAndValue[1]) );
     }
@@ -686,13 +792,15 @@ InputStream::ReadType InputStream::start( InputIterator* inIterator )
 {
     _fields.clear();
     _fields.push_back( "Start" );
-    
+
     ReadType type = READ_UNKNOWN;
     _in = inIterator;
     if ( !_in )
         throwException( "InputStream: Null stream specified." );
     if ( getException() ) return type;
-    
+
+    _in->setInputStream(this);
+
     // Check OSG header information
     unsigned int version = 0;
     if ( isBinary() )
@@ -700,9 +808,22 @@ InputStream::ReadType InputStream::start( InputIterator* inIterator )
         unsigned int typeValue;
         *this >> typeValue >> version;
         type = static_cast<ReadType>(typeValue);
-        
+
         unsigned int attributes; *this >> attributes;
+        if ( attributes&0x4 ) inIterator->setSupportBinaryBrackets( true );
         if ( attributes&0x2 ) _useSchemaData = true;
+
+        // Record custom domains
+        if ( attributes&0x1 )
+        {
+            unsigned int numDomains; *this >> numDomains;
+            for ( unsigned int i=0; i<numDomains; ++i )
+            {
+                std::string domainName; *this >> domainName;
+                int domainVersion; *this >> domainVersion;
+                _domainVersionMap[domainName] = domainVersion;
+            }
+        }
     }
     if ( !isBinary() )
     {
@@ -710,12 +831,19 @@ InputStream::ReadType InputStream::start( InputIterator* inIterator )
         if ( typeString=="Scene" ) type = READ_SCENE;
         else if ( typeString=="Image" ) type = READ_IMAGE;
         else if ( typeString=="Object" ) type = READ_OBJECT;
-        
+
         std::string osgName, osgVersion;
         *this >> PROPERTY("#Version") >> version;
         *this >> PROPERTY("#Generator") >> osgName >> osgVersion;
+
+        while ( matchString("#CustomDomain") )
+        {
+            std::string domainName; *this >> domainName;
+            int domainVersion; *this >> domainVersion;
+            _domainVersionMap[domainName] = domainVersion;
+        }
     }
-    
+
     // Record file version for back-compatibility checking of wrappers
     _fileVersion = version;
     _fields.pop_back();
@@ -726,20 +854,20 @@ void InputStream::decompress()
 {
     if ( !isBinary() ) return;
     _fields.clear();
-    
+
     std::string compressorName; *this >> compressorName;
     if ( compressorName!="0" )
     {
         std::string data;
         _fields.push_back( "Decompression" );
-        
+
         BaseCompressor* compressor = Registry::instance()->getObjectWrapperManager()->findCompressor(compressorName);
         if ( !compressor )
         {
             OSG_WARN << "InputStream::decompress(): No such compressor "
                                    << compressorName << std::endl;
         }
-        
+
         if ( !compressor->decompress(*(_in->getStream()), data) )
             throwException( "InputStream: Failed to decompress stream." );
         if ( getException() ) return;
@@ -748,7 +876,7 @@ void InputStream::decompress()
         _in->setStream( _dataDecompress );
         _fields.pop_back();
     }
-    
+
     if ( _useSchemaData )
     {
         _fields.push_back( "SchemaData" );
@@ -770,7 +898,7 @@ void InputStream::setWrapperSchema( const std::string& name, const std::string&
                                << name << std::endl;
         return;
     }
-    
+
     StringList schema, methods, keyAndValue;
     std::vector<int> types;
     split( properties, schema );
@@ -804,7 +932,7 @@ void InputStream::resetSchema()
 }
 
 template<typename T>
-void InputStream::readArrayImplementation( T* a, int read_size, bool useByteSwap )
+void InputStream::readArrayImplementation( T* a, unsigned int numComponentsPerElements, unsigned int componentSizeInBytes )
 {
     int size = 0;
     *this >> size >> BEGIN_BRACKET;
@@ -813,12 +941,8 @@ void InputStream::readArrayImplementation( T* a, int read_size, bool useByteSwap
         a->resize( size );
         if ( isBinary() )
         {
-            readCharArray( (char*)&((*a)[0]), read_size*size ); checkStream();
-            if ( useByteSwap && _byteSwap )
-            {
-                for ( int i=0; i<size; ++i )
-                    osg::swapBytes( (char*)&((*a)[i]), read_size );
-            }
+            readComponentArray( (char*)&((*a)[0]), size, numComponentsPerElements, componentSizeInBytes );
+            checkStream();
         }
         else
         {
