@@ -92,6 +92,14 @@ ObjectWrapper::ObjectWrapper( osg::Object* proto, const std::string& name,
     split( associates, _associates );
 }
 
+ObjectWrapper::ObjectWrapper( osg::Object* proto, const std::string& domain, const std::string& name,
+                              const std::string& associates )
+:   osg::Referenced(),
+    _proto(proto), _domain(domain), _name(name), _version(0)
+{
+    split( associates, _associates );
+}
+
 void ObjectWrapper::addSerializer( BaseSerializer* s, BaseSerializer::Type t )
 {
     s->_firstVersion = _version;
@@ -118,7 +126,7 @@ BaseSerializer* ObjectWrapper::getSerializer( const std::string& name )
         if ( (*itr)->getName()==name )
             return itr->get();
     }
-    
+
     for ( StringList::const_iterator itr=_associates.begin(); itr!=_associates.end(); ++itr )
     {
         const std::string& assocName = *itr;
@@ -129,7 +137,7 @@ BaseSerializer* ObjectWrapper::getSerializer( const std::string& name )
                                    << assocName << std::endl;
             continue;
         }
-        
+
         for ( SerializerList::iterator aitr=assocWrapper->_serializers.begin();
               aitr!=assocWrapper->_serializers.end(); ++aitr )
         {
@@ -143,12 +151,13 @@ BaseSerializer* ObjectWrapper::getSerializer( const std::string& name )
 bool ObjectWrapper::read( InputStream& is, osg::Object& obj )
 {
     bool readOK = true;
+    int inputVersion = is.getFileVersion(_domain);
     for ( SerializerList::iterator itr=_serializers.begin();
           itr!=_serializers.end(); ++itr )
     {
         BaseSerializer* serializer = itr->get();
-        if ( serializer->_firstVersion <= is.getFileVersion() &&
-             is.getFileVersion() <= serializer->_lastVersion)
+        if ( serializer->_firstVersion <= inputVersion &&
+             inputVersion <= serializer->_lastVersion )
         {
             if ( !serializer->read(is, obj) )
             {
@@ -169,19 +178,20 @@ bool ObjectWrapper::read( InputStream& is, osg::Object& obj )
      {
          (*itr)->objectRead(is, obj);
      }
-    
+
     return readOK;
 }
 
 bool ObjectWrapper::write( OutputStream& os, const osg::Object& obj )
 {
     bool writeOK = true;
+    int outputVersion = os.getFileVersion(_domain);
     for ( SerializerList::iterator itr=_serializers.begin();
           itr!=_serializers.end(); ++itr )
     {
         BaseSerializer* serializer = itr->get();
-        if ( serializer->_firstVersion <= OPENSCENEGRAPH_SOVERSION &&
-             OPENSCENEGRAPH_SOVERSION <= serializer->_lastVersion)
+        if ( serializer->_firstVersion <= outputVersion &&
+             outputVersion <= serializer->_lastVersion )
         {
             if ( !serializer->write(os, obj) )
             {
@@ -204,7 +214,7 @@ bool ObjectWrapper::readSchema( const StringList& properties, const std::vector<
     if ( !_backupSerializers.size() )
         _backupSerializers = _serializers;
     _serializers.clear();
-    
+
     unsigned int size = properties.size();
     unsigned int serializersSize = _backupSerializers.size();
     for ( unsigned int i=0; i<size; ++i )
@@ -215,7 +225,7 @@ bool ObjectWrapper::readSchema( const StringList& properties, const std::vector<
                                    << ": Incompatible serializers size" << std::endl;
             break;
         }
-        
+
         const std::string& prop = properties[i];
         if ( prop==_backupSerializers[i]->getName() )
         {
@@ -248,7 +258,7 @@ void ObjectWrapper::writeSchema( StringList& properties, std::vector<int>& types
     {
         properties.push_back( (*itr)->getName() );
     }
-    
+
     for ( std::vector<int>::iterator itr=_typeList.begin();
           itr!=_typeList.end(); ++itr )
     {
@@ -280,6 +290,31 @@ RegisterWrapperProxy::~RegisterWrapperProxy()
     }
 }
 
+////////////////////////////////////////////////////////////////////////////////////////////////////////
+//
+// RegisterCustomWrapperProxy
+//
+RegisterCustomWrapperProxy::RegisterCustomWrapperProxy(
+        osg::Object* proto, const std::string& domain, const std::string& name,
+        const std::string& associates, AddPropFunc func )
+{
+    _wrapper = new ObjectWrapper( proto, domain, name, associates );
+    if ( func ) (*func)( domain.c_str(), _wrapper.get() );
+
+    if (Registry::instance())
+    {
+        Registry::instance()->getObjectWrapperManager()->addWrapper( _wrapper.get() );
+    }
+}
+
+RegisterCustomWrapperProxy::~RegisterCustomWrapperProxy()
+{
+    if (Registry::instance())
+    {
+        Registry::instance()->getObjectWrapperManager()->removeWrapper( _wrapper.get() );
+    }
+}
+
 ////////////////////////////////////////////////////////////////////////////////////////////////////////
 //
 // ObjectWrapperManager
@@ -344,6 +379,9 @@ ObjectWrapperManager::ObjectWrapperManager()
     glTable.add( "GL_LIGHT6", GL_LIGHT6 );
     glTable.add( "GL_LIGHT7", GL_LIGHT7 );
 
+    glTable.add("GL_VERTEX_PROGRAM_POINT_SIZE", GL_VERTEX_PROGRAM_POINT_SIZE);
+    glTable.add("GL_VERTEX_PROGRAM_TWO_SIDE", GL_VERTEX_PROGRAM_TWO_SIDE);
+
     // Functions
     glTable.add( "NEVER", GL_NEVER );
     glTable.add( "LESS", GL_LESS );
@@ -414,7 +452,7 @@ ObjectWrapperManager::ObjectWrapperManager()
     glTable.add( "GL_COMPRESSED_RGBA_PVRTC_4BPPV1_IMG",GL_COMPRESSED_RGBA_PVRTC_4BPPV1_IMG );
     glTable.add( "GL_COMPRESSED_RGBA_PVRTC_2BPPV1_IMG",GL_COMPRESSED_RGBA_PVRTC_2BPPV1_IMG );
     glTable.add( "GL_ETC1_RGB8_OES",GL_ETC1_RGB8_OES );
-    
+
     // Texture source types
     glTable.add( "GL_BYTE", GL_BYTE );
     glTable.add( "GL_SHORT", GL_SHORT );
@@ -484,10 +522,15 @@ ObjectWrapperManager::ObjectWrapperManager()
     arrayTable.add( "Vec2bArray", ID_VEC2B_ARRAY );
     arrayTable.add( "Vec3bArray", ID_VEC3B_ARRAY );
     arrayTable.add( "Vec4bArray", ID_VEC4B_ARRAY );
+    arrayTable.add( "Vec2ubArray", ID_VEC2UB_ARRAY );
+    arrayTable.add( "Vec3ubArray", ID_VEC3UB_ARRAY );
     arrayTable.add( "Vec4ubArray", ID_VEC4UB_ARRAY );
     arrayTable.add( "Vec2sArray", ID_VEC2S_ARRAY );
     arrayTable.add( "Vec3sArray", ID_VEC3S_ARRAY );
     arrayTable.add( "Vec4sArray", ID_VEC4S_ARRAY );
+    arrayTable.add( "Vec2usArray", ID_VEC2US_ARRAY );
+    arrayTable.add( "Vec3usArray", ID_VEC3US_ARRAY );
+    arrayTable.add( "Vec4usArray", ID_VEC4US_ARRAY );
     arrayTable.add( "Vec2fArray", ID_VEC2_ARRAY );
     arrayTable.add( "Vec3fArray", ID_VEC3_ARRAY );
     arrayTable.add( "Vec4fArray", ID_VEC4_ARRAY );
