@@ -19,15 +19,21 @@
 #include <osgDB/ObjectWrapper>
 #include <fstream>
 #include <sstream>
+#include <stdlib.h>
 
 using namespace osgDB;
 
 OutputStream::OutputStream( const osgDB::Options* options )
-:   _writeImageHint(WRITE_USE_IMAGE_HINT), _useSchemaData(false)
+:   _writeImageHint(WRITE_USE_IMAGE_HINT), _useSchemaData(false), _useRobustBinaryFormat(true)
 {
+    BEGIN_BRACKET.set( "{", +INDENT_VALUE );
+    END_BRACKET.set( "}", -INDENT_VALUE );
+
     if ( !options ) return;
     _options = options;
-    
+
+    if ( options->getPluginStringData("RobustBinaryFormat")=="false" )
+        _useRobustBinaryFormat = false;
     if ( options->getPluginStringData("SchemaData")=="true" )
         _useSchemaData = true;
     if ( !options->getPluginStringData("SchemaFile").empty() )
@@ -42,12 +48,31 @@ OutputStream::OutputStream( const osgDB::Options* options )
         else if ( hintString=="UseExternal" ) _writeImageHint = WRITE_USE_EXTERNAL;
         else if ( hintString=="WriteOut" ) _writeImageHint = WRITE_EXTERNAL_FILE;
     }
+
+    if ( !options->getPluginStringData("CustomDomains").empty() )
+    {
+        StringList domains, keyAndValue;
+        split( options->getPluginStringData("CustomDomains"), domains, ';' );
+        for ( unsigned int i=0; i<domains.size(); ++i )
+        {
+            split( domains[i], keyAndValue, ':' );
+            if ( keyAndValue.size()>1 )
+                _domainVersionMap[keyAndValue.front()] = atoi(keyAndValue.back().c_str());
+        }
+    }
 }
 
 OutputStream::~OutputStream()
 {
 }
 
+int OutputStream::getFileVersion( const std::string& d ) const
+{
+    if ( d.empty() ) return OPENSCENEGRAPH_SOVERSION;
+    VersionMap::const_iterator itr = _domainVersionMap.find(d);
+    return itr==_domainVersionMap.end() ? 0 : itr->second;
+}
+
 OutputStream& OutputStream::operator<<( const osg::Vec2b& v )
 { *this << v.x() << v.y(); return *this; }
 
@@ -57,6 +82,12 @@ OutputStream& OutputStream::operator<<( const osg::Vec3b& v )
 OutputStream& OutputStream::operator<<( const osg::Vec4b& v )
 { *this << v.x() << v.y() << v.z() << v.w(); return *this; }
 
+OutputStream& OutputStream::operator<<( const osg::Vec2ub& v )
+{ *this << v.x() << v.y(); return *this; }
+
+OutputStream& OutputStream::operator<<( const osg::Vec3ub& v )
+{ *this << v.x() << v.y() << v.z(); return *this; }
+
 OutputStream& OutputStream::operator<<( const osg::Vec4ub& v )
 { *this << v.r() << v.g() << v.b() << v.a(); return *this; }
 
@@ -69,6 +100,15 @@ OutputStream& OutputStream::operator<<( const osg::Vec3s& v )
 OutputStream& OutputStream::operator<<( const osg::Vec4s& v )
 { *this << v.x() << v.y() << v.z() << v.w(); return *this; }
 
+OutputStream& OutputStream::operator<<( const osg::Vec2us& v )
+{ *this << v.x() << v.y(); return *this; }
+
+OutputStream& OutputStream::operator<<( const osg::Vec3us& v )
+{ *this << v.x() << v.y() << v.z(); return *this; }
+
+OutputStream& OutputStream::operator<<( const osg::Vec4us& v )
+{ *this << v.x() << v.y() << v.z() << v.w(); return *this; }
+
 OutputStream& OutputStream::operator<<( const osg::Vec2f& v )
 { *this << v.x() << v.y(); return *this; }
 
@@ -156,7 +196,7 @@ void OutputStream::writeArray( const osg::Array* a )
         *this << std::endl;
         return;
     }
-    
+
     switch ( a->getType() )
     {
     case osg::Array::ByteArrayType:
@@ -203,6 +243,14 @@ void OutputStream::writeArray( const osg::Array* a )
         *this << MAPPEE(ArrayType, ID_VEC4B_ARRAY);
         writeArrayImplementation( static_cast<const osg::Vec4bArray*>(a), a->getNumElements() );
         break;
+    case osg::Array::Vec2ubArrayType:
+        *this << MAPPEE(ArrayType, ID_VEC2UB_ARRAY);
+        writeArrayImplementation( static_cast<const osg::Vec2ubArray*>(a), a->getNumElements() );
+        break;
+    case osg::Array::Vec3ubArrayType:
+        *this << MAPPEE(ArrayType, ID_VEC3UB_ARRAY);
+        writeArrayImplementation( static_cast<const osg::Vec3ubArray*>(a), a->getNumElements() );
+        break;
     case osg::Array::Vec4ubArrayType:
         *this << MAPPEE(ArrayType, ID_VEC4UB_ARRAY);
         writeArrayImplementation( static_cast<const osg::Vec4ubArray*>(a), a->getNumElements() );
@@ -219,6 +267,18 @@ void OutputStream::writeArray( const osg::Array* a )
         *this << MAPPEE(ArrayType, ID_VEC4S_ARRAY);
         writeArrayImplementation( static_cast<const osg::Vec4sArray*>(a), a->getNumElements() );
         break;
+    case osg::Array::Vec2usArrayType:
+        *this << MAPPEE(ArrayType, ID_VEC2US_ARRAY);
+        writeArrayImplementation( static_cast<const osg::Vec2usArray*>(a), a->getNumElements() );
+        break;
+    case osg::Array::Vec3usArrayType:
+        *this << MAPPEE(ArrayType, ID_VEC3US_ARRAY);
+        writeArrayImplementation( static_cast<const osg::Vec3usArray*>(a), a->getNumElements() );
+        break;
+    case osg::Array::Vec4usArrayType:
+        *this << MAPPEE(ArrayType, ID_VEC4US_ARRAY);
+        writeArrayImplementation( static_cast<const osg::Vec4usArray*>(a), a->getNumElements() );
+        break;
     case osg::Array::Vec2ArrayType:
         *this << MAPPEE(ArrayType, ID_VEC2_ARRAY);
         writeArrayImplementation( static_cast<const osg::Vec2Array*>(a), a->getNumElements() );
@@ -251,14 +311,14 @@ void OutputStream::writeArray( const osg::Array* a )
 void OutputStream::writePrimitiveSet( const osg::PrimitiveSet* p )
 {
     if ( !p ) return;
-    
+
     switch ( p->getType() )
     {
     case osg::PrimitiveSet::DrawArraysPrimitiveType:
         *this << MAPPEE(PrimitiveType, ID_DRAWARRAYS);
         {
             const osg::DrawArrays* da = static_cast<const osg::DrawArrays*>(p);
-            *this << MAPPEE(PrimitiveType, da->getMode())
+            *this << MAPPEE(PrimitiveType, da->getMode()) << da->getNumInstances()
                   << da->getFirst() << da->getCount() << std::endl;
         }
         break;
@@ -266,7 +326,7 @@ void OutputStream::writePrimitiveSet( const osg::PrimitiveSet* p )
         *this << MAPPEE(PrimitiveType, ID_DRAWARRAY_LENGTH);
         {
             const osg::DrawArrayLengths* dl = static_cast<const osg::DrawArrayLengths*>(p);
-            *this << MAPPEE(PrimitiveType, dl->getMode()) << dl->getFirst();
+            *this << MAPPEE(PrimitiveType, dl->getMode()) << dl->getNumInstances() << dl->getFirst();
             writeArrayImplementation( dl, dl->size(), 4 );
         }
         break;
@@ -274,7 +334,7 @@ void OutputStream::writePrimitiveSet( const osg::PrimitiveSet* p )
         *this << MAPPEE(PrimitiveType, ID_DRAWELEMENTS_UBYTE);
         {
             const osg::DrawElementsUByte* de = static_cast<const osg::DrawElementsUByte*>(p);
-            *this << MAPPEE(PrimitiveType, de->getMode());
+            *this << MAPPEE(PrimitiveType, de->getMode()) << de->getNumInstances();
             writeArrayImplementation( de, de->size(), 4 );
         }
         break;
@@ -282,7 +342,7 @@ void OutputStream::writePrimitiveSet( const osg::PrimitiveSet* p )
         *this << MAPPEE(PrimitiveType, ID_DRAWELEMENTS_USHORT);
         {
             const osg::DrawElementsUShort* de = static_cast<const osg::DrawElementsUShort*>(p);
-            *this << MAPPEE(PrimitiveType, de->getMode());
+            *this << MAPPEE(PrimitiveType, de->getMode()) << de->getNumInstances();
             writeArrayImplementation( de, de->size(), 4 );
         }
         break;
@@ -290,7 +350,7 @@ void OutputStream::writePrimitiveSet( const osg::PrimitiveSet* p )
         *this << MAPPEE(PrimitiveType, ID_DRAWELEMENTS_UINT);
         {
             const osg::DrawElementsUInt* de = static_cast<const osg::DrawElementsUInt*>(p);
-            *this << MAPPEE(PrimitiveType, de->getMode());
+            *this << MAPPEE(PrimitiveType, de->getMode()) << de->getNumInstances();
             writeArrayImplementation( de, de->size(), 4 );
         }
         break;
@@ -303,22 +363,18 @@ void OutputStream::writeImage( const osg::Image* img )
 {
     if ( !img ) return;
 
-    // std::string name = img->libraryName();
-    // name += std::string("::") + img->className();
+    std::string name = img->libraryName();
+    name += std::string("::") + img->className();
 
     bool newID = false;
     unsigned int id = findOrCreateObjectID( img, newID );
 
-    // *this << name << BEGIN_BRACKET << std::endl;       // Write object name
-    *this << PROPERTY("UniqueID") << id << std::endl;  // Write image ID
+    *this << PROPERTY("ClassName") << name << std::endl;   // Write object name
+    *this << PROPERTY("UniqueID") << id << std::endl;      // Write image ID
     if ( getException() ) return;
 
     if (newID)
     {
-        *this << PROPERTY("FileName"); writeWrappedString(img->getFileName()); *this << std::endl;
-        *this << PROPERTY("WriteHint") << (int)img->getWriteHint();
-        if ( getException() ) return;
-
         int decision = IMAGE_EXTERNAL;
         switch ( _writeImageHint )
         {
@@ -334,19 +390,30 @@ void OutputStream::writeImage( const osg::Image* img )
             break;
         }
 
-        *this << decision << std::endl;
+
+        std::string imageFileName = img->getFileName();
         if ( decision==IMAGE_WRITE_OUT || _writeImageHint==WRITE_EXTERNAL_FILE )
         {
-            bool result = osgDB::writeImageFile( *img, img->getFileName() );
-            OSG_NOTICE << "OutputStream::writeImage(): Write image data to external file "
-                                    << img->getFileName() << std::endl;
+            if (imageFileName.empty())
+            {
+                OSG_NOTICE<<"Empty Image::FileName resetting to image.dds"<<std::endl;
+                imageFileName = "image.dds";
+            }
+
+            bool result = osgDB::writeImageFile( *img, imageFileName );
+            OSG_NOTICE << "OutputStream::writeImage(): Write image data to external file " << imageFileName << std::endl;
             if ( !result )
             {
-                OSG_WARN << "OutputStream::writeImage(): Failed to write "
-                                    << img->getFileName() << std::endl;
+                OSG_WARN << "OutputStream::writeImage(): Failed to write " << img->getFileName() << std::endl;
             }
         }
 
+        *this << PROPERTY("FileName"); writeWrappedString(imageFileName); *this << std::endl;
+        *this << PROPERTY("WriteHint") << (int)img->getWriteHint();
+        if ( getException() ) return;
+
+        *this << decision << std::endl;
+
         switch ( decision )
         {
         case IMAGE_INLINE_DATA:
@@ -362,15 +429,33 @@ void OutputStream::writeImage( const osg::Image* img )
 
                 // _data
                 unsigned int size = img->getTotalSizeInBytesIncludingMipmaps();
-                writeSize(size); writeCharArray( (char*)img->data(), size );
+                writeSize(size);
+
+                for(osg::Image::DataIterator img_itr(img); img_itr.valid(); ++img_itr)
+                {
+                    writeCharArray( (char*)img_itr.data(), img_itr.size() );
+                }
 
                 // _mipmapData
-                const osg::Image::MipmapDataType& levels = img->getMipmapLevels();
-                writeSize(levels.size());
-                for ( osg::Image::MipmapDataType::const_iterator itr=levels.begin();
-                    itr!=levels.end(); ++itr )
+                unsigned int numMipmaps = img->getNumMipmapLevels()-1;
+                writeSize(numMipmaps);
+                int s = img->s();
+                int t = img->t();
+                int r = img->r();
+                unsigned int offset = 0;
+                for (unsigned int i=0; i<numMipmaps; ++i)
                 {
-                    *this << *itr;
+                    unsigned int size = osg::Image::computeImageSizeInBytes(s,t,r,img->getPixelFormat(),img->getDataType(),img->getPacking());
+                    offset += size;
+
+                    *this << offset;
+
+                    s >>= 1;
+                    t >>= 1;
+                    r >>= 1;
+                    if (s<1) s=1;
+                    if (t<1) t=1;
+                    if (r<1) r=1;
                 }
             }
             break;
@@ -455,7 +540,6 @@ void OutputStream::writeObjectFields( const osg::Object* obj )
                                 << name << std::endl;
         return;
     }
-    _fields.push_back( name );
 
     const StringList& associates = wrapper->getAssociates();
     for ( StringList::const_iterator itr=associates.begin(); itr!=associates.end(); ++itr )
@@ -475,7 +559,7 @@ void OutputStream::writeObjectFields( const osg::Object* obj )
                 StringList properties;
                 std::vector<int> types;
                 assocWrapper->writeSchema( properties, types );
-                
+
                 unsigned int size = osg::minimum( properties.size(), types.size() );
                 if ( size>0 )
                 {
@@ -495,34 +579,55 @@ void OutputStream::writeObjectFields( const osg::Object* obj )
 
         _fields.pop_back();
     }
-    _fields.pop_back();
-
 }
 
 void OutputStream::start( OutputIterator* outIterator, OutputStream::WriteType type )
 {
     _fields.clear();
     _fields.push_back( "Start" );
-    
+
     _out = outIterator;
     if ( !_out )
         throwException( "OutputStream: Null stream specified." );
     if ( getException() ) return;
-    
+
     if ( isBinary() )
     {
         *this << (unsigned int)type << (unsigned int)OPENSCENEGRAPH_SOVERSION;
-        
+
         bool useCompressSource = false;
         unsigned int attributes = 0;
 
+        // From SOVERSION 98, start to support custom wrapper domains, enabling the attribute bit
+        if ( _domainVersionMap.size()>0 ) attributes |= 0x1;
+
         if ( _useSchemaData )
         {
             attributes |= 0x2;  // Record if we use inbuilt schema data or not
             useCompressSource = true;
         }
+
+        // From SOVERSION 98, start to support binary begin/end brackets so we can easily ignore
+        // errors and unsupport classes, enabling the attribute bit
+        if ( _useRobustBinaryFormat )
+        {
+            outIterator->setSupportBinaryBrackets( true );
+            attributes |= 0x4;
+        }
         *this << attributes;
-        
+
+        // Record all custom versions
+        if ( _domainVersionMap.size()>0 )
+        {
+            unsigned int numDomains = _domainVersionMap.size();
+            *this << numDomains;
+            for ( VersionMap::iterator itr=_domainVersionMap.begin();
+                  itr!=_domainVersionMap.end(); ++itr )
+            {
+                *this << itr->first << itr->second;
+            }
+        }
+
         if ( !_compressorName.empty() )
         {
             BaseCompressor* compressor = Registry::instance()->getObjectWrapperManager()->findCompressor(_compressorName);
@@ -539,7 +644,7 @@ void OutputStream::start( OutputIterator* outIterator, OutputStream::WriteType t
         }
         if ( !_compressorName.empty() ) *this << _compressorName;
         else *this << std::string("0");  // No compressor
-        
+
         // Compressors and inbuilt schema use a new stream, which will be merged with the original one at the end.
         if ( useCompressSource )
         {
@@ -557,11 +662,19 @@ void OutputStream::start( OutputIterator* outIterator, OutputStream::WriteType t
         case WRITE_OBJECT: typeString = "Object"; break;
         default: break;
         }
-        
+
         *this << typeString << std::endl;
         *this << PROPERTY("#Version") << (unsigned int)OPENSCENEGRAPH_SOVERSION << std::endl;
         *this << PROPERTY("#Generator") << std::string("OpenSceneGraph")
               << std::string(osgGetVersion()) << std::endl;
+        if ( _domainVersionMap.size()>0 )
+        {
+            for ( VersionMap::iterator itr=_domainVersionMap.begin();
+                  itr!=_domainVersionMap.end(); ++itr )
+            {
+                *this << PROPERTY("#CustomDomain") << itr->first << itr->second << std::endl;
+            }
+        }
         *this << std::endl;
     }
     _fields.pop_back();
@@ -571,29 +684,29 @@ void OutputStream::compress( std::ostream* ostream )
 {
     _fields.clear();
     if ( !isBinary() ) return;
-    
+
     std::stringstream schemaSource;
     if ( _useSchemaData )
     {
         _fields.push_back( "SchemaData" );
-        
+
         std::string schemaData;
-        for ( std::map<std::string, std::string>::iterator itr=_inbuiltSchemaMap.begin();
+        for ( SchemaMap::iterator itr=_inbuiltSchemaMap.begin();
               itr!=_inbuiltSchemaMap.end(); ++itr )
         {
             schemaData += itr->first + '=';
             schemaData += itr->second;
             schemaData += '\n';
         }
-        
+
         int size = schemaData.size();
         schemaSource.write( (char*)&size, INT_SIZE );
         schemaSource.write( schemaData.c_str(), size );
-        
+
         _inbuiltSchemaMap.clear();
         _fields.pop_back();
     }
-    
+
     if ( !_compressorName.empty() )
     {
         _fields.push_back( "Compression" );
@@ -603,7 +716,7 @@ void OutputStream::compress( std::ostream* ostream )
             _fields.pop_back();
             return;
         }
-        
+
         if ( !compressor->compress(*ostream, schemaSource.str() + _compressSource.str()) )
             throwException( "OutputStream: Failed to compress stream." );
         if ( getException() ) return;
@@ -625,11 +738,11 @@ void OutputStream::writeSchema( std::ostream& fout )
     {
         ObjectWrapper* wrapper = itr->second.get();
         fout << itr->first << " =";
-        
+
         StringList properties;
         std::vector<int> types;
         wrapper->writeSchema( properties, types );
-        
+
         std::string propertiesString;
         unsigned int size = osg::minimum( properties.size(), types.size() );
         for ( unsigned int i=0; i<size; ++i )
