@@ -43,7 +43,8 @@ OrbitManipulator::OrbitManipulator( int flags )
 
 /// Constructor.
 OrbitManipulator::OrbitManipulator( const OrbitManipulator& om, const CopyOp& copyOp )
-   : inherited( om, copyOp ),
+   : osg::Object(om, copyOp),
+     inherited( om, copyOp ),
      _center( om._center ),
      _rotation( om._rotation ),
      _distance( om._distance ),
@@ -151,7 +152,7 @@ void OrbitManipulator::getTransformation( osg::Vec3d& eye, osg::Vec3d& center, o
 /** Sets the transformation by heading. Heading is given as an angle in radians giving a azimuth in xy plane.
     Its meaning is similar to longitude used in cartography and navigation.
     Positive number is going to the east direction.*/
-void OrbitManipulator::setHeading( double azimuth ) 
+void OrbitManipulator::setHeading( double azimuth )
 {
     CoordinateFrame coordinateFrame = getCoordinateFrame( _center );
     Vec3d localUp = getUpVector( coordinateFrame );
@@ -261,7 +262,7 @@ bool OrbitManipulator::handleMouseWheel( const GUIEventAdapter& ea, GUIActionAda
             // perform zoom
             zoomModel( -_wheelZoomFactor, true );
             us.requestRedraw();
-            us.requestContinuousUpdate( false );
+            us.requestContinuousUpdate( isAnimating() || _thrown );
             return true;
         }
 
@@ -297,7 +298,7 @@ bool OrbitManipulator::performMovementMiddleMouseButton( const double eventTimeD
 
 
 // doc in parent
-bool OrbitManipulator::performMovementRightMouseButton( const double eventTimeDelta, const double dx, const double dy )
+bool OrbitManipulator::performMovementRightMouseButton( const double eventTimeDelta, const double /*dx*/, const double dy )
 {
     // zoom model
     zoomModel( dy * getThrowScale( eventTimeDelta ), true );
@@ -383,7 +384,7 @@ void OrbitManipulator::rotateTrackball( const float px0, const float py0,
     osg::Vec3d axis;
     float angle;
 
-    trackball( axis, angle, px1, py1, px0, py0 );
+    trackball( axis, angle, px0 + (px1-px0)*scale, py0 + (py1-py0)*scale, px0, py0 );
 
     Quat new_rotate;
     new_rotate.makeRotate( angle, axis );
