@@ -1,16 +1,18 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
+#include <stdlib.h>
+
 
 #include<OpenThreads/ScopedLock>
 
@@ -25,6 +27,7 @@
 #include <osg/Notify>
 #include <osg/io_utils>
 #include <osg/Timer>
+#include <osg/ImageUtils>
 
 using namespace osgParticle;
 
@@ -32,86 +35,42 @@ using namespace osgParticle;
 
 static float random(float min,float max) { return min + (max-min)*(float)rand()/(float)RAND_MAX; }
 
-static void fillSpotLightImage(unsigned char* ptr, const osg::Vec4& centerColour, const osg::Vec4& backgroudColour, unsigned int size, float power)
-{
-    if (size==1)
-    {
-        float r = 0.5f;
-        osg::Vec4 color = centerColour*r+backgroudColour*(1.0f-r);
-        *ptr++ = (unsigned char)((color[0])*255.0f);
-        *ptr++ = (unsigned char)((color[1])*255.0f);
-        *ptr++ = (unsigned char)((color[2])*255.0f);
-        *ptr++ = (unsigned char)((color[3])*255.0f);
-        return;
-    }
-
-    float mid = (float(size)-1.0f)*0.5f;
-    float div = 2.0f/float(size);
-    for(unsigned int r=0;r<size;++r)
-    {
-        //unsigned char* ptr = image->data(0,r,0);
-        for(unsigned int c=0;c<size;++c)
-        {
-            float dx = (float(c) - mid)*div;
-            float dy = (float(r) - mid)*div;
-            float r = powf(1.0f-sqrtf(dx*dx+dy*dy),power);
-            if (r<0.0f) r=0.0f;
-            osg::Vec4 color = centerColour*r+backgroudColour*(1.0f-r);
-            *ptr++ = (unsigned char)((color[0])*255.0f);
-            *ptr++ = (unsigned char)((color[1])*255.0f);
-            *ptr++ = (unsigned char)((color[2])*255.0f);
-            *ptr++ = (unsigned char)((color[3])*255.0f);
-        }
-    }
-}
-
-static osg::Image* createSpotLightImage(const osg::Vec4& centerColour, const osg::Vec4& backgroudColour, unsigned int size, float power)
-{
-
-#if 0
-    osg::Image* image = new osg::Image;
-    unsigned char* ptr = image->data(0,0,0);
-    fillSpotLightImage(ptr, centerColour, backgroudColour, size, power);
-
-    return image;
-#else
-    osg::Image* image = new osg::Image;
-    osg::Image::MipmapDataType mipmapData;
-    unsigned int s = size;
-    unsigned int totalSize = 0;
-    unsigned i;
-    for(i=0; s>0; s>>=1, ++i)
-    {
-        if (i>0) mipmapData.push_back(totalSize);
-        totalSize += s*s*4;
-    }
-
-    unsigned char* ptr = new unsigned char[totalSize];
-    image->setImage(size, size, size, GL_RGBA, GL_RGBA, GL_UNSIGNED_BYTE, ptr, osg::Image::USE_NEW_DELETE,1);
-
-    image->setMipmapLevels(mipmapData);
-
-    s = size;
-    for(i=0; s>0; s>>=1, ++i)
-    {
-        fillSpotLightImage(ptr, centerColour, backgroudColour, s, power);
-        ptr += s*s*4;
-    }
-
-    return image;
-#endif    
-}
-
-
-PrecipitationEffect::PrecipitationEffect()
+PrecipitationEffect::PrecipitationEffect():
+    _previousFrameTime(FLT_MAX)
 {
     setNumChildrenRequiringUpdateTraversal(1);
-    
+
     setUpGeometries(1024);
 
     rain(0.5);
 }
 
+
+PrecipitationEffect::PrecipitationEffect(const PrecipitationEffect& copy, const osg::CopyOp& copyop):
+    osg::Node(copy,copyop),
+    _previousFrameTime(FLT_MAX)
+{
+    setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+1);
+
+    _wind = copy._wind;
+    _particleSpeed = copy._particleSpeed;
+    _particleSize = copy._particleSize;
+    _particleColor = copy._particleColor;
+    _maximumParticleDensity = copy._maximumParticleDensity;
+    _cellSize = copy._cellSize;
+    _nearTransition = copy._nearTransition;
+    _farTransition = copy._farTransition;
+
+    _fog = copy._fog.valid() ? dynamic_cast<osg::Fog*>(copy._fog->clone(copyop)) : 0;
+
+
+    _useFarLineSegments = copy._useFarLineSegments;
+
+    _dirty = true;
+
+    update();
+}
+
 void PrecipitationEffect::rain(float intensity)
 {
     _wind.set(0.0f,0.0f,0.0f);
@@ -122,7 +81,7 @@ void PrecipitationEffect::rain(float intensity)
     _cellSize.set(5.0f / (0.25f+intensity), 5.0f / (0.25f+intensity), 5.0f);
     _nearTransition = 25.f;
     _farTransition = 100.0f - 60.0f*sqrtf(intensity);
-    
+
     if (!_fog) _fog = new osg::Fog;
 
     _fog->setMode(osg::Fog::EXP);
@@ -130,7 +89,7 @@ void PrecipitationEffect::rain(float intensity)
     _fog->setColor(osg::Vec4(0.5, 0.5, 0.5, 1.0));
 
     _useFarLineSegments = false;
-    
+
     _dirty = true;
 
     update();
@@ -146,7 +105,7 @@ void PrecipitationEffect::snow(float intensity)
     _cellSize.set(5.0f / (0.25f+intensity), 5.0f / (0.25f+intensity), 5.0f);
     _nearTransition = 25.f;
     _farTransition = 100.0f - 60.0f*sqrtf(intensity);
-    
+
     if (!_fog) _fog = new osg::Fog;
 
     _fog->setMode(osg::Fog::EXP);
@@ -154,35 +113,27 @@ void PrecipitationEffect::snow(float intensity)
     _fog->setColor(osg::Vec4(0.6, 0.6, 0.6, 1.0));
 
     _useFarLineSegments = false;
-    
+
     _dirty = true;
 
     update();
 }
 
-PrecipitationEffect::PrecipitationEffect(const PrecipitationEffect& copy, const osg::CopyOp& copyop):
-    osg::Node(copy,copyop)
-{
-    setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+1);            
-    _dirty = true;
-    update();
-}
-
 void PrecipitationEffect::compileGLObjects(osg::RenderInfo& renderInfo) const
 {
-    if (_quadGeometry.valid()) 
+    if (_quadGeometry.valid())
     {
         _quadGeometry->compileGLObjects(renderInfo);
         if (_quadGeometry->getStateSet()) _quadGeometry->getStateSet()->compileGLObjects(*renderInfo.getState());
     }
 
-    if (_lineGeometry.valid()) 
+    if (_lineGeometry.valid())
     {
         _lineGeometry->compileGLObjects(renderInfo);
         if (_lineGeometry->getStateSet()) _lineGeometry->getStateSet()->compileGLObjects(*renderInfo.getState());
     }
 
-    if (_pointGeometry.valid()) 
+    if (_pointGeometry.valid())
     {
         _pointGeometry->compileGLObjects(renderInfo);
         if (_pointGeometry->getStateSet()) _pointGeometry->getStateSet()->compileGLObjects(*renderInfo.getState());
@@ -195,19 +146,20 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
     if (nv.getVisitorType() == osg::NodeVisitor::UPDATE_VISITOR)
     {
         if (_dirty) update();
-    
+
         if (nv.getFrameStamp())
         {
             double currentTime = nv.getFrameStamp()->getSimulationTime();
-            static double previousTime = currentTime;
-            double delta = currentTime - previousTime;
+            if (_previousFrameTime==FLT_MAX) _previousFrameTime = currentTime;
+
+            double delta = currentTime - _previousFrameTime;
             _origin += _wind * delta;
-            previousTime = currentTime;
+            _previousFrameTime = currentTime;
         }
 
         return;
     }
-    
+
     if (nv.getVisitorType() == osg::NodeVisitor::NODE_VISITOR)
     {
         if (_dirty) update();
@@ -220,10 +172,10 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
                 compileGLObjects(globjVisitor->getRenderInfo());
             }
         }
-    
+
         return;
     }
-    
+
 
     if (nv.getVisitorType() != osg::NodeVisitor::CULL_VISITOR)
     {
@@ -237,59 +189,59 @@ void PrecipitationEffect::traverse(osg::NodeVisitor& nv)
     }
 
     ViewIdentifier viewIndentifier(cv, nv.getNodePath());
-    
+
     {
         PrecipitationDrawableSet* precipitationDrawableSet = 0;
-        
+
         {
             OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
             precipitationDrawableSet = &(_viewDrawableMap[viewIndentifier]);
-        }
-                
-        if (!precipitationDrawableSet->_quadPrecipitationDrawable)
-        {
-            precipitationDrawableSet->_quadPrecipitationDrawable = new PrecipitationDrawable;
-            precipitationDrawableSet->_quadPrecipitationDrawable->setRequiresPreviousMatrix(true);
-            precipitationDrawableSet->_quadPrecipitationDrawable->setGeometry(_quadGeometry.get());
-            precipitationDrawableSet->_quadPrecipitationDrawable->setStateSet(_quadStateSet.get());
-            precipitationDrawableSet->_quadPrecipitationDrawable->setDrawType(GL_QUADS);
-            
-            precipitationDrawableSet->_linePrecipitationDrawable = new PrecipitationDrawable;
-            precipitationDrawableSet->_linePrecipitationDrawable->setRequiresPreviousMatrix(true);
-            precipitationDrawableSet->_linePrecipitationDrawable->setGeometry(_lineGeometry.get());
-            precipitationDrawableSet->_linePrecipitationDrawable->setStateSet(_lineStateSet.get());
-            precipitationDrawableSet->_linePrecipitationDrawable->setDrawType(GL_LINES);
 
-            precipitationDrawableSet->_pointPrecipitationDrawable = new PrecipitationDrawable;
-            precipitationDrawableSet->_pointPrecipitationDrawable->setRequiresPreviousMatrix(false);
-            precipitationDrawableSet->_pointPrecipitationDrawable->setGeometry(_pointGeometry.get());
-            precipitationDrawableSet->_pointPrecipitationDrawable->setStateSet(_pointStateSet.get());
-            precipitationDrawableSet->_pointPrecipitationDrawable->setDrawType(GL_POINTS);
+            if (!precipitationDrawableSet->_quadPrecipitationDrawable)
+            {
+                precipitationDrawableSet->_quadPrecipitationDrawable = new PrecipitationDrawable;
+                precipitationDrawableSet->_quadPrecipitationDrawable->setRequiresPreviousMatrix(true);
+                precipitationDrawableSet->_quadPrecipitationDrawable->setGeometry(_quadGeometry.get());
+                precipitationDrawableSet->_quadPrecipitationDrawable->setStateSet(_quadStateSet.get());
+                precipitationDrawableSet->_quadPrecipitationDrawable->setDrawType(GL_QUADS);
+
+                precipitationDrawableSet->_linePrecipitationDrawable = new PrecipitationDrawable;
+                precipitationDrawableSet->_linePrecipitationDrawable->setRequiresPreviousMatrix(true);
+                precipitationDrawableSet->_linePrecipitationDrawable->setGeometry(_lineGeometry.get());
+                precipitationDrawableSet->_linePrecipitationDrawable->setStateSet(_lineStateSet.get());
+                precipitationDrawableSet->_linePrecipitationDrawable->setDrawType(GL_LINES);
+
+                precipitationDrawableSet->_pointPrecipitationDrawable = new PrecipitationDrawable;
+                precipitationDrawableSet->_pointPrecipitationDrawable->setRequiresPreviousMatrix(false);
+                precipitationDrawableSet->_pointPrecipitationDrawable->setGeometry(_pointGeometry.get());
+                precipitationDrawableSet->_pointPrecipitationDrawable->setStateSet(_pointStateSet.get());
+                precipitationDrawableSet->_pointPrecipitationDrawable->setDrawType(GL_POINTS);
+            }
         }
-        
+
         cull(*precipitationDrawableSet, cv);
-        
+
         cv->pushStateSet(_stateset.get());
         float depth = 0.0f;
 
         if (!precipitationDrawableSet->_quadPrecipitationDrawable->getCurrentCellMatrixMap().empty())
         {
             cv->pushStateSet(precipitationDrawableSet->_quadPrecipitationDrawable->getStateSet());
-            cv->addDrawableAndDepth(precipitationDrawableSet->_quadPrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);    
+            cv->addDrawableAndDepth(precipitationDrawableSet->_quadPrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);
             cv->popStateSet();
         }
 
         if (!precipitationDrawableSet->_linePrecipitationDrawable->getCurrentCellMatrixMap().empty())
         {
             cv->pushStateSet(precipitationDrawableSet->_linePrecipitationDrawable->getStateSet());
-            cv->addDrawableAndDepth(precipitationDrawableSet->_linePrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);    
+            cv->addDrawableAndDepth(precipitationDrawableSet->_linePrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);
             cv->popStateSet();
         }
 
         if (!precipitationDrawableSet->_pointPrecipitationDrawable->getCurrentCellMatrixMap().empty())
         {
             cv->pushStateSet(precipitationDrawableSet->_pointPrecipitationDrawable->getStateSet());
-            cv->addDrawableAndDepth(precipitationDrawableSet->_pointPrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);    
+            cv->addDrawableAndDepth(precipitationDrawableSet->_pointPrecipitationDrawable.get(),cv->getModelViewMatrix(),depth);
             cv->popStateSet();
         }
 
@@ -302,7 +254,7 @@ void PrecipitationEffect::update()
 {
     _dirty = false;
 
-    osg::notify(osg::INFO)<<"PrecipitationEffect::update()"<<std::endl;
+    OSG_INFO<<"PrecipitationEffect::update()"<<std::endl;
 
     float length_u = _cellSize.x();
     float length_v = _cellSize.y();
@@ -318,16 +270,16 @@ void PrecipitationEffect::update()
     _inverse_du.set(1.0f/length_u, 0.0f, 0.0f);
     _inverse_dv.set(0.0f, 1.0f/length_v, 0.0f);
     _inverse_dw.set(0.0f, 0.0f, 1.0f/length_w);
-    
-    osg::notify(osg::INFO)<<"Cell size X="<<length_u<<std::endl;
-    osg::notify(osg::INFO)<<"Cell size Y="<<length_v<<std::endl;
-    osg::notify(osg::INFO)<<"Cell size Z="<<length_w<<std::endl;
-    
+
+    OSG_INFO<<"Cell size X="<<length_u<<std::endl;
+    OSG_INFO<<"Cell size Y="<<length_v<<std::endl;
+    OSG_INFO<<"Cell size Z="<<length_w<<std::endl;
+
 
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
         _viewDrawableMap.clear();
-    }    
+    }
 
     // set up state/
     {
@@ -350,13 +302,13 @@ void PrecipitationEffect::update()
         }
         else _inversePeriodUniform->set(1.0f/_period);
 
-        if (!_particleSizeUniform)
+        if (!_particleColorUniform)
         {
             _particleColorUniform = new osg::Uniform("particleColour", _particleColor);
             _stateset->addUniform(_particleColorUniform.get());
         }
         else _particleColorUniform->set(_particleColor);
-        
+
         if (!_particleSizeUniform)
         {
             _particleSizeUniform = new osg::Uniform("particleSize", _particleSize);
@@ -365,11 +317,11 @@ void PrecipitationEffect::update()
         else _particleSizeUniform->set(_particleSize);
 
     }
-        
+
 }
 
-void PrecipitationEffect::createGeometry(unsigned int numParticles, 
-                    osg::Geometry* quad_geometry, 
+void PrecipitationEffect::createGeometry(unsigned int numParticles,
+                    osg::Geometry* quad_geometry,
                     osg::Geometry* line_geometry,
                     osg::Geometry* point_geometry)
 {
@@ -378,7 +330,7 @@ void PrecipitationEffect::createGeometry(unsigned int numParticles,
     osg::Vec2 offset10(1.0f,0.0f);
     osg::Vec2 offset01(0.0f,1.0f);
     osg::Vec2 offset11(1.0f,1.0f);
-    
+
     osg::Vec2 offset0(0.5f,0.0f);
     osg::Vec2 offset1(0.5f,1.0f);
 
@@ -391,7 +343,7 @@ void PrecipitationEffect::createGeometry(unsigned int numParticles,
     if (quad_geometry)
     {
         quad_geometry->setName("quad");
-    
+
         quad_vertices = new osg::Vec3Array(numParticles*4);
         quad_offsets = new osg::Vec2Array(numParticles*4);
 
@@ -431,7 +383,7 @@ void PrecipitationEffect::createGeometry(unsigned int numParticles,
     for(unsigned int i=0; i< numParticles; ++i)
     {
         osg::Vec3 pos( random(0.0f, 1.0f), random(0.0f, 1.0f), random(0.0f, 1.0f));
-    
+
         // quad particles
         if (quad_vertices)
         {
@@ -444,7 +396,7 @@ void PrecipitationEffect::createGeometry(unsigned int numParticles,
             (*quad_offsets)[i*4+2] = offset11;
             (*quad_offsets)[i*4+3] = offset10;
         }
-                
+
         // line particles
         if (line_vertices)
         {
@@ -453,7 +405,7 @@ void PrecipitationEffect::createGeometry(unsigned int numParticles,
             (*line_offsets)[i*2] = offset0;
             (*line_offsets)[i*2+1] = offset1;
         }
-        
+
         // point particles
         if (point_vertices)
         {
@@ -468,9 +420,9 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
     unsigned int quadRenderBin = 13;
     unsigned int lineRenderBin = 12;
     unsigned int pointRenderBin = 11;
-    
-    
-    osg::notify(osg::INFO)<<"PrecipitationEffect::setUpGeometries("<<numParticles<<")"<<std::endl;
+
+
+    OSG_INFO<<"PrecipitationEffect::setUpGeometries("<<numParticles<<")"<<std::endl;
 
     bool needGeometryRebuild = false;
 
@@ -487,15 +439,15 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
         _lineGeometry->setUseVertexBufferObjects(true);
         needGeometryRebuild = true;
     }
-    
+
     if (!_pointGeometry || _pointGeometry->getVertexArray()->getNumElements() != numParticles)
     {
         _pointGeometry = new osg::Geometry;
         _pointGeometry->setUseVertexBufferObjects(true);
         needGeometryRebuild = true;
     }
-    
-    if (needGeometryRebuild) 
+
+    if (needGeometryRebuild)
     {
         createGeometry(numParticles, _quadGeometry.get(), _lineGeometry.get(), _pointGeometry.get());
     }
@@ -504,13 +456,13 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
     if (!_quadStateSet)
     {
         _quadStateSet = new osg::StateSet;
-        
+
         osg::Program* program = new osg::Program;
         _quadStateSet->setAttribute(program);
         _quadStateSet->setRenderBinDetails(quadRenderBin,"DepthSortedBin");
 
 #ifdef USE_LOCAL_SHADERS
-        char vertexShaderSource[] = 
+        char vertexShaderSource[] =
             "uniform float inversePeriod;\n"
             "uniform vec4 particleColour;\n"
             "uniform float particleSize;\n"
@@ -553,9 +505,10 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "    v1.xy += dp*texCoord.x;\n"
             "    \n"
             "    gl_Position = gl_ProjectionMatrix * v1;\n"
+            "    gl_ClipVertex = v1;\n"
             "}\n";
 
-        char fragmentShaderSource[] = 
+        char fragmentShaderSource[] =
             "uniform sampler2D baseTexture;\n"
             "varying vec2 texCoord;\n"
             "varying vec4 colour;\n"
@@ -584,7 +537,7 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
         _lineStateSet->setRenderBinDetails(lineRenderBin,"DepthSortedBin");
 
 #ifdef USE_LOCAL_SHADERS
-        char vertexShaderSource[] = 
+        char vertexShaderSource[] =
             "uniform float inversePeriod;\n"
             "uniform vec4 particleColour;\n"
             "uniform float particleSize;\n"
@@ -624,9 +577,10 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "    v1.xyz += dv*texCoord.y;\n"
             "    \n"
             "    gl_Position = gl_ProjectionMatrix * v1;\n"
+            "    gl_ClipVertex = v1;\n"
             "}\n";
 
-        char fragmentShaderSource[] = 
+        char fragmentShaderSource[] =
             "uniform sampler2D baseTexture;\n"
             "varying vec2 texCoord;\n"
             "varying vec4 colour;\n"
@@ -654,7 +608,7 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
         _pointStateSet->setAttribute(program);
 
 #ifdef USE_LOCAL_SHADERS
-        char vertexShaderSource[] = 
+        char vertexShaderSource[] =
             "uniform float inversePeriod;\n"
             "uniform vec4 particleColour;\n"
             "uniform float particleSize;\n"
@@ -681,9 +635,10 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
             "    gl_PointSize = ceil(pointSize);\n"
             "    \n"
             "    colour.a = 0.05+(pointSize*pointSize)/(gl_PointSize*gl_PointSize);\n"
+            "    gl_ClipVertex = gl_ModelViewMatrix * v_current;\n"
             "}\n";
 
-        char fragmentShaderSource[] = 
+        char fragmentShaderSource[] =
             "uniform sampler2D baseTexture;\n"
             "varying vec4 colour;\n"
             "\n"
@@ -704,7 +659,12 @@ void PrecipitationEffect::setUpGeometries(unsigned int numParticles)
         osg::PointSprite *sprite = new osg::PointSprite();
         _pointStateSet->setTextureAttributeAndModes(0, sprite, osg::StateAttribute::ON);
 
-        _pointStateSet->setMode(GL_VERTEX_PROGRAM_POINT_SIZE, osg::StateAttribute::ON);
+        #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+            _pointStateSet->setMode(GL_VERTEX_PROGRAM_POINT_SIZE, osg::StateAttribute::ON);
+        #else
+            OSG_NOTICE<<"Warning: ParticleEffect::setUpGeometries(..) not fully implemented."<<std::endl;
+        #endif
+
         _pointStateSet->setRenderBinDetails(pointRenderBin,"DepthSortedBin");
     }
 
@@ -732,18 +692,18 @@ void PrecipitationEffect::cull(PrecipitationDrawableSet& pds, osgUtil::CullVisit
 
     osg::Matrix inverse_modelview;
     inverse_modelview.invert(*(cv->getModelViewMatrix()));
-    
+
     osg::Vec3 eyeLocal = osg::Vec3(0.0f,0.0f,0.0f) * inverse_modelview;
-    //osg::notify(osg::NOTICE)<<"  eyeLocal "<<eyeLocal<<std::endl;
-    
+    //OSG_NOTICE<<"  eyeLocal "<<eyeLocal<<std::endl;
+
     float eye_k = (eyeLocal-_origin)*_inverse_dw;
     osg::Vec3 eye_kPlane = eyeLocal-_dw*eye_k-_origin;
-    
-    // osg::notify(osg::NOTICE)<<"  eye_kPlane "<<eye_kPlane<<std::endl;
-    
+
+    // OSG_NOTICE<<"  eye_kPlane "<<eye_kPlane<<std::endl;
+
     float eye_i = eye_kPlane*_inverse_du;
     float eye_j = eye_kPlane*_inverse_dv;
-    
+
     osg::Polytope frustum;
     frustum.setToUnitFrustum(false,false);
     frustum.transformProvidingInverse(*(cv->getProjectionMatrix()));
@@ -752,7 +712,7 @@ void PrecipitationEffect::cull(PrecipitationDrawableSet& pds, osgUtil::CullVisit
     float i_delta = _farTransition * _inverse_du.x();
     float j_delta = _farTransition * _inverse_dv.y();
     float k_delta = 1;//_nearTransition * _inverse_dw.z();
-    
+
     int i_min = (int)floor(eye_i - i_delta);
     int j_min = (int)floor(eye_j - j_delta);
     int k_min = (int)floor(eye_k - k_delta);
@@ -760,8 +720,8 @@ void PrecipitationEffect::cull(PrecipitationDrawableSet& pds, osgUtil::CullVisit
     int i_max = (int)ceil(eye_i + i_delta);
     int j_max = (int)ceil(eye_j + j_delta);
     int k_max = (int)ceil(eye_k + k_delta);
-    
-    //osg::notify(osg::NOTICE)<<"i_delta="<<i_delta<<" j_delta="<<j_delta<<" k_delta="<<k_delta<<std::endl;
+
+    //OSG_NOTICE<<"i_delta="<<i_delta<<" j_delta="<<j_delta<<" k_delta="<<k_delta<<std::endl;
 
     unsigned int numTested=0;
     unsigned int numInFrustum=0;
@@ -777,19 +737,19 @@ void PrecipitationEffect::cull(PrecipitationDrawableSet& pds, osgUtil::CullVisit
             {
                 float startTime = (float)(i)*iCyle + (float)(j)*jCyle;
                 startTime = (startTime-floor(startTime))*_period;
-    
+
                 if (build(eyeLocal, i,j,k, startTime, pds, frustum, cv)) ++numInFrustum;
                 ++numTested;
             }
         }
     }
 
-    
+
 #ifdef DO_TIMING
     osg::Timer_t endTick = osg::Timer::instance()->tick();
 
-    osg::notify(osg::NOTICE)<<"time for cull "<<osg::Timer::instance()->delta_m(startTick,endTick)<<"ms  numTested="<<numTested<<" numInFrustum"<<numInFrustum<<std::endl;
-    osg::notify(osg::NOTICE)<<"     quads "<<pds._quadPrecipitationDrawable->getCurrentCellMatrixMap().size()<<"   lines "<<pds._linePrecipitationDrawable->getCurrentCellMatrixMap().size()<<"   points "<<pds._pointPrecipitationDrawable->getCurrentCellMatrixMap().size()<<std::endl;
+    OSG_NOTICE<<"time for cull "<<osg::Timer::instance()->delta_m(startTick,endTick)<<"ms  numTested="<<numTested<<" numInFrustum"<<numInFrustum<<std::endl;
+    OSG_NOTICE<<"     quads "<<pds._quadPrecipitationDrawable->getCurrentCellMatrixMap().size()<<"   lines "<<pds._linePrecipitationDrawable->getCurrentCellMatrixMap().size()<<"   points "<<pds._pointPrecipitationDrawable->getCurrentCellMatrixMap().size()<<std::endl;
 #endif
 }
 
@@ -800,7 +760,7 @@ bool PrecipitationEffect::build(const osg::Vec3 eyeLocal, int i, int j, int k, f
 
     osg::BoundingBox bb(position.x(), position.y(), position.z()+scale.z(),
                         position.x()+scale.x(), position.y()+scale.y(), position.z());
-                        
+
     if (!frustum.contains(bb)) return false;
 
     osg::Vec3 center = position + scale*0.5f;
@@ -837,9 +797,9 @@ bool PrecipitationEffect::build(const osg::Vec3 eyeLocal, int i, int j, int k, f
     }
 
     *mymodelview = *(cv->getModelViewMatrix());
-    mymodelview->preMult(osg::Matrix::translate(position));
-    mymodelview->preMult(osg::Matrix::scale(scale));
-    
+    mymodelview->preMultTranslate(position);
+    mymodelview->preMultScale(scale);
+
     cv->updateCalculatedNearFar(*(cv->getModelViewMatrix()),bb);
 
     return true;
@@ -873,20 +833,23 @@ PrecipitationEffect::PrecipitationDrawable::PrecipitationDrawable(const Precipit
 
 void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::RenderInfo& renderInfo) const
 {
-    if (!_geometry) return;
+#if defined(OSG_GL_MATRICES_AVAILABLE)
+
+if (!_geometry) return;
+
 
     const osg::Geometry::Extensions* extensions = osg::Geometry::getExtensions(renderInfo.getContextID(),true);
-    
+
     // save OpenGL matrices
     glPushMatrix();
-    
+
     if (_requiresPreviousMatrix)
     {
         renderInfo.getState()->setActiveTextureUnit(0);
         glMatrixMode( GL_TEXTURE );
         glPushMatrix();
     }
-    
+
     typedef std::vector<const CellMatrixMap::value_type*> DepthMatrixStartTimeVector;
     DepthMatrixStartTimeVector orderedEntries;
     orderedEntries.reserve(_currentCellMatrixMap.size());
@@ -897,9 +860,9 @@ void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::RenderI
     {
         orderedEntries.push_back(&(*citr));
     }
-    
+
     std::sort(orderedEntries.begin(),orderedEntries.end(),LessFunctor());
-        
+
     for(DepthMatrixStartTimeVector::reverse_iterator itr = orderedEntries.rbegin();
         itr != orderedEntries.rend();
         ++itr)
@@ -917,13 +880,13 @@ void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::RenderI
             {
                 // load previous frame modelview matrix for motion blurr effect
                 glMatrixMode( GL_TEXTURE );
-                glLoadMatrix(pitr->second.modelview.ptr());    
+                glLoadMatrix(pitr->second.modelview.ptr());
             }
             else
             {
                 // use current modelview matrix as "previous" frame value, cancelling motion blurr effect
                 glMatrixMode( GL_TEXTURE );
-                glLoadMatrix((*itr)->second.modelview.ptr());    
+                glLoadMatrix((*itr)->second.modelview.ptr());
             }
         }
         else
@@ -932,7 +895,7 @@ void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::RenderI
         }
 
         _geometry->draw(renderInfo);
-        
+
         unsigned int numVertices = osg::minimum(_geometry->getVertexArray()->getNumElements(), _numberOfVertices);
         glDrawArrays(_drawType, 0, numVertices);
     }
@@ -943,8 +906,9 @@ void PrecipitationEffect::PrecipitationDrawable::drawImplementation(osg::RenderI
         glPopMatrix();
         glMatrixMode( GL_MODELVIEW );
     }
-    
-    glPopMatrix();
 
-    
+    glPopMatrix();
+#else
+    OSG_NOTICE<<"Warning: ParticleEffect::drawImplementation(..) not fully implemented."<<std::endl;
+#endif
 }
