@@ -10,6 +10,7 @@
 #include <osg/MatrixTransform>
 #include <osg/BlendFunc>
 #include <osg/TexEnvCombine>
+#include <osg/CullFace>
 
 #include <osgDB/Registry>
 #include <osgDB/FileUtils>
@@ -30,6 +31,7 @@
 #include <map>
 #include <iostream>
 #include <sstream>
+#include <limits>
 #include <assert.h>
 
 using namespace std;
@@ -206,11 +208,10 @@ protected:
 
         osg::Texture2D* createTexture(Lib3dsTextureMap *texture,const char* label,bool& transparancy);
         StateSetInfo createStateSet(Lib3dsMaterial *materials);
-        osg::Drawable* createDrawable(Lib3dsMesh *meshes,FaceList& faceList, const osg::Matrix * matrix, StateSetInfo & ssi);
+        osg::Drawable* createDrawable(Lib3dsMesh *meshes,FaceList& faceList, const osg::Matrix * matrix, StateSetInfo & ssi, bool smoothVertexNormals);
 
         std::string _directory;
         bool _useSmoothingGroups;
-        bool _usePerVertexNormals;
 
         // MIKEC
         osg::Node* processMesh(StateSetMap& drawStateMap,osg::Group* parent,Lib3dsMesh* mesh, const osg::Matrix * matrix);
@@ -238,6 +239,7 @@ ReaderWriter3DS::ReaderWriter3DS()
     //supportsOption("OutputTextureFiles","Write out the texture images to file");
     //supportsOption("flipTexture", "flip texture upside-down");
     supportsOption("extended3dsFilePaths", "(Write option) Keeps long texture filenames (not 8.3) when exporting 3DS, but can lead to compatibility problems.");
+    supportsOption("preserveMaterialNames", "(Write option) Preserve original material names, up to 64 characters. This can lead to compatibility problems.");
     supportsOption("noMatrixTransforms", "(Read option) Set the plugin to apply matrices into the mesh vertices (\"old behaviour\") instead of restoring them (\"new behaviour\"). You may use this option to avoid a few rounding errors.");
     supportsOption("checkForEspilonIdentityMatrices", "(Read option) If not set, then consider \"almost identity\" matrices to be identity ones (in case of rounding errors).");
     supportsOption("restoreMatrixTransformsNoMeshes", "(Read option) Makes an exception to the behaviour when 'noMatrixTransforms' is not set for mesh instances. When a mesh instance has a transform on it, the reader creates a MatrixTransform above the Geode. If you don't want the hierarchy to be modified, then you can use this option to merge the transform into vertices.");
@@ -265,7 +267,6 @@ ReaderWriter3DS::ReaderWriter3DS()
 
 ReaderWriter3DS::ReaderObject::ReaderObject(const osgDB::ReaderWriter::Options* options) :
     _useSmoothingGroups(true),
-    _usePerVertexNormals(true),
     options(options),
     noMatrixTransforms(false),
     checkForEspilonIdentityMatrices(false),
@@ -274,8 +275,8 @@ ReaderWriter3DS::ReaderObject::ReaderObject(const osgDB::ReaderWriter::Options*
     if (options)
     {
         std::istringstream iss(options->getOptionString());
-        std::string opt; 
-        while (iss >> opt) 
+        std::string opt;
+        while (iss >> opt)
         {
             if (opt == "noMatrixTransforms")
                 noMatrixTransforms = true;
@@ -388,6 +389,11 @@ void ReaderWriter3DS::ReaderObject::addDrawableFromFace(osg::Geode * geode, Face
             flitr!=faceList.end();
             ++flitr)
         {
+            // ChrisD: Worth bearing in mind that this splitting up of
+            // faces into smoothing groups is only correct for faces
+            // belonging to a single smoothing group. The smoothing group
+            // value is actually a bitmask for all the smoothing groups that
+            // a face may belong to.
             smoothingFaceMap[mesh->faces[*flitr].smoothing_group].push_back(*flitr);
         }
 
@@ -395,11 +401,15 @@ void ReaderWriter3DS::ReaderObject::addDrawableFromFace(osg::Geode * geode, Face
             sitr!=smoothingFaceMap.end();
             ++sitr)
         {
+            // We only compute smoothed vertex normals for faces with non-zero smoothing group value.
+            const unsigned int smoothing_group = sitr->first;
+            bool smoothVertexNormals = (smoothing_group != 0);
+
             // each smoothing group to have its own geom
             // to ensure the vertices on adjacent groups
             // don't get shared.
-            FaceList& smoothFaceMap = sitr->second;
-            osg::ref_ptr<osg::Drawable> drawable = createDrawable(mesh,smoothFaceMap,matrix,ssi);
+            FaceList& smoothFaceList = sitr->second;
+            osg::ref_ptr<osg::Drawable> drawable = createDrawable(mesh, smoothFaceList, matrix, ssi, smoothVertexNormals);
             if (drawable.valid())
             {
                 if (ssi.stateset)
@@ -410,7 +420,9 @@ void ReaderWriter3DS::ReaderObject::addDrawableFromFace(osg::Geode * geode, Face
     }
     else // ignore smoothing groups.
     {
-        osg::ref_ptr<osg::Drawable> drawable = createDrawable(mesh,faceList,matrix,ssi);
+        // Create drawable with no smoothing of normals.
+        bool smoothVertexNormals = false;
+        osg::ref_ptr<osg::Drawable> drawable = createDrawable(mesh, faceList, matrix, ssi, smoothVertexNormals);
         if (drawable.valid())
         {
             if (ssi.stateset)
@@ -559,10 +571,10 @@ osg::Node* ReaderWriter3DS::ReaderObject::processNode(StateSetMap& drawStateMap,
 
     if (group)
     {
-        if (strcmp(node->name, "$$$DUMMY") == 0) 
+        if (strcmp(node->name, "$$$DUMMY") == 0)
         {
             if (node->type == LIB3DS_NODE_MESH_INSTANCE)
-                group->setName(reinterpret_cast<Lib3dsMeshInstanceNode *>(node)->instance_name); 
+                group->setName(reinterpret_cast<Lib3dsMeshInstanceNode *>(node)->instance_name);
         }
         else if (node->type == LIB3DS_NODE_MESH_INSTANCE && strlen(reinterpret_cast<Lib3dsMeshInstanceNode *>(node)->instance_name) != 0)
             group->setName(reinterpret_cast<Lib3dsMeshInstanceNode *>(node)->instance_name);
@@ -589,18 +601,37 @@ osg::Node* ReaderWriter3DS::ReaderObject::processNode(StateSetMap& drawStateMap,
             meshAppliedMatPtr = &meshMat;
         }
 
+        // MIKEC: "group" handles the node transform, siblings, and our child geometry if we don't want an MT node for it
+        // if noMatrixTransforms is not set, we create a transform node for the mesh's matrix
+        osg::Group* meshTransform=NULL;
+
+        if ((noMatrixTransforms==false) && meshAppliedMatPtr) { // we are allowed to have, and need another matrixtransform
+            meshTransform=new osg::MatrixTransform(meshMat);
+            meshAppliedMatPtr=NULL; // since meshTransform applies it
+
+            meshTransform->setName("3DSMeshMatrix");
+            if (group) group->addChild(meshTransform);
+        } else {
+            meshTransform=group; // don't need the meshTransform node - note group can be NULL
+        }
+
         if (group)
         {
             // add our geometry to group (where our children already are)
             // creates geometry under modifier node
-            processMesh(drawStateMap,group,mesh,meshAppliedMatPtr);
+            processMesh(drawStateMap,meshTransform,mesh,meshAppliedMatPtr);
             return group;
         }
         else
         {
-            // didnt use group for children
-            // return a ptr directly to the Geode for this mesh
-            return processMesh(drawStateMap,NULL,mesh,meshAppliedMatPtr);
+            // didnt use group for children, return a ptr directly to the Geode for this mesh
+            // there is no group node but may have a meshTransform node to hold the meshes matrix
+            if (meshTransform) {
+                processMesh(drawStateMap,meshTransform,mesh,meshAppliedMatPtr);
+                return meshTransform;
+            } else { // no group or meshTransform nodes - create a new Geode and return that
+                return processMesh(drawStateMap,NULL,mesh,meshAppliedMatPtr);
+            }
         }
 
     }
@@ -782,19 +813,19 @@ osgDB::ReaderWriter::ReadResult ReaderWriter3DS::constructFrom3dsFile(Lib3dsFile
     // We can traverse by meshes (old method, broken for pivot points, but otherwise works), or by nodes (new method, not so well tested yet)
     // if your model is broken, especially wrt object positions try setting this flag. If that fixes it,
     // send me the model
-    bool traverse_nodes=false;
+    bool traverse_nodes=true;
 
     // MIKEC: have found 3ds files with NO node structure - only meshes, for this case we fall back to the old traverse-by-meshes code
     // Loading and re-exporting these files from 3DS produces a file with correct node structure, so perhaps these are not 100% conformant?
     if (f->nodes == NULL)
     {
         OSG_WARN<<"Warning: in 3ds loader: file has no nodes, traversing by meshes instead"<< std::endl;
-        traverse_nodes=true;
+        traverse_nodes=false;
     }
 
     osg::Node* group = NULL;
 
-    if (traverse_nodes) // old method
+    if (!traverse_nodes) // old method, traverse by mesh
     {
         group = new osg::Group();
         for (int imesh=0; imesh<f->nmeshes; ++imesh)
@@ -830,159 +861,277 @@ osgDB::ReaderWriter::ReadResult ReaderWriter3DS::constructFrom3dsFile(Lib3dsFile
     return group;
 }
 
-/**
-use matrix to pretransform geometry, or NULL to do nothing
-*/
-osg::Drawable* ReaderWriter3DS::ReaderObject::createDrawable(Lib3dsMesh *m,FaceList& faceList, const osg::Matrix * matrix, StateSetInfo & ssi)
+struct RemappedFace
 {
-    osg::Geometry * geom = new osg::Geometry;
-    unsigned int i;
+    Lib3dsFace* face;        // Original face definition.
+    osg::Vec3f normal;
+    unsigned int index[3];   // Indices to OSG vertex/normal/texcoord arrays.
+};
 
-    std::vector<int> orig2NewMapping;
-    orig2NewMapping.reserve(m->nvertices);
-    for(i=0;i<m->nvertices;++i) orig2NewMapping.push_back(-1);
+struct VertexParams
+{
+    VertexParams() : matrix(NULL), smoothNormals(false), scaleUV(1.f, 1.f), offsetUV(0.f, 0.f) { }
+    const osg::Matrix* matrix;
+    bool smoothNormals;
+    osg::Vec2f scaleUV;
+    osg::Vec2f offsetUV;
+};
 
-    unsigned int noVertex=0;
-    FaceList::iterator fitr;
-    for (fitr=faceList.begin();
-        fitr!=faceList.end();
-        ++fitr)
+static bool isFaceValid(const Lib3dsMesh* mesh, const Lib3dsFace* face)
+{
+    return
+        face->index[0] < mesh->nvertices &&
+        face->index[1] < mesh->nvertices &&
+        face->index[2] < mesh->nvertices;
+}
+
+bool isNumber(float x)
+{
+    // This looks like it should always be true,
+    // but it's false if x is a NaN.
+    return (x == x);
+}
+
+
+/* ChrisD: addVertex handles the averaging of normals and spltting of vertices
+   required to implement normals for smoothing groups. When a shared
+   vertex is encountered when smoothing is required, normals are added
+   and normalized. When a shared vertex is encountered when smoothing is
+   not required, we must split the vertex if a different normal is required.
+   For example if we are processing a cube mesh with no smoothing group
+   made from 12 triangles and 8 vertices, the resultant mesh should have
+   24 vertices to accomodate the 3 different normals at each vertex.
+  */
+static void addVertex(
+    const Lib3dsMesh* mesh,
+    RemappedFace& remappedFace,
+    unsigned int i,
+    osg::Geometry* geometry,
+    std::vector<int>& origToNewMapping,
+    std::vector<int>& splitVertexChain,
+    const VertexParams& params)
+{
+    osg::Vec3Array* vertices = (osg::Vec3Array*)geometry->getVertexArray();
+    osg::Vec3Array* normals = (osg::Vec3Array*)geometry->getNormalArray();
+    osg::Vec2Array* texCoords = (osg::Vec2Array*)geometry->getTexCoordArray(0);
+
+    unsigned int index = remappedFace.face->index[i];
+    if (origToNewMapping[index] == -1)
     {
+        int newIndex = vertices->size();
+        remappedFace.index[i] = newIndex;
+        origToNewMapping[index] = newIndex;
 
-        Lib3dsFace& face = m->faces[*fitr];
+        // Add the vertex position
+        osg::Vec3 vertex = copyLib3dsVec3ToOsgVec3(mesh->vertices[index]);
+        if (params.matrix) vertex = vertex * (*params.matrix);
+        vertices->push_back(vertex);
 
-        if (orig2NewMapping[face.index[0]]<0)
-            orig2NewMapping[face.index[0]] = noVertex++;
+        // Add the vertex normal
+        normals->push_back(remappedFace.normal);
 
-        if (orig2NewMapping[face.index[1]]<0)
-            orig2NewMapping[face.index[1]] = noVertex++;
-
-        if (orig2NewMapping[face.index[2]]<0)
-            orig2NewMapping[face.index[2]] = noVertex++;
-
-    }
-
-    // create vertices.
-
-    osg::ref_ptr<osg::Vec3Array> osg_coords = new osg::Vec3Array(noVertex);
-    geom->setVertexArray(osg_coords.get());
-
-    for (i=0; i<m->nvertices; ++i)
-    {
-        if (orig2NewMapping[i]>=0)
+        // Add the texture coordinate.
+        if (texCoords)
         {
-            if (matrix)
-            {
-                (*osg_coords)[orig2NewMapping[i]].set( copyLib3dsVec3ToOsgVec3(m->vertices[i]) * (*matrix) );
+            osg::Vec2f texCoord(mesh->texcos[index][0], mesh->texcos[index][1]);
+            texCoord = componentMultiply(texCoord, params.scaleUV);
+            texCoord += params.offsetUV;
+            if (!isNumber(texCoord.x()) || !isNumber(texCoord.y())) {
+                OSG_WARN << "NaN found in texcoord" << std::endl;
+                texCoord.set(0,0);
             }
-            else
-            {
-                // original no transform code.
-                (*osg_coords)[orig2NewMapping[i]].set( copyLib3dsVec3ToOsgVec3(m->vertices[i]) );
-            }
-        }
-    }
-
-    // create texture coords if needed.
-    if (m->texcos)
-    {
-        osg::ref_ptr<osg::Vec2Array> osg_tcoords = new osg::Vec2Array(noVertex);
-        geom->setTexCoordArray(0, osg_tcoords.get());
-
-        // Texture 0 parameters (only one texture supported for now)
-        float scaleU(1.f), scaleV(1.f);
-        float offsetU(0.f), offsetV(0.f);
-        if (ssi.lib3dsmat && *(ssi.lib3dsmat->texture1_map.name))     // valid texture = name not empty
-        {
-            Lib3dsTextureMap & tex3ds = ssi.lib3dsmat->texture1_map;
-            scaleU = tex3ds.scale[0];
-            scaleV = tex3ds.scale[1];
-            offsetU = tex3ds.offset[0];
-            offsetV = tex3ds.offset[1];
-            if (tex3ds.rotation != 0) OSG_NOTICE << "3DS texture rotation not supported yet" << std::endl;
-            //TODO: tint_1, tint_2, tint_r, tint_g, tint_b
+            texCoords->push_back(texCoord);
         }
 
-        for (i=0; i<m->nvertices; ++i)
-        {
-            if (orig2NewMapping[i]>=0) (*osg_tcoords)[orig2NewMapping[i]].set(m->texcos[i][0]*scaleU + offsetU, m->texcos[i][1]*scaleV + offsetV);
-        }
-    }
-
-    // create normals
-    // Sukender: 3DS file format doesn't store normals (that is to say they're recomputed each time).
-    // When using per vertex normals, we could use either vertex computation, or face computation (and copy the normal to each vertex). Here we use the latter one.
-    if (_usePerVertexNormals)
-    {
-        //Lib3dsVector * normals = new Lib3dsVector[m->nfaces*3];
-        //lib3ds_mesh_calculate_vertex_normals(m, normals);
-        scoped_array<Lib3dsVector> normals( new Lib3dsVector[m->nfaces] );        // Temporary array
-        lib3ds_mesh_calculate_face_normals(m, normals.get());
-        osg::ref_ptr<osg::Vec3Array> osg_normals = new osg::Vec3Array(noVertex);
-
-        // initialize normal list to zero's.
-        for (i=0; i<noVertex; ++i)
-        {
-            (*osg_normals)[i].set(0.0f,0.0f,0.0f);
-        }
-
-        for (fitr=faceList.begin();
-            fitr!=faceList.end();
-            ++fitr)
-        {
-            Lib3dsFace& face = m->faces[*fitr];
-            osg::Vec3f osgNormal( copyLib3dsVec3ToOsgVec3(normals[*fitr]) );
-            if (matrix) osgNormal = osg::Matrix::transform3x3(osgNormal, *matrix);
-            osgNormal.normalize();
-            (*osg_normals)[orig2NewMapping[face.index[0]]] = osgNormal;
-            (*osg_normals)[orig2NewMapping[face.index[1]]] = osgNormal;
-            (*osg_normals)[orig2NewMapping[face.index[2]]] = osgNormal;
-        }
-
-        geom->setNormalArray(osg_normals.get());
-        geom->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
+        // New vertex, so not split yet.
+        splitVertexChain.push_back(-1);
     }
     else
     {
-        scoped_array<Lib3dsVector> normals ( new Lib3dsVector[m->nfaces] );
-        lib3ds_mesh_calculate_face_normals(m, normals.get());
-        osg::ref_ptr<osg::Vec3Array> osg_normals = new osg::Vec3Array(faceList.size());
-        osg::Vec3Array::iterator normal_itr = osg_normals->begin();
-        for (fitr=faceList.begin();
-            fitr!=faceList.end();
-            ++fitr)
+        int newIndex = origToNewMapping[index];
+        if (params.smoothNormals)
         {
-            osg::Vec3f osgNormal( copyLib3dsVec3ToOsgVec3(normals[*fitr]) );
-            if (matrix) osgNormal = osg::Matrix::transform3x3(osgNormal, *matrix);
-            osgNormal.normalize();
-            *(normal_itr++) = osgNormal;
+            // Average the normals on the shared vertex.
+            remappedFace.index[i] = newIndex;
+            osg::Vec3f normal = (*normals)[newIndex];
+            normal += remappedFace.normal;
+            normal.normalize();
+            (*normals)[newIndex] = normal;
+        }
+        else
+        {
+            // Find a split vertex chained from newIndex which has the 'same' normal.
+            int sharedVertexIndex = newIndex;
+            do
+            {
+                osg::Vec3f normal = (*normals)[sharedVertexIndex];
+                float normalDifference = (remappedFace.normal - normal).length2();
+                if (normalDifference < 1e-6) break;
+                sharedVertexIndex = splitVertexChain[sharedVertexIndex];
+            } while (sharedVertexIndex != -1);
+
+            if (sharedVertexIndex == -1)
+            {
+                // When different normals on a shared vertex required, split the vertex.
+                int splitVertexIndex = vertices->size();
+                remappedFace.index[i] = splitVertexIndex;
+                vertices->push_back((*vertices)[newIndex]);
+                normals->push_back(remappedFace.normal);
+                if (texCoords)
+                {
+                    texCoords->push_back((*texCoords)[newIndex]);
+                }
+                splitVertexChain[newIndex] = splitVertexIndex;
+                splitVertexChain.push_back(-1);
+            }
+            else
+            {
+                // When normals on shared vertex are identical (or very similar), keep it shared.
+                remappedFace.index[i] = sharedVertexIndex;
+            }
+        }
+    }
+}
+
+static bool addFace(
+    const Lib3dsMesh* mesh,
+    RemappedFace& remappedFace,
+    osg::Geometry* geometry,
+    std::vector<int>& origToNewMapping,
+    std::vector<int>& splitVertexChain,
+    const VertexParams& params)
+{
+    if (isFaceValid(mesh, remappedFace.face))
+    {
+        addVertex(mesh, remappedFace, 0, geometry, origToNewMapping, splitVertexChain, params);
+        addVertex(mesh, remappedFace, 1, geometry, origToNewMapping, splitVertexChain, params);
+        addVertex(mesh, remappedFace, 2, geometry, origToNewMapping, splitVertexChain, params);
+        return true;
+    }
+    else
+    {
+        // Avoids crash with corrupted files.
+        remappedFace.face = NULL;
+        return false;
+    }
+}
+
+template <typename Prim>
+void fillTriangles(osg::Geometry & geom, const std::vector<RemappedFace> & remappedFaces, unsigned int numIndices) {
+    //if (m->nvertices < std::numeric_limits<unsigned short>::max()) {
+    osg::ref_ptr<Prim> elements( new Prim(osg::PrimitiveSet::TRIANGLES, numIndices) );
+    typename Prim::iterator index_itr( elements->begin() );
+    for (unsigned int i = 0; i < remappedFaces.size(); ++i)
+    {
+        const RemappedFace& remappedFace = remappedFaces[i];
+        if (remappedFace.face != NULL)
+        {
+            *(index_itr++) = remappedFace.index[0];
+            *(index_itr++) = remappedFace.index[1];
+            *(index_itr++) = remappedFace.index[2];
+        }
+    }
+    geom.addPrimitiveSet(elements.get());
+}
+
+
+/**
+use matrix to pretransform geometry, or NULL to do nothing
+*/
+osg::Drawable* ReaderWriter3DS::ReaderObject::createDrawable(Lib3dsMesh *m,FaceList& faceList, const osg::Matrix * matrix, StateSetInfo & ssi, bool smoothVertexNormals)
+{
+    // Avoid creating geoms for empty face list because otherwise osg asserts/crashes during render traversal.
+    if (faceList.empty()) return NULL;
+
+    osg::Geometry * geom = new osg::Geometry;
+
+    VertexParams params;
+    params.matrix = matrix;
+    params.smoothNormals = smoothVertexNormals;
+
+    std::vector<RemappedFace> remappedFaces(faceList.size());
+
+    scoped_array<Lib3dsVector> normals( new Lib3dsVector[m->nfaces] );        // Temporary array
+    lib3ds_mesh_calculate_face_normals(m, normals.get());
+
+    osg::ref_ptr<osg::Vec3Array> osg_vertices = new osg::Vec3Array();
+    osg_vertices->reserve(m->nvertices);
+    geom->setVertexArray(osg_vertices.get());
+
+    osg::ref_ptr<osg::Vec3Array> osg_normals = new osg::Vec3Array();
+    osg_normals->reserve(m->nvertices);
+    geom->setNormalArray(osg_normals.get(), osg::Array::BIND_PER_VERTEX);
+
+    osg::ref_ptr<osg::Vec2Array> osg_texCoords = NULL;
+
+    if (m->texcos)
+    {
+        osg_texCoords = new osg::Vec2Array();
+        osg_texCoords->reserve(m->nvertices);
+        geom->setTexCoordArray(0, osg_texCoords.get());
+
+        // Texture 0 parameters (only one texture supported for now)
+        if (ssi.lib3dsmat && *(ssi.lib3dsmat->texture1_map.name))     // valid texture = name not empty
+        {
+            Lib3dsTextureMap & tex3ds = ssi.lib3dsmat->texture1_map;
+            params.scaleUV = osg::Vec2f(tex3ds.scale[0], tex3ds.scale[1]);
+            params.offsetUV = osg::Vec2f(tex3ds.offset[0], tex3ds.offset[1]);
+            if (tex3ds.rotation != 0) OSG_NOTICE << "3DS texture rotation not supported yet" << std::endl;
+            //TODO: tint_1, tint_2, tint_r, tint_g, tint_b
         }
-        geom->setNormalArray(osg_normals.get());
-        geom->setNormalBinding(osg::Geometry::BIND_PER_PRIMITIVE);
     }
 
+    // The map between lib3ds mesh vertex indices and remapped osg vertices.
+    std::vector<int> origToNewMapping(m->nvertices, -1);
+
+    // If osg vertices need to be split to hold a different vertex normal,
+    // splitVertexChain allows us to look them up.
+    std::vector<int> splitVertexChain;
+    splitVertexChain.reserve(m->nvertices);
+
+    unsigned int faceIndex = 0;
+    unsigned int faceCount = 0;
+    unsigned int maxVertexIndex = 0;
+    for (FaceList::iterator itr = faceList.begin();
+        itr != faceList.end();
+        ++itr, ++faceIndex)
+    {
+        osg::Vec3f normal(copyLib3dsVec3ToOsgVec3(normals[*itr]));
+        if (matrix) normal = osg::Matrix::transform3x3(normal, *(params.matrix));
+        normal.normalize();
+
+        Lib3dsFace& face = m->faces[*itr];
+        RemappedFace & rf = remappedFaces[faceIndex];
+        rf.face = &face;
+        rf.normal = normal;
+        if (addFace(m, rf, geom, origToNewMapping, splitVertexChain, params))
+        {
+            ++faceCount;
+        }
+        // Get the highest index
+        maxVertexIndex = osg::maximum(maxVertexIndex, rf.index[0]);
+        maxVertexIndex = osg::maximum(maxVertexIndex, rf.index[1]);
+        maxVertexIndex = osg::maximum(maxVertexIndex, rf.index[2]);
+    }
+
+    // 'Shrink to fit' all vertex arrays because potentially faceList refers to fewer vertices than the whole mesh.
+    // This will almost certainly be the case where mesh has been broken down into smoothing groups.
+    if (osg_vertices.valid() && osg_vertices->size() < osg_vertices->capacity()) osg_vertices->trim();
+    if (osg_normals.valid() && osg_normals->size() < osg_normals->capacity()) osg_normals->trim();
+    if (osg_texCoords.valid() && osg_texCoords->size() < osg_texCoords->capacity()) osg_texCoords->trim();
+
+    // Set geometry color to white.
     osg::ref_ptr<osg::Vec4ubArray> osg_colors = new osg::Vec4ubArray(1);
     (*osg_colors)[0].set(255,255,255,255);
-    geom->setColorArray(osg_colors.get());
-    geom->setColorBinding(osg::Geometry::BIND_OVERALL);
-
-    // create primitives
-    int numIndices = faceList.size()*3;
-    osg::ref_ptr<DrawElementsUShort> elements = new osg::DrawElementsUShort(osg::PrimitiveSet::TRIANGLES,numIndices);
-    DrawElementsUShort::iterator index_itr = elements->begin();
-
-    for (fitr=faceList.begin();
-        fitr!=faceList.end();
-        ++fitr)
-    {
-        Lib3dsFace& face = m->faces[*fitr];
-        *(index_itr++) = orig2NewMapping[face.index[0]];
-        *(index_itr++) = orig2NewMapping[face.index[1]];
-        *(index_itr++) = orig2NewMapping[face.index[2]];
-    }
-
-    geom->addPrimitiveSet(elements.get());
+    geom->setColorArray(osg_colors.get(), osg::Array::BIND_OVERALL);
 
+    // Create triangle primitives.
+    // Remapping may create additional vertices, thus creating indices over USHORT_MAX
+    if (maxVertexIndex < std::numeric_limits<unsigned short>::max()) {
+        fillTriangles<DrawElementsUShort>(*geom, remappedFaces, faceCount * 3);
+    } else {
+        fillTriangles<DrawElementsUInt>  (*geom, remappedFaces, faceCount * 3);
+   }
 #if 0
     osgUtil::TriStripVisitor tsv;
     tsv.stripify(*geom);
@@ -1023,8 +1172,9 @@ osg::Texture2D*  ReaderWriter3DS::ReaderObject::createTexture(Lib3dsTextureMap *
             }
             else
             {
-                OSG_WARN << "texture '"<<texture->name<<"' not found"<< std::endl;
-                return NULL;
+                // MIKEC: We can still continue to call osgDB::readRefImageFile in case user has a ReadFileCallback registered
+                //        in that case we just use the image's filename as it exists in the 3DS file
+                fileName=texture->name;
             }
         }
 
@@ -1055,7 +1205,7 @@ osg::Texture2D*  ReaderWriter3DS::ReaderObject::createTexture(Lib3dsTextureMap *
         osg_texture->setImage(osg_image.get());
         osg_texture->setName(texture->name);
         // does the texture support transparancy?
-        //transparency = ((texture->flags)&LIB3DS_TEXTURE_ALPHA_SOURCE)!=0;
+        transparency = ((texture->flags)&LIB3DS_TEXTURE_ALPHA_SOURCE)!=0;
 
         // what is the wrap mode of the texture.
         osg::Texture2D::WrapMode wm = ((texture->flags)&LIB3DS_TEXTURE_NO_TILE) ?
@@ -1098,6 +1248,7 @@ ReaderWriter3DS::StateSetInfo ReaderWriter3DS::ReaderObject::createStateSet(Lib3
     osg::Texture2D* texture1_map = createTexture(&(mat->texture1_map),"texture1_map",textureTransparency);
     if (texture1_map)
     {
+        if(textureTransparency) transparency = true;
         stateset->setTextureAttributeAndModes(unit, texture1_map, osg::StateAttribute::ON);
 
         double factor = mat->texture1_map.percent;
@@ -1110,8 +1261,8 @@ ReaderWriter3DS::StateSetInfo ReaderWriter3DS::ReaderObject::createStateSet(Lib3
             texenv->setSource2_RGB(osg::TexEnvCombine::CONSTANT);
             texenv->setConstantColor(osg::Vec4(factor, factor, factor, factor));
             stateset->setTextureAttributeAndModes(unit, texenv, osg::StateAttribute::ON);
-        } 
-        else 
+        }
+        else
         {
             // from an email from Eric Hamil, September 30, 2003.
             // According to the 3DS spec, and other
@@ -1134,23 +1285,23 @@ ReaderWriter3DS::StateSetInfo ReaderWriter3DS::ReaderObject::createStateSet(Lib3
             diffuse.set(0.8f,0.8f,0.8f);
             specular.set(0.0f,0.0f,0.0f);
 #endif
-        }        
+        }
 
         unit++;
     }
 
     // opacity
-    osg::Texture* opacity_map = createTexture(&(mat->opacity_map),"opacity_map", textureTransparency);
-    if (opacity_map && texture1_map)
+    osg::Texture2D* opacity_map = createTexture(&(mat->opacity_map),"opacity_map", textureTransparency);
+    if (opacity_map)
     {
-        if(texture1_map->getImage()->isImageTranslucent())
+        if(opacity_map->getImage()->isImageTranslucent())
         {
-            transparency = true; 
+            transparency = true;
 
             stateset->setTextureAttributeAndModes(unit, opacity_map, osg::StateAttribute::ON);
 
             double factor = mat->opacity_map.percent;
-            
+
                 osg::TexEnvCombine* texenv = new osg::TexEnvCombine();
                 texenv->setCombine_Alpha(osg::TexEnvCombine::INTERPOLATE);
                 texenv->setSource0_Alpha(osg::TexEnvCombine::TEXTURE);
@@ -1158,9 +1309,9 @@ ReaderWriter3DS::StateSetInfo ReaderWriter3DS::ReaderObject::createStateSet(Lib3
                 texenv->setSource2_Alpha(osg::TexEnvCombine::CONSTANT);
                 texenv->setConstantColor(osg::Vec4(factor, factor, factor, 1.0 - factor));
                 stateset->setTextureAttributeAndModes(unit, texenv, osg::StateAttribute::ON);
-            
+
             unit++;
-        } 
+        }
         else
         {
             osg::notify(WARN)<<"The plugin does not support images without alpha channel for opacity"<<std::endl;
@@ -1178,11 +1329,21 @@ ReaderWriter3DS::StateSetInfo ReaderWriter3DS::ReaderObject::createStateSet(Lib3
 
     if ((alpha < 1.0f) || transparency)
     {
-        //stateset->setAttributeAndModes(new osg::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));
+        stateset->setAttributeAndModes(new osg::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));
         stateset->setMode(GL_BLEND,osg::StateAttribute::ON);
         stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
     }
 
+    // Set back face culling state if single sided material applied.
+    // This seems like a reasonable assumption given that the backface cull option
+    // doesn't appear to be encoded directly in the 3DS format, and also because
+    // it mirrors the effect of code in 3DS writer which uses the the face culling
+    // attribute to determine the state of the 'two_sided' 3DS material being written.
+    if (!mat->two_sided)
+    {
+        stateset->setAttributeAndModes(new osg::CullFace(osg::CullFace::BACK));
+    }
+
 /*
     osg::ref_ptr<osg::Texture> texture1_mask = createTexture(&(mat->texture1_mask),"texture1_mask",textureTransparancy);
     osg::ref_ptr<osg::Texture> texture2_map = createTexture(&(mat->texture2_map),"texture2_map",textureTransparancy);
@@ -1253,7 +1414,7 @@ osgDB::ReaderWriter::WriteResult ReaderWriter3DS::doWriteNode(const osg::Node& n
     io.read_func = NULL;
     io.write_func = fileo_write_func;
     io.log_func = fileio_log_func;
-    
+
     Lib3dsFile * file3ds = lib3ds_file_new();
     if (!file3ds) return WriteResult(WriteResult::ERROR_IN_WRITING_FILE);
 
