@@ -756,6 +756,38 @@ ConvertFromInventor::postLOD(void* data, SoCallbackAction* action,
 
     return SoCallbackAction::CONTINUE;
 }
+
+// g++ (at least) guarantees thread-safe method-local static initialization, so moving construction of these maps to exploit
+class NormBindingMap : public std::map<SoNormalBinding::Binding, deprecated_osg::Geometry::AttributeBinding>
+{
+  public:
+    NormBindingMap()
+    {
+        (*this)[SoNormalBinding::OVERALL]            = deprecated_osg::Geometry::BIND_OVERALL;
+        (*this)[SoNormalBinding::PER_PART]           = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoNormalBinding::PER_PART_INDEXED]   = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoNormalBinding::PER_FACE]           = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoNormalBinding::PER_FACE_INDEXED]   = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoNormalBinding::PER_VERTEX]         = deprecated_osg::Geometry::BIND_PER_VERTEX;
+        (*this)[SoNormalBinding::PER_VERTEX_INDEXED] = deprecated_osg::Geometry::BIND_PER_VERTEX;
+    }
+};
+
+class ColBindingMap : public std::map<SoMaterialBinding::Binding, deprecated_osg::Geometry::AttributeBinding>
+{
+  public:
+    ColBindingMap()
+    {
+        (*this)[SoMaterialBinding::OVERALL]            = deprecated_osg::Geometry::BIND_OVERALL;
+        (*this)[SoMaterialBinding::PER_PART]           = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoMaterialBinding::PER_PART_INDEXED]   = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoMaterialBinding::PER_FACE]           = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoMaterialBinding::PER_FACE_INDEXED]   = deprecated_osg::Geometry::BIND_PER_PRIMITIVE;
+        (*this)[SoMaterialBinding::PER_VERTEX]         = deprecated_osg::Geometry::BIND_PER_VERTEX;
+        (*this)[SoMaterialBinding::PER_VERTEX_INDEXED] = deprecated_osg::Geometry::BIND_PER_VERTEX;
+    }
+};
+
 ///////////////////////////////////////////////////////////////////
 SoCallbackAction::Response
 ConvertFromInventor::preShape(void* data, SoCallbackAction* action,
@@ -769,45 +801,8 @@ ConvertFromInventor::preShape(void* data, SoCallbackAction* action,
     ConvertFromInventor* thisPtr = (ConvertFromInventor *) (data);
 
     // Normal and color binding map from Inventor to OSG
-    static std::map<SoNormalBinding::Binding, osg::Geometry::AttributeBinding>
-        normBindingMap;
-    static std::map<SoMaterialBinding::Binding, osg::Geometry::AttributeBinding>
-        colBindingMap;
-    static bool firstTime = true;
-    if (firstTime)
-    {
-        normBindingMap[SoNormalBinding::OVERALL]
-                                        = osg::Geometry::BIND_OVERALL;
-        normBindingMap[SoNormalBinding::PER_PART]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        normBindingMap[SoNormalBinding::PER_PART_INDEXED]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        normBindingMap[SoNormalBinding::PER_FACE]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        normBindingMap[SoNormalBinding::PER_FACE_INDEXED]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        normBindingMap[SoNormalBinding::PER_VERTEX]
-                                        = osg::Geometry::BIND_PER_VERTEX;
-        normBindingMap[SoNormalBinding::PER_VERTEX_INDEXED]
-                                        = osg::Geometry::BIND_PER_VERTEX;
-
-        colBindingMap[SoMaterialBinding::OVERALL]
-                                        = osg::Geometry::BIND_OVERALL;
-        colBindingMap[SoMaterialBinding::PER_PART]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        colBindingMap[SoMaterialBinding::PER_PART_INDEXED]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        colBindingMap[SoMaterialBinding::PER_FACE]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        colBindingMap[SoMaterialBinding::PER_FACE_INDEXED]
-                                        = osg::Geometry::BIND_PER_PRIMITIVE;
-        colBindingMap[SoMaterialBinding::PER_VERTEX]
-                                        = osg::Geometry::BIND_PER_VERTEX;
-        colBindingMap[SoMaterialBinding::PER_VERTEX_INDEXED]
-                                        = osg::Geometry::BIND_PER_VERTEX;
-
-        firstTime = false;
-    }
+    static NormBindingMap normBindingMap;
+    static ColBindingMap  colBindingMap;
 
     // Get normal and color binding
     if (node->isOfType(SoVertexShape::getClassTypeId()))
@@ -817,8 +812,8 @@ ConvertFromInventor::preShape(void* data, SoCallbackAction* action,
     }
     else
     {
-        thisPtr->normalBinding = osg::Geometry::BIND_PER_VERTEX;
-        thisPtr->colorBinding = osg::Geometry::BIND_PER_VERTEX;
+        thisPtr->normalBinding = deprecated_osg::Geometry::BIND_PER_VERTEX;
+        thisPtr->colorBinding = deprecated_osg::Geometry::BIND_PER_VERTEX;
     }
 
     // Check vertex ordering
@@ -869,7 +864,7 @@ ConvertFromInventor::postShape(void* data, SoCallbackAction* action,
 
 
     // Create a new Geometry
-    osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
+    osg::ref_ptr<deprecated_osg::Geometry> geometry = new deprecated_osg::Geometry;
 
 
     osg::ref_ptr<osg::Vec3Array> coords = new osg::Vec3Array(thisPtr->vertices.size());
@@ -878,7 +873,7 @@ ConvertFromInventor::postShape(void* data, SoCallbackAction* action,
     geometry->setVertexArray(coords.get());
 
     osg::ref_ptr<osg::Vec3Array> norms = NULL;
-    if (thisPtr->normalBinding == osg::Geometry::BIND_OVERALL)
+    if (thisPtr->normalBinding == deprecated_osg::Geometry::BIND_OVERALL)
     {
         norms = new osg::Vec3Array(1);
         const SbVec3f &norm = action->getNormal(0);
@@ -897,7 +892,7 @@ ConvertFromInventor::postShape(void* data, SoCallbackAction* action,
 
     // Set the colors
     osg::ref_ptr<osg::Vec4Array> cols;
-    if (thisPtr->colorBinding == osg::Geometry::BIND_OVERALL)
+    if (thisPtr->colorBinding == deprecated_osg::Geometry::BIND_OVERALL)
     {
         cols = new osg::Vec4Array(1);
         SbColor ambient, diffuse, specular, emission;
@@ -1093,12 +1088,21 @@ static osgDB::ReaderWriter::Options* createOptions()
 
 static osg::Image* loadImage(const char *fileName, osgDB::ReaderWriter::Options *options)
 {
-    osg::Image *osgImage = osgDB::readImageFile(fileName, options);
+    osg::ref_ptr<osg::Image> osgImage = osgDB::readImageFile(fileName, options);
 
     if (!osgImage)
+    {
         OSG_WARN << NOTIFY_HEADER << "Could not read texture file '" << fileName << "'.";
+        return 0;
+    }
 
-    return osgImage;
+    if (!osgImage->isDataContiguous())
+    {
+        OSG_WARN << NOTIFY_HEADER << "Inventor cannot handle non contiguous image data found in texture file '" << fileName << "'.";
+        return 0;
+    }
+
+    return osgImage.release();
 }
 
 SbBool SoTexture2Osg::readInstance(SoInput *in, unsigned short flags)
@@ -1264,7 +1268,7 @@ SbBool SoVRMLImageTextureOsg::readInstance(SoInput *in, unsigned short flags)
         if (url.getNum() && url[0].getLength())
         {
             osg::ref_ptr<osg::Image> image = loadImage(url[0].getString(), options);
-            if (!image->valid())
+            if (!image.valid())
             {
                 OSG_WARN << "Could not read texture file: " << url[0].getString() << std::endl;
                 this->setReadStatus(FALSE);
@@ -1868,6 +1872,17 @@ update: The mentioned bug is probably just for very old NVidia drivers (commit t
 
     return stateSet;
 }
+
+class TexWrapMap : public std::map<SoTexture2::Wrap, osg::Texture2D::WrapMode>
+{
+  public:
+    TexWrapMap()
+    {
+        (*this)[SoTexture2::CLAMP] = osg::Texture2D::CLAMP;
+        (*this)[SoTexture2::REPEAT] = osg::Texture2D::REPEAT;
+    }
+};
+
 ////////////////////////////////////////////////////////////////////
 osg::Texture2D*
 ConvertFromInventor::convertIVTexToOSGTex(const SoNode* soNode,
@@ -1937,14 +1952,7 @@ ConvertFromInventor::convertIVTexToOSGTex(const SoNode* soNode,
     // Set name
     osgTex->setName(soNode->getName().getString());
 
-    static std::map<SoTexture2::Wrap, osg::Texture2D::WrapMode> texWrapMap;
-    static bool firstTime = true;
-    if (firstTime)
-    {
-        texWrapMap[SoTexture2::CLAMP] = osg::Texture2D::CLAMP;
-        texWrapMap[SoTexture2::REPEAT] = osg::Texture2D::REPEAT;
-        firstTime = false;
-    }
+    static TexWrapMap texWrapMap;
 
     // Set texture wrap mode
 #ifdef __COIN__
@@ -2148,8 +2156,8 @@ void ConvertFromInventor::addVertex(SoCallbackAction* action,
     // Get the normal of the vertex
     SbVec3f norm = v->getNormal();
 
-    if ((normalBinding == osg::Geometry::BIND_PER_VERTEX) ||
-        (normalBinding == osg::Geometry::BIND_PER_PRIMITIVE && index == 0))
+    if ((normalBinding == deprecated_osg::Geometry::BIND_PER_VERTEX) ||
+        (normalBinding == deprecated_osg::Geometry::BIND_PER_PRIMITIVE && index == 0))
     {
         // What is this? Why to invert normals at CLOCKWISE vertex ordering?
         // PCJohn 2009-12-13
@@ -2159,18 +2167,18 @@ void ConvertFromInventor::addVertex(SoCallbackAction* action,
             normals.push_back(osg::Vec3(norm[0], norm[1], norm[2]));
     }
 
-    if (colorBinding == osg::Geometry::BIND_PER_VERTEX ||
-            colorBinding == osg::Geometry::BIND_PER_PRIMITIVE)
+    if (colorBinding == deprecated_osg::Geometry::BIND_PER_VERTEX ||
+            colorBinding == deprecated_osg::Geometry::BIND_PER_PRIMITIVE)
     {
         // Get the material/color
         SbColor ambient, diffuse, specular, emission;
         float transparency, shininess;
         action->getMaterial(ambient, diffuse, specular, emission, shininess,
                             transparency, v->getMaterialIndex());
-        if (colorBinding == osg::Geometry::BIND_PER_VERTEX)
+        if (colorBinding == deprecated_osg::Geometry::BIND_PER_VERTEX)
             colors.push_back(osg::Vec4(diffuse[0], diffuse[1], diffuse[2],
                                        1.0 - transparency));
-        else if (colorBinding == osg::Geometry::BIND_PER_PRIMITIVE && index == 0)
+        else if (colorBinding == deprecated_osg::Geometry::BIND_PER_PRIMITIVE && index == 0)
             colors.push_back(osg::Vec4(diffuse[0], diffuse[1], diffuse[2],
                                        1.0 - transparency));
     }
