@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -29,6 +29,8 @@
 #include "Document.h"
 #include "RecordInputStream.h"
 
+#include <osg/ValueObject>
+
 #include <algorithm>
 
 namespace flt {
@@ -88,7 +90,7 @@ void addDrawableAndReverseWindingOrder( osg::Geode* geode )
                         reverseWindingOrder( vertices, drawarray->getMode(), first, last );
                     }
 
-                    if( geom->getNormalBinding( ) == osg::Geometry::BIND_PER_VERTEX )
+                    if( osg::getBinding(geom->getNormalArray()) == osg::Array::BIND_PER_VERTEX )
                     {
                         osg::Vec3Array* normals = dynamic_cast<osg::Vec3Array*>(geom->getNormalArray());
                         if( normals )
@@ -102,7 +104,7 @@ void addDrawableAndReverseWindingOrder( osg::Geode* geode )
                         }
                     }
 
-                    if( geom->getColorBinding( ) == osg::Geometry::BIND_PER_VERTEX )
+                    if( osg::getBinding(geom->getColorArray()) == osg::Array::BIND_PER_VERTEX )
                     {
                         osg::Vec4Array* colors = dynamic_cast<osg::Vec4Array*>(geom->getColorArray());
                         if( colors )
@@ -254,7 +256,10 @@ public:
             {
                 // Use face color if vertex color is -1 in a gouraud polygon.
                 // http://www.multigen-paradigm.com/ubb/Forum1/HTML/000967.html
-                colors->push_back(_primaryColor);
+                // Incorporate Face transparency per osg-users thread "Open Flight
+                // characteristic not reflected in the current OSG" (Sept/Oct 2011)
+                colors->push_back(osg::Vec4(_primaryColor.r(), _primaryColor.g(),
+                    _primaryColor.b(), ( 1.0 - getTransparency() ) ));
             }
         }
 
@@ -344,7 +349,7 @@ protected:
     virtual void readRecord(RecordInputStream& in, Document& document)
     {
         std::string id = in.readString(8);
-        /*int32 IRColor =*/ in.readInt32();
+        int32 IRColor = in.readInt32();
         /*int16 relativePriority =*/ in.readInt16();
         _drawFlag = in.readUInt8(SOLID_NO_BACKFACE);
         uint8 texturedWhite = in.readUInt8();
@@ -355,8 +360,8 @@ protected:
         /*int detailTexture =*/ in.readInt16(-1);
         int textureIndex = in.readInt16(-1);
         int materialIndex = in.readInt16(-1);
-        /*int16 surface =*/ in.readInt16();
-        /*int16 feature =*/ in.readInt16();
+        int16 surface = in.readInt16();
+        int16 feature = in.readInt16();
         /*int32 IRMaterial =*/ in.readInt32(-1);
         _transparency = in.readUInt16(0);
         // version > 13
@@ -446,6 +451,24 @@ protected:
             stateset->setAttribute(material);
         }
 
+        // IRColor (IRC)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != IRColor)
+        {
+          _geometry->setUserValue("<UA:IRC>", IRColor);
+        }
+
+        // surface (SMC)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != surface)
+        {
+          _geometry->setUserValue("<UA:SMC>", surface);
+        }
+
+        // feature (FID)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != feature)
+        {
+          _geometry->setUserValue("<UA:FID>", feature);
+        }
+
         // Shaders
         if (shaderIndex >= 0)
         {
@@ -558,7 +581,7 @@ protected:
                     if (isGouraud())
                     {
                         // Color per vertex
-                        geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
+                        if (geometry->getColorArray()) geometry->getColorArray()->setBinding(osg::Array::BIND_PER_VERTEX);
                     }
                     else
                     {
@@ -566,21 +589,19 @@ protected:
                         osg::Vec4 col = getPrimaryColor();
                         col[3] = 1.0f - getTransparency();
 
-                        geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
                         osg::Vec4Array* colors = new osg::Vec4Array(1);
                         (*colors)[0] = col;
-                        geometry->setColorArray(colors);
+                        geometry->setColorArray(colors, osg::Array::BIND_OVERALL);
                     }
 
                     // Normal binding
                     if (isLit())
                     {
-                        geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX );
+                        if (geometry->getNormalArray()) geometry->getNormalArray()->setBinding(osg::Array::BIND_PER_VERTEX);
                     }
                     else
                     {
-                        geometry->setNormalBinding(osg::Geometry::BIND_OFF);
-                        geometry->setNormalArray(NULL);
+                        geometry->setNormalArray(0);
                     }
                 }
             }
@@ -641,7 +662,7 @@ protected:
 
                         billboard->getDrawable(i)->dirtyBound();
                     }
-                    
+
                     billboard->dirtyBound();
                 }
             }
@@ -917,7 +938,7 @@ protected:
     {
         std::string id = in.readString(8);
         in.forward(4);
-        /*int32 IRColor =*/ in.readInt32();
+        int32 IRColor = in.readInt32();
         /*int16 relativePriority =*/ in.readInt16();
         _drawFlag = in.readUInt8(SOLID_NO_BACKFACE);
         uint8 texturedWhite = in.readUInt8();
@@ -928,8 +949,8 @@ protected:
         /*int detailTexture =*/ in.readInt16(-1);
         int textureIndex = in.readInt16(-1);
         int materialIndex = in.readInt16(-1);
-        /*int16 surface =*/ in.readInt16();
-        /*int16 feature =*/ in.readInt16();
+        int16 surface = in.readInt16();
+        int16 feature = in.readInt16();
         /*int32 IRMaterial =*/ in.readInt32(-1);
         _transparency = in.readUInt16(0);
         // version > 13
@@ -1015,6 +1036,24 @@ protected:
             stateset->setAttribute(material);
         }
 
+        // IRColor (IRC)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != IRColor)
+        {
+          _geode->setUserValue("<UA:IRC>", IRColor);
+        }
+
+        // surface (SMC)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != surface)
+        {
+          _geode->setUserValue("<UA:SMC>", surface);
+        }
+
+        // feature (FID)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != feature)
+        {
+          _geode->setUserValue("<UA:FID>", feature);
+        }
+
         // Shaders
         if (shaderIndex >= 0)
         {
@@ -1137,7 +1176,7 @@ protected:
 
                         billboard->getDrawable(i)->dirtyBound();
                     }
-                    
+
                     billboard->dirtyBound();
                 }
             }
@@ -1245,7 +1284,7 @@ protected:
                 {
                     osg::Vec2f uv = in.readVec2f();
                     vertex.setUV(layer,uv);
-                    
+
                     if (!uv.valid())
                     {
                         OSG_NOTICE<<"Warning: data error detected in LocalVertexPool::readRecord uv="<<uv.x()<<" "<<uv.y()<<std::endl;
@@ -1337,7 +1376,7 @@ protected:
                 index = in.readUInt32();
                 break;
             }
-            
+
             if (index < vertexList->size())
             {
                 Vertex& vertex = (*vertexList)[index];
@@ -1372,7 +1411,7 @@ protected:
         if (mesh->isGouraud())
         {
             // Color per vertex
-            geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
+            if (geometry->getColorArray()) geometry->getColorArray()->setBinding(osg::Array::BIND_PER_VERTEX);
         }
         else
         {
@@ -1380,21 +1419,20 @@ protected:
             osg::Vec4 col = mesh->getPrimaryColor();
             col[3] = 1.0f - mesh->getTransparency();
 
-            geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
+
             osg::Vec4Array* colors = new osg::Vec4Array(1);
             (*colors)[0] = col;
-            geometry->setColorArray(colors);
+            geometry->setColorArray(colors, osg::Array::BIND_OVERALL);
         }
 
         // Normal binding
         if (mesh->isLit())
         {
-            geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX );
+            if (geometry->getNormalArray()) geometry->getNormalArray()->setBinding(osg::Array::BIND_PER_VERTEX);
         }
         else
         {
-            geometry->setNormalBinding(osg::Geometry::BIND_OFF);
-            geometry->setNormalArray(NULL);
+            geometry->setNormalArray(0);
         }
 
         mesh->addGeometry(*geometry);
