@@ -1,18 +1,18 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
 //
-// OpenFlight loader for OpenSceneGraph
+// OpenFlight (R) loader for OpenSceneGraph
 //
 //  Copyright (C) 2005-2007  Brede Johansen
 //
@@ -30,6 +30,23 @@
 #include "AttrData.h"
 #include "RecordInputStream.h"
 
+#if defined(OSG_GLES1_AVAILABLE) || defined(OSG_GLES2_AVAILABLE)
+    #define GL_RGB5                     0x8050
+    #define GL_RGBA4                    0x8056
+    #define GL_RGBA8                    0x8058
+    #define GL_RGBA12                   0x805A
+    #define GL_RGB12                    0x8053
+    #define GL_LUMINANCE12_ALPHA4       0x8046
+    #define GL_LUMINANCE12_ALPHA12      0x8047
+    #define GL_INTENSITY16              0x804D
+#endif
+
+#if defined(OSG_GL3_AVAILABLE)
+    #define GL_LUMINANCE12_ALPHA4       0x8046
+    #define GL_LUMINANCE12_ALPHA12      0x8047
+    #define GL_INTENSITY16              0x804D
+#endif
+
 namespace flt {
 
 class VertexPalette : public Record
@@ -49,11 +66,14 @@ protected:
         uint32 paletteSize = in.readUInt32();
 
         // Entries in vertex pool found by offset from start of this record.
-        const int RECORD_HEADER_SIZE = 4;
-        const int OFFSET = RECORD_HEADER_SIZE+sizeof(paletteSize);
+        const uint32 RECORD_HEADER_SIZE = 4;
+        const uint32 OFFSET = RECORD_HEADER_SIZE+sizeof(paletteSize);
 
         std::string buffer(paletteSize,'\0');
-        in.read(&buffer[OFFSET], paletteSize-OFFSET);
+        if (OFFSET < paletteSize)
+        {
+            in.read(&buffer[OFFSET], paletteSize-OFFSET);
+        }
 
         // Keep a copy of the vertex pool in memory for later reference.
         document.setVertexPool(new VertexPool(buffer));
@@ -200,8 +220,16 @@ protected:
         material->setDiffuse (osg::Material::FRONT_AND_BACK,osg::Vec4(diffuse,alpha));
         material->setSpecular(osg::Material::FRONT_AND_BACK,osg::Vec4(specular,alpha));
         material->setEmission(osg::Material::FRONT_AND_BACK,osg::Vec4(emissive,alpha));
-        material->setShininess(osg::Material::FRONT_AND_BACK,shininess);
         
+        if (shininess>=0.0f)
+        {        
+            material->setShininess(osg::Material::FRONT_AND_BACK,shininess);
+        }
+        else
+        {
+            OSG_INFO<<"Warning: OpenFlight shininess value out of range: "<<shininess<<std::endl;
+        }
+
         MaterialPool* mp = document.getOrCreateMaterialPool();
         (*mp)[index] = material;
     }
@@ -240,14 +268,22 @@ protected:
             /*uint32 flags =*/ in.readUInt32();
             std::string name = in.readString(12);
             in.forward(4*28);
-            
+
             osg::Material* material = new osg::Material;
             material->setAmbient(osg::Material::FRONT_AND_BACK,osg::Vec4(ambient,alpha));
             material->setDiffuse (osg::Material::FRONT_AND_BACK,osg::Vec4(diffuse,alpha));
             material->setSpecular(osg::Material::FRONT_AND_BACK,osg::Vec4(specular,alpha));
             material->setEmission(osg::Material::FRONT_AND_BACK,osg::Vec4(emissive,alpha));
-            material->setShininess(osg::Material::FRONT_AND_BACK,shininess);
-            
+
+            if (shininess>=0.0f)
+            {        
+                material->setShininess(osg::Material::FRONT_AND_BACK,shininess);
+            }
+            else
+            {
+                OSG_INFO<<"Warning: OpenFlight shininess value out of range: "<<shininess<<std::endl;
+            }
+
             MaterialPool* mp = document.getOrCreateMaterialPool();
             (*mp)[i] = material;
         }
@@ -266,7 +302,7 @@ public:
     TexturePalette() {}
 
     META_Record(TexturePalette)
-    
+
 protected:
 
     virtual ~TexturePalette() {}
@@ -449,7 +485,7 @@ protected:
         std::string pathname = osgDB::findDataFile(filename,document.getOptions());
         if (pathname.empty())
         {
-            osg::notify(osg::WARN) << "Can't find texture (" << index << ") " << filename << std::endl;
+            OSG_WARN << "Can't find texture (" << index << ") " << filename << std::endl;
             return;
         }
 
@@ -540,7 +576,7 @@ public:
 
     META_Record(LightSourcePalette)
 
-    enum LightType 
+    enum LightType
     {
         INFINITE_LIGHT = 0,
         LOCAL_LIGHT = 1,
@@ -638,9 +674,9 @@ protected:
         appearance->materialCode = in.readInt16();
         appearance->featureID = in.readInt16();
 
-        int32 backColorIndex = in.readInt32();        
-        appearance->backColor = document.getColorPool() ? 
-                            document.getColorPool()->getColor(backColorIndex) : 
+        int32 backColorIndex = in.readInt32();
+        appearance->backColor = document.getColorPool() ?
+                            document.getColorPool()->getColor(backColorIndex) :
                             osg::Vec4(1.0f, 1.0f, 1.0f, 1.0f);
 
         appearance->displayMode = in.readInt32();
@@ -676,14 +712,14 @@ protected:
         appearance->fadeOutDuration = in.readFloat32();
         appearance->LODRangeRatio = in.readFloat32();
         appearance->LODScale = in.readFloat32();
-          
+
         if(document.version() > VERSION_15_8)
             appearance->texturePatternIndex = in.readInt16(-1);
         else
            appearance->texturePatternIndex = -1;
- 
+
         // The final short is reserved; don't bother reading it.
- 
+
         // Add to pool
         LightPointAppearancePool* lpaPool = document.getOrCreateLightPointAppearancePool();
         (*lpaPool)[appearance->index] = appearance.get();
