@@ -114,7 +114,7 @@ void VBSPEntity::processFuncBrush()
         // Get the origin parameter's value
         std::string value = (*param).second;
 
-        // Parse the value into a vector 
+        // Parse the value into a vector
         entity_origin = getVector(value);
     }
     param = entity_parameters.find("angles");
@@ -123,7 +123,7 @@ void VBSPEntity::processFuncBrush()
         // Get the origin parameter's value
         std::string value = (*param).second;
 
-        // Parse the value into a vector 
+        // Parse the value into a vector
         entity_angles = getVector(value);
     }
 }
@@ -152,7 +152,7 @@ void VBSPEntity::processProp()
         // Get the origin parameter's value
         std::string value = (*param).second;
 
-        // Parse the value into a vector 
+        // Parse the value into a vector
         entity_origin = getVector(value);
     }
     param = entity_parameters.find("angles");
@@ -161,7 +161,7 @@ void VBSPEntity::processProp()
         // Get the origin parameter's value
         std::string value = (*param).second;
 
-        // Parse the value into a vector 
+        // Parse the value into a vector
         entity_angles = getVector(value);
     }
 }
@@ -560,8 +560,8 @@ ref_ptr<Group> VBSPEntity::createModelGeometry()
     }
     else
     {
-        notify(WARN) << "Couldn't find prop \"" << entity_model << "\".";
-        notify(WARN) << std::endl;
+        OSG_WARN << "Couldn't find prop \"" << entity_model << "\".";
+        OSG_WARN << std::endl;
 
         // Leave the group empty (no model to show)
         entityGroup = NULL;
