@@ -1,14 +1,14 @@
 /*
  * Copyright 2006 Sony Computer Entertainment Inc.
  *
- * Licensed under the SCEA Shared Source License, Version 1.0 (the "License"); you may not use this 
+ * Licensed under the SCEA Shared Source License, Version 1.0 (the "License"); you may not use this
  * file except in compliance with the License. You may obtain a copy of the License at:
  * http://research.scea.com/scea_shared_source_license.html
  *
- * Unless required by applicable law or agreed to in writing, software distributed under the License 
- * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
- * implied. See the License for the specific language governing permissions and limitations under the 
- * License. 
+ * Unless required by applicable law or agreed to in writing, software distributed under the License
+ * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+ * implied. See the License for the specific language governing permissions and limitations under the
+ * License.
  */
 
 #include "daeReader.h"
@@ -127,7 +127,7 @@ void daeReader::processBindMaterial( domBind_material *bm, domGeometry *geom, os
         osg::Drawable* drawable = geode->getDrawable(i);
         std::string materialName = drawable->getName();
         osg::Geometry *cachedGeometry = dynamic_cast<osg::Geometry*>(cachedGeode->getDrawable(i)->asGeometry());
-        
+
         domInstance_material_Array &ima = bm->getTechnique_common()->getInstance_material_array();
         std::string symbol;
         bool found = false;
@@ -135,7 +135,7 @@ void daeReader::processBindMaterial( domBind_material *bm, domGeometry *geom, os
         {
             symbol = ima[j]->getSymbol();
             if (symbol.compare(materialName) == 0)
-            { 
+            {
                 found = true;
                 domMaterial *mat = daeSafeCast< domMaterial >(getElementFromURI( ima[j]->getTarget()));
                 if (mat)
@@ -167,7 +167,7 @@ void daeReader::processBindMaterial( domBind_material *bm, domGeometry *geom, os
                     }
                     clonedGeometry->getTexCoordArrayList().clear();
 
-                    // 2. For each possible texture unit find the correct texcoord array and 
+                    // 2. For each possible texture unit find the correct texcoord array and
                     // indices from the cached drawable and place in the cloned drawable
                     // in the correct texture unit slot
                     unsigned int textureUnit(0);
@@ -200,12 +200,23 @@ void daeReader::processBindMaterial( domBind_material *bm, domGeometry *geom, os
 // 0..* <extra>
 void    daeReader::processMaterial(osg::StateSet *ss, domMaterial *mat )
 {
+    if (!mat)
+    {
+        return;
+    }
+    if (mat->getName()) {
+        ss->setName(mat->getName());
+    }
     _currentInstance_effect = mat->getInstance_effect();
+    if (!_currentInstance_effect)
+    {
+        return;
+    }
     domEffect *effect = daeSafeCast< domEffect >( getElementFromURI( _currentInstance_effect->getUrl() ) );
     if (effect)
     {
         processEffect(ss, effect);
-    
+
         //TODO: process all of the setParams that could happen here in the material. ESP. the textures
     }
     else
@@ -304,7 +315,6 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
         }
     }
 
-    xsNCName DiffuseTextureName = NULL;
     osg::ref_ptr< osg::Material > mat = new osg::Material();
     // <blinn>
     // elements:
@@ -328,11 +338,11 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
             OSG_WARN << "Currently no support for <texture> in Emission channel " << std::endl;
 
         processColorOrTextureType(ss, b->getAmbient(), osg::Material::AMBIENT, mat.get(), NULL,  &AmbientStateAttribute );
-        
+
         processColorOrTextureType(ss, b->getDiffuse(), osg::Material::DIFFUSE, mat.get(), NULL, &DiffuseStateAttribute );
-        if (DiffuseStateAttribute != NULL ) 
+        if (DiffuseStateAttribute != NULL )
         {
-            if (AmbientStateAttribute != NULL ) 
+            if (AmbientStateAttribute != NULL )
             {
                 // Set the ambient and diffuse colour white so that the incoming fragment colour ends up as a
                 // lit white colour. I modulate both textures onto this to approximate the lighting equation.
@@ -361,13 +371,10 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
                 ss->setTextureAttribute( textureUnit, new osg::TexEnv(osg::TexEnv::MODULATE) );
                 ss->setTextureAttribute( textureUnit, DiffuseStateAttribute );
             }
-
-            // Save the texture name for later
-            DiffuseTextureName = b->getDiffuse()->getTexture()->getTexture();
         }
         else
         {
-            if (NULL != AmbientStateAttribute  ) 
+            if (NULL != AmbientStateAttribute  )
                 OSG_WARN << "Ambient occlusion map only supported when diffuse texture also specified" << std::endl;
         }
 
@@ -381,7 +388,7 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
             ss->setAttributeAndModes(lightmodel, osg::StateAttribute::ON);
         }
 
-        processTransparencySettings(b->getTransparent(), b->getTransparency(), ss, mat.get(), DiffuseTextureName );
+        processTransparencySettings(b->getTransparent(), b->getTransparency(), ss, mat.get(), _pluginOptions.usePredefinedTextureUnits ? MAIN_TEXTURE_UNIT : 0 );
     }
     // <phong>
     // elements:
@@ -403,13 +410,13 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
         processColorOrTextureType(ss, p->getEmission(), osg::Material::EMISSION, mat.get(), NULL, &EmissionStateAttribute );
         if (NULL != EmissionStateAttribute)
             OSG_WARN << "Currently no support for <texture> in Emission channel " << std::endl;
-        
+
         processColorOrTextureType(ss, p->getAmbient(), osg::Material::AMBIENT, mat.get(), NULL,  &AmbientStateAttribute );
-        
+
         processColorOrTextureType(ss, p->getDiffuse(), osg::Material::DIFFUSE, mat.get(), NULL, &DiffuseStateAttribute );
-        if (DiffuseStateAttribute != NULL ) 
+        if (DiffuseStateAttribute != NULL )
         {
-            if (AmbientStateAttribute != NULL ) 
+            if (AmbientStateAttribute != NULL )
             {
                 // Set the ambient and diffuse colour white so that the incoming fragment colour ends up as a
                 // lit white colour. I modulate both textures onto this to approximate the lighting equation.
@@ -438,16 +445,13 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
                 ss->setTextureAttribute( textureUnit, new osg::TexEnv(osg::TexEnv::MODULATE) );
                 ss->setTextureAttribute( textureUnit, DiffuseStateAttribute );
             }
-
-            // Save the texture name for later
-            DiffuseTextureName = p->getDiffuse()->getTexture()->getTexture();
         }
         else
         {
-            if (NULL != AmbientStateAttribute  ) 
+            if (NULL != AmbientStateAttribute  )
                 OSG_WARN << "Ambient occlusion map only supported when diffuse texture also specified" << std::endl;
         }
-        
+
         if (processColorOrTextureType(ss, p->getSpecular(), osg::Material::SPECULAR, mat.get(), p->getShininess() ) && (NULL != DiffuseStateAttribute) )
         {
             // Diffuse texture will defeat specular highlighting
@@ -458,7 +462,7 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
             ss->setAttributeAndModes(lightmodel, osg::StateAttribute::ON);
         }
 
-        processTransparencySettings(p->getTransparent(), p->getTransparency(), ss, mat.get(), DiffuseTextureName );
+        processTransparencySettings(p->getTransparent(), p->getTransparency(), ss, mat.get(), _pluginOptions.usePredefinedTextureUnits ? MAIN_TEXTURE_UNIT : 0 );
     }
     // <lambert>
     // elements:
@@ -478,13 +482,13 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
         processColorOrTextureType(ss, l->getEmission(), osg::Material::EMISSION, mat.get(), NULL, &EmissionStateAttribute );
         if (NULL != EmissionStateAttribute)
             OSG_WARN << "Currently no support for <texture> in Emission channel " << std::endl;
-        
+
         processColorOrTextureType(ss, l->getAmbient(), osg::Material::AMBIENT, mat.get(), NULL,  &AmbientStateAttribute);
 
         processColorOrTextureType(ss, l->getDiffuse(), osg::Material::DIFFUSE, mat.get(), NULL, &DiffuseStateAttribute );
-        if (DiffuseStateAttribute != NULL ) 
+        if (DiffuseStateAttribute != NULL )
         {
-            if (AmbientStateAttribute != NULL ) 
+            if (AmbientStateAttribute != NULL )
             {
                 // Set the ambient and diffuse colour white so that the incoming fragment colour ends up as a
                 // lit white colour. I modulate both textures onto this to approximate the lighting equation.
@@ -513,17 +517,14 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
                 ss->setTextureAttribute( textureUnit, new osg::TexEnv(osg::TexEnv::MODULATE) );
                 ss->setTextureAttribute( textureUnit, DiffuseStateAttribute );
             }
-
-            // Save the texture name for later
-            DiffuseTextureName = l->getDiffuse()->getTexture()->getTexture();
         }
         else
         {
-            if (NULL != AmbientStateAttribute  ) 
+            if (NULL != AmbientStateAttribute  )
                 OSG_WARN << "Ambient occlusion map only supported when diffuse texture also specified" << std::endl;
         }
 
-        processTransparencySettings(l->getTransparent(), l->getTransparency(), ss, mat.get(), DiffuseTextureName );
+        processTransparencySettings(l->getTransparent(), l->getTransparency(), ss, mat.get(), _pluginOptions.usePredefinedTextureUnits ? MAIN_TEXTURE_UNIT : 0 );
     }
     // <constant>
     // elements:
@@ -537,7 +538,7 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
     {
         osg::Texture2D *sa = NULL;
         processColorOrTextureType(ss, c->getEmission(), osg::Material::EMISSION, mat.get(), NULL, &sa );
-        if (sa != NULL ) 
+        if (sa != NULL )
         {
             unsigned int textureUnit( _pluginOptions.usePredefinedTextureUnits ? MAIN_TEXTURE_UNIT : 0);
             ss->setTextureMode( textureUnit, GL_TEXTURE_2D, GL_TRUE );
@@ -548,7 +549,7 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
         // Use the emission colour as the main colour in transparency calculations
         mat->setDiffuse(osg::Material::FRONT_AND_BACK, mat->getEmission(osg::Material::FRONT_AND_BACK));
 
-        processTransparencySettings(c->getTransparent(), c->getTransparency(), ss, mat.get(), NULL );
+        processTransparencySettings(c->getTransparent(), c->getTransparency(), ss, mat.get(), _pluginOptions.usePredefinedTextureUnits ? MAIN_TEXTURE_UNIT : 0 );
 
         // Kill the lighting
         mat->setAmbient(osg::Material::FRONT_AND_BACK, osg::Vec4(0.0, 0.0, 0.0, 1.0));
@@ -569,7 +570,7 @@ void daeReader::processProfileCOMMON(osg::StateSet *ss, domProfile_COMMON *pc )
 //        1        texcoord
 //        0..*    extra
 bool daeReader::processColorOrTextureType(const osg::StateSet* ss,
-                                          domCommon_color_or_texture_type *cot, 
+                                          domCommon_color_or_texture_type *cot,
                                           osg::Material::ColorMode channel,
                                           osg::Material *mat,
                                           domCommon_float_or_param_type *fop,
@@ -591,7 +592,7 @@ bool daeReader::processColorOrTextureType(const osg::StateSet* ss,
         if (cot->getColor() != NULL )
         {
             domFloat4 &f4 = cot->getColor()->getValue();
-            mat->setEmission( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], f4[3] ) );
+            mat->setEmission( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], (f4.getCount()==4)? f4[3] : 1.0 ) );
             retVal = true;
         }
         else if (cot->getParam() != NULL)
@@ -612,7 +613,7 @@ bool daeReader::processColorOrTextureType(const osg::StateSet* ss,
             }
             else
                 OSG_WARN << "Currently no support for <texture> in Emission channel " << std::endl;
-        }       
+        }
         else
         {
             OSG_WARN << "Missing <color>, <param> or <texture> in Emission channel " << std::endl;
@@ -623,7 +624,7 @@ bool daeReader::processColorOrTextureType(const osg::StateSet* ss,
         if (cot->getColor() != NULL )
         {
             domFloat4 &f4 = cot->getColor()->getValue();
-            mat->setAmbient( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], f4[3] ) );
+            mat->setAmbient( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], (f4.getCount()==4)? f4[3] : 1.0 ) );
             retVal = true;
         }
         else if (cot->getParam() != NULL)
@@ -653,10 +654,10 @@ bool daeReader::processColorOrTextureType(const osg::StateSet* ss,
     }
     else if (channel == osg::Material::DIFFUSE )
     {
-        if (cot->getColor() != NULL )
+        if (cot->getColor() != NULL)
         {
             domFloat4 &f4 = cot->getColor()->getValue();
-            mat->setDiffuse( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], f4[3] ) );
+            mat->setDiffuse( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], (f4.getCount()==4)? f4[3] : 1.0 ) );
             retVal = true;
         }
         else if (cot->getTexture() != NULL)
@@ -707,7 +708,7 @@ bool daeReader::processColorOrTextureType(const osg::StateSet* ss,
         if (cot->getColor() != NULL )
         {
             domFloat4 &f4 = cot->getColor()->getValue();
-            mat->setSpecular( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], f4[3] ) );
+            mat->setSpecular( osg::Material::FRONT_AND_BACK, osg::Vec4( f4[0], f4[1], f4[2], (f4.getCount()==4)? f4[3] : 1.0 ) );
             retVal = true;
         }
         else if (cot->getParam() != NULL)
@@ -775,7 +776,7 @@ bool daeReader::GetFloat4Param(xsNCName Reference, domFloat4 &f4) const
         }
     }
 
-    domCommon_newparam_type *cnp = daeSafeCast< domCommon_newparam_type >( el ); 
+    domCommon_newparam_type *cnp = daeSafeCast< domCommon_newparam_type >( el );
     domFx_newparam_common *npc = daeSafeCast< domFx_newparam_common >( el );
     if ((cnp != NULL) && (NULL != cnp->getFloat4()))
     {
@@ -822,7 +823,7 @@ bool daeReader::GetFloatParam(xsNCName Reference, domFloat &f) const
         }
     }
 
-    domCommon_newparam_type *cnp = daeSafeCast< domCommon_newparam_type >( el ); 
+    domCommon_newparam_type *cnp = daeSafeCast< domCommon_newparam_type >( el );
     domFx_newparam_common *npc = daeSafeCast< domFx_newparam_common >( el );
     if ((cnp != NULL) && (NULL != cnp->getFloat()))
     {
@@ -1011,8 +1012,8 @@ osg::Texture2D* daeReader::processTexture(
     domImage *dImg = NULL;
 
     std::string target = std::string("./") + std::string(tex->getTexture());
-    OSG_NOTICE<<"processTexture("<<target<<")"<<std::endl;
-    
+    OSG_INFO<<"processTexture("<<target<<")"<<std::endl;
+
     daeSIDResolver res1( _currentEffect, target.c_str() );
     daeElement *el = res1.getElement();
 
@@ -1028,7 +1029,7 @@ osg::Texture2D* daeReader::processTexture(
     }
     else
     {
-        domCommon_newparam_type *cnp = daeSafeCast< domCommon_newparam_type >( el ); 
+        domCommon_newparam_type *cnp = daeSafeCast< domCommon_newparam_type >( el );
         domFx_newparam_common *npc = daeSafeCast< domFx_newparam_common >( el );
 
         if (cnp != NULL )
@@ -1046,6 +1047,12 @@ osg::Texture2D* daeReader::processTexture(
             return NULL;
         }
 
+        if (sampler->getSource() == NULL || sampler->getSource()->getValue() == NULL)
+        {
+            OSG_WARN << "Could not locate source for sampler2D" << std::endl;
+            return NULL;
+        }
+
         //find the newparam for the surface based on the sampler2D->source value
         target = std::string("./") + std::string( sampler->getSource()->getValue() );
         daeSIDResolver res2( _currentEffect, target.c_str() );
@@ -1055,7 +1062,7 @@ osg::Texture2D* daeReader::processTexture(
             OSG_WARN << "Could not locate newparam for source " << sampler->getSource()->getValue() << std::endl;
             return NULL;
         }
-        cnp = daeSafeCast< domCommon_newparam_type >( el ); 
+        cnp = daeSafeCast< domCommon_newparam_type >( el );
         npc = daeSafeCast< domFx_newparam_common >( el );
 
         if (cnp != NULL )
@@ -1096,11 +1103,11 @@ osg::Texture2D* daeReader::processTexture(
             parameters.wrap_t = getWrapMode(sampler->getWrap_s()->getValue());
         }
 
-        if (sampler->getMinfilter())
+        if (sampler->getMinfilter() && sampler->getMinfilter()->getValue() != FX_SAMPLER_FILTER_COMMON_NONE)
         {
             parameters.filter_min = getFilterMode(sampler->getMinfilter()->getValue(), true);
         }
-        if (sampler->getMagfilter())
+        if (sampler->getMagfilter() && sampler->getMagfilter()->getValue() != FX_SAMPLER_FILTER_COMMON_NONE)
         {
             parameters.filter_mag = getFilterMode(sampler->getMagfilter()->getValue(), false);
         }
@@ -1159,8 +1166,9 @@ In <blinn>, <constant>, <lambert>, and <phong>, the child element <transparent>
 optional opaque attribute whose valid values are:
  A_ONE (the default): Takes the transparency information from the colors alpha channel, where the value 1.0 is opaque.
  RGB_ZERO: Takes the transparency information from the colors red, green, and blue channels, where the value 0.0 is opaque,
-with each channel modulated independently.
-In the Specification, this is described in the FX Reference chapter in the
+ with each channel modulated independently.
+
+ In the Specification, this is described in the FX Reference chapter in the
 common_color_or_texture_type entry, along with a description of how transparency works in the
 Getting Started with COLLADA FX chapter in the Determining Transparency section.
 
@@ -1170,15 +1178,16 @@ Collada Digital Asset Schema Release 1.5.0 Release Notes
 The <transparent> elements opaque attribute now allows, in addition to A_ONE and RGB_ZERO, the following values:
  A_ZERO: Takes the transparency information from the colors alpha channel, where the value 0.0 is opaque.
  RGB_ONE: Takes the transparency information from the colors red, green, and blue channels, where the value 1.0
-* is opaque, with each channel modulated independently.
-* When we update to a version of the dom using that schema we will need to modify the code below 
+ is opaque, with each channel modulated independently.
+
+When we update to a version of the dom using that schema we will need to modify the code below
 */
 
 void daeReader::processTransparencySettings( domCommon_transparent_type *ctt,
                                             domCommon_float_or_param_type *pTransparency,
                                             osg::StateSet *ss,
                                             osg::Material *material,
-                                            xsNCName diffuseTextureName  )
+                                            unsigned int diffuseTextureUnit  )
 {
     if (ss == NULL)
         return;
@@ -1256,7 +1265,7 @@ void daeReader::processTransparencySettings( domCommon_transparent_type *ctt,
         if (!strictTransparency)
         {
             const osg::Texture* pMainTexture = dynamic_cast<osg::Texture*>(
-                ss->getTextureAttribute(MAIN_TEXTURE_UNIT, osg::StateAttribute::TEXTURE));
+                ss->getTextureAttribute(diffuseTextureUnit, osg::StateAttribute::TEXTURE));
             bool haveTranslucentTexture = pMainTexture &&
                 pMainTexture->getImage(0) && pMainTexture->getImage(0)->isImageTranslucent();
             strictTransparency = !haveTranslucentTexture;
@@ -1297,10 +1306,10 @@ bool daeReader::copyTextureCoordinateSet(const osg::StateSet* ss, const osg::Geo
     {
         if (!strcmp(bvia[k]->getSemantic(), texCoordSetName.c_str()) && !strcmp(bvia[k]->getInput_semantic(), COMMON_PROFILE_INPUT_TEXCOORD))
         {
-            unsigned set = bvia[k]->getInput_set();
+            unsigned int set = bvia[k]->getInput_set();
             if (set < cachedGeometry->getNumTexCoordArrays())
             {
-            clonedGeometry->setTexCoordData(localTextureUnit, cachedGeometry->getTexCoordData(set));
+                clonedGeometry->setTexCoordArray(localTextureUnit, const_cast<osg::Array*>(cachedGeometry->getTexCoordArray(set)));
             }
             else
             {
@@ -1312,9 +1321,31 @@ bool daeReader::copyTextureCoordinateSet(const osg::StateSet* ss, const osg::Geo
     if (k == bvia.getCount())
     {
         OSG_WARN << "Failed to find matching <bind_vertex_input> for " << texCoordSetName << std::endl;
-        if (cachedGeometry->getNumTexCoordArrays())
+
+        //bind_vertex_input failed, we try bind
+        const domInstance_material::domBind_Array &ba = im->getBind_array();
+        for (k = 0; k < ba.getCount(); k++)
         {
-        clonedGeometry->setTexCoordData(localTextureUnit, cachedGeometry->getTexCoordData(0));
+            if (!strcmp(ba[k]->getSemantic(), texCoordSetName.c_str()) )
+            {
+                IdToCoordIndexMap::const_iterator it = _texCoordIdMap.find(ba[k]->getTarget());
+                if (it!=_texCoordIdMap.end()&& (cachedGeometry->getNumTexCoordArrays()>it->second))
+                {
+                  clonedGeometry->setTexCoordArray(localTextureUnit, const_cast<osg::Array*>(cachedGeometry->getTexCoordArray(it->second)));
+                }
+                else
+                {
+                    OSG_WARN << "Texture coordinate set " << ba[k]->getTarget() << " not found." << std::endl;
+                }
+                break;
+            }
+        }
+        if (k == ba.getCount())
+        {
+            if (cachedGeometry->getNumTexCoordArrays())
+            {
+                clonedGeometry->setTexCoordArray(localTextureUnit, const_cast<osg::Array*>(cachedGeometry->getTexCoordArray(0)));
+            }
         }
     }
     return true;
