@@ -5,14 +5,14 @@
 #include <osgDB/FileNameUtils>
 
 
-static osg::Texture::WrapMode convertWrap(KFbxFileTexture::EWrapMode wrap)
+static osg::Texture::WrapMode convertWrap(FbxFileTexture::EWrapMode wrap)
 {
-    return wrap == KFbxFileTexture::eREPEAT ?
+    return wrap == FbxFileTexture::eRepeat ?
         osg::Texture2D::REPEAT : osg::Texture2D::CLAMP_TO_EDGE;
 }
 
 StateSetContent
-FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
+FbxMaterialToOsgStateSet::convert(const FbxSurfaceMaterial* pFbxMat)
 {
     FbxMaterialMap::const_iterator it = _fbxMaterialMap.find(pFbxMat);
     if (it != _fbxMaterialMap.end())
@@ -20,23 +20,23 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
 
     osg::ref_ptr<osg::Material> pOsgMat = new osg::Material;
     pOsgMat->setName(pFbxMat->GetName());
-    
+
     StateSetContent result;
 
     result.material = pOsgMat;
 
-    fbxString shadingModel = pFbxMat->ShadingModel.Get();
+    FbxString shadingModel = pFbxMat->ShadingModel.Get();
 
-    const KFbxSurfaceLambert* pFbxLambert = KFbxCast<KFbxSurfaceLambert>(pFbxMat);
+    const FbxSurfaceLambert* pFbxLambert = FbxCast<FbxSurfaceLambert>(pFbxMat);
 
     // diffuse map...
-    const KFbxProperty lProperty = pFbxMat->FindProperty(KFbxSurfaceMaterial::sDiffuse);
+    const FbxProperty lProperty = pFbxMat->FindProperty(FbxSurfaceMaterial::sDiffuse);
     if (lProperty.IsValid())
     {
-        int lNbTex = lProperty.GetSrcObjectCount(KFbxFileTexture::ClassId);
+        int lNbTex = lProperty.GetSrcObjectCount<FbxFileTexture>();
         for (int lTextureIndex = 0; lTextureIndex < lNbTex; lTextureIndex++)
         {
-            KFbxFileTexture* lTexture = KFbxCast<KFbxFileTexture>(lProperty.GetSrcObject(KFbxFileTexture::ClassId, lTextureIndex));
+            FbxFileTexture* lTexture = FbxCast<FbxFileTexture>(lProperty.GetSrcObject<FbxFileTexture>(lTextureIndex));
             if (lTexture)
             {
                 result.diffuseTexture = fbxTextureToOsgTexture(lTexture);
@@ -50,14 +50,24 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
         }
     }
 
+    double transparencyColorFactor = 1.0;
+    bool useTransparencyColorFactor = false;
+
     // opacity map...
-    const KFbxProperty lOpacityProperty = pFbxMat->FindProperty(KFbxSurfaceMaterial::sTransparentColor);
+    const FbxProperty lOpacityProperty = pFbxMat->FindProperty(FbxSurfaceMaterial::sTransparentColor);
     if (lOpacityProperty.IsValid())
     {
-        int lNbTex = lOpacityProperty.GetSrcObjectCount(KFbxFileTexture::ClassId);
+        FbxDouble3 transparentColor = lOpacityProperty.Get<FbxDouble3>();
+        // If transparent color is defined set the transparentFactor to gray scale value of transparentColor
+        if (transparentColor[0] < 1.0 || transparentColor[1] < 1.0 || transparentColor[2] < 1.0) {
+            transparencyColorFactor = transparentColor[0]*0.30 + transparentColor[1]*0.59 + transparentColor[2]*0.11;
+            useTransparencyColorFactor = true;
+        }         
+
+        int lNbTex = lOpacityProperty.GetSrcObjectCount<FbxFileTexture>();
         for (int lTextureIndex = 0; lTextureIndex < lNbTex; lTextureIndex++)
         {
-            KFbxFileTexture* lTexture = KFbxCast<KFbxFileTexture>(lOpacityProperty.GetSrcObject(KFbxFileTexture::ClassId, lTextureIndex));
+            FbxFileTexture* lTexture = FbxCast<FbxFileTexture>(lOpacityProperty.GetSrcObject<FbxFileTexture>(lTextureIndex));
             if (lTexture)
             {
                 // TODO: if texture image does NOT have an alpha channel, should it be added?
@@ -74,17 +84,17 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
     }
 
     // reflection map...
-    const KFbxProperty lReflectionProperty = pFbxMat->FindProperty(KFbxSurfaceMaterial::sReflection);
+    const FbxProperty lReflectionProperty = pFbxMat->FindProperty(FbxSurfaceMaterial::sReflection);
     if (lReflectionProperty.IsValid())
     {
-        int lNbTex = lReflectionProperty.GetSrcObjectCount(KFbxFileTexture::ClassId);
+        int lNbTex = lReflectionProperty.GetSrcObjectCount<FbxFileTexture>();
         for (int lTextureIndex = 0; lTextureIndex < lNbTex; lTextureIndex++)
         {
-            KFbxFileTexture* lTexture = KFbxCast<KFbxFileTexture>(lReflectionProperty.GetSrcObject(KFbxFileTexture::ClassId, lTextureIndex));
+            FbxFileTexture* lTexture = FbxCast<FbxFileTexture>(lReflectionProperty.GetSrcObject<FbxFileTexture>(lTextureIndex));
             if (lTexture)
             {
                 // support only spherical reflection maps...
-                if (KFbxFileTexture::eUMT_ENVIRONMENT == lTexture->CurrentMappingType.Get())
+                if (FbxFileTexture::eUMT_ENVIRONMENT == lTexture->CurrentMappingType.Get())
                 {
                     result.reflectionTexture = fbxTextureToOsgTexture(lTexture);
                     result.reflectionChannel = lTexture->UVSet.Get();
@@ -97,13 +107,13 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
     }
 
     // emissive map...
-    const KFbxProperty lEmissiveProperty = pFbxMat->FindProperty(KFbxSurfaceMaterial::sEmissive);
+    const FbxProperty lEmissiveProperty = pFbxMat->FindProperty(FbxSurfaceMaterial::sEmissive);
     if (lEmissiveProperty.IsValid())
     {
-        int lNbTex = lEmissiveProperty.GetSrcObjectCount(KFbxFileTexture::ClassId);
+        int lNbTex = lEmissiveProperty.GetSrcObjectCount<FbxFileTexture>();
         for (int lTextureIndex = 0; lTextureIndex < lNbTex; lTextureIndex++)
         {
-            KFbxFileTexture* lTexture = KFbxCast<KFbxFileTexture>(lEmissiveProperty.GetSrcObject(KFbxFileTexture::ClassId, lTextureIndex));
+            FbxFileTexture* lTexture = FbxCast<FbxFileTexture>(lEmissiveProperty.GetSrcObject<FbxFileTexture>(lTextureIndex));
             if (lTexture)
             {
                 result.emissiveTexture = fbxTextureToOsgTexture(lTexture);
@@ -117,15 +127,37 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
         }
     }
 
+    // ambient map...
+    const FbxProperty lAmbientProperty = pFbxMat->FindProperty(FbxSurfaceMaterial::sAmbient);
+    if (lAmbientProperty.IsValid())
+    {
+        int lNbTex = lAmbientProperty.GetSrcObjectCount<FbxFileTexture>();
+        for (int lTextureIndex = 0; lTextureIndex < lNbTex; lTextureIndex++)
+        {
+            FbxFileTexture* lTexture = FbxCast<FbxFileTexture>(lAmbientProperty.GetSrcObject<FbxFileTexture>(lTextureIndex));
+            if (lTexture)
+            {
+                result.ambientTexture = fbxTextureToOsgTexture(lTexture);
+                result.ambientChannel = lTexture->UVSet.Get();
+                result.ambientScaleU = lTexture->GetScaleU();
+                result.ambientScaleV = lTexture->GetScaleV();
+            }
+
+            //For now only allow 1 texture
+            break;
+        }
+    }
+
     if (pFbxLambert)
     {
-        fbxDouble3 color = pFbxLambert->Diffuse.Get();
+        FbxDouble3 color = pFbxLambert->Diffuse.Get();
         double factor = pFbxLambert->DiffuseFactor.Get();
+        double transparencyFactor = useTransparencyColorFactor ? transparencyColorFactor : pFbxLambert->TransparencyFactor.Get();
         pOsgMat->setDiffuse(osg::Material::FRONT_AND_BACK, osg::Vec4(
             static_cast<float>(color[0] * factor),
             static_cast<float>(color[1] * factor),
             static_cast<float>(color[2] * factor),
-            static_cast<float>(1.0 - pFbxLambert->TransparencyFactor.Get())));
+            static_cast<float>(1.0 - transparencyFactor)));
 
         color = pFbxLambert->Ambient.Get();
         factor = pFbxLambert->AmbientFactor.Get();
@@ -146,7 +178,7 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
         // get maps factors...
         result.diffuseFactor = pFbxLambert->DiffuseFactor.Get();
 
-        if (const KFbxSurfacePhong* pFbxPhong = KFbxCast<KFbxSurfacePhong>(pFbxLambert))
+        if (const FbxSurfacePhong* pFbxPhong = FbxCast<FbxSurfacePhong>(pFbxLambert))
         {
             color = pFbxPhong->Specular.Get();
             factor = pFbxPhong->SpecularFactor.Get();
@@ -155,9 +187,12 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
                 static_cast<float>(color[1] * factor),
                 static_cast<float>(color[2] * factor),
                 1.0f));
-
+            // Since Maya and 3D studio Max stores their glossiness values in exponential format (2^(log2(x)) 
+            // We need to linearize to values between 0-100 and then scale to values between 0-128.
+            // Glossiness values above 100 will result in shininess larger than 128.0 and will be clamped
+            double shininess = (64.0 * log (pFbxPhong->Shininess.Get())) / (5.0 * log(2.0));
             pOsgMat->setShininess(osg::Material::FRONT_AND_BACK,
-                static_cast<float>(pFbxPhong->Shininess.Get()));
+                static_cast<float>(shininess));
 
             // get maps factors...
             result.reflectionFactor = pFbxPhong->ReflectionFactor.Get();
@@ -182,7 +217,7 @@ FbxMaterialToOsgStateSet::convert(const KFbxSurfaceMaterial* pFbxMat)
 }
 
 osg::ref_ptr<osg::Texture2D>
-FbxMaterialToOsgStateSet::fbxTextureToOsgTexture(const KFbxFileTexture* fbx)
+FbxMaterialToOsgStateSet::fbxTextureToOsgTexture(const FbxFileTexture* fbx)
 {
     ImageMap::iterator it = _imageMap.find(fbx->GetFileName());
     if (it != _imageMap.end())
