@@ -9,6 +9,14 @@
 //#include <iostream>
 
 
+#ifndef AVCODEC_MAX_AUDIO_FRAME_SIZE
+#define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000
+#endif
+
+
+#if LIBAVCODEC_VERSION_MAJOR < 56
+   #define AV_CODEC_ID_NONE CODEC_ID_NONE
+#endif
 
 namespace osgFFmpeg {
 
@@ -16,7 +24,54 @@ static int decode_audio(AVCodecContext *avctx, int16_t *samples,
                          int *frame_size_ptr,
                          const uint8_t *buf, int buf_size)
 {
-#if LIBAVCODEC_VERSION_MAJOR >= 53 || (LIBAVCODEC_VERSION_MAJOR==52 && LIBAVCODEC_VERSION_MINOR>=32)
+#if LIBAVCODEC_VERSION_MAJOR >= 56
+
+    AVFrame *frame = av_frame_alloc();
+
+    if (!frame) return AVERROR(ENOMEM);
+
+    AVPacket avpkt;
+    av_init_packet(&avpkt);
+    avpkt.data = const_cast<uint8_t *>(buf);
+    avpkt.size = buf_size;
+
+    int got_frame = 0;
+    int result = avcodec_decode_audio4(avctx, frame, &got_frame, &avpkt);
+
+    if (result>=0 && got_frame)
+    {
+        int ch, plane_size;
+        int planar = av_sample_fmt_is_planar(avctx->sample_fmt);
+        int data_size = av_samples_get_buffer_size(&plane_size, avctx->channels, frame->nb_samples, avctx->sample_fmt, 1);
+        if (*frame_size_ptr < data_size)
+        {
+            av_log(avctx, AV_LOG_ERROR, "output buffer size is too small for "
+            "the current frame (%d < %d)\n", *frame_size_ptr, data_size);
+            av_frame_free(&frame);
+            return AVERROR(EINVAL);
+        }
+        memcpy(samples, frame->extended_data[0], plane_size);
+        if (planar && avctx->channels > 1)
+        {
+            uint8_t *out = ((uint8_t *)samples) + plane_size;
+            for (ch = 1; ch < avctx->channels; ch++)
+            {
+                memcpy(out, frame->extended_data[ch], plane_size);
+                out += plane_size;
+            }
+        }
+        *frame_size_ptr = data_size;
+    }
+    else
+    {
+        *frame_size_ptr = 0;
+    }
+
+    av_frame_free(&frame);
+
+    return result;
+
+#elif LIBAVCODEC_VERSION_MAJOR >= 53 || (LIBAVCODEC_VERSION_MAJOR==52 && LIBAVCODEC_VERSION_MINOR>=32)
 
     // following code segment copied from ffmpeg's avcodec_decode_audio2()
     // implementation to avoid warnings about deprecated function usage.
@@ -72,10 +127,34 @@ void FFmpegDecoderAudio::open(AVStream * const stream)
 
         m_frequency = m_context->sample_rate;
         m_nb_channels = m_context->channels;
-        m_sample_format = osg::AudioStream::SampleFormat(m_context->sample_fmt);
+
+        OSG_NOTICE<<"FFmpegDecoderAudio::open(..), m_nb_channels="<<m_nb_channels<<", m_context->sample_fmt="<<m_context->sample_fmt<<std::endl;
+
+
+        switch (m_context->sample_fmt)
+        {
+        case AV_SAMPLE_FMT_NONE:
+            throw std::runtime_error("invalid audio format AV_SAMPLE_FMT_NONE");
+        case AV_SAMPLE_FMT_U8:
+            m_sample_format = osg::AudioStream::SAMPLE_FORMAT_U8;
+            break;
+        case AV_SAMPLE_FMT_S16:
+            m_sample_format = osg::AudioStream::SAMPLE_FORMAT_S16;
+            break;
+        case AV_SAMPLE_FMT_S32:
+            m_sample_format = osg::AudioStream::SAMPLE_FORMAT_S32;
+            break;
+        case AV_SAMPLE_FMT_FLT:
+            m_sample_format = osg::AudioStream::SAMPLE_FORMAT_F32;
+            break;
+        case AV_SAMPLE_FMT_DBL:
+            throw std::runtime_error("unhandled audio format AV_SAMPLE_FMT_DBL");
+        default:
+            throw std::runtime_error("unknown audio format");
+        }
 
         // Check stream sanity
-        if (m_context->codec_id == CODEC_ID_NONE)
+        if (m_context->codec_id == AV_CODEC_ID_NONE)
             throw std::runtime_error("invalid audio codec");;
 
         // Find the decoder for the audio stream
@@ -89,7 +168,7 @@ void FFmpegDecoderAudio::open(AVStream * const stream)
         //    m_context->flags |= CODEC_FLAG_TRUNCATED;
 
         // Open codec
-        if (avcodec_open(m_context, p_codec) < 0)
+        if (avcodec_open2(m_context, p_codec, NULL) < 0)
             throw std::runtime_error("avcodec_open() failed");
     }
 
@@ -211,7 +290,7 @@ void FFmpegDecoderAudio::fillBuffer(void * const buffer, size_t size)
 void FFmpegDecoderAudio::decodeLoop()
 {
     const bool skip_audio = ! validContext() || ! m_audio_sink.valid();
-    
+
     if (! skip_audio && ! m_audio_sink->playing())
     {
         m_clocks.audioSetDelay(m_audio_sink->getDelay());
@@ -319,7 +398,7 @@ size_t FFmpegDecoderAudio::decodeFrame(void * const buffer, const size_t size)
 
             // If we have some data, return it and come back for more later.
             if (data_size > 0)
-                return data_size; 
+                return data_size;
         }
 
         // Get next packet
