@@ -1,4 +1,4 @@
-/* 
+/*
  The following code was based on code from the following location:
     http://www.graphics.cornell.edu/online/formats/rgbe/
 
@@ -8,7 +8,7 @@
     developed by Greg Ward.  It handles the conversions between rgbe and
     pixels consisting of floats.  The data is assumed to be an array of floats.
     By default there are three floats per pixel in the order red, green, blue.
-    (RGBE_DATA_??? values control this.)  Only the mimimal header reading and 
+    (RGBE_DATA_??? values control this.)  Only the mimimal header reading and
     writing is implemented.  Each routine does error checking and will return
     a status value as defined below.  This code is intended as a skeleton so
     feel free to modify it to suit your needs.
@@ -39,13 +39,12 @@ public:
 protected:
 
 // can read or write pixels in chunks of any size including single pixels
-    static bool writePixelsNoRLE( std::ostream& fout, float* data, int numpixels);
+    static bool writeNoRLE( std::ostream& fout, const osg::Image* image);
     static bool writePixelsRAW(  std::ostream& fout, unsigned char* data, int numpixels);
 
-// read or write run length encoded files 
+// read or write run length encoded files
 // must be called to read or write whole scanlines
     static bool writeBytesRLE(std::ostream& fout, unsigned char *data, int numbytes);
-    static bool writePixelsRLE( std::ostream& fout, float* data, int scanline_width, int num_scanlines );
 
 // inline conversions
     inline static void float2rgbe(unsigned char rgbe[4], float red, float green, float blue);
