@@ -18,14 +18,24 @@
 // probably especially important for istream which lacks extension information.
 // Is there information we can use in the OSG options parameter?
 
-// For ImageIO framework and also LaunchServices framework (for UTIs)
-#include <ApplicationServices/ApplicationServices.h>
+
+#import "TargetConditionals.h"
+#if (TARGET_OS_IPHONE)
+    #import <UIKit/UIKit.h>
+    #import <ImageIO/ImageIO.h>
+    #import <CoreGraphics/CoreGraphics.h>
+    #import <Foundation/Foundation.h>
+    #import <MobileCoreServices/MobileCoreServices.h>
+#else
+    #include <ApplicationServices/ApplicationServices.h>
+#endif
 // For the vImage framework (part of the Accerlate framework)
 #include <Accelerate/Accelerate.h>
 
 // Used because CGDataProviderCreate became deprecated in 10.5
 #include <AvailabilityMacros.h>
 
+
 #include <osg/GL>
 #include <osg/Notify>
 #include <osg/Image>
@@ -41,7 +51,7 @@
 /**************************************************************
  ***** Begin Callback functions for istream block reading *****
  **************************************************************/
- 
+
 // This callback reads some bytes from an istream and copies it
 // to a Quartz buffer (supplied by Apple framework).
 size_t MyProviderGetBytesCallback(void* istream_userdata, void* quartz_buffer, size_t the_count)
@@ -140,7 +150,7 @@ CGImageRef CreateCGImageFromDataStream(std::istream& fin)
         MyProviderRewindCallback,
         MyProviderReleaseInfoCallback
     };
-    
+
     CGDataProviderRef data_provider = CGDataProviderCreateSequential(&fin, &provider_callbacks);
 
 
@@ -153,7 +163,7 @@ CGImageRef CreateCGImageFromDataStream(std::istream& fin)
         MyProviderRewindCallback,
         MyProviderReleaseInfoCallback
     };
-    
+
     CGDataProviderRef data_provider = CGDataProviderCreate(&fin, &provider_callbacks);
 #endif
     // If we had a way of hinting at what the data type is, we could
@@ -169,7 +179,7 @@ CGImageRef CreateCGImageFromDataStream(std::istream& fin)
     }
 
     image_ref = CGImageSourceCreateImageAtIndex(source_ref, 0, NULL);
-    
+
     /* Don't need the SourceRef any more (error or not) */
     CFRelease(source_ref);
 
@@ -194,13 +204,13 @@ CGImageRef CreateCGImageFromFile(const char* the_path)
     );
     if(!cf_string)
     {
-        osg::notify(osg::WARN) << "CreateCGImageFromFile :: could not create CCFSTring" << std::endl;
+        OSG_WARN << "CreateCGImageFromFile :: could not create CCFSTring" << std::endl;
         return NULL;
     }
- 
+
     /* Create a CFURL from a CFString */
     the_url = CFURLCreateWithFileSystemPath(
-        NULL, 
+        NULL,
         cf_string,
         kCFURLPOSIXPathStyle,
         false
@@ -208,21 +218,21 @@ CGImageRef CreateCGImageFromFile(const char* the_path)
 
     /* Don't need the CFString any more (error or not) */
     CFRelease(cf_string);
-    
+
     if(!the_url)
     {
-        osg::notify(osg::WARN) << "CreateCGImageFromFile :: could not create CFUrl" << std::endl;
+        OSG_WARN << "CreateCGImageFromFile :: could not create CFUrl" << std::endl;
         return NULL;
     }
 
-    
+
     source_ref = CGImageSourceCreateWithURL(the_url, NULL);
     /* Don't need the URL any more (error or not) */
     CFRelease(the_url);
 
     if(!source_ref)
     {
-        osg::notify(osg::WARN) << "CreateCGImageFromFile :: could not create ImageSource" << std::endl;
+        OSG_WARN << "CreateCGImageFromFile :: could not create ImageSource" << std::endl;
         return NULL;
     }
 
@@ -230,9 +240,9 @@ CGImageRef CreateCGImageFromFile(const char* the_path)
     // contain multiple items).
     image_ref = CGImageSourceCreateImageAtIndex(source_ref, 0, NULL);
     if (!image_ref) {
-        osg::notify(osg::WARN) << "CreateCGImageFromFile :: could not get Image" << std::endl;
+        OSG_WARN << "CreateCGImageFromFile :: could not get Image" << std::endl;
     }
-    
+
     /* Don't need the SourceRef any more (error or not) */
     CFRelease(source_ref);
 
@@ -244,7 +254,7 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
 {
     /* This code is adapted from Apple's Documentation found here:
      * http://developer.apple.com/documentation/GraphicsImaging/Conceptual/OpenGL-MacProgGuide/index.html
-     * Listing 9-4††Using a Quartz image as a texture source.
+     * Listing 9-4Using a Quartz image as a texture source.
      * Unfortunately, this guide doesn't show what to do about
      * non-RGBA image formats so I'm making the rest up
      * (and it's probably all wrong).
@@ -252,19 +262,19 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
 
     size_t the_width = CGImageGetWidth(image_ref);
     size_t the_height = CGImageGetHeight(image_ref);
-    CGRect the_rect = {{0, 0}, {the_width, the_height}};
+    CGRect the_rect = {{0.0f, 0.0f}, {static_cast<CGFloat>(the_width), static_cast<CGFloat>(the_height)}};
 
     size_t bits_per_pixel = CGImageGetBitsPerPixel(image_ref);
     size_t bytes_per_row = CGImageGetBytesPerRow(image_ref);
 //    size_t bits_per_component = CGImageGetBitsPerComponent(image_ref);
     size_t bits_per_component = 8;
-    
+
     CGImageAlphaInfo alpha_info = CGImageGetAlphaInfo(image_ref);
-    
+
     GLint internal_format;
     GLenum pixel_format;
     GLenum data_type;
-    
+
     void* image_data = calloc(the_width * 4, the_height);
 
     CGColorSpaceRef color_space;
@@ -288,7 +298,7 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
                 internal_format = GL_RGBA8;
                 pixel_format = GL_BGRA_EXT;
                 data_type = GL_UNSIGNED_INT_8_8_8_8_REV;
-                
+
                 bytes_per_row = the_width*4;
 //                color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
                 color_space = CGColorSpaceCreateDeviceRGB();
@@ -297,7 +307,7 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
                 bitmap_info = kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Big; /* XRGB Big Endian */
 #else
                 bitmap_info = kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Little; /* XRGB Little Endian */
-#endif 
+#endif
             }
             else
             {
@@ -324,24 +334,26 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
             bitmap_info = kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Big; /* XRGB Big Endian */
 #else
             bitmap_info = kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Little; /* XRGB Little Endian */
-#endif       
+#endif
             break;
         }
         //
         // Tatsuhiro Nishioka
         // 16 bpp grayscale (8 bit white and 8 bit alpha) causes invalid argument combination
-        // in CGBitmapContextCreate. 
+        // in CGBitmapContextCreate.
         // I guess it is safer to handle 16 bit grayscale image as 32-bit RGBA image.
         // It works at least on FlightGear
         //
         case 16:
         case 32:
+        case 48:
+        case 64:
         {
 
             internal_format = GL_RGBA8;
             pixel_format = GL_BGRA_EXT;
             data_type = GL_UNSIGNED_INT_8_8_8_8_REV;
-            
+
             bytes_per_row = the_width*4;
 //            color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
             color_space = CGColorSpaceCreateDeviceRGB();
@@ -351,18 +363,19 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
             bitmap_info = kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Big; /* XRGB Big Endian */
 #else
             bitmap_info = kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Little; /* XRGB Little Endian */
-#endif 
+#endif
             break;
         }
         default:
         {
-            // osg::notify(osg::WARN) << "Unknown file type in " << fileName.c_str() << " with " << origDepth << std::endl;
+            // OSG_WARN << "Unknown file type in " << fileName.c_str() << " with " << origDepth << std::endl;
             return NULL;
             break;
         }
 
     }
-    
+
+
     // Sets up a context to be drawn to with image_data as the area to be drawn to
     CGContextRef bitmap_context = CGBitmapContextCreate(
         image_data,
@@ -373,51 +386,31 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
         color_space,
         bitmap_info
     );
-    
+
+    CGContextTranslateCTM(bitmap_context, 0, the_height);
+    CGContextScaleCTM(bitmap_context, 1.0, -1.0);
     // Draws the image into the context's image_data
     CGContextDrawImage(bitmap_context, the_rect, image_ref);
 
     CGContextRelease(bitmap_context);
 
-    // 
-    // Reverse the premultiplied alpha for avoiding unexpected darker edges
-    // by Tatsuhiro Nishioka (based on SDL's workaround on the similar issue)
-    // http://bugzilla.libsdl.org/show_bug.cgi?id=868
-    // 
-    if (bits_per_pixel > 8 && (bitmap_info & kCGBitmapAlphaInfoMask) == kCGImageAlphaPremultipliedFirst) {
-        int i, j;
-        GLubyte *pixels = (GLubyte *)image_data;
-        for (i = the_height * the_width; i--; ) {
-            GLuint *value = (GLuint *)pixels;
-#if __BIG_ENDIAN__
-            // 
-            // swap endian of each pixel for avoiding weird colors on ppc macs
-            // by Tatsuhiro Nishioka
-            // FIXME: I've tried many combinations of pixel_format, internal_format, and data_type 
-            // but none worked well. Therefore I tried endian swapping, which seems working with gif,png,tiff,tga,and psd.
-            // (for grayscaled tga and non-power-of-two tga, I can't guarantee since test images (made with Gimp) 
-            // get corrupted on Preview.app ...
-            *value = ((*value) >> 24) | (((*value) << 8) & 0x00FF0000) | (((*value) >> 8) & 0x0000FF00) | ((*value) << 24);
-#endif
-            GLubyte alpha = pixels[3];
-            if (alpha) {
-                for (j = 0; j < 3; ++j) {
-                    pixels[j] = (pixels[j] * 255) / alpha;
-                }
-            }
-            pixels += 4;
-        }
-    }
+    if (!image_data)
+        return NULL;
+
+    // alpha is premultiplied with rgba, undo it
+
+    vImage_Buffer vb;
+    vb.data = image_data;
+    vb.height = the_height;
+    vb.width = the_width;
+    vb.rowBytes = the_width * 4;
+    vImageUnpremultiplyData_RGBA8888(&vb, &vb, 0);
 
-    //
-    // Workaround for ignored alpha channel
-    // by Tatsuhiro Nishioka
-    // FIXME: specifying GL_UNSIGNED_INT_8_8_8_8_REV or GL_UNSIGNED_INT_8_8_8_8 ignores the alpha channel.
     // changing it to GL_UNSIGNED_BYTE seems working, but I'm not sure if this is a right way.
     //
     data_type = GL_UNSIGNED_BYTE;
     osg::Image* osg_image = new osg::Image;
-    
+
     osg_image->setImage(
         the_width,
         the_height,
@@ -429,7 +422,6 @@ osg::Image* CreateOSGImageFromCGImage(CGImageRef image_ref)
         osg::Image::USE_MALLOC_FREE // Assumption: osg_image takes ownership of image_data and will free
     );
 
-    osg_image->flipVertical();
     return osg_image;
 
 
@@ -457,7 +449,7 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
      * least `(width * bitsPerComponent * number of components + 7)/8' bytes.
      */
     size_t target_bytes_per_row;
-     
+
     CGColorSpaceRef color_space;
     CGBitmapInfo bitmap_info;
     /* From what I can figure out so far...
@@ -474,7 +466,7 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
      * we don't want to touch the data, so again we turn to the vImage framework
      * and invert the data.
      */
-    vImage_Buffer vimage_buffer_in = 
+    vImage_Buffer vimage_buffer_in =
     {
         (void*)osg_image.data(), // need to override const, but we don't modify the data so it's safe
         image_height,
@@ -489,7 +481,7 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
         image_height,
         image_width,
         0 // will fill-in in switch
-    };    
+    };
     vImage_Error vimage_error_flag;
 
     // FIXME: Do I want to use format, type, or internalFormat?
@@ -499,17 +491,18 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
         {
             bitmap_info = kCGImageAlphaNone;
             target_bytes_per_row = (image_width * 8 + 7)/8;
-            color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericGray);
+            //color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericGray);
+            color_space = CGColorSpaceCreateDeviceGray();
             if(NULL == color_space)
             {
                 return NULL;
             }
-            
+
             //    out_image_data = calloc(target_bytes_per_row, image_height);
-            out_image_data = malloc(target_bytes_per_row * image_height);            
+            out_image_data = malloc(target_bytes_per_row * image_height);
             if(NULL == out_image_data)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
                 CGColorSpaceRelease(color_space);
                 return NULL;
             }
@@ -525,11 +518,11 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             );
             if(vimage_error_flag != kvImageNoError)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData for GL_LUMINANCE, vImageVerticalReflect_Planar8 failed with vImage Error Code: " << vimage_error_flag << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData for GL_LUMINANCE, vImageVerticalReflect_Planar8 failed with vImage Error Code: " << vimage_error_flag << std::endl;
                 free(out_image_data);
                 CGColorSpaceRelease(color_space);
                 return NULL;
-            }            
+            }
 
 
             break;
@@ -542,12 +535,12 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             // http://developer.apple.com/qa/qa2001/qa1037.html
             // colorSpace=NULL is for alpha only
             color_space = NULL;
-            
+
             //    out_image_data = calloc(target_bytes_per_row, image_height);
-            out_image_data = malloc(target_bytes_per_row * image_height);            
+            out_image_data = malloc(target_bytes_per_row * image_height);
             if(NULL == out_image_data)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
                 return NULL;
             }
 
@@ -562,10 +555,10 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             );
             if(vimage_error_flag != kvImageNoError)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData for GL_ALPHA, vImageVerticalReflect_Planar8 failed with vImage Error Code: " << vimage_error_flag << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData for GL_ALPHA, vImageVerticalReflect_Planar8 failed with vImage Error Code: " << vimage_error_flag << std::endl;
                 free(out_image_data);
                 return NULL;
-            }            
+            }
 
 
             break;
@@ -582,18 +575,19 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
         {
             bitmap_info = kCGImageAlphaNoneSkipFirst;
             target_bytes_per_row = (image_width * 8 * 4 + 7)/8;
-            color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
-            if(NULL == color_space)
+            //color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+            color_space = CGColorSpaceCreateDeviceRGB();
+             if(NULL == color_space)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, CGColorSpaceCreateWithName failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, CGColorSpaceCreateWithName failed" << std::endl;
                 return NULL;
             }
-            
+
             //    out_image_data = calloc(target_bytes_per_row, image_height);
-            out_image_data = malloc(target_bytes_per_row * image_height);            
+            out_image_data = malloc(target_bytes_per_row * image_height);
             if(NULL == out_image_data)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
                 CGColorSpaceRelease(color_space);
                 return NULL;
             }
@@ -611,7 +605,7 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             );
             if(vimage_error_flag != kvImageNoError)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, vImageConvert_RGB888toARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, vImageConvert_RGB888toARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
                 free(out_image_data);
                 CGColorSpaceRelease(color_space);
                 return NULL;
@@ -624,29 +618,30 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             );
             if(vimage_error_flag != kvImageNoError)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, vImageAffineWarp_ARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, vImageAffineWarp_ARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
                 free(out_image_data);
                 CGColorSpaceRelease(color_space);
                 return NULL;
-            }            
-            
+            }
+
             break;
         }
         case GL_RGBA:
         {
             bitmap_info = kCGImageAlphaPremultipliedLast;
             target_bytes_per_row = osg_image.getRowSizeInBytes();
-            color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+            //color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+            color_space = CGColorSpaceCreateDeviceRGB();
             if(NULL == color_space)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, CGColorSpaceCreateWithName failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, CGColorSpaceCreateWithName failed" << std::endl;
                 return NULL;
             }
             //    out_image_data = calloc(target_bytes_per_row, image_height);
-            out_image_data = malloc(target_bytes_per_row * image_height);            
+            out_image_data = malloc(target_bytes_per_row * image_height);
             if(NULL == out_image_data)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
                 CGColorSpaceRelease(color_space);
                 return NULL;
             }
@@ -660,11 +655,11 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             );
             if(vimage_error_flag != kvImageNoError)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, vImageAffineWarp_ARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, vImageAffineWarp_ARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
                 free(out_image_data);
                 CGColorSpaceRelease(color_space);
                 return NULL;
-            }            
+            }
             break;
         }
         case GL_BGRA:
@@ -675,7 +670,7 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
                 bitmap_info = kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Big; /* XRGB Big Endian */
 #else
                 bitmap_info = kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Little; /* XRGB Little Endian */
-#endif 
+#endif
             }
             else
             {
@@ -684,17 +679,18 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
             }
 
             target_bytes_per_row = osg_image.getRowSizeInBytes();
-            color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+            //color_space = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+            color_space = CGColorSpaceCreateDeviceRGB();
             if(NULL == color_space)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, CGColorSpaceCreateWithName failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, CGColorSpaceCreateWithName failed" << std::endl;
                 return NULL;
             }
             //    out_image_data = calloc(target_bytes_per_row, image_height);
-            out_image_data = malloc(target_bytes_per_row * image_height);            
+            out_image_data = malloc(target_bytes_per_row * image_height);
             if(NULL == out_image_data)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, malloc failed" << std::endl;
                 CGColorSpaceRelease(color_space);
                 return NULL;
             }
@@ -708,18 +704,18 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
                                                                );
             if(vimage_error_flag != kvImageNoError)
             {
-                osg::notify(osg::WARN) << "In CreateCGImageFromOSGData, vImageAffineWarp_ARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
+                OSG_WARN << "In CreateCGImageFromOSGData, vImageAffineWarp_ARGB8888 failed with vImage Error Code: " << vimage_error_flag << std::endl;
                 free(out_image_data);
                 CGColorSpaceRelease(color_space);
                 return NULL;
-            }            
+            }
             break;
         }
         // FIXME: Handle other cases.
         // Use vImagePermuteChannels_ARGB8888 to swizzle bytes
         default:
         {
-            osg::notify(osg::WARN) << "In CreateCGImageFromOSGData: Sorry support for this format is not implemented." << std::endl;
+            OSG_WARN << "In CreateCGImageFromOSGData: Sorry support for this format is not implemented." << std::endl;
             return NULL;
             break;
         }
@@ -742,7 +738,7 @@ CGImageRef CreateCGImageFromOSGData(const osg::Image& osg_image)
         free(out_image_data);
         return NULL;
     }
-    
+
 
     /* Make an image out of our bitmap; does a cheap vm_copy of the bitmap */
     CGImageRef image_ref = CGBitmapContextCreateImage(bitmap_context);
@@ -779,23 +775,23 @@ CGImageDestinationRef CreateCGImageDestinationFromFile(const char* the_path,  co
     {
         return NULL;
     }
-    
+
     /* Create a CFURL from a CFString */
     the_url = CFURLCreateWithFileSystemPath(
-        NULL, 
+        NULL,
         cf_string,
         kCFURLPOSIXPathStyle,
         false
     );
-    
+
     /* Don't need the CFString any more (error or not) */
     CFRelease(cf_string);
-    
+
     if(!the_url)
     {
         return NULL;
     }
-    
+
     if(the_options)
     {
         std::istringstream iss(the_options->getOptionString());
@@ -803,14 +799,14 @@ CGImageDestinationRef CreateCGImageDestinationFromFile(const char* the_path,  co
         while (iss >> opt)
         {
             // Not handled: The user could do something stupid and specify both PNG and JPEG options.
-            
+
             if(opt=="PNG_COMPRESSION")
             {
                 found_png_option = true;
                 // I don't see an option to set PNG compression levels in the API so this info is unused.
                 int level;
                 iss >> level;
-                                
+
             }
             else if(opt=="JPEG_QUALITY")
             {
@@ -824,7 +820,7 @@ CGImageDestinationRef CreateCGImageDestinationFromFile(const char* the_path,  co
             }
         }
     }
-    
+
 
     CFStringRef path_extension = CFURLCopyPathExtension(the_url);
     if(NULL == path_extension)
@@ -856,15 +852,15 @@ CGImageDestinationRef CreateCGImageDestinationFromFile(const char* the_path,  co
         CFRelease(path_extension);
     }
 
-                                                      
+
     dest_ref =  CGImageDestinationCreateWithURL(
-        the_url, 
-        uti_type, 
+        the_url,
+        uti_type,
         1, // image file will contain only one image
         NULL
     );
 
-    
+
     CFRelease(uti_type);
     CFRelease(the_url);
 
@@ -876,20 +872,20 @@ CGImageDestinationRef CreateCGImageDestinationFromFile(const char* the_path,  co
         CFStringRef the_keys[1];
         CFNumberRef the_values[1];
         CFDictionaryRef the_dict;
-                
+
         the_keys[0] = kCGImageDestinationLossyCompressionQuality;
         the_values[0] = CFNumberCreate(
             NULL,
             kCFNumberFloat32Type,
             &compression_quality
         );
-        
+
         the_dict = CFDictionaryCreate(NULL, (const void**)&the_keys, (const void**)&the_values, 1, &kCFCopyStringDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
         CFRelease(the_values[0]);
 
         // Now that we have the dict, actually set the property.
         CGImageDestinationSetProperties(dest_ref, the_dict);
-        
+
         CFRelease(the_dict);
     }
 
@@ -906,13 +902,13 @@ CGImageDestinationRef CreateCGImageDestinationFromDataStream(std::ostream& fout,
     bool found_png_option = false;
     bool found_jpeg_option = false;
     float compression_quality = 1.0f;
-    
+
     CGDataConsumerCallbacks consumer_callbacks =
     {
         MyConsumerPutBytesCallback,
         MyConsumerReleaseInfoCallback
     };
-    
+
     CGDataConsumerRef data_consumer = CGDataConsumerCreate(&fout, &consumer_callbacks);
 
     if(the_options)
@@ -922,14 +918,14 @@ CGImageDestinationRef CreateCGImageDestinationFromDataStream(std::ostream& fout,
         while (iss >> opt)
         {
             // Not handled: The user could do something stupid and specify both PNG and JPEG options.
-            
+
             if(opt=="PNG_COMPRESSION")
             {
                 found_png_option = true;
                 // I don't see an option to set PNG compression levels in the API so this info is unused.
                 int level;
                 iss >> level;
-                
+
             }
             else if(opt=="JPEG_QUALITY")
             {
@@ -943,7 +939,7 @@ CGImageDestinationRef CreateCGImageDestinationFromDataStream(std::ostream& fout,
             }
         }
     }
-    
+
 
     if(found_jpeg_option)
     {
@@ -961,7 +957,7 @@ CGImageDestinationRef CreateCGImageDestinationFromDataStream(std::ostream& fout,
             kUTTypeImage // "public.image"
         );
     }
-    
+
 
     // If we had a way of hinting at what the data type is, we could
     // pass this hint in the second parameter.
@@ -971,11 +967,11 @@ CGImageDestinationRef CreateCGImageDestinationFromDataStream(std::ostream& fout,
         1, // image file will contain only one image
         NULL
     );
-    
+
     CGDataConsumerRelease(data_consumer);
     CFRelease(uti_type);
-    
-    
+
+
     // Not handled: The user could do something stupid and specify both PNG and JPEG options.
     if(found_jpeg_option)
     {
@@ -983,23 +979,23 @@ CGImageDestinationRef CreateCGImageDestinationFromDataStream(std::ostream& fout,
         CFStringRef the_keys[1];
         CFNumberRef the_values[1];
         CFDictionaryRef the_dict;
-        
+
         the_keys[0] = kCGImageDestinationLossyCompressionQuality;
         the_values[0] = CFNumberCreate(
                                        NULL,
                                        kCFNumberFloat32Type,
                                        &compression_quality
                                        );
-        
+
         the_dict = CFDictionaryCreate(NULL, (const void**)&the_keys, (const void**)&the_values, 1, &kCFCopyStringDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
         CFRelease(the_values[0]);
-        
+
         // Now that we have the dict, actually set the property.
         CGImageDestinationSetProperties(dest_ref, the_dict);
-        
+
         CFRelease(the_dict);
     }
-    
+
     return dest_ref;
 }
 /**************************************************************
@@ -1015,12 +1011,12 @@ public:
     ReaderWriterImageIO()
     {
 
-         supportsExtension("jpg",   "jpg image file"); 
+         supportsExtension("jpg",   "jpg image file");
          supportsExtension("jpeg",  "jpeg image file");
          supportsExtension("jpe",   "jpe image file");
          supportsExtension("jp2",   "jp2 image file");
-         supportsExtension("tiff",  "tiff image file"); 
-         supportsExtension("tif",   "tif image file");               
+         supportsExtension("tiff",  "tiff image file");
+         supportsExtension("tif",   "tif image file");
          supportsExtension("gif",   "gif image file");
          supportsExtension("png",   "png image file");
          supportsExtension("pict",  "pict image file");
@@ -1033,14 +1029,14 @@ public:
          supportsExtension("tga",   "tga image file");
          supportsExtension("targa", "targa image file");
          supportsExtension("psd",   "psd image file");
-         
+
          supportsExtension("pdf",   "pdf image file");
          supportsExtension("eps",   "eps image file");
          supportsExtension("epi",   "epi image file");
          supportsExtension("epsf",  "epsf image file");
          supportsExtension("epsi",  "epsi image file");
          supportsExtension("ps",    "postscript image file");
-         
+
          supportsExtension("dng",   "dng image file");
          supportsExtension("cr2",   "cr2 image file");
          supportsExtension("crw",   "crw image file");
@@ -1081,12 +1077,12 @@ public:
        // Apple's UTI guide:
        // http://developer.apple.com/documentation/Carbon/Conceptual/understanding_utis/utilist/chapter_4_section_1.html
       return
-         osgDB::equalCaseInsensitive(extension,"jpg") || 
+         osgDB::equalCaseInsensitive(extension,"jpg") ||
          osgDB::equalCaseInsensitive(extension,"jpeg") ||
          osgDB::equalCaseInsensitive(extension,"jpe") ||
          osgDB::equalCaseInsensitive(extension,"jp2") ||
-         osgDB::equalCaseInsensitive(extension,"tiff") || 
-         osgDB::equalCaseInsensitive(extension,"tif") ||               
+         osgDB::equalCaseInsensitive(extension,"tiff") ||
+         osgDB::equalCaseInsensitive(extension,"tif") ||
          osgDB::equalCaseInsensitive(extension,"gif") ||
          osgDB::equalCaseInsensitive(extension,"png") ||
          osgDB::equalCaseInsensitive(extension,"pict") ||
@@ -1099,14 +1095,14 @@ public:
          osgDB::equalCaseInsensitive(extension,"tga") ||
          osgDB::equalCaseInsensitive(extension,"targa") ||
          osgDB::equalCaseInsensitive(extension,"psd") ||
-         
+
          osgDB::equalCaseInsensitive(extension,"pdf") ||
          osgDB::equalCaseInsensitive(extension,"eps") ||
          osgDB::equalCaseInsensitive(extension,"epi") ||
          osgDB::equalCaseInsensitive(extension,"epsf") ||
          osgDB::equalCaseInsensitive(extension,"epsi") ||
          osgDB::equalCaseInsensitive(extension,"ps") ||
-         
+
          osgDB::equalCaseInsensitive(extension,"dng") ||
          osgDB::equalCaseInsensitive(extension,"cr2") ||
          osgDB::equalCaseInsensitive(extension,"crw") ||
@@ -1133,7 +1129,7 @@ public:
    }
 
 
-    
+
     ReadResult readImageStream(std::istream& fin) const
     {
         // Call ImageIO to load the image.
@@ -1142,7 +1138,7 @@ public:
 
         // Create an osg::Image from the CGImageRef.
         osg::Image* osg_image = CreateOSGImageFromCGImage(cg_image_ref);
-    
+
         CFRelease(cg_image_ref);
         return osg_image;
     }
@@ -1152,19 +1148,21 @@ public:
         ReadResult read_result = readImageStream(fin);
         return read_result;
     }
-    
+
     ReadResult readImageFile(const std::string& file_name) const
     {
-        osg::notify(osg::INFO) << "imageio readImageFile: " << file_name << std::endl;
-        
+        OSG_INFO << "imageio readImageFile: " << file_name << std::endl;
+
         // Call ImageIO to load the image.
         CGImageRef cg_image_ref = CreateCGImageFromFile(file_name.c_str());
         if (NULL == cg_image_ref) return ReadResult::FILE_NOT_FOUND;
 
         // Create an osg::Image from the CGImageRef.
         osg::Image* osg_image = CreateOSGImageFromCGImage(cg_image_ref);
-    
+
         CFRelease(cg_image_ref);
+        if (!osg_image)
+            return ReadResult::INSUFFICIENT_MEMORY_TO_LOAD;
 
         return osg_image;
     }
@@ -1180,9 +1178,9 @@ public:
 #if 1
         ReadResult read_result = readImageFile(full_file_name);
 #else
-        // Only here to help test istream backend. The file version is better because 
+        // Only here to help test istream backend. The file version is better because
         // the filenname.extension could potentially be used by ImageIO to hint what the format type is.
-        std::ifstream istream(full_file_name.c_str(), std::ios::in | std::ios::binary);
+        osgDB::ifstream istream(full_file_name.c_str(), std::ios::in | std::ios::binary);
         if(!istream) return ReadResult::FILE_NOT_HANDLED;
         ReadResult read_result = readImage(istream);
 #endif
@@ -1197,18 +1195,23 @@ public:
 
     WriteResult writeImageStream(const osg::Image& osg_image, std::ostream& fout, const osgDB::ReaderWriter::Options* the_options) const
     {
+        if (!osg_image.isDataContiguous())
+        {
+            return WriteResult::FILE_NOT_HANDLED;
+        }
+
         WriteResult ret_val = WriteResult::ERROR_IN_WRITING_FILE;
 
         CGImageDestinationRef cg_dest_ref = CreateCGImageDestinationFromDataStream(fout, the_options);
         if (NULL == cg_dest_ref) return WriteResult::ERROR_IN_WRITING_FILE;
-        
+
         CGImageRef cg_image_ref = CreateCGImageFromOSGData(osg_image);
         if(NULL == cg_image_ref)
         {
             CFRelease(cg_dest_ref);
             return WriteResult::ERROR_IN_WRITING_FILE;
         }
-        
+
         CGImageDestinationAddImage(cg_dest_ref, cg_image_ref, NULL);
         if(CGImageDestinationFinalize(cg_dest_ref))
         {
@@ -1218,13 +1221,13 @@ public:
         {
             ret_val = WriteResult::ERROR_IN_WRITING_FILE;
         }
-        
+
         CFRelease(cg_image_ref);
         CFRelease(cg_dest_ref);
-        
-        return WriteResult::FILE_SAVED;    
+
+        return WriteResult::FILE_SAVED;
     }
-        
+
     virtual WriteResult writeImage(const osg::Image& osg_image, std::ostream& fout, const osgDB::ReaderWriter::Options* the_options) const
     {
         WriteResult write_result = writeImageStream(osg_image, fout, the_options);
@@ -1233,7 +1236,13 @@ public:
 
     WriteResult writeImageFile(const osg::Image& osg_image, const std::string& full_file_name, const osgDB::ReaderWriter::Options* the_options) const
     {
+        if (!osg_image.isDataContiguous())
+        {
+            return WriteResult::FILE_NOT_HANDLED;
+        }
+
         WriteResult ret_val = WriteResult::ERROR_IN_WRITING_FILE;
+
         // Call ImageIO to load the image.
         CGImageDestinationRef cg_dest_ref = CreateCGImageDestinationFromFile(full_file_name.c_str(), the_options);
         if (NULL == cg_dest_ref) return WriteResult::ERROR_IN_WRITING_FILE;
@@ -1254,10 +1263,10 @@ public:
         {
             ret_val = WriteResult::ERROR_IN_WRITING_FILE;
         }
-        
+
         CFRelease(cg_image_ref);
         CFRelease(cg_dest_ref);
-        
+
         return WriteResult::FILE_SAVED;
     }
 
@@ -1266,27 +1275,38 @@ public:
         std::string ext = osgDB::getFileExtension(file_name);
         if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
+        if (!osg_image.isDataContiguous())
+        {
+            return WriteResult::FILE_NOT_HANDLED;
+        }
+
+        WriteResult ret_val = WriteResult::ERROR_IN_WRITING_FILE;
 #if 1
         // FIXME: Something may need to provide a proper writable location for the files.
         std::string full_file_name;
         full_file_name = file_name;
         return writeImageFile(osg_image, full_file_name, the_options);
 #else
-        // Only here to help test ostream backend. The file version is better because 
+        // Only here to help test ostream backend. The file version is better because
         // the filenname.extension could potentially be used by ImageIO to hint what the format type is.
-        std::ofstream fout(file_name.c_str(), std::ios::out | std::ios::binary);
+        osgDB::ofstream fout(file_name.c_str(), std::ios::out | std::ios::binary);
         if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
         return writeImage(osg_image, fout, the_options);
-#endif        
+#endif
+    }
+
+    virtual ReadResult readObject(std::istream& fin,const osgDB::ReaderWriter::Options* options =NULL) const
+    {
+        return readImage(fin, options);
+    }
+
+    virtual ReadResult readObject(const std::string& file, const osgDB::ReaderWriter::Options* options =NULL) const
+    {
+        return readImage(file, options);
     }
 
 };
 
 // now register with Registry to instantiate the above
 // reader/writer.
-
-osgDB::RegisterReaderWriterProxy<ReaderWriterImageIO> g_readerWriter_ImageIO_Proxy;
-
-
-
-
+REGISTER_OSGPLUGIN(imageio, ReaderWriterImageIO)
