@@ -647,6 +647,12 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
 
         WriteResult::WriteStatus write_JPEG_file (std::ostream &fout, const osg::Image &img, int quality = 100) const
         {
+            if (!img.isDataContiguous())
+            {
+                OSG_WARN<<"Warning: Writing of image data, that is non contiguous, is not supported by JPEG plugin."<<std::endl;
+                return WriteResult::ERROR_IN_WRITING_FILE;
+            }
+
             int image_width = img.s();
             int image_height = img.t();
             if ( (image_width == 0) || (image_height == 0) )
@@ -659,6 +665,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             int image_components = 3;
             // Only cater for gray, alpha and RGB for now
             switch(img.getPixelFormat()) {
+              case(GL_DEPTH_COMPONENT):
               case(GL_LUMINANCE):
               case(GL_ALPHA): {
                   image_color_space = JCS_GRAYSCALE;
@@ -673,7 +680,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
               default:
               {
                   OSG_DEBUG << "ReaderWriterJPEG::write_JPEG_file - Error pixel format non supported" << std::endl;
-                return WriteResult::ERROR_IN_WRITING_FILE; break;              
+                return WriteResult::ERROR_IN_WRITING_FILE; break;
               }
             }
 
@@ -723,7 +730,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             {
                 return WriteResult::ERROR_IN_WRITING_FILE;
             }*/
-    
+
             //jpeg_stdio_dest(&cinfo, outfile);
             osgDBJPEG::jpeg_stream_dest(&cinfo, &fout);
 
@@ -828,7 +835,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             int r = 1;
 
             //int internalFormat = numComponents_ret;
-            int internalFormat = 
+            int internalFormat =
                 numComponents_ret == 1 ? GL_LUMINANCE :
                 numComponents_ret == 2 ? GL_LUMINANCE_ALPHA :
                 numComponents_ret == 3 ? GL_RGB :
