@@ -8,11 +8,11 @@
  * Modified by Robert Osfield to support per Drawable coord, normal and
  * texture coord arrays, bug fixes, and support for texture mapping.
  *
- * Writing support added 2007 by Stephan Huber, http://digitalmind.de, 
+ * Writing support added 2007 by Stephan Huber, http://digitalmind.de,
  * some ideas taken from the dae-plugin
  *
- * The Open Scene Graph (OSG) is a cross platform C++/OpenGL library for 
- * real-time rendering of large 3D photo-realistic models. 
+ * The Open Scene Graph (OSG) is a cross platform C++/OpenGL library for
+ * real-time rendering of large 3D photo-realistic models.
  * The OSG homepage is http://www.openscenegraph.org/
  */
 
@@ -27,6 +27,7 @@
 #include <osg/Node>
 #include <osg/MatrixTransform>
 #include <osg/Geode>
+#include <osg/Vec3f>
 
 #include <osg/Geometry>
 #include <osg/StateSet>
@@ -59,6 +60,7 @@ public:
         supportsOption("noRotation","Do not do the default rotate about X axis");
         supportsOption("noTesselateLargePolygons","Do not do the default tesselation of large polygons");
         supportsOption("noTriStripPolygons","Do not do the default tri stripping of polygons");
+        supportsOption("generateFacetNormals","generate facet normals for verticies without normals");
 
         supportsOption("DIFFUSE=<unit>", "Set texture unit for diffuse texture");
         supportsOption("AMBIENT=<unit>", "Set texture unit for ambient texture");
@@ -76,54 +78,54 @@ public:
     virtual ReadResult readNode(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const;
 
     virtual ReadResult readNode(std::istream& fin, const Options* options) const;
-    
-    virtual WriteResult writeObject(const osg::Object& obj,const std::string& fileName,const Options* options=NULL) const 
+
+    virtual WriteResult writeObject(const osg::Object& obj,const std::string& fileName,const Options* options=NULL) const
     {
         const osg::Node* node = dynamic_cast<const osg::Node*>(&obj);
         if (node)
             return writeNode(*node, fileName, options);
-        else 
-            return WriteResult(WriteResult::FILE_NOT_HANDLED); 
+        else
+            return WriteResult(WriteResult::FILE_NOT_HANDLED);
     }
-    
-    virtual WriteResult writeNode(const osg::Node& node,const std::string& fileName,const Options* options =NULL) const 
-    { 
+
+    virtual WriteResult writeNode(const osg::Node& node,const std::string& fileName,const Options* options =NULL) const
+    {
         if (!acceptsExtension(osgDB::getFileExtension(fileName)))
-            return WriteResult(WriteResult::FILE_NOT_HANDLED); 
-            
+            return WriteResult(WriteResult::FILE_NOT_HANDLED);
+
         osgDB::ofstream f(fileName.c_str());
         std::string materialFile = osgDB::getNameLessExtension(fileName) + ".mtl";
         OBJWriterNodeVisitor nv(f, osgDB::getSimpleFileName(materialFile));
-        
+
         // we must cast away constness
         (const_cast<osg::Node*>(&node))->accept(nv);
-        
+
         osgDB::ofstream mf(materialFile.c_str());
         nv.writeMaterials(mf);
-         
-        return WriteResult(WriteResult::FILE_SAVED); 
+
+        return WriteResult(WriteResult::FILE_SAVED);
     }
 
-    
-    virtual WriteResult writeObject(const osg::Object& obj,std::ostream& fout,const Options* options=NULL) const 
+
+    virtual WriteResult writeObject(const osg::Object& obj,std::ostream& fout,const Options* options=NULL) const
     {
         const osg::Node* node = dynamic_cast<const osg::Node*>(&obj);
         if (node)
             return writeNode(*node, fout, options);
-        else 
-            return WriteResult(WriteResult::FILE_NOT_HANDLED); 
+        else
+            return WriteResult(WriteResult::FILE_NOT_HANDLED);
     }
 
-    virtual WriteResult writeNode(const osg::Node& node,std::ostream& fout,const Options* =NULL) const 
-    { 
+    virtual WriteResult writeNode(const osg::Node& node,std::ostream& fout,const Options* =NULL) const
+    {
         // writing to a stream does not support materials
-        
+
         OBJWriterNodeVisitor nv(fout);
-        
+
         // we must cast away constness
         (const_cast<osg::Node*>(&node))->accept(nv);
-    
-        return WriteResult(WriteResult::FILE_SAVED); 
+
+        return WriteResult(WriteResult::FILE_SAVED);
     }
 
 
@@ -134,6 +136,7 @@ protected:
         bool rotate;
         bool noTesselateLargePolygons;
         bool noTriStripPolygons;
+        bool generateFacetNormals;
         bool fixBlackMaterials;
         // This is the order in which the materials will be assigned to texture maps, unless
         // otherwise overriden
@@ -142,16 +145,16 @@ protected:
     };
 
     typedef std::map< std::string, osg::ref_ptr<osg::StateSet> > MaterialToStateSetMap;
-    
-    void buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToSetSetMapObj, ObjOptionsStruct& localOptions) const;
-    
-    osg::Geometry* convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, bool& rotate) const;
-    
-    osg::Node* convertModelToSceneGraph(obj::Model& model, ObjOptionsStruct& localOptions) const;
+
+    void buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToSetSetMapObj, ObjOptionsStruct& localOptions, const Options* options) const;
+
+    osg::Geometry* convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, ObjOptionsStruct& localOptions) const;
+
+    osg::Node* convertModelToSceneGraph(obj::Model& model, ObjOptionsStruct& localOptions, const Options* options) const;
 
     inline osg::Vec3 transformVertex(const osg::Vec3& vec, const bool rotate) const ;
     inline osg::Vec3 transformNormal(const osg::Vec3& vec, const bool rotate) const ;
- 
+
     ObjOptionsStruct parseOptions(const Options* options) const;
 
 
@@ -188,22 +191,23 @@ REGISTER_OSGPLUGIN(obj, ReaderWriterOBJ)
 static void load_material_texture(    obj::Model &model,
                                     obj::Material::Map &map,
                                     osg::StateSet *stateset,
-                                    const unsigned int texture_unit  )
+                                    const unsigned int texture_unit,
+                                    const osgDB::Options* options)
 {
     std::string filename = map.name;
     if (!filename.empty())
     {
         osg::ref_ptr< osg::Image > image;
-        if ( !model.getDatabasePath().empty() ) 
+        if ( !model.getDatabasePath().empty() )
         {
-            // first try with database path of parent. 
-            image = osgDB::readRefImageFile(model.getDatabasePath()+'/'+filename);
+            // first try with database path of parent.
+            image = osgDB::readRefImageFile(model.getDatabasePath()+'/'+filename, options);
         }
-        
+
         if ( !image.valid() )
         {
             // if not already set then try the filename as is.
-            image = osgDB::readRefImageFile(filename);
+            image = osgDB::readRefImageFile(filename, options);
         }
 
         if ( image.valid() )
@@ -226,17 +230,17 @@ static void load_material_texture(    obj::Model &model,
             texture->setWrap(osg::Texture2D::WRAP_S, textureWrapMode);
             texture->setWrap(osg::Texture2D::WRAP_T, textureWrapMode);
             stateset->setTextureAttributeAndModes( texture_unit, texture,osg::StateAttribute::ON );
-            
+
             if ( map.type == obj::Material::Map::REFLECTION )
             {
                 osg::TexGen* texgen = new osg::TexGen;
                 texgen->setMode(osg::TexGen::SPHERE_MAP);
                 stateset->setTextureAttributeAndModes( texture_unit,texgen,osg::StateAttribute::ON );
             }
-            
+
             if  ( image->isImageTranslucent())
             {
-                osg::notify(osg::INFO)<<"Found transparent image"<<std::endl;
+                OSG_INFO<<"Found transparent image"<<std::endl;
                 stateset->setMode(GL_BLEND, osg::StateAttribute::ON);
                 stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
             }
@@ -249,12 +253,12 @@ static void load_material_texture(    obj::Model &model,
         osg::Matrix mat;
         if (map.uScale != 1.0f || map.vScale != 1.0f)
         {
-            osg::notify(osg::DEBUG_INFO) << "Obj TexMat scale=" << map.uScale << "," << map.vScale << std::endl;
+            OSG_DEBUG << "Obj TexMat scale=" << map.uScale << "," << map.vScale << std::endl;
             mat *= osg::Matrix::scale(map.uScale, map.vScale, 1.0);
         }
         if (map.uOffset != 0.0f || map.vOffset != 0.0f)
         {
-            osg::notify(osg::DEBUG_INFO) << "Obj TexMat offset=" << map.uOffset << "," << map.uOffset << std::endl;
+            OSG_DEBUG << "Obj TexMat offset=" << map.uOffset << "," << map.uOffset << std::endl;
             mat *= osg::Matrix::translate(map.uOffset, map.vOffset, 0.0);
         }
 
@@ -265,7 +269,7 @@ static void load_material_texture(    obj::Model &model,
 }
 
 
-void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToStateSetMap, ObjOptionsStruct& localOptions) const
+void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToStateSetMap, ObjOptionsStruct& localOptions, const Options* options) const
 {
     if (localOptions.fixBlackMaterials)
     {
@@ -288,7 +292,7 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
                 ++numNotBlack;
             }
         }
-        
+
         if (numNotBlack==0 && numBlack!=0)
         {
             for(itr = model.materialMap.begin();
@@ -305,13 +309,13 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
             }
         }
     }
-    
+
     for(obj::Model::MaterialMap::iterator itr = model.materialMap.begin();
         itr != model.materialMap.end();
         ++itr)
     {
         obj::Material& material = itr->second;
-       
+
         osg::ref_ptr< osg::StateSet > stateset = new osg::StateSet;
 
         bool isTransparent = false;
@@ -322,7 +326,7 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
         {
             osg::Material* osg_material = new osg::Material;
             stateset->setAttribute(osg_material);
-
+            osg_material->setName(material.name);
             osg_material->setAmbient(osg::Material::FRONT_AND_BACK,material.ambient);
             osg_material->setDiffuse(osg::Material::FRONT_AND_BACK,material.diffuse);
             osg_material->setEmission(osg::Material::FRONT_AND_BACK,material.emissive);
@@ -333,18 +337,18 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
                 osg_material->setSpecular(osg::Material::FRONT_AND_BACK, osg::Vec4(0,0,0,1));
             }
             osg_material->setShininess(osg::Material::FRONT_AND_BACK,(material.Ns/1000.0f)*128.0f ); // note OBJ shiniess is 0..1000.
-            
+
             if (material.ambient[3]!=1.0 ||
                 material.diffuse[3]!=1.0 ||
                 material.specular[3]!=1.0||
                 material.emissive[3]!=1.0)
             {
-                osg::notify(osg::INFO)<<"Found transparent material"<<std::endl;
+                OSG_INFO<<"Found transparent material"<<std::endl;
                 isTransparent = true;
             }
         }
-        
-        // If the user has explicitly set the required texture type to unit map via the options 
+
+        // If the user has explicitly set the required texture type to unit map via the options
         // string, then we load ONLY those textures that are in the map.
         if(localOptions.textureUnitAllocation.size()>0)
         {
@@ -363,7 +367,7 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
                         break;
                     }
                 }
-                if(index>=0) load_material_texture( model, material.maps[index], stateset.get(), unit );
+                if(index>=0) load_material_texture( model, material.maps[index], stateset.get(), unit, options );
             }
         }
         // If the user has set no options, then we load them up in the order contained in the enum. This
@@ -386,12 +390,12 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
                 }
                 if(index>=0)
                 {
-                    load_material_texture( model, material.maps[index], stateset.get(), unit );
+                    load_material_texture( model, material.maps[index], stateset.get(), unit, options );
                     unit++;
                 }
             }
         }
-      
+
         if (isTransparent)
         {
             stateset->setMode(GL_BLEND, osg::StateAttribute::ON);
@@ -402,18 +406,53 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
     }
 }
 
-osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, bool& rotate) const
+osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, ObjOptionsStruct& localOptions) const
 {
-    
+
     unsigned int numVertexIndices = 0;
     unsigned int numNormalIndices = 0;
     unsigned int numTexCoordIndices = 0;
-    
+
     unsigned int numPointElements = 0;
     unsigned int numPolylineElements = 0;
     unsigned int numPolygonElements = 0;
 
     obj::Model::ElementList::iterator itr;
+
+    if (localOptions.generateFacetNormals == true) {
+        for(itr=elementList.begin();
+                itr!=elementList.end();
+                    ++itr)
+        {
+            obj::Element& element = *(*itr);
+            if (element.dataType==obj::Element::POINTS || element.dataType==obj::Element::POLYLINE)
+                continue;
+
+            if (element.normalIndices.size() == 0) {
+                // fill in the normals
+                int a = element.vertexIndices[0];
+                int b = element.vertexIndices[1];
+                int c = element.vertexIndices[2];
+
+                osg::Vec3f ab(model.vertices[b]);
+                osg::Vec3f ac(model.vertices[c]);
+
+                ab -= model.vertices[a];
+                ac -= model.vertices[a];
+
+                osg::Vec3f Norm( ab ^ ac );
+                Norm.normalize();
+                int normal_idx = model.normals.size();
+                model.normals.push_back(Norm);
+
+                for (unsigned i=0 ; i < element.vertexIndices.size() ; i++)
+                    element.normalIndices.push_back(normal_idx);
+            }
+        }
+    }
+
+
+
     for(itr=elementList.begin();
         itr!=elementList.end();
         ++itr)
@@ -431,39 +470,38 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
     }
 
     if (numVertexIndices==0) return 0;
-    
+
     if (numNormalIndices!=0 && numNormalIndices!=numVertexIndices)
     {
-        osg::notify(osg::NOTICE)<<"Incorrect number of normals, ignore them"<<std::endl;
+        OSG_NOTICE<<"Incorrect number of normals, ignore them"<<std::endl;
         numNormalIndices = 0;
     }
-    
+
     if (numTexCoordIndices!=0 && numTexCoordIndices!=numVertexIndices)
     {
-        osg::notify(osg::NOTICE)<<"Incorrect number of normals, ignore them"<<std::endl;
+        OSG_NOTICE<<"Incorrect number of normals, ignore them"<<std::endl;
         numTexCoordIndices = 0;
     }
-    
+
     osg::Vec3Array* vertices = numVertexIndices ? new osg::Vec3Array : 0;
     osg::Vec3Array* normals = numNormalIndices ? new osg::Vec3Array : 0;
     osg::Vec2Array* texcoords = numTexCoordIndices ? new osg::Vec2Array : 0;
-    
+
     if (vertices) vertices->reserve(numVertexIndices);
     if (normals) normals->reserve(numNormalIndices);
     if (texcoords) texcoords->reserve(numTexCoordIndices);
-    
+
     osg::Geometry* geometry = new osg::Geometry;
     if (vertices) geometry->setVertexArray(vertices);
     if (normals)
     {
-        geometry->setNormalArray(normals);
-        geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
+        geometry->setNormalArray(normals, osg::Array::BIND_PER_VERTEX);
     }
     if (texcoords)
     {
         geometry->setTexCoordArray(0,texcoords);
     }
-    
+
 
     if (numPointElements>0)
     {
@@ -480,7 +518,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                     index_itr != element.vertexIndices.end();
                     ++index_itr)
                 {
-                    vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
+                    vertices->push_back(transformVertex(model.vertices[*index_itr],localOptions.rotate));
                     ++numPoints;
                 }
                 if (numNormalIndices)
@@ -489,7 +527,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.normalIndices.end();
                         ++index_itr)
                     {
-                        normals->push_back(transformNormal(model.normals[*index_itr],rotate));
+                        normals->push_back(transformNormal(model.normals[*index_itr],localOptions.rotate));
                     }
                 }
                 if (numTexCoordIndices)
@@ -507,7 +545,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
         osg::DrawArrays* drawArrays = new osg::DrawArrays(GL_POINTS,startPos,numPoints);
         geometry->addPrimitiveSet(drawArrays);
     }
-    
+
     if (numPolylineElements>0)
     {
         unsigned int startPos = vertices->size();
@@ -526,7 +564,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                     index_itr != element.vertexIndices.end();
                     ++index_itr)
                 {
-                    vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
+                    vertices->push_back(transformVertex(model.vertices[*index_itr],localOptions.rotate));
                 }
                 if (numNormalIndices)
                 {
@@ -534,7 +572,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.normalIndices.end();
                         ++index_itr)
                     {
-                        normals->push_back(transformNormal(model.normals[*index_itr],rotate));
+                        normals->push_back(transformNormal(model.normals[*index_itr],localOptions.rotate));
                     }
                 }
                 if (numTexCoordIndices)
@@ -558,7 +596,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
     if (numPolygonElements>0)
     {
         unsigned int startPos = vertices->size();
-        
+
         #ifdef USE_DRAWARRAYLENGTHS
             osg::DrawArrayLengths* drawArrayLengths = new osg::DrawArrayLengths(GL_POLYGON,startPos);
             geometry->addPrimitiveSet(drawArrayLengths);
@@ -572,6 +610,11 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
             if (element.dataType==obj::Element::POLYGON)
             {
 
+
+
+
+
+
                 #ifdef USE_DRAWARRAYLENGTHS
                     drawArrayLengths->push_back(element.vertexIndices.size());
                 #else
@@ -589,7 +632,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                     }
                 #endif
 
-            
+
                 if (model.needReverse(element))
                 {
                     // need to reverse so add to OSG arrays in same order as in OBJ, as OSG assume anticlockwise ordering.
@@ -597,7 +640,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.vertexIndices.rend();
                         ++index_itr)
                     {
-                        vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
+                        vertices->push_back(transformVertex(model.vertices[*index_itr],localOptions.rotate));
                     }
                     if (numNormalIndices)
                     {
@@ -605,9 +648,11 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                             index_itr != element.normalIndices.rend();
                             ++index_itr)
                         {
-                            normals->push_back(transformNormal(model.normals[*index_itr],rotate));
+                            normals->push_back(transformNormal(model.normals[*index_itr],localOptions.rotate));
                         }
                     }
+
+
                     if (numTexCoordIndices)
                     {
                         for(obj::Element::IndexList::reverse_iterator index_itr = element.texCoordIndices.rbegin();
@@ -625,7 +670,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.vertexIndices.end();
                         ++index_itr)
                     {
-                        vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
+                        vertices->push_back(transformVertex(model.vertices[*index_itr],localOptions.rotate));
                     }
                     if (numNormalIndices)
                     {
@@ -633,7 +678,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                             index_itr != element.normalIndices.end();
                             ++index_itr)
                         {
-                            normals->push_back(transformNormal(model.normals[*index_itr],rotate));
+                            normals->push_back(transformNormal(model.normals[*index_itr],localOptions.rotate));
                         }
                     }
                     if (numTexCoordIndices)
@@ -651,11 +696,11 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
 
 
     }
-    
+
     return geometry;
 }
 
-osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptionsStruct& localOptions) const
+osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptionsStruct& localOptions, const Options* options) const
 {
 
     if (model.elementStateMap.empty()) return 0;
@@ -663,42 +708,47 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptio
     osg::Group* group = new osg::Group;
 
     // set up the materials
-    MaterialToStateSetMap materialToSetSetMap;
-    buildMaterialToStateSetMap(model, materialToSetSetMap, localOptions);
+    MaterialToStateSetMap materialToStateSetMap;
+    buildMaterialToStateSetMap(model, materialToStateSetMap, localOptions, options);
 
     // go through the groups of related elements and build geometry from them.
     for(obj::Model::ElementStateMap::iterator itr=model.elementStateMap.begin();
         itr!=model.elementStateMap.end();
         ++itr)
     {
-    
+
         const obj::ElementState& es = itr->first;
         obj::Model::ElementList& el = itr->second;
 
-        osg::Geometry* geometry = convertElementListToGeometry(model,el,localOptions.rotate);
+        osg::Geometry* geometry = convertElementListToGeometry(model,el,localOptions);
 
         if (geometry)
         {
+            MaterialToStateSetMap::const_iterator it = materialToStateSetMap.find(es.materialName);
+            if (it == materialToStateSetMap.end())
+            {
+                OSG_WARN << "Obj unable to find material '" << es.materialName << "'" << std::endl;
+            }
 
-            osg::StateSet* stateset = materialToSetSetMap[es.materialName].get();
+            osg::StateSet* stateset = materialToStateSetMap[es.materialName].get();
             geometry->setStateSet(stateset);
-        
+
             // tesseleate any large polygons
             if (!localOptions.noTesselateLargePolygons)
             {
                 osgUtil::Tessellator tessellator;
                 tessellator.retessellatePolygons(*geometry);
             }
-            
+
             // tri strip polygons to improve graphics peformance
             if (!localOptions.noTriStripPolygons)
             {
                 osgUtil::TriStripVisitor tsv;
                 tsv.stripify(*geometry);
             }
-            
+
             // if no normals present add them.
-            if (!geometry->getNormalArray() || geometry->getNormalArray()->getNumElements()==0)
+            if (localOptions.generateFacetNormals==false && (!geometry->getNormalArray() || geometry->getNormalArray()->getNumElements()==0))
             {
                 osgUtil::SmoothingVisitor sv;
                 sv.smooth(*geometry);
@@ -707,7 +757,7 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptio
 
             osg::Geode* geode = new osg::Geode;
             geode->addDrawable(geometry);
-            
+
             if (es.objectName.empty())
             {
                 geode->setName(es.groupName);
@@ -735,6 +785,7 @@ ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::Rea
     localOptions.rotate = true;
     localOptions.noTesselateLargePolygons = false;
     localOptions.noTriStripPolygons = false;
+    localOptions.generateFacetNormals = false;
     localOptions.fixBlackMaterials = true;
 
     if (options!=NULL)
@@ -752,7 +803,7 @@ ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::Rea
             {
                 pre_equals = opt.substr(0,found);
                 post_equals = opt.substr(found+1);
-            } 
+            }
             else
             {
                 pre_equals = opt;
@@ -761,18 +812,22 @@ ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::Rea
             if (pre_equals == "noRotation")
             {
                 localOptions.rotate = false;
-            }                
+            }
             else if (pre_equals == "noTesselateLargePolygons")
             {
                 localOptions.noTesselateLargePolygons = true;
-            }                
+            }
             else if (pre_equals == "noTriStripPolygons")
             {
                 localOptions.noTriStripPolygons = true;
             }
+            else if (pre_equals == "generateFacetNormals")
+            {
+                localOptions.generateFacetNormals = true;
+            }
             else if (post_equals.length()>0)
-            {    
-                obj::Material::Map::TextureMapType type = obj::Material::Map::UNKNOWN;                        
+            {
+                obj::Material::Map::TextureMapType type = obj::Material::Map::UNKNOWN;
                 // Now we check to see if we have anything forcing a texture allocation
                 if        (pre_equals == "DIFFUSE")            type = obj::Material::Map::DIFFUSE;
                 else if (pre_equals == "AMBIENT")            type = obj::Material::Map::AMBIENT;
@@ -782,12 +837,12 @@ ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::Rea
                 else if (pre_equals == "BUMP")                type = obj::Material::Map::BUMP;
                 else if (pre_equals == "DISPLACEMENT")        type = obj::Material::Map::DISPLACEMENT;
                 else if (pre_equals == "REFLECTION")        type = obj::Material::Map::REFLECTION;
-                
+
                 if (type!=obj::Material::Map::UNKNOWN)
                 {
                     int unit = atoi(post_equals.c_str());    // (probably should use istringstream rather than atoi)
                     localOptions.textureUnitAllocation.push_back(std::make_pair(unit,(obj::Material::Map::TextureMapType) type));
-                    osg::notify(osg::NOTICE)<<"Obj Found map in options, ["<<pre_equals<<"]="<<unit<<std::endl;
+                    OSG_NOTICE<<"Obj Found map in options, ["<<pre_equals<<"]="<<unit<<std::endl;
                 }
             }
         }
@@ -804,27 +859,26 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(const std::string& fil
 
     std::string fileName = osgDB::findDataFile( file, options );
     if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
-    
-    
+
+
     osgDB::ifstream fin(fileName.c_str());
     if (fin)
     {
-    
-        // code for setting up the database path so that internally referenced file are searched for on relative paths. 
+
+        // code for setting up the database path so that internally referenced file are searched for on relative paths.
         osg::ref_ptr<Options> local_opt = options ? static_cast<Options*>(options->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
-        local_opt->setDatabasePath(osgDB::getFilePath(fileName));
+        local_opt->getDatabasePathList().push_front(osgDB::getFilePath(fileName));
 
         obj::Model model;
         model.setDatabasePath(osgDB::getFilePath(fileName.c_str()));
         model.readOBJ(fin, local_opt.get());
-        
+
         ObjOptionsStruct localOptions = parseOptions(options);
-        
-        
-        osg::Node* node = convertModelToSceneGraph(model,localOptions);
+
+        osg::Node* node = convertModelToSceneGraph(model, localOptions, local_opt.get());
         return node;
     }
-    
+
     return ReadResult::FILE_NOT_HANDLED;
 }
 
@@ -832,14 +886,16 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(std::istream& fin, con
 {
     if (fin)
     {
+        fin.imbue(std::locale::classic());
+
         obj::Model model;
         model.readOBJ(fin, options);
-        
+
         ObjOptionsStruct localOptions = parseOptions(options);
-        
-        osg::Node* node = convertModelToSceneGraph(model, localOptions);
+
+        osg::Node* node = convertModelToSceneGraph(model, localOptions, options);
         return node;
     }
-    
+
     return ReadResult::FILE_NOT_HANDLED;
 }
