@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2004 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2004 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -16,6 +16,7 @@
 #include <fstream>
 #include <string>
 #include <stdio.h>
+#include <functional>
 
 #include "obj.h"
 
@@ -31,7 +32,8 @@ using namespace obj;
 static std::string strip( const std::string& ss )
 {
     std::string result;
-    result.assign( ss.begin() + ss.find_first_not_of( ' ' ), ss.begin() + 1 + ss.find_last_not_of( ' ' ) );
+    result.assign( std::find_if( ss.begin(), ss.end(), std::not1( std::ptr_fun< int, int >( isspace ) ) ),
+                   std::find_if( ss.rbegin(), ss.rend(), std::not1( std::ptr_fun< int, int >( isspace ) ) ).base() );
     return( result );
 }
 
@@ -47,7 +49,7 @@ static Material::Map parseTextureMap( const std::string& ss, Material::Map::Text
     {
         if (s[0] != '-')
             break;
- 
+
         int n;
         if (s[1] == 's' || s[1] == 'o')
         {
@@ -107,7 +109,7 @@ static Material::Map parseTextureMap( const std::string& ss, Material::Map::Text
         s = strip(s.substr(n));
     }
 
-    map.name = s;
+    map.name = osgDB::convertFileNameToNativeStyle(s);
     map.type = type;
     return map;
 }
@@ -136,7 +138,7 @@ bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
                 // OSG_NOTICE<<"We have dos line ending"<<std::endl;
                 if (skipNewline)
                 {
-                    skipNewline = false; 
+                    skipNewline = false;
                     *ptr++ = ' ';
                     continue;
                 }
@@ -146,7 +148,7 @@ bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
             // OSG_NOTICE<<"We have mac line ending"<<std::endl;
             if (skipNewline)
             {
-                skipNewline = false; 
+                skipNewline = false;
                 *ptr++ = ' ';
                 continue;
             }
@@ -166,7 +168,7 @@ bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
         else if (c=='\\' && (p=='\r' || p=='\n'))
         {
             // need to keep return;
-            skipNewline = true;   
+            skipNewline = true;
         }
         else if (c!=std::ifstream::traits_type::eof()) // don't copy eof.
         {
@@ -178,10 +180,10 @@ bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
                 *ptr++ = c;
             }
         }
-        
-        
+
+
     }
-    
+
     // strip trailing spaces
     while (ptr>line && *(ptr-1)==' ')
     {
@@ -189,16 +191,16 @@ bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
     }
 
     *ptr = 0;
-    
+
     if (changeTabsToSpaces)
     {
-        
+
         for(ptr = line; *ptr != 0; ++ptr)
         {
             if (*ptr == '\t') *ptr=' ';
         }
     }
-    
+
     return true;
 }
 
@@ -220,6 +222,7 @@ bool Model::readMTL(std::istream& fin)
     const int LINE_SIZE = 4096;
     char line[LINE_SIZE];
     float r = 1.0f, g = 1.0f, b = 1.0f, a = 1.0f;
+    bool usingDissolve = false;
 
     Material* material = 0;// &(materialMap[""]);
     std::string filename;
@@ -236,9 +239,11 @@ bool Model::readMTL(std::istream& fin)
         {
             if (strncmp(line,"newmtl ",7)==0)
             {
-                std::string materialName(line+7);
+                // get material name and left- and right-trim all the white-space
+                std::string materialName(strip(line+7));
                 material = & materialMap[materialName];
                 material->name = materialName;
+                usingDissolve = false;
             }
             else if (material)
             {
@@ -405,19 +410,52 @@ bool Model::readMTL(std::istream& fin)
 
                     if (fieldsRead==1) material->Ni = Ni;
                 }
+                //
+                // Tr - transparency
+                //
+                // Seems that the world did not agreed about the specification of the item.
+                //
+                // Some thinks that value of 1 means opaque material and 0 transparent material,
+                // such as http://people.sc.fsu.edu/~jburkardt/data/mtl/mtl.html .
+                //
+                // However, 3ds Max export uses the opposite: 0 means opaque material and
+                // 1 completely transparent material. These 3ds Max exported files
+                // carry the following signature as the first line in the file (*.obj, *.mtl):
+                // # 3ds Max Wavefront OBJ Exporter v0.97b - (c)2007 guruware
+                //
+                // Moreover, at least one model uses Tr followed by two numbers.
+                // Such model can be downloaded from http://graphics.cs.williams.edu/data/meshes/cube.zip
+                // (part of the following collection: http://graphics.cs.williams.edu/data/meshes.xml).
+                //
+                // Current solution: As we do not know what is the correct interpretation of
+                // the value 0 and value 1 for Tr, we will rely on d (dissolve) parameter instead
+                // whenever it is present. This seems to fix the problem on large number of models.
+                //
                 else if (strncmp(line,"Tr ",3)==0)
                 {
-                    float alpha=1.0f;
-                    unsigned int fieldsRead = sscanf(line+3,"%f", &alpha);
-
-                    if (fieldsRead==1)
+                    if( !usingDissolve )
                     {
-                        material->ambient[3] = alpha;
-                        material->diffuse[3] = alpha;
-                        material->specular[3] = alpha;
-                        material->emissive[3] = alpha;
+                        float alpha=1.0f;
+                        unsigned int fieldsRead = sscanf(line+3,"%f", &alpha);
+
+                        if (fieldsRead==1)
+                        {
+                            material->ambient[3] = alpha;
+                            material->diffuse[3] = alpha;
+                            material->specular[3] = alpha;
+                            material->emissive[3] = alpha;
+                        }
                     }
                 }
+                //
+                // d - dissolve (pseudo-transparency)
+                //
+                // Dissolve of value 1 means completely opaque material
+                // and value of 0 results in completely transparent material.
+                //
+                // To be compatible with 3D Max obj exporter,
+                // d takes precedence over Tr (handled through usingDissolve variable).
+                //
                 else if (strncmp(line,"d ",2)==0)
                 {
                     float alpha=1.0f;
@@ -429,6 +467,7 @@ bool Model::readMTL(std::istream& fin)
                         material->diffuse[3] = alpha;
                         material->specular[3] = alpha;
                         material->emissive[3] = alpha;
+                        usingDissolve = true;
                     }
                 }
                 else if (strncmp(line,"map_Ka ",7)==0)
@@ -447,7 +486,7 @@ bool Model::readMTL(std::istream& fin)
                 }
                 // map_opacity doesn't exist in the spec, but was already in the plugin
                 // so leave it or plugin will break for some users
-                else if (strncmp(line,"map_opacity ",12)==0) 
+                else if (strncmp(line,"map_opacity ",12)==0)
                 {
                     material->maps.push_back(parseTextureMap(strip(line+12),Material::Map::OPACITY));
                 }
@@ -495,7 +534,7 @@ bool Model::readMTL(std::istream& fin)
             {
                 OSG_NOTICE <<"*** line not handled *** :"<<line<<std::endl;
             }
-        
+
         }
 
     }
@@ -562,7 +601,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
                      strncmp(line,"f ",2)==0)
             {
                 char* ptr = line+2;
-                
+
                 Element* element = new Element( (line[0]=='p') ? Element::POINTS :
                                                 (line[0]=='l') ? Element::POLYLINE :
                                                 Element::POLYGON );
@@ -574,7 +613,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
                 {
                     // skip white space
                     while(*ptr==' ') ++ptr;
-                    
+
                     if (sscanf(ptr, "%d/%d/%d", &vi, &ti, &ni) == 3)
                     {
                         // OSG_NOTICE<<"   vi="<<vi<<"/ti="<<ti<<"/ni="<<ni<<std::endl;
@@ -602,19 +641,19 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
 
                     // skip to white space or end of line
                     while(*ptr!=' ' && *ptr!=0) ++ptr;
-                
+
                 }
-                
+
                 if (!element->normalIndices.empty() && element->normalIndices.size() != element->vertexIndices.size())
                 {
                     element->normalIndices.clear();
                 }
-                
+
                 if (!element->texCoordIndices.empty() && element->texCoordIndices.size() != element->vertexIndices.size())
                 {
                     element->texCoordIndices.clear();
                 }
-                
+
                 if (!element->vertexIndices.empty())
                 {
                     Element::CoordinateCombination coordateCombination = element->getCoordinateCombination();
@@ -704,7 +743,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
                 int smoothingGroup=0;
                 if (strncmp(line+2,"off",3)==0) smoothingGroup = 0;
                 else sscanf(line+2,"%d",&smoothingGroup);
-                
+
                 if (currentElementState.smoothingGroup != smoothingGroup)
                 {
                     currentElementState.smoothingGroup = smoothingGroup;
@@ -715,7 +754,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
             {
                 OSG_NOTICE <<"*** line not handled *** :"<<line<<std::endl;
             }
-        
+
         }
 
     }
@@ -725,7 +764,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
     OSG_NOTICE <<"texcoords :"<<texcoords.size()<<std::endl;
     OSG_NOTICE <<"materials :"<<materialMap.size()<<std::endl;
     OSG_NOTICE <<"elementStates :"<<elementStateMap.size()<<std::endl;
-    
+
     unsigned int pos=0;
     for(ElementStateMap::iterator itr=elementStateMap.begin();
         itr!=elementStateMap.end();
@@ -739,7 +778,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
         OSG_NOTICE<<"    es.materialName="<<es.materialName<<std::endl;
         OSG_NOTICE<<"    es.smoothGroup="<<es.smoothingGroup<<std::endl;
         OSG_NOTICE<<"    ElementList ="<<el.size()<<std::endl;
-        
+
     }
 #endif
     return true;
@@ -753,7 +792,7 @@ void Model::addElement(Element* element)
         currentElementList = & (elementStateMap[currentElementState]);
     }
     currentElementList->push_back(element);
-    
+
 }
 
 osg::Vec3 Model::averageNormal(const Element& element) const
@@ -766,7 +805,7 @@ osg::Vec3 Model::averageNormal(const Element& element) const
         normal += normals[*itr];
     }
     normal.normalize();
-    
+
     return normal;
 }
 
@@ -782,13 +821,13 @@ osg::Vec3 Model::computeNormal(const Element& element) const
         normal += localNormal;
     }
     normal.normalize();
-    
+
     return normal;
 }
 
 bool Model::needReverse(const Element& element) const
 {
     if (element.normalIndices.empty()) return false;
-    
+
     return computeNormal(element)*averageNormal(element) < 0.0f;
 }
