@@ -87,11 +87,11 @@ convert_16_to_24(const unsigned char * const src, unsigned char * const dest)
     unsigned int t0 = src[0];
     unsigned int t1 = src[1];
                                  /* r */
-    dest[0] = (unsigned char) (t0 & 0x1f) << 2;
+    dest[0] = (unsigned char) (t1 & 0x7c) << 1;
                                  /* g */
-    dest[1] = (unsigned char) (t1 & 0x7c) >> 2;
+    dest[1] = (unsigned char) ((t1 & 0x03) << 6) | ((t0 & 0xe0) >> 2);
                                  /*b */
-    dest[2] = (unsigned char) ((t1 & 0x3)<<3) | ((t0&0xe)>>5);
+    dest[2] = (unsigned char) (t0 & 0x1f) << 3;
 }
 
 
@@ -102,12 +102,12 @@ convert_16_to_32(const unsigned char * const src, unsigned char * const dest)
     unsigned int t0 = src[0];
     unsigned int t1 = src[1];
                                  /* r */
-    dest[0] = (unsigned char) (t0 & 0x1f) << 2;
+    dest[0] = (unsigned char) (t1 & 0x7c) << 1;
                                  /* g */
-    dest[1] = (unsigned char) (t1 & 0x7c) >> 2;
+    dest[1] = (unsigned char) ((t1 & 0x03) << 6) | ((t0 & 0xe0) >> 2);
                                  /*b */
-    dest[2] = (unsigned char) ((t1 & 0x3)<<3) | ((t0&0xe)>>5);
-    dest[3] = (t1&0x70)?255:0;   /* a */
+    dest[2] = (unsigned char) (t0 & 0x1f) << 3;
+    dest[3] = (t1 & 0x80)?255:0;   /* a */
 }
 
 
@@ -468,12 +468,12 @@ int headerlen)
 class ReaderWriterTGA : public osgDB::ReaderWriter
 {
     public:
-    
+
         ReaderWriterTGA()
         {
             supportsExtension("tga","Tga Image format");
         }
-        
+
         virtual const char* className() const { return "TGA Image Reader"; }
 
         ReadResult readTGAStream(std::istream& fin) const
@@ -542,9 +542,11 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             if(rr.validImage()) rr.getImage()->setFileName(file);
             return rr;
         }
-        
+
         bool saveTGAStream(const osg::Image& image, std::ostream& fout) const
         {
+            if (!image.data()) return false;
+
             // At present, I will only save the image to unmapped RGB format
             // Other data types can be added soon with different options
             // The format description can be found at:
@@ -553,9 +555,7 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             int width = image.s(), height = image.t();
             int numPerPixel = image.computeNumComponents(pixelFormat);
             int pixelMultiplier = (image.getDataType()==GL_FLOAT ? 255 : 1);
-            const unsigned char* data = image.data();
-            if ( !data ) return false;
-            
+
             // Headers
             fout.put(0);  // Identification field size
             fout.put(0);  // Color map type
@@ -569,7 +569,7 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             fout.put(height&0xff); fout.put((height&0xff00)>>8);  // Height of image
             fout.put(numPerPixel * 8);  // Image pixel size
             fout.put(0);  // Image descriptor
-            
+
             // Swap red/blue channels for BGR images
             int r = 0, g = 1, b = 2;
             if( pixelFormat == GL_BGR || pixelFormat == GL_BGRA )
@@ -581,7 +581,7 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             // Data
             for (int y=0; y<height; ++y)
             {
-                const unsigned char* ptr = data + y * width * numPerPixel;
+                const unsigned char* ptr = image.data(0,y);
                 for (int x=0; x<width; ++x)
                 {
                     int off = x * numPerPixel;
@@ -602,7 +602,7 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             }
             return true;
         }
-        
+
         virtual WriteResult writeImage(const osg::Image& image, std::ostream& fout, const Options*) const
         {
             if (saveTGAStream(image, fout))
@@ -610,12 +610,12 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             else
                 return WriteResult::ERROR_IN_WRITING_FILE;
         }
-        
+
         virtual WriteResult writeImage(const osg::Image& image, const std::string& fileName, const Options* options) const
         {
             std::string ext = osgDB::getFileExtension(fileName);
             if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
-            
+
             osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
             if (!fout) return WriteResult::ERROR_IN_WRITING_FILE;
             return writeImage(image, fout, options);
