@@ -248,7 +248,7 @@ simage_tiff_error(char * buffer, int buflen)
             strncpy(buffer, "TIFF loader: Error opening file", buflen);
             break;
         case ERR_READ:
-            strncpy(buffer, "TIFF loader: Error reding/decoding file", buflen);
+            strncpy(buffer, "TIFF loader: Error reading/decoding file", buflen);
             break;
         case ERR_MEM:
             strncpy(buffer, "TIFF loader: Out of memory error", buflen);
@@ -347,7 +347,7 @@ invert_row(unsigned char *ptr, unsigned char *data, int n, int invert, uint16 bi
     {
         unsigned short *ptr1 = (unsigned short *)ptr;
         unsigned short *data1 = (unsigned short *)data;
-        
+
         while (n--)
         {
             if (invert) *ptr1++ = 65535 - *data1++;
@@ -358,7 +358,7 @@ invert_row(unsigned char *ptr, unsigned char *data, int n, int invert, uint16 bi
     {
         float *ptr1 = (float *)ptr;
         float *data1 = (float *)data;
-        
+
         while (n--)
         {
             if (invert) *ptr1++ = 1.0 - *data1++;
@@ -404,7 +404,7 @@ static void interleave_row(unsigned char *ptr,
         unsigned short *red1 = (unsigned short *)red;
         unsigned short *green1 = (unsigned short *)green;
         unsigned short *blue1 = (unsigned short *)blue;
-        
+
         while (n--)
         {
             *ptr1++ = *red1++;
@@ -419,7 +419,7 @@ static void interleave_row(unsigned char *ptr,
         float *red1 = (float *)red;
         float *green1 = (float *)green;
         float *blue1 = (float *)blue;
-        
+
         while (n--)
         {
             *ptr1++ = *red1++;
@@ -452,7 +452,7 @@ static void interleave_row(unsigned char *ptr,
         unsigned short *green1 = (unsigned short *)green;
         unsigned short *blue1 = (unsigned short *)blue;
         unsigned short *alpha1 = (unsigned short *)alpha;
-        
+
         while (n--)
         {
             *ptr1++ = *red1++;
@@ -468,7 +468,7 @@ static void interleave_row(unsigned char *ptr,
         float *green1 = (float *)green;
         float *blue1 = (float *)blue;
         float *alpha1 = (float *)alpha;
-        
+
         while (n--)
         {
             *ptr1++ = *red1++;
@@ -595,7 +595,7 @@ simage_tiff_load(std::istream& fin,
         TIFFClose(in);
         return NULL;
     }
-    
+
     if (TIFFGetField(in, TIFFTAG_IMAGEWIDTH, &w) != 1 ||
         TIFFGetField(in, TIFFTAG_IMAGELENGTH, &h) != 1 ||
         TIFFGetField(in, TIFFTAG_PLANARCONFIG, &config) != 1)
@@ -605,7 +605,7 @@ simage_tiff_load(std::istream& fin,
         return NULL;
     }
 
-        
+
     TIFFGetField(in, TIFFTAG_DATATYPE, &dataType);
     OSG_INFO<<"TIFFTAG_DATATYPE="<<dataType<<std::endl;
 
@@ -620,18 +620,18 @@ simage_tiff_load(std::istream& fin,
     // if it has a palette, data returned is 3 byte rgb
     // so set format to 3.
     if (photometric == PHOTOMETRIC_PALETTE)
-        format = 3; 
+        format = 3;
     else
         format = samplesperpixel * bitspersample / 8;
-    
-    
+
+
     int bytespersample = bitspersample / 8;
     int bytesperpixel = bytespersample * samplesperpixel;
-    
+
     OSG_INFO<<"format="<<format<<std::endl;
     OSG_INFO<<"bytespersample="<<bytespersample<<std::endl;
     OSG_INFO<<"bytesperpixel="<<bytesperpixel<<std::endl;
-    
+
     buffer = new unsigned char [w*h*format];
 
     if (!buffer)
@@ -665,7 +665,7 @@ simage_tiff_load(std::istream& fin,
                     tifferror = ERR_READ;
                     break;
                 }
-                invert_row(currPtr, inbuf, w, photometric == PHOTOMETRIC_MINISWHITE, bitspersample);
+                invert_row(currPtr, inbuf, samplesperpixel*w, photometric == PHOTOMETRIC_MINISWHITE, bitspersample);
                 currPtr -= format*w;
             }
             break;
@@ -767,16 +767,16 @@ simage_tiff_load(std::istream& fin,
 class ReaderWriterTIFF : public osgDB::ReaderWriter
 {
     public:
-    
+
         ReaderWriterTIFF()
         {
             supportsExtension("tiff","Tiff image format");
             supportsExtension("tif","Tiff image format");
         }
-        
+
         virtual const char* className() const { return "TIFF Image Reader"; }
         virtual bool acceptsExtension(const std::string& extension) const
-        { 
+        {
             if( osgDB::equalCaseInsensitive(extension,"tiff")) return true;
             if( osgDB::equalCaseInsensitive(extension,"tif") ) return true;
             return false;
@@ -792,10 +792,10 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
 
             imageData = simage_tiff_load(fin, width_ret, height_ret, numComponents_ret, bitspersample_ret);
 
-            if (imageData==NULL) 
+            if (imageData==NULL)
             {
                 char err_msg[256];
-                simage_tiff_error( err_msg, sizeof(err_msg)); 
+                simage_tiff_error( err_msg, sizeof(err_msg));
                 OSG_WARN << err_msg << std::endl;
                 return ReadResult::FILE_NOT_HANDLED;
             }
@@ -813,8 +813,8 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
                 numComponents_ret == 4 ? GL_RGBA : (GLenum)-1;
 
 
-            unsigned int dataType = 
-                bitspersample_ret == 8 ? GL_UNSIGNED_BYTE : 
+            unsigned int dataType =
+                bitspersample_ret == 8 ? GL_UNSIGNED_BYTE :
                 bitspersample_ret == 16 ? GL_UNSIGNED_SHORT :
                 bitspersample_ret == 32 ? GL_FLOAT : (GLenum)-1;
 
@@ -829,8 +829,37 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             return pOsgImage;
         }
 
-        WriteResult::WriteStatus writeTIFStream(std::ostream& fout, const osg::Image& img) const
+        WriteResult::WriteStatus writeTIFStream(std::ostream& fout, const osg::Image& img, const osgDB::ReaderWriter::Options* options) const
         {
+            int compressionType = COMPRESSION_PACKBITS;
+            if (options) {
+                std::istringstream iss(options->getOptionString());
+                std::string opt;
+                while (iss >> opt) {
+                    opt = osgDB::convertToLowerCase(opt);
+
+                    std::size_t eqInd = opt.find("=");
+                    if (opt.substr(0, eqInd) == "tiff_compression") {
+                        std::string compressTypeOpt;
+                        compressTypeOpt = opt.substr(eqInd + 1);
+                        compressTypeOpt = osgDB::convertToLowerCase(compressTypeOpt);
+                        if (compressTypeOpt == "packbits") {
+                            compressionType = COMPRESSION_PACKBITS;
+                        }
+                        else if (compressTypeOpt == "lzw") {
+                            compressionType = COMPRESSION_LZW;
+                        }
+                        else if (compressTypeOpt == "jpeg") {
+                            compressionType = COMPRESSION_JPEG;
+                        }
+                        else if (compressTypeOpt == "none") {
+                            compressionType = COMPRESSION_NONE;
+                        }
+                    }
+                }
+            }
+
+
             //Code is based from the following article on CodeProject.com
             //http://www.codeproject.com/bitmap/BitmapsToTiffs.asp
 
@@ -847,7 +876,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
                                     libtiffOStreamSizeProc, //Custom size function
                                     libtiffStreamMapProc, //Custom map function
                                     libtiffStreamUnmapProc); //Custom unmap function
-            
+
             if(image == NULL)
             {
                 return WriteResult::ERROR_IN_WRITING_FILE;
@@ -857,10 +886,12 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
                 case GL_DEPTH_COMPONENT:
                 case GL_LUMINANCE:
                 case GL_ALPHA:
+                case GL_RED:
                     photometric = PHOTOMETRIC_MINISBLACK;
                     samplesPerPixel = 1;
                     break;
                 case GL_LUMINANCE_ALPHA:
+                case GL_RG:
                     photometric = PHOTOMETRIC_MINISBLACK;
                     samplesPerPixel = 2;
                     break;
@@ -877,16 +908,22 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
                     break;
             }
 
+            uint32 rowsperstrip = 0;
+            
             switch(img.getDataType()){
                 case GL_FLOAT:
                     TIFFSetField(image, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_IEEEFP);
-                    TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, 1);
+                    rowsperstrip = 1;
                     bitsPerSample = 32;
                     break;
                 case GL_SHORT:
                     TIFFSetField(image, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_INT);
                     bitsPerSample = 16;
                     break;
+                case GL_UNSIGNED_SHORT:
+                    TIFFSetField(image, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_UINT);
+                    bitsPerSample = 16;
+                    break;
                 default:
                     bitsPerSample = 8;
                     break;
@@ -897,13 +934,13 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             TIFFSetField(image, TIFFTAG_BITSPERSAMPLE,bitsPerSample);
             TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL,samplesPerPixel);
             TIFFSetField(image, TIFFTAG_PHOTOMETRIC, photometric);
-            TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_PACKBITS); 
+            TIFFSetField(image, TIFFTAG_COMPRESSION, compressionType);
             TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
             TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
-
-            //uint32 rowsperstrip = TIFFDefaultStripSize(image, -1); 
-            //TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, rowsperstrip);
             
+            if (rowsperstrip==0) rowsperstrip = TIFFDefaultStripSize(image, 0);
+            TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, rowsperstrip);
+
             // Write the information to the file
             for(int i = 0; i < img.t(); ++i) {
                 TIFFWriteScanline(image,(tdata_t)img.data(0,img.t()-i-1),i,0);
@@ -945,9 +982,9 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             return rr;
         }
 
-        virtual WriteResult writeImage(const osg::Image& img,std::ostream& fout,const osgDB::ReaderWriter::Options* /*options*/) const
+        virtual WriteResult writeImage(const osg::Image& img,std::ostream& fout,const osgDB::ReaderWriter::Options* options) const
         {
-            WriteResult::WriteStatus ws = writeTIFStream(fout,img);
+            WriteResult::WriteStatus ws = writeTIFStream(fout,img, options);
             return ws;
         }
 
