@@ -25,11 +25,20 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
 
     //TileMapper* tileMapper = dynamic_cast<TileMapper*>(nv.getUserData());
     //Modified by Brad Anderegg (May-27-08) because the black listing process appears to make tiles switch lods
-    //when they clearly shouldnt, in the worst cases a tile will page out that is right in front of you.  
+    //when they clearly shouldnt, in the worst cases a tile will page out that is right in front of you.
     bool forceUseOfFirstChild = /*tileMapper ? (tileMapper->isNodeBlackListed(this)) :*/ false;
 
     double timeStamp = nv.getFrameStamp()?nv.getFrameStamp()->getReferenceTime():0.0;
     bool updateTimeStamp = nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR;
+    unsigned int frameNumber = nv.getFrameStamp()?nv.getFrameStamp()->getFrameNumber():0;
+
+    // set the frame number of the traversal so that external nodes can find out how active this
+    // node is.
+    if (nv.getFrameStamp() &&
+        nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR)
+    {
+        setFrameNumberOfLastTraversal(nv.getFrameStamp()->getFrameNumber());
+    }
 
     switch(nv.getTraversalMode())
     {
@@ -49,13 +58,16 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
 
             for(unsigned int i=0;i<maxRangeSize;++i)
             {
-                if (forceUseOfFirstChild || 
+                if (forceUseOfFirstChild ||
                     (_rangeList[i].first<=distance && distance<_rangeList[i].second))
                 {
                     if (i<_children.size())
                     {
                         if (updateTimeStamp)
+                        {
                             _perRangeDataList[i]._timeStamp=timeStamp;
+                            _perRangeDataList[i]._frameNumber=frameNumber;
+                        }
 
                         _children[i]->accept(nv);
                         lastChildTraversed = (int)i;
@@ -78,7 +90,7 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
                     //std::cout<<"    to child "<<numChildren-1<<std::endl;
                     if (updateTimeStamp)
                         _perRangeDataList[numChildren-1]._timeStamp=timeStamp;
-                        
+
                     _children[numChildren-1]->accept(nv);
                 }
 
@@ -93,7 +105,7 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
 
                     //std::cout<<"    requesting child "<<_fileNameList[numChildren]<<" priotity = "<<priority<<std::endl;
                     nv.getDatabaseRequestHandler()->requestNodeFile(_perRangeDataList[numChildren]._filename,
-                                                                    this,
+                                                                    nv.getNodePath(),
                                                                     priority,
                                                                     nv.getFrameStamp(),
                                                                     _perRangeDataList[numChildren]._databaseRequest);
