@@ -30,6 +30,8 @@
 trpgCheckable::trpgCheckable()
 {
     valid = false;
+    handle = -1;
+    writeHandle = false;
 }
 trpgCheckable::~trpgCheckable()
 {
