@@ -5,7 +5,7 @@
  * VRML2 file converter for OpenSceneGraph.
  *
  * authors :
- *           Johan Nouvel (johan_nouvel@yahoo.com) for the writeNode function. 
+ *           Johan Nouvel (johan_nouvel@yahoo.com) for the writeNode function.
  *
  *           Jan Ciger (jan.ciger@gmail.com),
  *           Tolga Abaci (tolga.abaci@gmail.com),
@@ -21,6 +21,11 @@
 #include <string>
 #include <map>
 
+#include <osg/Config>
+#ifndef OSG_USE_DEPRECATED_GEOMETRY_METHODS 
+#define OSG_USE_DEPRECATED_GEOMETRY_METHODS 1
+#endif
+
 #include <osg/Node>
 #include <osg/Geometry>
 
