@@ -1,7 +1,7 @@
 // -*-c++-*-
 
 /*
- * $Id: directx.cpp 7747 2007-12-24 15:19:52Z robert $
+ * $Id: directx.cpp 13865 2013-10-21 09:56:57Z robert $
  *
  * Loader for DirectX .x files.
  * Copyright (c)2002-2006 Ulrich Hertlein <u.hertlein@sandbox.de>
@@ -35,6 +35,7 @@
 #include <math.h>
 
 #include <osg/Notify>
+#include <osgDB/fstream>
 
 using namespace DX;
 using namespace std;
@@ -58,20 +59,21 @@ void Object::clear()
     _meshes.clear();
 }
 
-bool Object::load(const char* filename)
+bool Object::load(std::istream& fin)
 {
-    if (!filename)
+    // read header
+    char buf[256];
+    if (!fin.getline(buf, sizeof(buf))) {
+        OSG_WARN << "Failed to read DirectX header\n";
         return false;
-
-    ifstream fin(filename);
-    if (fin.bad()) {
-        osg::notify(osg::WARN) << "Object::load: Unable to open: " << filename << endl;
+    }
+    if (strstr(buf, "xof") == 0) {
+        OSG_WARN << "No 'xof' found in DirectX header\n";
         return false;
     }
 
+    // read sections
     parseSection(fin);
-    fin.close();
-
     return true;
 }
 
@@ -106,7 +108,7 @@ Material * Object::findMaterial(const std::string & name)
  **********************************************************************/
 
 // Parse section
-void Object::parseSection(ifstream& fin)
+void Object::parseSection(std::istream& fin)
 {
     char buf[256];
     vector<string> token;
@@ -130,6 +132,7 @@ void Object::parseSection(ifstream& fin)
                 Mesh * mesh = new Mesh(this);
                 _meshes.push_back(mesh);
                 mesh->parseMesh(fin);
+                OSG_INFO << "Mesh " << (token.size()>1?token[1]:"") << endl;
             }
             else if (token[0] == "Material") {
                 //
@@ -143,19 +146,16 @@ void Object::parseSection(ifstream& fin)
                 }
                 parseMaterial(fin, mm);
                 _globalMaterials.push_back(mm);
+                OSG_INFO << "Material " << (token.size()>1?token[1]:"") << endl;
             }
             else if (token[0] == "Frame") {
-                parseFrame(fin);
+                //parseFrame(fin);
+                parseSection(fin);
             }
             else {
-                //cerr << "!!! Begin section " << token[0] << endl;
+                OSG_DEBUG << "!!! Begin section " << token[0] << endl;
                 parseSection(fin);
             }
         }
     }
 }
-
-// Parse frame
-void Object::parseFrame(ifstream& /*fin*/)
-{
-}
