@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -39,7 +39,7 @@ GlyphTexture::GlyphTexture():
     setWrap(WRAP_T, CLAMP_TO_EDGE);
 }
 
-GlyphTexture::~GlyphTexture() 
+GlyphTexture::~GlyphTexture()
 {
 }
 
@@ -54,9 +54,9 @@ int GlyphTexture::compare(const osg::StateAttribute& rhs) const
 
 bool GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
 {
-    int maxAxis = std::max(glyph->s(), glyph->t());
+    int maxAxis = osg::maximum(glyph->s(), glyph->t());
     int margin = _margin + (int)((float)maxAxis * _marginRatio);
-    
+
     int width = glyph->s()+2*margin;
     int height = glyph->t()+2*margin;
 
@@ -68,7 +68,7 @@ bool GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
 
         // record the position in which the texture will be stored.
         posX = _partUsedX+margin;
-        posY = _usedY+margin;        
+        posY = _usedY+margin;
 
         // move used markers on.
         _partUsedX += width;
@@ -76,7 +76,7 @@ bool GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
 
         return true;
     }
-    
+
     // start an new row.
     if (width <= getTextureWidth() &&
         height <= (getTextureHeight()-_partUsedY))
@@ -86,7 +86,7 @@ bool GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
         _usedY = _partUsedY;
 
         posX = _partUsedX+margin;
-        posY = _usedY+margin;        
+        posY = _usedY+margin;
 
         // move used markers on.
         _partUsedX += width;
@@ -121,7 +121,7 @@ void GlyphTexture::addGlyph(Glyph* glyph, int posX, int posY)
 
 void GlyphTexture::apply(osg::State& state) const
 {
-    // get the contextID (user defined ID of 0 upwards) for the 
+    // get the contextID (user defined ID of 0 upwards) for the
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
 
@@ -152,9 +152,26 @@ void GlyphTexture::apply(osg::State& state) const
 
     // get the texture object for the current contextID.
     TextureObject* textureObject = getTextureObject(contextID);
-    
+
     bool newTextureObject = (textureObject == 0);
 
+    #if defined(OSG_GLES2_AVAILABLE)
+    bool requiresGenerateMipmapCall = false;
+
+    // need to look to see generate mip map call is required.
+    switch(_min_filter)
+    {
+    case NEAREST_MIPMAP_NEAREST:
+    case NEAREST_MIPMAP_LINEAR:
+    case LINEAR_MIPMAP_NEAREST:
+    case LINEAR_MIPMAP_LINEAR:
+        requiresGenerateMipmapCall = generateMipMapSupported;
+        break;
+    default:
+        break;
+    }
+    #endif
+
     if (newTextureObject)
     {
         GLint maxTextureSize = 256;
@@ -166,7 +183,7 @@ void GlyphTexture::apply(osg::State& state) const
             OSG_WARN<<"         Please set OSG_MAX_TEXTURE_SIZE lenvironment variable to "<<maxTextureSize<<" and re-run application."<<std::endl;
             return;
         }
-        
+
         // being bound for the first time, need to allocate the texture
 
         _textureObjectBuffer[contextID] = textureObject = osg::Texture::generateTextureObject(
@@ -177,7 +194,6 @@ void GlyphTexture::apply(osg::State& state) const
 
         applyTexParameters(GL_TEXTURE_2D,state);
 
-        
         // need to look at generate mip map extension if mip mapping required.
         switch(_min_filter)
         {
@@ -187,7 +203,9 @@ void GlyphTexture::apply(osg::State& state) const
         case LINEAR_MIPMAP_LINEAR:
             if (generateMipMapSupported)
             {
+            #if !defined(OSG_GLES2_AVAILABLE)
                 glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_TRUE);
+            #endif
             }
             else glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, LINEAR);
             break;
@@ -195,14 +213,19 @@ void GlyphTexture::apply(osg::State& state) const
             // not mip mapping so no problems.
             break;
         }
-        
+
         unsigned int imageDataSize = getTextureHeight()*getTextureWidth();
         unsigned char* imageData = new unsigned char[imageDataSize];
         for(unsigned int i=0; i<imageDataSize; ++i)
         {
             imageData[i] = 0;
         }
-        
+
+        glPixelStorei(GL_UNPACK_ALIGNMENT,1);
+
+        #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+        glPixelStorei(GL_UNPACK_ROW_LENGTH,getTextureWidth());
+        #endif
 
         // allocate the texture memory.
         glTexImage2D( GL_TEXTURE_2D, 0, GL_ALPHA,
@@ -210,15 +233,15 @@ void GlyphTexture::apply(osg::State& state) const
                 GL_ALPHA,
                 GL_UNSIGNED_BYTE,
                 imageData );
-                
+
         delete [] imageData;
-    
+
     }
     else
     {
         // reuse texture by binding.
         textureObject->bind();
-        
+
         if (getTextureParameterDirty(contextID))
         {
             applyTexParameters(GL_TEXTURE_2D,state);
@@ -226,7 +249,7 @@ void GlyphTexture::apply(osg::State& state) const
 
 
     }
-    
+
     static const GLubyte* s_renderer = 0;
     static bool s_subloadAllGlyphsTogether = false;
     if (!s_renderer)
@@ -241,9 +264,9 @@ void GlyphTexture::apply(osg::State& state) const
             // subloading bugs by loading all at once.
             s_subloadAllGlyphsTogether = true;
         }
-        
-        if (s_renderer && 
-            ((strstr((const char*)s_renderer,"Radeon")!=0) || 
+
+        if (s_renderer &&
+            ((strstr((const char*)s_renderer,"Radeon")!=0) ||
             (strstr((const char*)s_renderer,"RADEON")!=0) ||
             (strstr((const char*)s_renderer,"ALL-IN-WONDER")!=0)))
         {
@@ -274,7 +297,13 @@ void GlyphTexture::apply(osg::State& state) const
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
-        if (!s_subloadAllGlyphsTogether)
+        bool subloadAllGlyphsTogether = s_subloadAllGlyphsTogether;
+
+        #if defined(OSG_GLES2_AVAILABLE)
+        if (requiresGenerateMipmapCall) subloadAllGlyphsTogether = true;
+        #endif
+
+        if (!subloadAllGlyphsTogether)
         {
             if (newTextureObject)
             {
@@ -286,7 +315,7 @@ void GlyphTexture::apply(osg::State& state) const
                 }
             }
             else // just subload the new entries.
-            {            
+            {
                 // default way of subloading as required.
                 //std::cout<<"subloading"<<std::endl;
                 for(GlyphPtrList::iterator itr=glyphsWereSubloading.begin();
@@ -296,10 +325,10 @@ void GlyphTexture::apply(osg::State& state) const
                     (*itr)->subload();
                 }
             }
-            
+
             // clear the list since we have now subloaded them.
             glyphsWereSubloading.clear();
-            
+
         }
         else
         {
@@ -328,7 +357,7 @@ void GlyphTexture::apply(osg::State& state) const
                     for( int s = 0; s < (*itr)->s(); s++ )
                     {
                         int sindex = (t*(*itr)->s()+s);
-                        int dindex =  
+                        int dindex =
                             ((((*itr)->getTexturePositionY()+t) * getTextureWidth()) +
                             ((*itr)->getTexturePositionX()+s));
 
@@ -343,12 +372,22 @@ void GlyphTexture::apply(osg::State& state) const
             // clear the list since we have now subloaded them.
             glyphsWereSubloading.clear();
 
+            glPixelStorei(GL_UNPACK_ALIGNMENT,1);
+
+            #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+            glPixelStorei(GL_UNPACK_ROW_LENGTH,getTextureWidth());
+            #endif
+
             // Subload the image once
-            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 
+            glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0,
                     getTextureWidth(),
                     getTextureHeight(),
                     GL_ALPHA, GL_UNSIGNED_BYTE, local_data );
 
+            #if defined(OSG_GLES2_AVAILABLE)
+            if (requiresGenerateMipmapCall) glGenerateMipmap(GL_TEXTURE_2D);
+            #endif
+
             delete [] local_data;
 
         }
@@ -379,7 +418,7 @@ void GlyphTexture::resizeGLObjectBuffers(unsigned int maxSize)
 
     unsigned int initialSize = _glyphsToSubload.size();
     _glyphsToSubload.resize(maxSize);
-    
+
     for(unsigned i=initialSize; i<_glyphsToSubload.size(); ++i)
     {
         for(GlyphRefList::iterator itr = _glyphs.begin();
@@ -475,13 +514,17 @@ void Glyph::subload() const
 
     glPixelStorei(GL_UNPACK_ALIGNMENT,getPacking());
 
+    #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+    glPixelStorei(GL_UNPACK_ROW_LENGTH,getRowLength());
+    #endif
+
     glTexSubImage2D(GL_TEXTURE_2D,0,
                     _texturePosX,_texturePosY,
                     s(),t(),
                     (GLenum)getPixelFormat(),
                     (GLenum)getDataType(),
                     data());
-                    
+
     errorNo = glGetError();
     if (errorNo!=GL_NO_ERROR)
     {
@@ -496,8 +539,8 @@ void Glyph::subload() const
                                  "\t                "<<s()<<" ,"<<t()<<std::endl<<hex<<
                                  "\t                0x"<<(GLenum)getPixelFormat()<<std::endl<<
                                  "\t                0x"<<(GLenum)getDataType()<<std::endl<<
-                                 "\t                0x"<<(unsigned long)data()<<");"<<dec<<std::endl;
-    }                    
+                                 "\t                "<<static_cast<const void*>(data())<<");"<<dec<<std::endl;
+    }
 }
 
 Glyph3D::Glyph3D(Font* font, unsigned int glyphCode):
@@ -514,7 +557,6 @@ Glyph3D::Glyph3D(Font* font, unsigned int glyphCode):
 
 void Glyph3D::setThreadSafeRefUnref(bool threadSafe)
 {
-    GlyphGeometries _glyphGeometries;
     for(GlyphGeometries::iterator itr = _glyphGeometries.begin();
         itr != _glyphGeometries.end();
         ++itr)
