@@ -1,14 +1,14 @@
 /* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  * Copyright (C) 2010 Tim Moore
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -60,33 +60,29 @@ struct GeometryArrayGatherer
     typedef std::vector<osg::Array*> ArrayList;
 
     GeometryArrayGatherer(osg::Geometry& geometry)
-        : _useDrawElements(true)
     {
-        add(geometry.getVertexArray(),osg::Geometry::BIND_PER_VERTEX);
-        add(geometry.getNormalArray(),geometry.getNormalBinding());
-        add(geometry.getColorArray(),geometry.getColorBinding());
-        add(geometry.getSecondaryColorArray(),geometry.getSecondaryColorBinding());
-        add(geometry.getFogCoordArray(),geometry.getFogCoordBinding());
+        add(geometry.getVertexArray());
+        add(geometry.getNormalArray());
+        add(geometry.getColorArray());
+        add(geometry.getSecondaryColorArray());
+        add(geometry.getFogCoordArray());
         unsigned int i;
         for(i=0;i<geometry.getNumTexCoordArrays();++i)
         {
-            add(geometry.getTexCoordArray(i),osg::Geometry::BIND_PER_VERTEX);
+            add(geometry.getTexCoordArray(i));
         }
         for(i=0;i<geometry.getNumVertexAttribArrays();++i)
         {
-            add(geometry.getVertexAttribArray(i),geometry.getVertexAttribBinding(i));
+            add(geometry.getVertexAttribArray(i));
         }
     }
 
-    void add(osg::Array* array, osg::Geometry::AttributeBinding binding)
+    void add(osg::Array* array)
     {
-        if (binding == osg::Geometry::BIND_PER_VERTEX)
+        if (array && array->getBinding()==osg::Array::BIND_PER_VERTEX)
         {
-            if (array)
-                _arrayList.push_back(array);
+            _arrayList.push_back(array);
         }
-        else if (binding == osg::Geometry::BIND_PER_PRIMITIVE)
-            _useDrawElements = false;
     }
 
     void accept(osg::ArrayVisitor& av)
@@ -98,11 +94,8 @@ struct GeometryArrayGatherer
             (*itr)->accept(av);
         }
     }
-    
+
     ArrayList _arrayList;
-    // True if geometry contains bindings that are compatible with
-    // DrawElements.
-    bool _useDrawElements;
 };
 
 // Compare vertices in a mesh using all their attributes. The vertices
@@ -113,7 +106,7 @@ struct VertexAttribComparitor : public GeometryArrayGatherer
         : GeometryArrayGatherer(geometry)
     {
     }
-    
+
     bool operator() (unsigned int lhs, unsigned int rhs) const
     {
         for(ArrayList::const_iterator itr=_arrayList.begin();
@@ -125,7 +118,7 @@ struct VertexAttribComparitor : public GeometryArrayGatherer
             if (compare==1) return false;
         }
         return false;
-    }   
+    }
 
     int compare(unsigned int lhs, unsigned int rhs)
     {
@@ -141,7 +134,7 @@ struct VertexAttribComparitor : public GeometryArrayGatherer
     }
 
 protected:
-    VertexAttribComparitor& operator = (const VertexAttribComparitor&) { return *this; }    
+    VertexAttribComparitor& operator = (const VertexAttribComparitor&) { return *this; }
 
 };
 
@@ -150,9 +143,9 @@ class RemapArray : public osg::ArrayVisitor
 {
     public:
         RemapArray(const IndexList& remapping):_remapping(remapping) {}
-        
+
         const IndexList& _remapping;
-        
+
         template<class T>
         inline void remap(T& array)
         {
@@ -165,7 +158,7 @@ class RemapArray : public osg::ArrayVisitor
             }
             array.erase(array.begin()+_remapping.size(),array.end());
         }
-        
+
         virtual void apply(osg::Array&) {}
         virtual void apply(osg::ByteArray& array) { remap(array); }
         virtual void apply(osg::ShortArray& array) { remap(array); }
@@ -179,12 +172,12 @@ class RemapArray : public osg::ArrayVisitor
         virtual void apply(osg::Vec2Array& array) { remap(array); }
         virtual void apply(osg::Vec3Array& array) { remap(array); }
         virtual void apply(osg::Vec4Array& array) { remap(array); }
-        
+
         virtual void apply(osg::Vec4ubArray& array) { remap(array); }
 
-        virtual void apply(osg::Vec2bArray& array) { remap(array); }        
-        virtual void apply(osg::Vec3bArray& array) { remap(array); }        
-        virtual void apply(osg::Vec4bArray& array) { remap(array); }        
+        virtual void apply(osg::Vec2bArray& array) { remap(array); }
+        virtual void apply(osg::Vec3bArray& array) { remap(array); }
+        virtual void apply(osg::Vec4bArray& array) { remap(array); }
 
         virtual void apply(osg::Vec2sArray& array) { remap(array); }
         virtual void apply(osg::Vec3sArray& array) { remap(array); }
@@ -193,7 +186,7 @@ class RemapArray : public osg::ArrayVisitor
         virtual void apply(osg::Vec2dArray& array) { remap(array); }
         virtual void apply(osg::Vec3dArray& array) { remap(array); }
         virtual void apply(osg::Vec4dArray& array) { remap(array); }
-        
+
         virtual void apply(osg::MatrixfArray& array) { remap(array); }
 protected:
 
@@ -231,17 +224,15 @@ typedef osg::TriangleIndexFunctor<MyTriangleOperator> MyTriangleIndexFunctor;
 
 void IndexMeshVisitor::makeMesh(Geometry& geom)
 {
-        if (geom.getNormalBinding()==osg::Geometry::BIND_PER_PRIMITIVE ||
-        geom.getNormalBinding()==osg::Geometry::BIND_PER_PRIMITIVE_SET) return;
+    if (geom.containsDeprecatedData()) geom.fixDeprecatedData();
 
-    if (geom.getColorBinding()==osg::Geometry::BIND_PER_PRIMITIVE ||
-        geom.getColorBinding()==osg::Geometry::BIND_PER_PRIMITIVE_SET) return;
-    
-    if (geom.getSecondaryColorBinding()==osg::Geometry::BIND_PER_PRIMITIVE ||
-        geom.getSecondaryColorBinding()==osg::Geometry::BIND_PER_PRIMITIVE_SET) return;
+    if (osg::getBinding(geom.getNormalArray())==osg::Array::BIND_PER_PRIMITIVE_SET) return;
 
-    if (geom.getFogCoordBinding()==osg::Geometry::BIND_PER_PRIMITIVE ||
-        geom.getFogCoordBinding()==osg::Geometry::BIND_PER_PRIMITIVE_SET) return;
+    if (osg::getBinding(geom.getColorArray())==osg::Array::BIND_PER_PRIMITIVE_SET) return;
+
+    if (osg::getBinding(geom.getSecondaryColorArray())==osg::Array::BIND_PER_PRIMITIVE_SET) return;
+
+    if (osg::getBinding(geom.getFogCoordArray())==osg::Array::BIND_PER_PRIMITIVE_SET) return;
 
     // no point optimizing if we don't have enough vertices.
     if (!geom.getVertexArray() || geom.getVertexArray()->getNumElements()<3) return;
@@ -275,21 +266,14 @@ void IndexMeshVisitor::makeMesh(Geometry& geom)
               || type == PrimitiveSet::DrawElementsUIntPrimitiveType))
             numNonIndexedPrimitives++;
     }
-    
+
     // nothing to index
     if (!numSurfacePrimitives || !numNonIndexedPrimitives) return;
 
-    // check to see if vertex attributes indices exists, if so expand them to remove them
-    if (geom.suitableForOptimization())
-    {
-        // removing coord indices
-        OSG_INFO<<"TriStripVisitor::stripify(Geometry&): Removing attribute indices"<<std::endl;
-        geom.copyToAndOptimize(geom);
-    }
-
+    // duplicate shared arrays as it isn't safe to rearrange vertices when arrays are shared.
+    if (geom.containsSharedArrays()) geom.duplicateSharedArrays();
 
     // compute duplicate vertices
-    
     typedef std::vector<unsigned int> IndexList;
     unsigned int numVertices = geom.getVertexArray()->getNumElements();
     IndexList indices(numVertices);
@@ -298,7 +282,7 @@ void IndexMeshVisitor::makeMesh(Geometry& geom)
     {
         indices[i] = i;
     }
-    
+
     VertexAttribComparitor arrayComparitor(geom);
     std::sort(indices.begin(),indices.end(),arrayComparitor);
 
@@ -311,7 +295,7 @@ void IndexMeshVisitor::makeMesh(Geometry& geom)
             lastUnique = i;
             ++numUnique;
         }
-        
+
     }
     IndexList remapDuplicatesToOrignals(numVertices);
     lastUnique = 0;
@@ -332,7 +316,7 @@ void IndexMeshVisitor::makeMesh(Geometry& geom)
             }
             lastUnique = i;
         }
-        
+
     }
     unsigned int min_index = indices[lastUnique];
     for(j=lastUnique+1;j<i;++j)
@@ -345,30 +329,30 @@ void IndexMeshVisitor::makeMesh(Geometry& geom)
     }
 
 
-    // copy the arrays.    
+    // copy the arrays.
     IndexList finalMapping(numVertices);
     IndexList copyMapping;
     copyMapping.reserve(numUnique);
     unsigned int currentIndex=0;
     for(i=0;i<numVertices;++i)
     {
-        if (remapDuplicatesToOrignals[i]==i) 
+        if (remapDuplicatesToOrignals[i]==i)
         {
             finalMapping[i] = currentIndex;
             copyMapping.push_back(i);
             currentIndex++;
         }
     }
-    
+
     for(i=0;i<numVertices;++i)
     {
-        if (remapDuplicatesToOrignals[i]!=i) 
+        if (remapDuplicatesToOrignals[i]!=i)
         {
             finalMapping[i] = finalMapping[remapDuplicatesToOrignals[i]];
         }
     }
-   
-    
+
+
     MyTriangleIndexFunctor taf;
     taf._remapIndices.swap(finalMapping);
 
@@ -838,7 +822,7 @@ void VertexCacheVisitor::doVertexOptimization(Geometry& geom,
         }
         assert(triToAdd != 0 && triToAdd->score > 0.0);
         // Add triangle vertices, and remove triangle from the
-        // vertices that use it. 
+        // vertices that use it.
         triToAdd->score = -1.0f;
         unsigned triToAddIdx = triToAdd - &triangles[0];
         for (unsigned i = 0; i < 3; ++i)
@@ -991,7 +975,7 @@ struct CacheRecorder : public TriangleIndexFunctor<CacheRecordOperator>
 void VertexCacheMissVisitor::doGeometry(Geometry& geom)
 {
     Array* vertArray = geom.getVertexArray();
-    if (!vertArray)
+    if (!vertArray || vertArray->getNumElements()==0)
         return;
     Geometry::PrimitiveSetList& primSets = geom.getPrimitiveSetList();
     CacheRecorder recorder(_cacheSize);
@@ -1055,12 +1039,12 @@ public:
     virtual void apply(osg::Vec2Array& array) { remap(array); }
     virtual void apply(osg::Vec3Array& array) { remap(array); }
     virtual void apply(osg::Vec4Array& array) { remap(array); }
-    
+
     virtual void apply(osg::Vec4ubArray& array) { remap(array); }
 
-    virtual void apply(osg::Vec2bArray& array) { remap(array); }        
-    virtual void apply(osg::Vec3bArray& array) { remap(array); }        
-    virtual void apply(osg::Vec4bArray& array) { remap(array); }        
+    virtual void apply(osg::Vec2bArray& array) { remap(array); }
+    virtual void apply(osg::Vec3bArray& array) { remap(array); }
+    virtual void apply(osg::Vec4bArray& array) { remap(array); }
 
     virtual void apply(osg::Vec2sArray& array) { remap(array); }
     virtual void apply(osg::Vec3sArray& array) { remap(array); }
@@ -1069,7 +1053,7 @@ public:
     virtual void apply(osg::Vec2dArray& array) { remap(array); }
     virtual void apply(osg::Vec3dArray& array) { remap(array); }
     virtual void apply(osg::Vec4dArray& array) { remap(array); }
-    
+
     virtual void apply(osg::MatrixfArray& array) { remap(array); }
 };
 
@@ -1081,10 +1065,10 @@ struct VertexReorderOperator
     unsigned seq;
     vector<unsigned> remap;
 
-    VertexReorderOperator() : seq(0) 
+    VertexReorderOperator() : seq(0)
     {
     }
-    
+
     void inline doVertex(unsigned v)
     {
         if (remap[v] == Remapper::invalidIndex)
@@ -1104,7 +1088,7 @@ struct VertexReorder : public TriangleIndexFunctor<VertexReorderOperator>
     {
         remap.resize(numVerts, Remapper::invalidIndex);
     }
-    
+
 };
 }
 
@@ -1133,12 +1117,9 @@ inline void reorderDrawElements(DE& drawElements,
 void VertexAccessOrderVisitor::optimizeOrder(Geometry& geom)
 {
     Array* vertArray = geom.getVertexArray();
-    if (!vertArray)
+    if (!vertArray || vertArray->getNumElements()==0)
         return;
     Geometry::PrimitiveSetList& primSets = geom.getPrimitiveSetList();
-    GeometryArrayGatherer gatherer(geom);
-    if (!gatherer._useDrawElements)
-        return;
     VertexReorder vr(vertArray->getNumElements());
     for (Geometry::PrimitiveSetList::iterator itr = primSets.begin(),
              end = primSets.end();
@@ -1153,6 +1134,11 @@ void VertexAccessOrderVisitor::optimizeOrder(Geometry& geom)
             return;
         ps->accept(vr);
     }
+
+    // duplicate shared arrays as it isn't safe to rearrange vertices when arrays are shared.
+    if (geom.containsSharedArrays()) geom.duplicateSharedArrays();
+    GeometryArrayGatherer gatherer(geom);
+
     Remapper remapper(vr.remap);
     gatherer.accept(remapper);
     for (Geometry::PrimitiveSetList::iterator itr = primSets.begin(),
