@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -28,7 +28,7 @@ namespace PlaneIntersectorUtils
     {
         typedef std::vector<osg::Vec4d> Polyline;
         Polyline _polyline;
-        
+
         void reverse()
         {
             unsigned int s=0;
@@ -37,32 +37,32 @@ namespace PlaneIntersectorUtils
             {
                 std::swap(_polyline[s],_polyline[e]);
             }
-            
+
         }
     };
 
     class PolylineConnector
     {
     public:
-        
+
         typedef std::map<osg::Vec4d, osg::ref_ptr<RefPolyline> > PolylineMap;
         typedef std::vector< osg::ref_ptr<RefPolyline> > PolylineList;
-        
+
         PolylineList    _polylines;
         PolylineMap     _startPolylineMap;
         PolylineMap     _endPolylineMap;
         osg::ref_ptr<osg::EllipsoidModel>   _em;
-        
+
 
         void add(const osg::Vec3d& v1, const osg::Vec3d& v2)
         {
             add(osg::Vec4d(v1,0.0), osg::Vec4d(v2,0.0));
         }
-        
+
         void add(const osg::Vec4d& v1, const osg::Vec4d& v2)
         {
             if (v1==v2) return;
-        
+
             PolylineMap::iterator v1_start_itr = _startPolylineMap.find(v1);
             PolylineMap::iterator v1_end_itr = _endPolylineMap.find(v1);
 
@@ -76,7 +76,7 @@ namespace PlaneIntersectorUtils
             unsigned int v2_connections = 0;
             if (v2_start_itr != _startPolylineMap.end()) ++v2_connections;
             if (v2_end_itr != _endPolylineMap.end()) ++v2_connections;
-            
+
             if (v1_connections==0) // v1 is no connected to anything.
             {
                 if (v2_connections==0)
@@ -97,13 +97,13 @@ namespace PlaneIntersectorUtils
                     }
                     else
                     {
-                        osg::notify(osg::NOTICE)<<"Error: should not get here!"<<std::endl;
+                        OSG_NOTICE<<"Error: should not get here!"<<std::endl;
                     }
                 }
                 else // if (v2_connections==2)
                 {
                     // v2 connects to a start and an end - must have a loop in the list!
-                    osg::notify(osg::NOTICE)<<"v2="<<v2<<" must connect to a start and an end - must have a loop!!!!!."<<std::endl;
+                    OSG_NOTICE<<"v2="<<v2<<" must connect to a start and an end - must have a loop!!!!!."<<std::endl;
                 }
             }
             else if (v2_connections==0) // v1 is no connected to anything.
@@ -121,24 +121,24 @@ namespace PlaneIntersectorUtils
                     }
                     else
                     {
-                        osg::notify(osg::NOTICE)<<"Error: should not get here!"<<std::endl;
+                        OSG_NOTICE<<"Error: should not get here!"<<std::endl;
                     }
                 }
                 else // if (v1_connections==2)
                 {
                     // v1 connects to a start and an end - must have a loop in the list!
-                    osg::notify(osg::NOTICE)<<"v1="<<v1<<" must connect to a start and an end - must have a loop!!!!!."<<std::endl;
+                    OSG_NOTICE<<"v1="<<v1<<" must connect to a start and an end - must have a loop!!!!!."<<std::endl;
                 }
             }
-            else 
+            else
             {
                 // v1 and v2 connect to existing lines, now need to fuse them together.
                 bool v1_connected_to_start = v1_start_itr != _startPolylineMap.end();
                 bool v1_connected_to_end = v1_end_itr != _endPolylineMap.end();
-                
+
                 bool v2_connected_to_start = v2_start_itr != _startPolylineMap.end();
                 bool v2_connected_to_end = v2_end_itr != _endPolylineMap.end();
-                
+
                 if (v1_connected_to_start)
                 {
                     if (v2_connected_to_start)
@@ -151,7 +151,7 @@ namespace PlaneIntersectorUtils
                     }
                     else
                     {
-                        osg::notify(osg::NOTICE)<<"Error: should not get here!"<<std::endl;
+                        OSG_NOTICE<<"Error: should not get here!"<<std::endl;
                     }
                 }
                 else if (v1_connected_to_end)
@@ -166,16 +166,16 @@ namespace PlaneIntersectorUtils
                     }
                     else
                     {
-                        osg::notify(osg::NOTICE)<<"Error: should not get here!"<<std::endl;
+                        OSG_NOTICE<<"Error: should not get here!"<<std::endl;
                     }
                 }
                 else
                 {
-                    osg::notify(osg::NOTICE)<<"Error: should not get here!"<<std::endl;
+                    OSG_NOTICE<<"Error: should not get here!"<<std::endl;
                 }
             }
         }
-        
+
         void newline(const osg::Vec4d& v1, const osg::Vec4d& v2)
         {
             RefPolyline* polyline = new RefPolyline;
@@ -184,7 +184,7 @@ namespace PlaneIntersectorUtils
             _startPolylineMap[v1] = polyline;
             _endPolylineMap[v2] = polyline;
         }
-        
+
         void insertAtStart(const osg::Vec4d& v, PolylineMap::iterator v_start_itr)
         {
             // put v1 at the start of its poyline
@@ -198,7 +198,7 @@ namespace PlaneIntersectorUtils
             _startPolylineMap.erase(v_start_itr);
 
         }
-        
+
         void insertAtEnd(const osg::Vec4d& v, PolylineMap::iterator v_end_itr)
         {
             // put v1 at the end of its poyline
@@ -229,11 +229,11 @@ namespace PlaneIntersectorUtils
 
             // reverse the first polyline
             poly1->reverse();
-            
+
             // add the second polyline to the first
             poly1->_polyline.insert( poly1->_polyline.end(),
                                      poly2->_polyline.begin(), poly2->_polyline.end() );
-            
+
             _startPolylineMap[poly1->_polyline.front()] = poly1;
             _endPolylineMap[poly1->_polyline.back()] = poly1;
 
@@ -243,17 +243,17 @@ namespace PlaneIntersectorUtils
         {
             osg::ref_ptr<RefPolyline> end_poly = end_itr->second;
             osg::ref_ptr<RefPolyline> start_poly = start_itr->second;
-            
+
             PolylineMap::iterator end_start_poly_itr = _endPolylineMap.find(start_poly->_polyline.back());
-            
+
             // add start_poly to end of end_poly
             end_poly->_polyline.insert( end_poly->_polyline.end(),
                                         start_poly->_polyline.begin(), start_poly->_polyline.end() );
-                    
+
             // reassign the end of the start poly so that it now points to the merged end_poly
             end_start_poly_itr->second = end_poly;
 
-            
+
             // remove entries for the end of the end_poly and the start of the start_poly
             _endPolylineMap.erase(end_itr);
             _startPolylineMap.erase(start_itr);
@@ -284,11 +284,11 @@ namespace PlaneIntersectorUtils
 
             // reverse the first polyline
             poly2->reverse();
-            
+
             // add the second polyline to the first
             poly1->_polyline.insert( poly1->_polyline.end(),
                                      poly2->_polyline.begin(), poly2->_polyline.end() );
-            
+
             _startPolylineMap[poly1->_polyline.front()] = poly1;
             _endPolylineMap[poly1->_polyline.back()] = poly1;
 
@@ -307,35 +307,35 @@ namespace PlaneIntersectorUtils
 
         void report()
         {
-            osg::notify(osg::NOTICE)<<"report()"<<std::endl;
+            OSG_NOTICE<<"report()"<<std::endl;
 
-            osg::notify(osg::NOTICE)<<"start:"<<std::endl;
+            OSG_NOTICE<<"start:"<<std::endl;
             for(PolylineMap::iterator sitr = _startPolylineMap.begin();
                 sitr != _startPolylineMap.end();
                 ++sitr)
             {
-                osg::notify(osg::NOTICE)<<"  line - start = "<<sitr->first<<" polyline size = "<<sitr->second->_polyline.size()<<std::endl;
+                OSG_NOTICE<<"  line - start = "<<sitr->first<<" polyline size = "<<sitr->second->_polyline.size()<<std::endl;
             }
 
-            osg::notify(osg::NOTICE)<<"ends:"<<std::endl;
+            OSG_NOTICE<<"ends:"<<std::endl;
             for(PolylineMap::iterator eitr = _endPolylineMap.begin();
                 eitr != _endPolylineMap.end();
                 ++eitr)
             {
-                osg::notify(osg::NOTICE)<<"  line - end = "<<eitr->first<<" polyline size = "<<eitr->second->_polyline.size()<<std::endl;
+                OSG_NOTICE<<"  line - end = "<<eitr->first<<" polyline size = "<<eitr->second->_polyline.size()<<std::endl;
             }
 
             for(PolylineList::iterator pitr = _polylines.begin();
                 pitr != _polylines.end();
                 ++pitr)
             {
-                osg::notify(osg::NOTICE)<<"polyline:"<<std::endl;
+                OSG_NOTICE<<"polyline:"<<std::endl;
                 RefPolyline::Polyline& polyline = (*pitr)->_polyline;
                 for(RefPolyline::Polyline::iterator vitr = polyline.begin();
                     vitr != polyline.end();
                     ++vitr)
                 {
-                    osg::notify(osg::NOTICE)<<"  "<<*vitr<<std::endl;
+                    OSG_NOTICE<<"  "<<*vitr<<std::endl;
                 }
             }
 
@@ -344,29 +344,29 @@ namespace PlaneIntersectorUtils
 
         void fuse()
         {
-             osg::notify(osg::NOTICE)<<"supposed to be doing a fuse..."<<std::endl;
+             OSG_NOTICE<<"supposed to be doing a fuse..."<<std::endl;
         }
 
-        
+
     };
 
     struct TriangleIntersector
     {
 
-        osg::Plane _plane;
-        osg::Polytope _polytope;
-        bool _hit;
-        osg::ref_ptr<osg::RefMatrix>        _matrix;
-        bool _recordHeightsAsAttributes;
-        osg::ref_ptr<osg::EllipsoidModel>   _em;
-        
+        osg::Plane                              _plane;
+        osg::Polytope                           _polytope;
+        bool                                    _hit;
+        osg::ref_ptr<osg::RefMatrix>            _matrix;
+        bool                                    _recordHeightsAsAttributes;
+        osg::ref_ptr<osg::EllipsoidModel>       _em;
+        bool                                    _limitOneIntersection;
+
         PolylineConnector _polylineConnector;
 
-
-        TriangleIntersector()
-        {
-            _hit = false;
-        }
+        TriangleIntersector():
+            _hit(false),
+            _recordHeightsAsAttributes(false),
+            _limitOneIntersection(false) {}
 
         void set(const osg::Plane& plane, const osg::Polytope& polytope, osg::RefMatrix* matrix, bool recordHeightsAsAttributes, osg::EllipsoidModel* em)
         {
@@ -401,101 +401,103 @@ namespace PlaneIntersectorUtils
                     osg::Plane& plane = *itr;
                     double ds = distance(plane,vs);
                     double de = distance(plane,ve);
-                    
+
                     if (ds<0.0)
                     {
                         if (de<0.0)
                         {
-                            // osg::notify(osg::NOTICE)<<"Discard segment "<<std::endl;
+                            // OSG_NOTICE<<"Discard segment "<<std::endl;
                             return;
                         }
-                        
-                        // osg::notify(osg::NOTICE)<<"Trim start vs="<<vs;
+
+                        // OSG_NOTICE<<"Trim start vs="<<vs;
 
                         double div = 1.0/(de-ds);
                         vs = vs*(de*div) - ve*(ds*div);
 
-                        // osg::notify(osg::NOTICE)<<" after vs="<<vs<<std::endl;
-                        
+                        // OSG_NOTICE<<" after vs="<<vs<<std::endl;
+
                     }
                     else if (de<0.0)
                     {
-                        // osg::notify(osg::NOTICE)<<"Trim end ve="<<ve;
+                        // OSG_NOTICE<<"Trim end ve="<<ve;
 
                         double div = 1.0/(ds-de);
                         ve = ve*(ds*div) - vs*(de*div);
 
-                        // osg::notify(osg::NOTICE)<<" after ve="<<ve<<std::endl;
-                        
+                        // OSG_NOTICE<<" after ve="<<ve<<std::endl;
+
                     }
-                    
-                } 
-                // osg::notify(osg::NOTICE)<<"Segment fine"<<std::endl;
+
+                }
+                // OSG_NOTICE<<"Segment fine"<<std::endl;
 
                 _polylineConnector.add(vs,ve);
-                
+
             }
         }
 
         inline void operator () (const osg::Vec3& v1,const osg::Vec3& v2,const osg::Vec3& v3, bool)
         {
-        
+
+            if (_limitOneIntersection && _hit) return;
+
             double d1 = _plane.distance(v1);
             double d2 = _plane.distance(v2);
             double d3 = _plane.distance(v3);
-            
+
             unsigned int numBelow = 0;
             unsigned int numAbove = 0;
             unsigned int numOnPlane = 0;
             if (d1<0) ++numBelow;
             else if (d1>0) ++numAbove;
             else ++numOnPlane;
-        
+
             if (d2<0) ++numBelow;
             else if (d2>0) ++numAbove;
             else ++numOnPlane;
-        
+
             if (d3<0) ++numBelow;
             else if (d3>0) ++numAbove;
             else ++numOnPlane;
-            
+
             // trivially discard triangles that are completely one side of the plane
             if (numAbove==3 || numBelow==3) return;
-            
+
             _hit = true;
 
             if (numOnPlane==3)
             {
                 // triangle lives wholy in the plane
-                osg::notify(osg::NOTICE)<<"3"<<std::endl;
+                OSG_NOTICE<<"3"<<std::endl;
                 return;
             }
 
             if (numOnPlane==2)
             {
                 // one edge lives wholy in the plane
-                osg::notify(osg::NOTICE)<<"2"<<std::endl;
+                OSG_NOTICE<<"2"<<std::endl;
                 return;
             }
 
             if (numOnPlane==1)
             {
                 // one point lives wholy in the plane
-                osg::notify(osg::NOTICE)<<"1"<<std::endl;
+                OSG_NOTICE<<"1"<<std::endl;
                 return;
             }
 
             osg::Vec4d v[2];
             unsigned int numIntersects = 0;
-            
+
             osg::Vec4d p1(v1, v1.z());
             osg::Vec4d p2(v2, v2.z());
             osg::Vec4d p3(v3, v3.z());
-            
+
             if (_em.valid())
             {
                 double latitude, longitude, height;
-                
+
                 if (_matrix.valid())
                 {
                     osg::Vec3d tp = v1 * (*_matrix);
@@ -548,9 +550,9 @@ namespace PlaneIntersectorUtils
                 }
                 else
                 {
-                    osg::notify(osg::NOTICE)<<"!!! too many intersecting edges found !!!"<<std::endl;
+                    OSG_NOTICE<<"!!! too many intersecting edges found !!!"<<std::endl;
                 }
-                 
+
             }
 
             add(v[0],v[1]);
@@ -590,6 +592,7 @@ Intersector* PlaneIntersector::clone(osgUtil::IntersectionVisitor& iv)
     {
         osg::ref_ptr<PlaneIntersector> pi = new PlaneIntersector(_plane, _polytope);
         pi->_parent = this;
+        pi->_intersectionLimit = this->_intersectionLimit;
         pi->_recordHeightsAsAttributes = _recordHeightsAsAttributes;
         pi->_em = _em;
         return pi.release();
@@ -600,18 +603,18 @@ Intersector* PlaneIntersector::clone(osgUtil::IntersectionVisitor& iv)
     osg::Matrix matrix;
     switch (_coordinateFrame)
     {
-        case(WINDOW): 
+        case(WINDOW):
             if (iv.getWindowMatrix()) matrix.preMult( *iv.getWindowMatrix() );
             if (iv.getProjectionMatrix()) matrix.preMult( *iv.getProjectionMatrix() );
             if (iv.getViewMatrix()) matrix.preMult( *iv.getViewMatrix() );
             if (iv.getModelMatrix()) matrix.preMult( *iv.getModelMatrix() );
             break;
-        case(PROJECTION): 
+        case(PROJECTION):
             if (iv.getProjectionMatrix()) matrix.preMult( *iv.getProjectionMatrix() );
             if (iv.getViewMatrix()) matrix.preMult( *iv.getViewMatrix() );
             if (iv.getModelMatrix()) matrix.preMult( *iv.getModelMatrix() );
             break;
-        case(VIEW): 
+        case(VIEW):
             if (iv.getViewMatrix()) matrix.preMult( *iv.getViewMatrix() );
             if (iv.getModelMatrix()) matrix.preMult( *iv.getModelMatrix() );
             break;
@@ -628,6 +631,7 @@ Intersector* PlaneIntersector::clone(osgUtil::IntersectionVisitor& iv)
 
     osg::ref_ptr<PlaneIntersector> pi = new PlaneIntersector(plane, transformedPolytope);
     pi->_parent = this;
+    pi->_intersectionLimit = this->_intersectionLimit;
     pi->_recordHeightsAsAttributes = _recordHeightsAsAttributes;
     pi->_em = _em;
     return pi.release();
@@ -635,6 +639,7 @@ Intersector* PlaneIntersector::clone(osgUtil::IntersectionVisitor& iv)
 
 bool PlaneIntersector::enter(const osg::Node& node)
 {
+    if (reachedLimit()) return false;
     return !node.isCullingActive() ||
            ( _plane.intersect(node.getBound())==0 && _polytope.contains(node.getBound()) );
 }
@@ -648,15 +653,17 @@ void PlaneIntersector::leave()
 
 void PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable* drawable)
 {
-    // osg::notify(osg::NOTICE)<<"PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable* drawable)"<<std::endl;
+    // OSG_NOTICE<<"PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable* drawable)"<<std::endl;
 
+    if (reachedLimit()) return;
     if ( _plane.intersect( drawable->getBound() )!=0 ) return;
     if ( !_polytope.contains( drawable->getBound() ) ) return;
 
-    // osg::notify(osg::NOTICE)<<"Succed PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable* drawable)"<<std::endl;
+    // OSG_NOTICE<<"Succed PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable* drawable)"<<std::endl;
 
     osg::TriangleFunctor<PlaneIntersectorUtils::TriangleIntersector> ti;
     ti.set(_plane, _polytope, iv.getModelMatrix(), _recordHeightsAsAttributes, _em.get());
+    ti._limitOneIntersection = (_intersectionLimit == LIMIT_ONE_PER_DRAWABLE || _intersectionLimit == LIMIT_ONE);
     drawable->accept(ti);
 
     ti._polylineConnector.consolidatePolylineLists();
@@ -664,7 +671,7 @@ void PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable
     if (ti._hit)
     {
         Intersections& intersections = getIntersections();
-     
+
         for(PlaneIntersectorUtils::PolylineConnector::PolylineList::iterator pitr = ti._polylineConnector._polylines.begin();
             pitr != ti._polylineConnector._polylines.end();
             ++pitr)
@@ -676,10 +683,10 @@ void PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable
             Intersection& new_intersection = intersections[pos];
 
             new_intersection.matrix = iv.getModelMatrix();
-    
+
             new_intersection.polyline.reserve((*pitr)->_polyline.size());
             if (_recordHeightsAsAttributes) new_intersection.attributes.reserve((*pitr)->_polyline.size());
-            
+
             for(PlaneIntersectorUtils::RefPolyline::Polyline::iterator vitr = (*pitr)->_polyline.begin();
                 vitr != (*pitr)->_polyline.end();
                 ++vitr)
@@ -700,6 +707,6 @@ void PlaneIntersector::intersect(osgUtil::IntersectionVisitor& iv, osg::Drawable
 void PlaneIntersector::reset()
 {
     Intersector::reset();
-    
+
     _intersections.clear();
 }
