@@ -85,6 +85,8 @@
 // custom UIView-class handling creation and display of frame/render buffers plus receives touch input
 // ----------------------------------------------------------------------------------------------------------
 
+typedef std::map<void*, unsigned int> TouchPointsIdMapping;
+
 @interface GraphicsWindowIOSGLView : UIView
 {
     @private
@@ -107,13 +109,18 @@
         
         // for multisampled antialiased rendering
         GLuint _msaaFramebuffer, _msaaRenderBuffer, _msaaDepthBuffer;
+        
+        TouchPointsIdMapping* _touchPointsIdMapping;
+        unsigned int _lastTouchPointId;
+
+
     
 }
 
 - (void)setGraphicsWindow: (osgViewer::GraphicsWindowIOS*) win;
 - (osgViewer::GraphicsWindowIOS*) getGraphicsWindow;
 - (void)setOpenGLContext: (EAGLContext*) context;
-
+- (void)updateDimensions;
 - (BOOL)createFramebuffer;
 - (void)destroyFramebuffer;
 - (void)swapBuffers;
@@ -155,12 +162,72 @@
 
 }
 
+
+- (unsigned int)computeTouchId: (UITouch*) touch mayCleanup: (bool)may_cleanup
+{
+    unsigned int result(0);
+    
+    if (!_touchPointsIdMapping) {
+        _lastTouchPointId = 0;
+        _touchPointsIdMapping = new TouchPointsIdMapping();
+    }
+    
+    switch([touch phase])
+    {
+    
+        case UITouchPhaseBegan:
+            {
+                TouchPointsIdMapping::iterator itr = _touchPointsIdMapping->find(touch);
+                // std::cout << "new: " << touch << " num: " << _touchPointsIdMapping->size() << " found: " << (itr != _touchPointsIdMapping->end()) << std::endl;
+                 
+                if (itr == _touchPointsIdMapping->end()) 
+                {
+                    (*_touchPointsIdMapping)[touch] = result = _lastTouchPointId;
+                    _lastTouchPointId++;
+                    break;
+                }
+               
+            }
+            // missing "break" by intention!
+        
+        case UITouchPhaseMoved:
+        case UITouchPhaseStationary:
+            {
+                result = (*_touchPointsIdMapping)[touch];
+            }
+            break;
+       
+        case UITouchPhaseEnded:
+        case UITouchPhaseCancelled:
+            {
+                TouchPointsIdMapping::iterator itr = _touchPointsIdMapping->find(touch);
+                // std::cout<< "remove: " << touch << " num: " << _touchPointsIdMapping->size() << " found: " << (itr != _touchPointsIdMapping->end()) << std::endl;
+                if (itr != _touchPointsIdMapping->end()) {
+                    result = itr->second;
+                    if(may_cleanup)
+                        _touchPointsIdMapping->erase(itr);
+                }
+                if(_touchPointsIdMapping->size() == 0) {
+                    _lastTouchPointId = 0;
+                }
+                // std::cout<< "remove: " << touch << " num: " << _touchPointsIdMapping->size() << std::endl;
+            }
+            break;
+            
+        default:
+            break;
+    }
+        
+    return result;
+}
+
+
 - (osg::Vec2) convertPointToPixel: (osg::Vec2) point
 {
     //get the views contentscale factor and multiply the point by it
     float scale = 1.0f;
     
-#ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
     scale = self.contentScaleFactor;
 #endif
     return osg::Vec2(point.x()*scale, point.y()*scale);
@@ -170,6 +237,8 @@
 -(void) setGraphicsWindow: (osgViewer::GraphicsWindowIOS*) win
 {
     _win = win;
+    _touchPointsIdMapping = new TouchPointsIdMapping();
+    _lastTouchPointId = 0;
 }
 
 - (osgViewer::GraphicsWindowIOS*) getGraphicsWindow {
@@ -198,16 +267,22 @@
         // Get the layer
         CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
         
-        eaglLayer.opaque = YES;//need to look into this, can't remember why it's here, i.e. do I set it to no for alphaed window?
+        osgViewer::GraphicsWindowIOS::WindowData* win_data(NULL);
+        if (_win->getTraits()->inheritedWindowData.valid())
+            win_data = dynamic_cast<osgViewer::GraphicsWindowIOS::WindowData*>(_win->getTraits()->inheritedWindowData.get());
+        
+        eaglLayer.opaque = win_data ? !win_data->getCreateTransparentView() : YES;
+        bool retained_backing = win_data ? win_data->getUseRetainedBacking() : NO;
+
         if(_win->getTraits()->alpha > 0)
         {
             //create layer with alpha channel RGBA8
             eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
-                                            [NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];
+                                            [NSNumber numberWithBool:retained_backing], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];
         }else{
             //else no alpha, IOS uses RBG565
             eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
-                                            [NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGB565, kEAGLDrawablePropertyColorFormat, nil];
+                                            [NSNumber numberWithBool:retained_backing], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGB565, kEAGLDrawablePropertyColorFormat, nil];
 
         }
     }
@@ -222,19 +297,43 @@
 - (void) dealloc
 {
     OSG_INFO << "GraphicsWindowIOSGLView::dealloc" << std::endl;
-    
+    if(_touchPointsIdMapping) 
+        delete _touchPointsIdMapping;
+    _touchPointsIdMapping = NULL;
     [super dealloc];
 }
 
 - (void)layoutSubviews {
-    /*
-    [EAGLContext setCurrentContext:_context];
-    [self destroyFramebuffer];
-    [self createFramebuffer];
-    */
+    [super layoutSubviews];
+    [self updateDimensions];
 }
 
 
+- (void) setFrame:(CGRect)frame
+{
+    [super setFrame:frame];
+    [self updateDimensions];
+}
+
+
+- (void) updateDimensions
+{
+    if (_win)
+    {
+        CGRect frame = self.bounds;
+        osg::Vec2 pointOrigin = osg::Vec2(frame.origin.x,frame.origin.y);
+        osg::Vec2 pointSize = osg::Vec2(frame.size.width,frame.size.height);
+        osg::Vec2 pixelOrigin = [(GraphicsWindowIOSGLView*)(self) convertPointToPixel:pointOrigin];
+        osg::Vec2 pixelSize = [(GraphicsWindowIOSGLView*)(self) convertPointToPixel:pointSize];
+        
+        OSG_INFO << "updateDimensions, resize to "
+            <<  pixelOrigin.x() << " " << pixelOrigin.y() << " " 
+            << pixelSize.x() << " " << pixelSize.y() 
+            << std::endl;
+        _win->resized(pixelOrigin.x(), pixelOrigin.y(), pixelSize.x(), pixelSize.y());
+    }
+}
+
 - (BOOL)createFramebuffer {
 
     _msaaFramebuffer = _msaaRenderBuffer = 0;
@@ -255,6 +354,37 @@
     
     osg::notify(osg::DEBUG_INFO) << "GraphicsWindowIOS::createFramebuffer INFO: Created GL RenderBuffer of size " << _backingWidth << ", " << _backingHeight << " ." << std::endl;
 
+#if __IPHONE_OS_VERSION_MIN_REQUIRED >= 50000
+    //on ios 5 we have to use a packed depth stencil buffer if we want stencil
+    if(_win->getTraits()->depth > 0) {
+        //add stencil if requested
+        if(_win->getTraits()->stencil > 0) {
+            // Create a packed depth stencil buffer.
+            glGenRenderbuffersOES(1, &_depthRenderbuffer);
+            glBindRenderbufferOES(GL_RENDERBUFFER_OES, _depthRenderbuffer);
+            
+            glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH24_STENCIL8_OES, _backingWidth, _backingHeight);
+            
+            glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES,
+                                         GL_RENDERBUFFER_OES, _depthRenderbuffer);
+            glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_STENCIL_ATTACHMENT_OES,
+                                         GL_RENDERBUFFER_OES, _depthRenderbuffer);
+        }else{
+            //normal depth buffer
+            glGenRenderbuffersOES(1, &_depthRenderbuffer);
+            glBindRenderbufferOES(GL_RENDERBUFFER_OES, _depthRenderbuffer);
+            if(_win->getTraits()->depth == 16)
+            {
+                glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, _backingWidth, _backingHeight);
+            }else if(_win->getTraits()->depth == 24){
+                glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT24_OES, _backingWidth, _backingHeight);
+            }
+
+            glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, _depthRenderbuffer);
+        }
+    }
+    
+#else
     //add depth if requested
     if(_win->getTraits()->depth > 0) {
         glGenRenderbuffersOES(1, &_depthRenderbuffer);
@@ -265,6 +395,13 @@
         }else if(_win->getTraits()->depth == 24){
             glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT24_OES, _backingWidth, _backingHeight);
         }
+
+#if defined(GL_DEPTH_COMPONENT32_OES)
+        else if(_win->getTraits()->depth == 32){
+            glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT32_OES, _backingWidth, _backingHeight);
+        }
+#endif
+
         glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, _depthRenderbuffer);
     }
     
@@ -274,11 +411,12 @@
         glBindRenderbufferOES(GL_RENDERBUFFER_OES, _stencilBuffer);
         glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_STENCIL_INDEX8_OES, _backingWidth, _backingHeight);
         glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_STENCIL_ATTACHMENT_OES, GL_RENDERBUFFER_OES, _stencilBuffer);
-    }    
+    }  
+#endif
     
     //MSAA only available for >= 4.0 sdk
     
-#ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
     
     if(_win->getTraits()->sampleBuffers > 0) 
     {
@@ -298,9 +436,14 @@
         glGenRenderbuffersOES(1, &_msaaDepthBuffer); 
         glBindRenderbufferOES(GL_RENDERBUFFER_OES, _msaaDepthBuffer);
         
-        glRenderbufferStorageMultisampleAPPLE(GL_RENDERBUFFER_OES, _win->getTraits()->samples, ( _win->getTraits()->depth == 16) ? GL_DEPTH_COMPONENT16_OES : GL_DEPTH_COMPONENT24_OES, _backingWidth , _backingHeight);
+        GLuint attachmentType = (_win->getTraits()->stencil > 0) ? GL_DEPTH24_STENCIL8_OES : ((_win->getTraits()->depth == 16) ? GL_DEPTH_COMPONENT16_OES : GL_DEPTH_COMPONENT24_OES);
+        glRenderbufferStorageMultisampleAPPLE(GL_RENDERBUFFER_OES, _win->getTraits()->samples, attachmentType, _backingWidth , _backingHeight);
+        
         glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, _msaaDepthBuffer);
-    
+        if (_win->getTraits()->stencil > 0) 
+        {
+            glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_STENCIL_ATTACHMENT_OES, GL_RENDERBUFFER_OES, _msaaDepthBuffer);
+        }
     }
 #endif
     
@@ -333,17 +476,17 @@
     }
     
     if(_stencilBuffer) {
-        glDeleteFramebuffersOES(1, &_stencilBuffer);
+        glDeleteRenderbuffersOES(1, &_stencilBuffer);
         _stencilBuffer = 0;
     }
     
     if(_msaaRenderBuffer) {
-        glDeleteFramebuffersOES(1, &_msaaRenderBuffer);
+        glDeleteRenderbuffersOES(1, &_msaaRenderBuffer);
         _msaaRenderBuffer = 0;
     }
     
     if(_msaaDepthBuffer) {
-        glDeleteFramebuffersOES(1, &_msaaDepthBuffer);
+        glDeleteRenderbuffersOES(1, &_msaaDepthBuffer);
         _msaaDepthBuffer = 0;
     }
 
@@ -359,7 +502,7 @@
 - (void)swapBuffers {
 
 
-#ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)    
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)    
     if(_msaaFramebuffer) 
     {
         glBindFramebufferOES(GL_FRAMEBUFFER_OES, _msaaFramebuffer);
@@ -384,7 +527,7 @@
     //re bind the frame buffer for next frames renders
     glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
     
-#ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
     if (_msaaFramebuffer)
         glBindFramebufferOES(GL_FRAMEBUFFER_OES, _msaaFramebuffer);;
 #endif
@@ -398,7 +541,7 @@
     //bind the frame buffer
     glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
     
-#ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
     if (_msaaFramebuffer)
         glBindFramebufferOES(GL_READ_FRAMEBUFFER_APPLE, _msaaFramebuffer);
 #endif
@@ -433,16 +576,18 @@
     {
         
         UITouch *touch = [[allTouches allObjects] objectAtIndex:i];
-        CGPoint pos = [touch locationInView:touch.view];
+        CGPoint pos = [touch locationInView:self];
         osg::Vec2 pixelPos = [self convertPointToPixel: osg::Vec2(pos.x,pos.y)];
+        unsigned int touch_id = [self computeTouchId: touch mayCleanup: FALSE];
         
         if (!osg_event) {
-            osg_event = _win->getEventQueue()->touchBegan(i, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
+            osg_event = _win->getEventQueue()->touchBegan(touch_id, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
         } else {
-            osg_event->addTouchPoint(i, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
+            osg_event->addTouchPoint(touch_id, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
         }
     }
     
+    [super touchesBegan:touches withEvent:event];
 }
 
 - (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
@@ -454,17 +599,18 @@
     for(int i=0; i<[allTouches count]; i++)
     {
         UITouch *touch = [[allTouches allObjects] objectAtIndex:i];
-        CGPoint pos = [touch locationInView:touch.view];
+        CGPoint pos = [touch locationInView:self];
         osg::Vec2 pixelPos = [self convertPointToPixel: osg::Vec2(pos.x,pos.y)];
-        
+        unsigned int touch_id = [self computeTouchId: touch mayCleanup: FALSE];
+
         if (!osg_event) {
-            osg_event = _win->getEventQueue()->touchMoved(i, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
+            osg_event = _win->getEventQueue()->touchMoved(touch_id, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
         } else {
-            osg_event->addTouchPoint(i, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
+            osg_event->addTouchPoint(touch_id, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y());
         }
-
-
     }
+    
+    [super touchesMoved:touches withEvent:event];    
 }
 
 - (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
@@ -477,18 +623,23 @@
     for(int i=0; i<[allTouches count]; i++)
     {
         UITouch *touch = [[allTouches allObjects] objectAtIndex:i];
-        CGPoint pos = [touch locationInView:touch.view];
+        CGPoint pos = [touch locationInView:self];
         osg::Vec2 pixelPos = [self convertPointToPixel: osg::Vec2(pos.x,pos.y)];
-        
+        unsigned int touch_id = [self computeTouchId: touch mayCleanup: TRUE];
         if (!osg_event) {
-            osg_event = _win->getEventQueue()->touchEnded(i, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y(), [touch tapCount]);
+            osg_event = _win->getEventQueue()->touchEnded(touch_id, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y(), [touch tapCount]);
         } else {
-            osg_event->addTouchPoint(i, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y(), [touch tapCount]);
+            osg_event->addTouchPoint(touch_id, [self convertTouchPhase: [touch phase]], pixelPos.x(), pixelPos.y(), [touch tapCount]);
         }
-
     }
+    
+    [super touchesEnded:touches withEvent:event];    
 }
 
+-(void) touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event 
+{
+    [self touchesEnded: touches withEvent:event];
+}
 
 @end
 
@@ -547,20 +698,7 @@
 
 - (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation duration:(NSTimeInterval)duration 
 {
-    osgViewer::GraphicsWindowIOS* win = [(GraphicsWindowIOSGLView*)(self.view) getGraphicsWindow];
-    if (win) {
-        CGRect frame = self.view.bounds;
-        osg::Vec2 pointOrigin = osg::Vec2(frame.origin.x,frame.origin.y);
-        osg::Vec2 pointSize = osg::Vec2(frame.size.width,frame.size.height);
-        osg::Vec2 pixelOrigin = [(GraphicsWindowIOSGLView*)(self.view) convertPointToPixel:pointOrigin];
-        osg::Vec2 pixelSize = [(GraphicsWindowIOSGLView*)(self.view) convertPointToPixel:pointSize];
-       OSG_INFO << "willAnimateRotationToInterfaceOrientation, resize to " 
-            <<  pixelOrigin.x() << " " << pixelOrigin.y() << " " 
-            << pixelSize.x() << " " << pixelSize.y() 
-            << std::endl;
-        win->resized(pixelOrigin.x(), pixelOrigin.y(), pixelSize.x(), pixelSize.y());
-    }
-
+    [(GraphicsWindowIOSGLView*)(self.view) updateDimensions];
 }
 
 
@@ -596,6 +734,9 @@ void GraphicsWindowIOS::init()
     //if -1.0 we use the screens scale factor
     _viewContentScaleFactor = -1.0f;
     _valid = _initialized = true;
+
+    // make sure the event queue has the correct window rectangle size and input range
+    getEventQueue()->syncWindowRectangleWithGraphcisContext();
 }
 
 
@@ -605,6 +746,8 @@ void GraphicsWindowIOS::init()
 
 bool GraphicsWindowIOS::realizeImplementation()
 {
+    if (_realized) return true;
+    
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     
     BOOL bar_hidden = (_traits->windowDecoration) ? NO: YES;
@@ -725,7 +868,7 @@ bool GraphicsWindowIOS::realizeImplementation()
     
     //Apply our content scale factor to our view, this is what converts the views points
     //size to our desired context size.
-#ifdef __IPHONE_4_0 && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
     theView.contentScaleFactor = _viewContentScaleFactor;
 #endif    
     [theView setGraphicsWindow: this];
@@ -742,11 +885,14 @@ bool GraphicsWindowIOS::realizeImplementation()
     
     // Attach view to window
     [_window addSubview: _view];
+    if ([_window isKindOfClass:[UIWindow class]])
+        _window.rootViewController = _viewController;
     [theView release];
     
     //if we own the window also make it visible
     if (_ownsWindow) 
     {
+        
         //show window
         [_window makeKeyAndVisible];
     }
@@ -755,6 +901,9 @@ bool GraphicsWindowIOS::realizeImplementation()
     
     // IOSs origin is top/left:
     getEventQueue()->getCurrentEventState()->setMouseYOrientation(osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS);
+
+    // make sure the event queue has the correct window rectangle size and input range
+    getEventQueue()->syncWindowRectangleWithGraphcisContext();
     
     _valid = _initialized = _realized = true;
     return _valid;
@@ -929,10 +1078,10 @@ bool GraphicsWindowIOS::setWindowRectangleImplementation(int x, int y, int width
 }
 
     
-void GraphicsWindowIOS::checkEvents()
+bool GraphicsWindowIOS::checkEvents()
 {
-    
-    
+
+    return !(getEventQueue()->empty());
 }
 
 
