@@ -29,6 +29,7 @@
 
 #include <X11/Xmd.h>
 #include <X11/keysym.h>
+#include <X11/XKBlib.h>
 #include <X11/cursorfont.h>
 
 #include <X11/Xmd.h>        /* For CARD16 */
@@ -373,6 +374,11 @@ bool GraphicsWindowX11::checkAndSendEventFullScreenIfNeeded(Display* display, in
     wsi->getScreenResolution(*_traits, screenWidth, screenHeight);
     bool isFullScreen = x == 0 && y == 0 && width == (int)screenWidth && height == (int)screenHeight && !windowDecoration;
 
+    if (isFullScreen) {
+        resized(x, y, width, height);
+        getEventQueue()->windowResize(x, y, width, height, getEventQueue()->getTime());
+    }
+
     Atom netWMStateAtom = XInternAtom(display, "_NET_WM_STATE", True);
     Atom netWMStateFullscreenAtom = XInternAtom(display, "_NET_WM_STATE_FULLSCREEN", True);
 
@@ -448,7 +454,7 @@ bool GraphicsWindowX11::setWindowDecorationImplementation(bool flag)
             // if traits says not resize we want to set the functions to exlude MWM_FUNC_RESIZE,
             // but this bitmask needs to be set if the MWM_FUNC_ALL bit is already set in order to toggle it off.
             if (_traits.valid() && !_traits->supportsResize) wmHints.functions = wmHints.functions | MWM_FUNC_RESIZE;
-            
+
         }
         else
         {
@@ -458,7 +464,7 @@ bool GraphicsWindowX11::setWindowDecorationImplementation(bool flag)
             wmHints.inputMode = 0;
             wmHints.status = 0;
         }
-        
+
         XChangeProperty( display, _window, atom, atom, 32, PropModeReplace, (unsigned char *)&wmHints,  5 );
         result = true;
     }
@@ -489,7 +495,7 @@ bool GraphicsWindowX11::setWindowRectangleImplementation(int x, int y, int width
 
     XFlush(display);
     XSync(display, 0);
-    
+
     // add usleep here to give window manager a chance to handle the request, if
     // we don't add this sleep then any X11 calls right afterwards can produce
     // X11 errors.
@@ -712,7 +718,7 @@ void GraphicsWindowX11::init()
     }
 
     // get any shared GLX contexts
-    GraphicsHandleX11* graphicsHandleX11 = dynamic_cast<GraphicsHandleX11*>(_traits->sharedContext);
+    GraphicsHandleX11* graphicsHandleX11 = dynamic_cast<GraphicsHandleX11*>(_traits->sharedContext.get());
     Context sharedContext = graphicsHandleX11 ? graphicsHandleX11->getContext() : 0;
 
     #ifdef OSG_USE_EGL
@@ -753,7 +759,7 @@ void GraphicsWindowX11::init()
         if (_traits->samples) { attributes.push_back(EGL_SAMPLES); attributes.push_back(_traits->samples); }
 
         attributes.push_back(EGL_RENDERABLE_TYPE); attributes.push_back(OSG_EGL_OPENGL_TARGET_BIT);
-        
+
         attributes.push_back(EGL_NONE);
         attributes.push_back(EGL_NONE);
 
@@ -835,8 +841,7 @@ void GraphicsWindowX11::init()
         }
     }
 
-
-
+    getEventQueue()->syncWindowRectangleWithGraphcisContext();
 }
 
 bool GraphicsWindowX11::createWindow()
@@ -910,7 +915,7 @@ bool GraphicsWindowX11::createWindow()
             doFullSceenWorkAround = true;
         }
     }
-    
+
     _window = XCreateWindow( _display, _parent,
                              x,
                              y,
@@ -947,7 +952,7 @@ bool GraphicsWindowX11::createWindow()
 
     setWindowDecoration(_traits->windowDecoration);
 
-    
+
     useCursor(_traits->useCursor);
 
     _deleteWindow = XInternAtom (_display, "WM_DELETE_WINDOW", False);
@@ -957,11 +962,14 @@ bool GraphicsWindowX11::createWindow()
     XFlush( _display );
     XSync( _display, 0 );
 
-    // now update the window dimensions to account for any size changes made by the window manager,
-    XGetWindowAttributes( _display, _window, &watt );
 
-    if (_traits->x != watt.x || _traits->y != watt.y
-        ||_traits->width != watt.width || _traits->height != watt.height)
+    // get window geometry relative to root window/screen
+    Window child_return;
+    int windowX, windowY;
+    XGetWindowAttributes( _display, _window, &watt );
+    XTranslateCoordinates( _display, _window, watt.root, watt.x, watt.y, &windowX, &windowY, &child_return);
+
+    if (_traits->x != windowX || _traits->y != windowY ||_traits->width != watt.width || _traits->height != watt.height)
     {
 
         if (doFullSceenWorkAround)
@@ -973,9 +981,10 @@ bool GraphicsWindowX11::createWindow()
             XSync(_display, 0);
 
             XGetWindowAttributes( _display, _window, &watt );
+            XTranslateCoordinates( _display, _window, watt.root, watt.x, watt.y, &windowX, &windowY, &child_return);
         }
 
-        resized( watt.x, watt.y, watt.width, watt.height );
+        resized( windowX, windowY, watt.width, watt.height );
     }
 
     //OSG_NOTICE<<"After sync apply.x = "<<watt.x<<" watt.y="<<watt.y<<" width="<<watt.width<<" height="<<watt.height<<std::endl;
@@ -1050,7 +1059,9 @@ bool GraphicsWindowX11::realizeImplementation()
 
     XMapWindow( _display, _window );
 
-//    Window temp = _window;
+    getEventQueue()->syncWindowRectangleWithGraphcisContext();
+
+    //    Window temp = _window;
 //    XSetWMColormapWindows( _display, _window, &temp, 1);
 
     _realized = true;
@@ -1190,9 +1201,9 @@ void GraphicsWindowX11::swapBuffersImplementation()
     }
 }
 
-void GraphicsWindowX11::checkEvents()
+bool GraphicsWindowX11::checkEvents()
 {
-    if (!_realized) return;
+    if (!_realized) return false;
 
     Display* display = _eventDisplay;
 
@@ -1200,6 +1211,8 @@ void GraphicsWindowX11::checkEvents()
     double eventTime = baseTime;
     double resizeTime = eventTime;
     _timeOfLastCheckEvents = getEventQueue()->getTime();
+    if (baseTime>_timeOfLastCheckEvents) baseTime = _timeOfLastCheckEvents;
+
 
     // OSG_NOTICE<<"GraphicsWindowX11::checkEvents() : getEventQueue()->getCurrentEventState()->getGraphicsContext()="<<getEventQueue()->getCurrentEventState()->getGraphicsContext()<<std::endl;
 
@@ -1220,10 +1233,10 @@ void GraphicsWindowX11::checkEvents()
         {
             case ClientMessage:
             {
-                OSG_NOTICE<<"ClientMessage event received"<<std::endl;
+                OSG_INFO<<"ClientMessage event received"<<std::endl;
                 if (static_cast<Atom>(ev.xclient.data.l[0]) == _deleteWindow)
                 {
-                    OSG_NOTICE<<"DeleteWindow event received"<<std::endl;
+                    OSG_INFO<<"DeleteWindow event received"<<std::endl;
                     // FIXME only do if _ownsWindow ?
                     getEventQueue()->closeWindow(eventTime);
                 }
@@ -1243,17 +1256,15 @@ void GraphicsWindowX11::checkEvents()
                 break;
 
             case DestroyNotify :
-                OSG_NOTICE<<"DestroyNotify event received"<<std::endl;
+                OSG_INFO<<"DestroyNotify event received"<<std::endl;
                 _realized =  false;
                 _valid = false;
                 break;
 
             case ConfigureNotify :
             {
-                OSG_INFO<<"ConfigureNotify x="<<ev.xconfigure.x<<" y="<<ev.xconfigure.y<<" width="<<ev.xconfigure.width<<", height="<<ev.xconfigure.height<<std::endl;
-
                 if (windowX != ev.xconfigure.x ||
-                    windowX != ev.xconfigure.y ||
+                    windowY != ev.xconfigure.y ||
                     windowWidth != ev.xconfigure.width ||
                     windowHeight != ev.xconfigure.height)
                 {
@@ -1276,8 +1287,6 @@ void GraphicsWindowX11::checkEvents()
                     XGetWindowAttributes(display, _window, &watt );
                 while( watt.map_state != IsViewable );
 
-                OSG_INFO<<"MapNotify x="<<watt.x<<" y="<<watt.y<<" width="<<watt.width<<", height="<<watt.height<<std::endl;
-
                 if (windowWidth != watt.width || windowHeight != watt.height)
                 {
                     resizeTime = eventTime;
@@ -1467,11 +1476,13 @@ void GraphicsWindowX11::checkEvents()
 
                 if( ev.xbutton.button == Button4 )
                 {
-                    getEventQueue()->mouseScroll(osgGA::GUIEventAdapter::SCROLL_UP, eventTime);
+                    //ignore scroll release as X11 is generating button press and release pairs for scroll wheel motion and we only need only event for each scroll action
+                    //getEventQueue()->mouseScroll(osgGA::GUIEventAdapter::SCROLL_UP, eventTime);
                 }
                 else if( ev.xbutton.button == Button5)
                 {
-                    getEventQueue()->mouseScroll(osgGA::GUIEventAdapter::SCROLL_DOWN, eventTime);
+                    //ignore scroll release as X11 is generating button press and release pairs for scroll wheel motion and we only need only event for each scroll action
+                    //getEventQueue()->mouseScroll(osgGA::GUIEventAdapter::SCROLL_DOWN, eventTime);
                 }
                 else
                 {
@@ -1532,19 +1543,30 @@ void GraphicsWindowX11::checkEvents()
             }
 
             default:
-                OSG_NOTICE<<"Other event "<<ev.type<<std::endl;
+                OSG_INFO<<"Other event "<<ev.type<<std::endl;
                 break;
 
         }
         _lastEventType = ev.type;
     }
 
+
+    // get window geometry relative to root window/screen
+    {
+        XWindowAttributes watt;
+        Window child_return;
+
+        XGetWindowAttributes( display, _window, &watt );
+        XTranslateCoordinates(display, _window, watt.root, watt.x, watt.y, &windowX, &windowY, &child_return);
+    }
+
     // send window resize event if window position or size was changed
     if (windowX != _traits->x ||
         windowY != _traits->y ||
         windowWidth != _traits->width ||
         windowHeight != _traits->height)
     {
+
         resized(windowX, windowY, windowWidth, windowHeight);
         getEventQueue()->windowResize(windowX, windowY, windowWidth, windowHeight, resizeTime);
 
@@ -1555,6 +1577,8 @@ void GraphicsWindowX11::checkEvents()
             requestRedraw();
         }
     }
+
+    return !(getEventQueue()->empty());
 }
 
 void GraphicsWindowX11::grabFocus()
@@ -1612,7 +1636,7 @@ void GraphicsWindowX11::adaptKey(XKeyEvent& keyevent, int& keySymbol, int& unmod
         keySymbol = buffer_return[0];
     }
 
-    unmodifiedKeySymbol = XKeycodeToKeysym(keyevent.display, keyevent.keycode, 0);
+    unmodifiedKeySymbol = XkbKeycodeToKeysym(keyevent.display, keyevent.keycode, 0, 0);
 }
 
 // Function to inject artificial key presses/releases.
@@ -2140,31 +2164,61 @@ extern "C" void graphicswindow_X11(void)
     osg::GraphicsContext::setWindowingSystemInterface(new X11WindowingSystemInterface);
 }
 
-
 void GraphicsWindowX11::raiseWindow()
 {
     Display* display = getDisplayToUse();
-    XWindowAttributes winAttrib;
+    if(!display) return;
 
-    Window root_return, parent_return, *children;
-    unsigned int nchildren, i=0;
-    XTextProperty windowName;
-    bool xraise = false;
-
-
-    XQueryTree(display, _parent, &root_return, &parent_return, &children, &nchildren);
-    while (!xraise &&  i<nchildren)
+    // get handles to window props of interest
+    Atom stateAbove = XInternAtom(display, "_NET_WM_STATE_ABOVE", True);
+    Atom stateAtom = XInternAtom(display, "_NET_WM_STATE", True);
+    // check that atoms are supported
+    if(stateAbove != None && stateAtom != None)
     {
-    XGetWMName(display,children[i++],&windowName);
-        if ((windowName.nitems != 0) && (strcmp(_traits->windowName.c_str(),(const char *)windowName.value) == 0)) xraise = true;
+        // fill an XEvent struct to send
+        XEvent xev;
+        xev.xclient.type = ClientMessage;
+        xev.xclient.serial = 0;
+        xev.xclient.send_event = True;
+        xev.xclient.window = _window;
+        xev.xclient.message_type = stateAtom;
+        xev.xclient.format = 32;
+        xev.xclient.data.l[0] = 1;
+        xev.xclient.data.l[1] = stateAbove;
+        xev.xclient.data.l[2] = 0;
+
+        XSendEvent(display, RootWindow(display, DefaultScreen(display)),
+                   False,  SubstructureRedirectMask | SubstructureNotifyMask, &xev);
     }
-    if (xraise) XRaiseWindow(display,_window);
     else
     {
-    XGetWindowAttributes(display, _window, &winAttrib);
-    XReparentWindow(display, _window, _parent, winAttrib.x, winAttrib.y);
+        // one or both of _NET_WM_STATE and _NET_WM_STATE_ABOVE aren't supported
+        // try to use XRaiseWindow
+        XWindowAttributes winAttrib;
+
+        Window root_return, parent_return, *children;
+        unsigned int nchildren, i=0;
+        XTextProperty windowName;
+        bool xraise = false;
+
+        // get the window tree around our current window
+        XQueryTree(display, _parent, &root_return, &parent_return, &children, &nchildren);
+        while (!xraise &&  i<nchildren)
+        {
+            XGetWMName(display,children[i++],&windowName);
+            if ((windowName.nitems != 0) && (strcmp(_traits->windowName.c_str(),(const char *)windowName.value) == 0))
+                xraise = true;
+        }
+        if (xraise)
+            XRaiseWindow(display,_window);
+        else
+        {
+            XGetWindowAttributes(display, _window, &winAttrib);
+            XReparentWindow(display, _window, _parent, winAttrib.x, winAttrib.y);
+        }
+
+        XFree(children);
     }
-    XFree(children);
 
     XFlush(display);
     XSync(display,0);
