@@ -219,7 +219,7 @@ bool StatsHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdap
                         }
                     }
 
-                    for(unsigned int i = viewer->getViewerStats()->getEarliestFrameNumber(); i<= viewer->getViewerStats()->getLatestFrameNumber()-1; ++i)
+                    for(unsigned int i = viewer->getViewerStats()->getEarliestFrameNumber(); i< viewer->getViewerStats()->getLatestFrameNumber(); ++i)
                     {
                         for(StatsList::iterator itr = statsList.begin();
                             itr != statsList.end();
@@ -235,6 +235,9 @@ bool StatsHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdap
                 return true;
             }
         }
+        case(osgGA::GUIEventAdapter::RESIZE):
+            setWindowSize(ea.getWindowWidth(), ea.getWindowHeight());
+            break;
         default: break;
     }
 
@@ -263,8 +266,8 @@ void StatsHandler::reset()
     _camera->removeChildren( 0, _camera->getNumChildren() );
 }
 
-void StatsHandler::addUserStatsLine(const std::string& label, const osg::Vec4& textColor, const osg::Vec4& barColor, 
-                                    const std::string& timeTakenName, float multiplier, bool average, bool averageInInverseSpace, 
+void StatsHandler::addUserStatsLine(const std::string& label, const osg::Vec4& textColor, const osg::Vec4& barColor,
+                                    const std::string& timeTakenName, float multiplier, bool average, bool averageInInverseSpace,
                                     const std::string& beginTimeName, const std::string& endTimeName, float maxValue)
 {
     _userStatsLines.push_back(UserStatsLine(label, textColor, barColor, timeTakenName, multiplier, average, averageInInverseSpace, beginTimeName, endTimeName, maxValue));
@@ -315,22 +318,37 @@ void StatsHandler::setUpHUDCamera(osgViewer::ViewerBase* viewer)
 
     _camera->setGraphicsContext(context);
 
-    _camera->setViewport(0, 0, context->getTraits()->width, context->getTraits()->height);
-    
     _camera->setRenderOrder(osg::Camera::POST_RENDER, 10);
 
-    _camera->setProjectionMatrix(osg::Matrix::ortho2D(0.0,_statsWidth,0.0,_statsHeight));
     _camera->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
     _camera->setViewMatrix(osg::Matrix::identity());
+    setWindowSize(context->getTraits()->width, context->getTraits()->height);
 
     // only clear the depth buffer
     _camera->setClearMask(0);
+    _camera->setAllowEventFocus(false);
 
     _camera->setRenderer(new Renderer(_camera.get()));
 
     _initialized = true;
 }
 
+void StatsHandler::setWindowSize(int width, int height)
+{
+    if (width <= 0 || height <= 0)
+        return;
+
+    _camera->setViewport(0, 0, width, height);
+    if (fabs(height*_statsWidth) <= fabs(width*_statsHeight))
+    {
+        _camera->setProjectionMatrix(osg::Matrix::ortho2D(0.0,width*_statsHeight/height,0.0,_statsHeight));
+    }
+    else
+    {
+        _camera->setProjectionMatrix(osg::Matrix::ortho2D(0.0,_statsWidth,_statsHeight-height*_statsWidth/width,_statsHeight));
+    }
+}
+
 // Drawcallback to draw averaged attribute
 struct AveragedValueTextDrawCallback : public virtual osg::Drawable::DrawCallback
 {
@@ -682,8 +700,7 @@ osg::Geometry* StatsHandler::createBackgroundRectangle(const osg::Vec3& pos, con
 
     osg::Vec4Array* colors = new osg::Vec4Array;
     colors->push_back(color);
-    geometry->setColorArray(colors);
-    geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
+    geometry->setColorArray(colors, osg::Array::BIND_OVERALL);
 
     osg::DrawElementsUShort *base =  new osg::DrawElementsUShort(osg::PrimitiveSet::TRIANGLE_FAN,0);
     base->push_back(0);
@@ -731,8 +748,7 @@ protected:
 
             osg::Vec4Array* colors = new osg::Vec4Array;
             colors->push_back(color);
-            setColorArray(colors);
-            setColorBinding(osg::Geometry::BIND_OVERALL);
+            setColorArray(colors, osg::Array::BIND_OVERALL);
 
             setDrawCallback(new GraphUpdateCallback(pos, width, height, viewerStats, stats, max, nameBegin, nameEnd));
         }
@@ -863,8 +879,7 @@ osg::Geometry* StatsHandler::createGeometry(const osg::Vec3& pos, float height,
 
     osg::Vec4Array* colours = new osg::Vec4Array;
     colours->push_back(colour);
-    geometry->setColorArray(colours);
-    geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
+    geometry->setColorArray(colours, osg::Array::BIND_OVERALL);
 
     geometry->addPrimitiveSet(new osg::DrawArrays(GL_QUADS, 0, numBlocks*4));
 
@@ -1017,8 +1032,7 @@ osg::Geometry* StatsHandler::createFrameMarkers(const osg::Vec3& pos, float heig
 
     osg::Vec4Array* colours = new osg::Vec4Array;
     colours->push_back(colour);
-    geometry->setColorArray(colours);
-    geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
+    geometry->setColorArray(colours, osg::Array::BIND_OVERALL);
 
     geometry->addPrimitiveSet(new osg::DrawArrays(GL_LINES, 0, numBlocks*2));
 
@@ -1044,8 +1058,7 @@ osg::Geometry* StatsHandler::createTick(const osg::Vec3& pos, float height, cons
 
     osg::Vec4Array* colours = new osg::Vec4Array;
     colours->push_back(colour);
-    geometry->setColorArray(colours);
-    geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
+    geometry->setColorArray(colours, osg::Array::BIND_OVERALL);
 
     geometry->addPrimitiveSet(new osg::DrawArrays(GL_LINES, 0, numTicks*2));
 
@@ -1209,16 +1222,16 @@ void StatsHandler::setUpScene(osgViewer::ViewerBase* viewer)
             pos.x() = _leftPos;
 
             UserStatsLine& line = _userStatsLines[i];
-            createTimeStatsLine(line.label, pos, line.textColor, line.barColor, viewer->getViewerStats(), viewer->getViewerStats(), 
+            createTimeStatsLine(line.label, pos, line.textColor, line.barColor, viewer->getViewerStats(), viewer->getViewerStats(),
                 line.timeTakenName, line.multiplier, line.average, line.averageInInverseSpace, line.beginTimeName, line.endTimeName);
-        
+
             pos.y() -= _characterSize*_lineHeight;
         }
 
         {
             pos.x() = _leftPos;
 
-            createTimeStatsLine("Event", pos, colorUpdate, colorUpdateAlpha, viewer->getViewerStats(), viewer->getViewerStats(), 
+            createTimeStatsLine("Event", pos, colorUpdate, colorUpdateAlpha, viewer->getViewerStats(), viewer->getViewerStats(),
                 "Event traversal time taken", 1000.0, true, false, "Event traversal begin time", "Event traversal end time");
 
             pos.y() -= _characterSize*_lineHeight;
@@ -1227,7 +1240,7 @@ void StatsHandler::setUpScene(osgViewer::ViewerBase* viewer)
         {
             pos.x() = _leftPos;
 
-            createTimeStatsLine("Update", pos, colorUpdate, colorUpdateAlpha, viewer->getViewerStats(), viewer->getViewerStats(), 
+            createTimeStatsLine("Update", pos, colorUpdate, colorUpdateAlpha, viewer->getViewerStats(), viewer->getViewerStats(),
                 "Update traversal time taken", 1000.0, true, false, "Update traversal begin time", "Update traversal end time");
 
             pos.y() -= _characterSize*_lineHeight;
@@ -1600,9 +1613,9 @@ void StatsHandler::setUpScene(osgViewer::ViewerBase* viewer)
 }
 
 void StatsHandler::createTimeStatsLine(const std::string& lineLabel,
-                                       osg::Vec3 pos, const osg::Vec4& textColor, const osg::Vec4& barColor, 
-                                       osg::Stats* viewerStats, osg::Stats* stats, 
-                                       const std::string& timeTakenName, float multiplier, bool average, bool averageInInverseSpace, 
+                                       osg::Vec3 pos, const osg::Vec4& textColor, const osg::Vec4& barColor,
+                                       osg::Stats* viewerStats, osg::Stats* stats,
+                                       const std::string& timeTakenName, float multiplier, bool average, bool averageInInverseSpace,
                                        const std::string& beginTimeName, const std::string& endTimeName)
 {
     osg::ref_ptr<osgText::Text> label = new osgText::Text;
@@ -1661,7 +1674,7 @@ void StatsHandler::createCameraTimeStats(osg::Vec3& pos, bool acquireGPUStats, o
     {
         pos.x() = _leftPos;
 
-        createTimeStatsLine("Cull", pos, colorCull, colorCullAlpha, viewerStats, stats, 
+        createTimeStatsLine("Cull", pos, colorCull, colorCullAlpha, viewerStats, stats,
             "Cull traversal time taken", 1000.0, true, false, "Cull traversal begin time", "Cull traversal end time");
 
         pos.y() -= _characterSize*_lineHeight;
@@ -1670,7 +1683,7 @@ void StatsHandler::createCameraTimeStats(osg::Vec3& pos, bool acquireGPUStats, o
     {
         pos.x() = _leftPos;
 
-        createTimeStatsLine("Draw", pos, colorDraw, colorDrawAlpha, viewerStats, stats, 
+        createTimeStatsLine("Draw", pos, colorDraw, colorDrawAlpha, viewerStats, stats,
             "Draw traversal time taken", 1000.0, true, false, "Draw traversal begin time", "Draw traversal end time");
 
         pos.y() -= _characterSize*_lineHeight;
@@ -1680,7 +1693,7 @@ void StatsHandler::createCameraTimeStats(osg::Vec3& pos, bool acquireGPUStats, o
     {
         pos.x() = _leftPos;
 
-        createTimeStatsLine("GPU", pos, colorGPU, colorGPUAlpha, viewerStats, stats, 
+        createTimeStatsLine("GPU", pos, colorGPU, colorGPUAlpha, viewerStats, stats,
             "GPU draw time taken", 1000.0, true, false, "GPU draw begin time", "GPU draw end time");
 
         pos.y() -= _characterSize*_lineHeight;
@@ -1690,8 +1703,8 @@ void StatsHandler::createCameraTimeStats(osg::Vec3& pos, bool acquireGPUStats, o
 
 void StatsHandler::getUsage(osg::ApplicationUsage& usage) const
 {
-    usage.addKeyboardMouseBinding("s","On screen stats.");
-    usage.addKeyboardMouseBinding("S","Output stats to console.");
+    usage.addKeyboardMouseBinding(_keyEventTogglesOnScreenStats,"On screen stats.");
+    usage.addKeyboardMouseBinding(_keyEventPrintsOutStats,"Output stats to console.");
 }
 
 }
