@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
 *
-* This library is open source and may be redistributed and/or modified under  
-* the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+* This library is open source and may be redistributed and/or modified under
+* the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
-* 
+*
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
 */
 
@@ -20,8 +20,10 @@
 
 #include <osgDB/FileNameUtils>
 
+#include <osg/Version>
 #include <osg/Geometry>
 #include <osg/TexMat>
+#include <osg/Texture2D>
 #include <osg/TextureRectangle>
 #include <osg/io_utils>
 
@@ -66,16 +68,16 @@ WindowSizeHandler::WindowSizeHandler() :
 
 void WindowSizeHandler::getUsage(osg::ApplicationUsage &usage) const
 {
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventToggleFullscreen), "Toggle full screen.");
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventWindowedResolutionUp), "Increase the screen resolution (in windowed mode).");
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventWindowedResolutionDown), "Decrease the screen resolution (in windowed mode).");
+    usage.addKeyboardMouseBinding(_keyEventToggleFullscreen, "Toggle full screen.");
+    usage.addKeyboardMouseBinding(_keyEventWindowedResolutionUp, "Increase the screen resolution (in windowed mode).");
+    usage.addKeyboardMouseBinding(_keyEventWindowedResolutionDown, "Decrease the screen resolution (in windowed mode).");
 }
 
 bool WindowSizeHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActionAdapter &aa)
 {
     osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
     if (!view) return false;
-    
+
     osgViewer::ViewerBase* viewer = view->getViewerBase();
 
     if (viewer == NULL)
@@ -91,8 +93,12 @@ bool WindowSizeHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActio
         {
             if (_toggleFullscreen == true && ea.getKey() == _keyEventToggleFullscreen)
             {
-                osgViewer::Viewer::Windows    windows;
 
+                // sleep to allow any viewer rendering threads to complete before we
+                // resize the window
+                OpenThreads::Thread::microSleep(100000);
+
+                osgViewer::Viewer::Windows windows;
                 viewer->getWindows(windows);
                 for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
                     itr != windows.end();
@@ -100,13 +106,18 @@ bool WindowSizeHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActio
                 {
                     toggleFullscreen(*itr);
                 }
+
+                aa.requestRedraw();
                 return true;
             }
             else if (_changeWindowedResolution == true && ea.getKey() == _keyEventWindowedResolutionUp)
             {
+                // sleep to allow any viewer rendering threads to complete before we
+                // resize the window
+                OpenThreads::Thread::microSleep(100000);
+
                 // Increase resolution
                 osgViewer::Viewer::Windows    windows;
-
                 viewer->getWindows(windows);
                 for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
                     itr != windows.end();
@@ -114,13 +125,18 @@ bool WindowSizeHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActio
                 {
                     changeWindowedResolution(*itr, true);
                 }
+
+                aa.requestRedraw();
                 return true;
             }
             else if (_changeWindowedResolution == true && ea.getKey() == _keyEventWindowedResolutionDown)
             {
+                // sleep to allow any viewer rendering threads to complete before we
+                // resize the window
+                OpenThreads::Thread::microSleep(100000);
+
                 // Decrease resolution
                 osgViewer::Viewer::Windows    windows;
-
                 viewer->getWindows(windows);
                 for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
                     itr != windows.end();
@@ -128,6 +144,8 @@ bool WindowSizeHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActio
                 {
                     changeWindowedResolution(*itr, false);
                 }
+
+                aa.requestRedraw();
                 return true;
             }
             break;
@@ -142,9 +160,9 @@ void WindowSizeHandler::toggleFullscreen(osgViewer::GraphicsWindow *window)
 {
     osg::GraphicsContext::WindowingSystemInterface    *wsi = osg::GraphicsContext::getWindowingSystemInterface();
 
-    if (wsi == NULL) 
+    if (wsi == NULL)
     {
-        osg::notify(osg::NOTICE) << "Error, no WindowSystemInterface available, cannot toggle window fullscreen." << std::endl;
+        OSG_NOTICE << "Error, no WindowSystemInterface available, cannot toggle window fullscreen." << std::endl;
         return;
     }
 
@@ -173,7 +191,7 @@ void WindowSizeHandler::toggleFullscreen(osgViewer::GraphicsWindow *window)
         resolution = _resolutionList[_currentResolutionIndex];
         window->setWindowDecoration(true);
         window->setWindowRectangle((screenWidth - (int)resolution.x()) / 2, (screenHeight - (int)resolution.y()) / 2, (int)resolution.x(), (int)resolution.y());
-        osg::notify(osg::INFO) << "Screen resolution = " << (int)resolution.x() << "x" << (int)resolution.y() << std::endl;  
+        OSG_INFO << "Screen resolution = " << (int)resolution.x() << "x" << (int)resolution.y() << std::endl;
     }
     else
     {
@@ -188,9 +206,9 @@ void WindowSizeHandler::changeWindowedResolution(osgViewer::GraphicsWindow *wind
 {
     osg::GraphicsContext::WindowingSystemInterface    *wsi = osg::GraphicsContext::getWindowingSystemInterface();
 
-    if (wsi == NULL) 
+    if (wsi == NULL)
     {
-        osg::notify(osg::NOTICE) << "Error, no WindowSystemInterface available, cannot toggle window fullscreen." << std::endl;
+        OSG_NOTICE << "Error, no WindowSystemInterface available, cannot toggle window fullscreen." << std::endl;
         return;
     }
 
@@ -245,7 +263,7 @@ void WindowSizeHandler::changeWindowedResolution(osgViewer::GraphicsWindow *wind
         resolution = _resolutionList[_currentResolutionIndex];
         window->setWindowDecoration(true);
         window->setWindowRectangle((screenWidth - (int)resolution.x()) / 2, (screenHeight - (int)resolution.y()) / 2, (int)resolution.x(), (int)resolution.y());
-        osg::notify(osg::INFO) << "Screen resolution = " << (int)resolution.x() << "x" << (int)resolution.y() << std::endl;  
+        OSG_INFO << "Screen resolution = " << (int)resolution.x() << "x" << (int)resolution.y() << std::endl;
 
         window->grabFocusIfPointerInWindow();
     }
@@ -290,15 +308,15 @@ ThreadingHandler::ThreadingHandler() :
 
 void ThreadingHandler::getUsage(osg::ApplicationUsage &usage) const
 {
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventChangeThreadingModel), "Toggle threading model.");
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventChangeEndBarrierPosition), "Toggle the placement of the end of frame barrier.");
+    usage.addKeyboardMouseBinding(_keyEventChangeThreadingModel, "Toggle threading model.");
+    usage.addKeyboardMouseBinding(_keyEventChangeEndBarrierPosition, "Toggle the placement of the end of frame barrier.");
 }
 
 bool ThreadingHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActionAdapter &aa)
 {
     osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
     if (!view) return false;
-    
+
     osgViewer::ViewerBase* viewerBase = view->getViewerBase();
     osgViewer::Viewer* viewer = dynamic_cast<Viewer*>(viewerBase);
 
@@ -317,37 +335,40 @@ bool ThreadingHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIAction
 
             if (_changeThreadingModel == true && ea.getKey() == _keyEventChangeThreadingModel && delta > 1.0)
             {
+
                 _tickOrLastKeyPress = osg::Timer::instance()->tick();
 
                 switch(viewerBase->getThreadingModel())
                 {
                 case(osgViewer::ViewerBase::SingleThreaded):
                     viewerBase->setThreadingModel(osgViewer::ViewerBase::CullDrawThreadPerContext);
-                    osg::notify(osg::NOTICE)<<"Threading model 'CullDrawThreadPerContext' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading model 'CullDrawThreadPerContext' selected."<<std::endl;
                     break;
                 case(osgViewer::ViewerBase::CullDrawThreadPerContext):
                     viewerBase->setThreadingModel(osgViewer::ViewerBase::DrawThreadPerContext);
-                    osg::notify(osg::NOTICE)<<"Threading model 'DrawThreadPerContext' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading model 'DrawThreadPerContext' selected."<<std::endl;
                     break;
                 case(osgViewer::ViewerBase::DrawThreadPerContext):
                     viewerBase->setThreadingModel(osgViewer::ViewerBase::CullThreadPerCameraDrawThreadPerContext);
-                    osg::notify(osg::NOTICE)<<"Threading model 'CullThreadPerCameraDrawThreadPerContext' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading model 'CullThreadPerCameraDrawThreadPerContext' selected."<<std::endl;
                     break;
                 case(osgViewer::ViewerBase::CullThreadPerCameraDrawThreadPerContext):
                     viewerBase->setThreadingModel(osgViewer::ViewerBase::SingleThreaded);
-                    osg::notify(osg::NOTICE)<<"Threading model 'SingleThreaded' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading model 'SingleThreaded' selected."<<std::endl;
                     break;
-#if 1                    
+#if 1
                 case(osgViewer::ViewerBase::AutomaticSelection):
                     viewerBase->setThreadingModel(osgViewer::ViewerBase::SingleThreaded);
-                    osg::notify(osg::NOTICE)<<"Threading model 'SingleThreaded' selected."<<std::endl;
-#else                    
+                    OSG_NOTICE<<"Threading model 'SingleThreaded' selected."<<std::endl;
+#else
                 case(osgViewer::ViewerBase::AutomaticSelection):
                     viewerBase->setThreadingModel(viewer->suggestBestThreadingModel());
-                    osg::notify(osg::NOTICE)<<"Threading model 'AutomaticSelection' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading model 'AutomaticSelection' selected."<<std::endl;
 #endif
                     break;
                 }
+
+                aa.requestRedraw();
                 return true;
             }
             if (viewer && _changeEndBarrierPosition == true && ea.getKey() == _keyEventChangeEndBarrierPosition)
@@ -356,13 +377,15 @@ bool ThreadingHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIAction
                 {
                 case(osgViewer::Viewer::BeforeSwapBuffers):
                     viewer->setEndBarrierPosition(osgViewer::Viewer::AfterSwapBuffers);
-                    osg::notify(osg::NOTICE)<<"Threading model 'AfterSwapBuffers' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading end of frame barrier position 'AfterSwapBuffers' selected."<<std::endl;
                     break;
                 case(osgViewer::Viewer::AfterSwapBuffers):
                     viewer->setEndBarrierPosition(osgViewer::Viewer::BeforeSwapBuffers);
-                    osg::notify(osg::NOTICE)<<"Threading model 'BeforeSwapBuffers' selected."<<std::endl;
+                    OSG_NOTICE<<"Threading end of frame barrier position 'BeforeSwapBuffers' selected."<<std::endl;
                     break;
                 }
+
+                aa.requestRedraw();
                 return true;
             }
             break;
@@ -384,8 +407,6 @@ RecordCameraPathHandler::RecordCameraPathHandler(const std::string& filename, fl
     _animStartTime(0),
     _lastFrameTime(osg::Timer::instance()->tick())
 {
-    _animPath = new osg::AnimationPath();
-
     const char* str = getenv("OSG_RECORD_CAMERA_PATH_FPS");
     if (str)
     {
@@ -399,8 +420,8 @@ RecordCameraPathHandler::RecordCameraPathHandler(const std::string& filename, fl
 
 void RecordCameraPathHandler::getUsage(osg::ApplicationUsage &usage) const
 {
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventToggleRecord), "Toggle camera path recording.");
-    usage.addKeyboardMouseBinding(reinterpret_cast<const char*>(&_keyEventTogglePlayback), "Toggle camera path playback.");
+    usage.addKeyboardMouseBinding(_keyEventToggleRecord, "Toggle camera path recording.");
+    usage.addKeyboardMouseBinding(_keyEventTogglePlayback, "Toggle camera path playback.");
 }
 
 bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActionAdapter &aa)
@@ -423,10 +444,10 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
         // If our internal _delta is finally large enough to warrant a ControlPoint
         // insertion, do so now. Be sure and reset the internal _delta, so we can start
         // calculating when the next insert should happen.
-        if (_currentlyRecording && _delta >= _interval)
+        if (_animPath.valid() && _currentlyRecording && _delta >= _interval)
         {
             const osg::Matrixd& m = view->getCamera()->getInverseViewMatrix();
-            double animationPathTime = osg::Timer::instance()->delta_s(_animStartTime, time);            
+            double animationPathTime = osg::Timer::instance()->delta_s(_animStartTime, time);
             _animPath->insert(animationPathTime, osg::AnimationPath::ControlPoint(m.getTrans(), m.getRotate()));
             _delta = 0.0f;
 
@@ -438,7 +459,7 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
 
         }
         else _delta += delta;
-        
+
         return true;
     }
 
@@ -456,8 +477,8 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
                 {
                     _currentlyRecording = true;
                     _animStartTime = osg::Timer::instance()->tick();
-                    _animPath->clear();
-                    
+                    _animPath = new osg::AnimationPath();
+
                     if (!_filename.empty())
                     {
                         std::stringstream ss;
@@ -468,8 +489,8 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
                             _autoinc++;
                         }
                         ss << "."<<osgDB::getFileExtension(_filename);
-                        
-                        osg::notify(osg::NOTICE) << "Recording camera path to file " << ss.str() << std::endl;
+
+                        OSG_NOTICE << "Recording camera path to file " << ss.str() << std::endl;
                         _fout.open( ss.str().c_str() );
 
                         // make sure doubles are not trucated by default stream precision = 6
@@ -477,7 +498,7 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
                     }
                     else
                     {
-                        osg::notify(osg::NOTICE)<<"Recording camera path."<<std::endl;
+                        OSG_NOTICE<<"Recording camera path."<<std::endl;
                     }
                 }
 
@@ -507,29 +528,39 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
                     _currentlyRecording = false;
                     _delta = 0.0f;
 
-                    // In the future this will need to be written continuously, rather
-                    // than all at once.
-                    osgDB::ofstream out(_filename.c_str());
-                    osg::notify(osg::NOTICE)<<"Writing camera file: "<<_filename<<std::endl;
-                    _animPath->write(out);
-                    out.close();
+                    if (_animPath.valid() && !_animPath->empty())
+                    {
+                        // In the future this will need to be written continuously, rather
+                        // than all at once.
+                        osgDB::ofstream out(_filename.c_str());
+                        OSG_NOTICE<<"Writing camera file: "<<_filename<<std::endl;
+                        _animPath->write(out);
+                        out.close();
+                    }
+                    else
+                    {
+                        OSG_NOTICE<<"No animation path to write out."<<std::endl;
+                    }
                 }
 
                 // The user has requested to BEGIN playback.
                 if (!_currentlyPlaying)
                 {
-                    _animPathManipulator = new osgGA::AnimationPathManipulator(_animPath.get());
-                    _animPathManipulator->home(ea,aa);
+                     if (_animPath.valid() && !_animPath->empty())
+                     {
+                        _animPathManipulator = new osgGA::AnimationPathManipulator(_animPath.get());
+                        _animPathManipulator->home(ea,aa);
 
 
-                    // If we successfully found our _filename file, set it and keep a copy
-                    // around of the original MatrixManipulator to restore later.
-                    if (_animPathManipulator.valid() && _animPathManipulator->valid())
-                    {
-                        _oldManipulator = view->getCameraManipulator();
-                        view->setCameraManipulator(_animPathManipulator.get());
-                        _currentlyPlaying = true;
-                    }
+                        // If we successfully found our _filename file, set it and keep a copy
+                        // around of the original CameraManipulator to restore later.
+                        if (_animPathManipulator.valid() && _animPathManipulator->valid())
+                        {
+                            _oldManipulator = view->getCameraManipulator();
+                            view->setCameraManipulator(_animPathManipulator.get());
+                            _currentlyPlaying = true;
+                        }
+                     }
                 }
 
                 // The user has requested to STOP playback.
@@ -542,7 +573,7 @@ bool RecordCameraPathHandler::handle(const osgGA::GUIEventAdapter &ea, osgGA::GU
                 }
 
                 return true;
-            }        
+            }
 
             break;
         }
@@ -574,16 +605,20 @@ bool LODScaleHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionA
             if (ea.getKey() == _keyEventIncreaseLODScale)
             {
                 camera->setLODScale(camera->getLODScale()*1.1);
-                osg::notify(osg::NOTICE)<<"LODScale = "<<camera->getLODScale()<<std::endl;
+                OSG_NOTICE<<"LODScale = "<<camera->getLODScale()<<std::endl;
+
+                aa.requestRedraw();
                 return true;
             }
 
             else if (ea.getKey() == _keyEventDecreaseLODScale)
             {
                 camera->setLODScale(camera->getLODScale()/1.1);
-                osg::notify(osg::NOTICE)<<"LODScale = "<<camera->getLODScale()<<std::endl;
+                OSG_NOTICE<<"LODScale = "<<camera->getLODScale()<<std::endl;
+
+                aa.requestRedraw();
                 return true;
-            }        
+            }
 
             break;
         }
@@ -596,33 +631,104 @@ bool LODScaleHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionA
 
 void LODScaleHandler::getUsage(osg::ApplicationUsage& usage) const
 {
+    usage.addKeyboardMouseBinding(_keyEventIncreaseLODScale,"Increase LODScale.");
+    usage.addKeyboardMouseBinding(_keyEventDecreaseLODScale,"Decrease LODScale.");
+}
+
+ToggleSyncToVBlankHandler::ToggleSyncToVBlankHandler():
+    _keyEventToggleSyncToVBlank('v')
+{
+}
+
+bool ToggleSyncToVBlankHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa)
+{
+    osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
+    if (!view) return false;
+
+    osgViewer::ViewerBase* viewer = view->getViewerBase();
+
+    if (viewer == NULL)
     {
-        std::ostringstream ostr;
-        ostr<<char(_keyEventIncreaseLODScale);
-        usage.addKeyboardMouseBinding(ostr.str(),"Increase LODScale.");
+        return false;
     }
-    
+
+    if (ea.getHandled()) return false;
+
+    switch(ea.getEventType())
     {
-        std::ostringstream ostr;
-        ostr<<char(_keyEventDecreaseLODScale);
-        usage.addKeyboardMouseBinding(ostr.str(),"Decrease LODScale.");
+        case(osgGA::GUIEventAdapter::KEYUP):
+        {
+            if (ea.getKey() == _keyEventToggleSyncToVBlank)
+            {
+                // Increase resolution
+                osgViewer::Viewer::Windows    windows;
+
+                viewer->getWindows(windows);
+                for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
+                    itr != windows.end();
+                    ++itr)
+                {
+                    (*itr)->setSyncToVBlank( !(*itr)->getSyncToVBlank() );
+                }
+
+                aa.requestRedraw();
+                return true;
+            }
+
+            break;
+        }
+    default:
+        break;
     }
+
+    return false;
 }
 
 
+void ToggleSyncToVBlankHandler::getUsage(osg::ApplicationUsage& usage) const
+{
+    usage.addKeyboardMouseBinding(_keyEventToggleSyncToVBlank,"Toggle SyncToVBlank.");
+}
+
+
+InteractiveImageHandler::InteractiveImageHandler(osg::Image* image) :
+    _image(image),
+    _texture(0),
+    _fullscreen(false),
+    _camera(0)
+{
+}
+
+InteractiveImageHandler::InteractiveImageHandler(osg::Image* image, osg::Texture2D* texture, osg::Camera* camera) :
+    _image(image),
+    _texture(texture),
+    _fullscreen(true),
+    _camera(camera)
+{
+    if (_camera.valid() && _camera->getViewport())
+    {
+        // Send an initial resize event (with the same size) so the image can
+        // resize itself initially.
+        double width = _camera->getViewport()->width();
+        double height = _camera->getViewport()->height();
+
+        resize(static_cast<int>(width), static_cast<int>(height));
+    }
+}
+
 bool InteractiveImageHandler::mousePosition(osgViewer::View* view, osg::NodeVisitor* nv, const osgGA::GUIEventAdapter& ea, int& x, int &y) const
 {
+    if (!view) return false;
+
     osgUtil::LineSegmentIntersector::Intersections intersections;
-    bool foundIntersection = view==0 ? false :
-        (nv==0 ? view->computeIntersections(ea.getX(), ea.getY(), intersections) :
-                 view->computeIntersections(ea.getX(), ea.getY(), nv->getNodePath(), intersections));
+    bool foundIntersection = (nv==0) ? view->computeIntersections(ea, intersections) :
+                                       view->computeIntersections(ea, nv->getNodePath(), intersections);
 
     if (foundIntersection)
     {
-
         osg::Vec2 tc(0.5f,0.5f);
 
-        // use the nearest intersection                 
+        // use the nearest intersection
         const osgUtil::LineSegmentIntersector::Intersection& intersection = *(intersections.begin());
         osg::Drawable* drawable = intersection.drawable.get();
         osg::Geometry* geometry = drawable ? drawable->asGeometry() : 0;
@@ -647,49 +753,49 @@ bool InteractiveImageHandler::mousePosition(osgViewer::View* view, osg::NodeVisi
                 osg::Vec2Array* texcoords_Vec2Array = dynamic_cast<osg::Vec2Array*>(texcoords);
                 if (texcoords_Vec2Array)
                 {
-                    // we have tex coord array so now we can compute the final tex coord at the point of intersection.                                
+                    // we have tex coord array so now we can compute the final tex coord at the point of intersection.
                     osg::Vec2 tc1 = (*texcoords_Vec2Array)[i1];
                     osg::Vec2 tc2 = (*texcoords_Vec2Array)[i2];
                     osg::Vec2 tc3 = (*texcoords_Vec2Array)[i3];
                     tc = tc1*r1 + tc2*r2 + tc3*r3;
                 }
             }
+
+            osg::TexMat* activeTexMat = 0;
+            osg::Texture* activeTexture = 0;
+
+            if (drawable->getStateSet())
+            {
+                osg::TexMat* texMat = dynamic_cast<osg::TexMat*>(drawable->getStateSet()->getTextureAttribute(0,osg::StateAttribute::TEXMAT));
+                if (texMat) activeTexMat = texMat;
+
+                osg::Texture* texture = dynamic_cast<osg::Texture*>(drawable->getStateSet()->getTextureAttribute(0,osg::StateAttribute::TEXTURE));
+                if (texture) activeTexture = texture;
+            }
+
+            if (activeTexMat)
+            {
+                osg::Vec4 tc_transformed = osg::Vec4(tc.x(), tc.y(), 0.0f,0.0f) * activeTexMat->getMatrix();
+                tc.x() = tc_transformed.x();
+                tc.y() = tc_transformed.y();
+            }
+
+            if (dynamic_cast<osg::TextureRectangle*>(activeTexture))
+            {
+                x = int( tc.x() );
+                y = int( tc.y() );
+            }
+            else if (_image.valid())
+            {
+                x = int( float(_image->s()) * tc.x() );
+                y = int( float(_image->t()) * tc.y() );
+            }
+
+            return true;
         }
 
-        osg::TexMat* activeTexMat = 0;
-        osg::Texture* activeTexture = 0;
-        
-        if (geometry->getStateSet())
-        {
-            osg::TexMat* texMat = dynamic_cast<osg::TexMat*>(geometry->getStateSet()->getTextureAttribute(0,osg::StateAttribute::TEXMAT));
-            if (texMat) activeTexMat = texMat;
-
-            osg::Texture* texture = dynamic_cast<osg::Texture*>(geometry->getStateSet()->getTextureAttribute(0,osg::StateAttribute::TEXTURE));
-            if (texture) activeTexture = texture;
-        }
-
-        if (activeTexMat)
-        {
-            osg::Vec4 tc_transformed = osg::Vec4(tc.x(), tc.y(), 0.0f,0.0f) * activeTexMat->getMatrix();
-            tc.x() = tc_transformed.x();
-            tc.y() = tc_transformed.y();
-        }
-
-        if (dynamic_cast<osg::TextureRectangle*>(activeTexture))
-        {
-            x = int( tc.x() );
-            y = int( tc.y() );
-        }
-        else if (_image.valid())
-        {
-            x = int( float(_image->s()) * tc.x() );
-            y = int( float(_image->t()) * tc.y() );
-        }
-
-
-        return true;
     }
-    
+
     return false;
 }
 
@@ -697,7 +803,7 @@ bool InteractiveImageHandler::mousePosition(osgViewer::View* view, osg::NodeVisi
 bool InteractiveImageHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa, osg::Object*, osg::NodeVisitor* nv)
 {
     if (ea.getHandled()) return false;
-    
+
     if (!_image) return false;
 
     switch(ea.getEventType())
@@ -721,11 +827,23 @@ bool InteractiveImageHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUI
             osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
             int x,y;
             bool sendKeyEvent = mousePosition(view, nv, ea, x, y);
-        
+
             if (sendKeyEvent)
             {
                 return _image->sendKeyEvent(ea.getKey(), ea.getEventType()==osgGA::GUIEventAdapter::KEYDOWN);
-            }          
+            }
+            break;
+        }
+        case (osgGA::GUIEventAdapter::RESIZE):
+        {
+            if (_fullscreen && _camera.valid())
+            {
+                _camera->setViewport(0, 0, ea.getWindowWidth(), ea.getWindowHeight());
+
+                resize(ea.getWindowWidth(), ea.getWindowHeight());
+                return true;
+            }
+            break;
         }
 
         default:
@@ -744,4 +862,17 @@ bool InteractiveImageHandler::cull(osg::NodeVisitor* nv, osg::Drawable*, osg::Re
     return false;
 }
 
+void InteractiveImageHandler::resize(int width, int height)
+{
+    if (_image.valid())
+    {
+        _image->scaleImage(width, height, 1);
+    }
+
+    // Make sure the texture does not rescale the image because
+    // it thinks it should still be the previous size...
+    if (_texture.valid())
+        _texture->setTextureSize(width, height);
+}
+
 }
