@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2009 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2009 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -229,7 +229,7 @@ void RayTracedTechnique::init()
             texture3D->setWrap(osg::Texture3D::WRAP_S,osg::Texture3D::CLAMP_TO_BORDER);
             texture3D->setWrap(osg::Texture3D::WRAP_T,osg::Texture3D::CLAMP_TO_BORDER);
             texture3D->setBorderColor(osg::Vec4(0.0,0.0,0.0,0.0));
-            if (image_3d->getPixelFormat()==GL_ALPHA || 
+            if (image_3d->getPixelFormat()==GL_ALPHA ||
                 image_3d->getPixelFormat()==GL_LUMINANCE)
             {
                 texture3D->setInternalFormatMode(osg::Texture3D::USE_USER_DEFINED_FORMAT);
@@ -303,7 +303,7 @@ void RayTracedTechnique::init()
 
             }
             else
-            {    
+            {
                 osg::Shader* fragmentShader = osgDB::readShaderFile(osg::Shader::FRAGMENT, "shaders/volume_mip.frag");
                 if (fragmentShader)
                 {
@@ -319,6 +319,8 @@ void RayTracedTechnique::init()
         else if (shadingModel==Isosurface)
         {
 
+            enableBlending = true;
+
             stateset->addUniform(cpv._isoProperty->getUniform());
 
             if (tf)
@@ -335,12 +337,12 @@ void RayTracedTechnique::init()
                 }
             }
             else
-            {    
+            {
                 osg::Shader* fragmentShader = osgDB::readShaderFile(osg::Shader::FRAGMENT, "shaders/volume_iso.frag");
                 if (fragmentShader)
                 {
                     OSG_INFO<<"Shader found"<<std::endl;
-                
+
                     program->addShader(fragmentShader);
                 }
                 else
@@ -351,7 +353,7 @@ void RayTracedTechnique::init()
                     program->addShader(new osg::Shader(osg::Shader::FRAGMENT, volume_iso_frag));
                 }
             }
-        } 
+        }
         else if (shadingModel==Light)
         {
             enableBlending = true;
@@ -371,7 +373,7 @@ void RayTracedTechnique::init()
 
             }
             else
-            {    
+            {
                 osg::Shader* fragmentShader = osgDB::readShaderFile(osg::Shader::FRAGMENT, "shaders/volume_lit.frag");
                 if (fragmentShader)
                 {
@@ -478,8 +480,7 @@ void RayTracedTechnique::init()
 
         osg::Vec4Array* colours = new osg::Vec4Array(1);
         (*colours)[0].set(1.0f,1.0f,1.0,1.0f);
-        geom->setColorArray(colours);
-        geom->setColorBinding(osg::Geometry::BIND_OVERALL);
+        geom->setColorArray(colours, osg::Array::BIND_OVERALL);
 
         osg::DrawElementsUShort* drawElements = new osg::DrawElementsUShort(GL_QUADS);
         // bottom
@@ -487,7 +488,7 @@ void RayTracedTechnique::init()
         drawElements->push_back(1);
         drawElements->push_back(2);
         drawElements->push_back(3);
-        
+
         // bottom
         drawElements->push_back(3);
         drawElements->push_back(2);
@@ -522,7 +523,7 @@ void RayTracedTechnique::init()
 
         geode->addDrawable(geom);
 
-    } 
+    }
 
     if (cpv._sampleDensityWhenMovingProperty.valid())
     {
@@ -531,7 +532,7 @@ void RayTracedTechnique::init()
     }
 }
 
-void RayTracedTechnique::update(osgUtil::UpdateVisitor* uv)
+void RayTracedTechnique::update(osgUtil::UpdateVisitor* /*uv*/)
 {
 //    OSG_NOTICE<<"RayTracedTechnique:update(osgUtil::UpdateVisitor* nv):"<<std::endl;
 }
@@ -597,8 +598,8 @@ void RayTracedTechnique::traverse(osg::NodeVisitor& nv)
         {
             update(uv);
             return;
-        }        
-        
+        }
+
     }
     else if (nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR)
     {
@@ -611,12 +612,12 @@ void RayTracedTechnique::traverse(osg::NodeVisitor& nv)
     }
 
 
-    if (_volumeTile->getDirty()) 
+    if (_volumeTile->getDirty())
     {
         OSG_INFO<<"******* Doing init ***********"<<std::endl;
         _volumeTile->init();
     }
 }
-    
+
 
 } // end of osgVolume namespace
