@@ -6,9 +6,10 @@ char volume_mip_frag[] = "uniform sampler3D baseTexture;\n"
                          "varying vec4 cameraPos;\n"
                          "varying vec4 vertexPos;\n"
                          "varying mat4 texgen;\n"
+                         "varying vec4 baseColor;\n"
                          "\n"
                          "void main(void)\n"
-                         "{ \n"
+                         "{\n"
                          "    vec4 t0 = vertexPos;\n"
                          "    vec4 te = cameraPos;\n"
                          "\n"
@@ -60,19 +61,17 @@ char volume_mip_frag[] = "uniform sampler3D baseTexture;\n"
                          "    t0 = t0 * texgen;\n"
                          "    te = te * texgen;\n"
                          "\n"
-                         "    const float max_iteratrions = 2048.0;\n"
+                         "    const float min_iterations = 2.0;\n"
+                         "    const float max_iterations = 2048.0;\n"
                          "    float num_iterations = ceil(length((te-t0).xyz)/SampleDensityValue);\n"
-                         "    if (num_iterations<2.0) num_iterations = 2.0;\n"
+                         "    if (num_iterations<min_iterations) num_iterations = min_iterations;\n"
+                         "    else if (num_iterations>max_iterations) num_iterations = max_iterations;\n"
+                         "    else if (num_iterations!=num_iterations) num_iterations = max_iterations;\n"
                          "\n"
-                         "    if (num_iterations>max_iteratrions) \n"
-                         "    {\n"
-                         "        num_iterations = max_iteratrions;\n"
-                         "    }\n"
-                         "\n"
-                         "    vec3 deltaTexCoord=(te-t0).xyz/float(num_iterations-1.0);\n"
+                         "    vec3 deltaTexCoord=(te-t0).xyz/(num_iterations-1.0);\n"
                          "    vec3 texcoord = t0.xyz;\n"
                          "\n"
-                         "    vec4 fragColor = vec4(0.0, 0.0, 0.0, 0.0); \n"
+                         "    vec4 fragColor = vec4(0.0, 0.0, 0.0, 0.0);\n"
                          "    while(num_iterations>0.0)\n"
                          "    {\n"
                          "        vec4 color = texture3D( baseTexture, texcoord);\n"
@@ -80,16 +79,17 @@ char volume_mip_frag[] = "uniform sampler3D baseTexture;\n"
                          "        {\n"
                          "            fragColor = color;\n"
                          "        }\n"
-                         "        texcoord += deltaTexCoord; \n"
+                         "        texcoord += deltaTexCoord;\n"
                          "\n"
                          "        --num_iterations;\n"
                          "    }\n"
                          "\n"
                          "    fragColor.w *= TransparencyValue;\n"
+                         "    if (fragColor.w>1.0) fragColor.w = 1.0;\n"
+                         "\n"
+                         "    fragColor *= baseColor;\n"
                          "\n"
-                         "    if (fragColor.w>1.0) fragColor.w = 1.0; \n"
                          "    if (fragColor.w<AlphaFuncValue) discard;\n"
-                         "    \n"
                          "    gl_FragColor = fragColor;\n"
                          "}\n"
                          "\n";
