@@ -10,13 +10,14 @@ char volume_tf_iso_frag[] = "uniform sampler3D baseTexture;\n"
                             "\n"
                             "varying vec4 cameraPos;\n"
                             "varying vec4 vertexPos;\n"
+                            "varying vec3 lightDirection;\n"
                             "varying mat4 texgen;\n"
+                            "varying vec4 baseColor;\n"
                             "\n"
                             "void main(void)\n"
-                            "{ \n"
+                            "{\n"
                             "    vec4 t0 = vertexPos;\n"
                             "    vec4 te = cameraPos;\n"
-                            "    vec3 eyeDirection = normalize((te-t0).xyz);\n"
                             "\n"
                             "    if (te.x>=0.0 && te.x<=1.0 &&\n"
                             "        te.y>=0.0 && te.y<=1.0 &&\n"
@@ -66,14 +67,13 @@ char volume_tf_iso_frag[] = "uniform sampler3D baseTexture;\n"
                             "    t0 = t0 * texgen;\n"
                             "    te = te * texgen;\n"
                             "\n"
-                            "    const float max_iteratrions = 2048.0;\n"
+                            "    const float min_iterations = 2.0;\n"
+                            "    const float max_iterations = 2048.0;\n"
                             "    float num_iterations = ceil(length((te-t0).xyz)/SampleDensityValue);\n"
-                            "    if (num_iterations<2.0) num_iterations = 2.0;\n"
+                            "    if (num_iterations<min_iterations) num_iterations = min_iterations;\n"
+                            "    else if (num_iterations>max_iterations) num_iterations = max_iterations;\n"
+                            "    else if (num_iterations!=num_iterations) num_iterations = max_iterations;\n"
                             "\n"
-                            "    if (num_iterations>max_iteratrions)\n"
-                            "    {\n"
-                            "        num_iterations = max_iteratrions;\n"
-                            "    }\n"
                             "\n"
                             "    vec3 deltaTexCoord=(t0-te).xyz/float(num_iterations-1.0);\n"
                             "    vec3 texcoord = te.xyz;\n"
@@ -83,7 +83,7 @@ char volume_tf_iso_frag[] = "uniform sampler3D baseTexture;\n"
                             "    vec3 deltaX = vec3(normalSampleDistance, 0.0, 0.0);\n"
                             "    vec3 deltaY = vec3(0.0, normalSampleDistance, 0.0);\n"
                             "    vec3 deltaZ = vec3(0.0, 0.0, normalSampleDistance);\n"
-                            "\n"
+                            "    \n"
                             "    while(num_iterations>0.0)\n"
                             "    {\n"
                             "\n"
@@ -110,13 +110,15 @@ char volume_tf_iso_frag[] = "uniform sampler3D baseTexture;\n"
                             "            if (grad.x!=0.0 || grad.y!=0.0 || grad.z!=0.0)\n"
                             "            {\n"
                             "                vec3 normal = normalize(grad);\n"
-                            "                float lightScale = 0.1 +  abs(dot(normal.xyz, eyeDirection))*0.9;\n"
+                            "                float lightScale = 0.1 +  max(0.0, dot(normal.xyz, lightDirection))*0.9;\n"
                             "\n"
                             "                color.x *= lightScale;\n"
                             "                color.y *= lightScale;\n"
                             "                color.z *= lightScale;\n"
                             "            }\n"
                             "\n"
+                            "            color *= baseColor;\n"
+                            "\n"
                             "            gl_FragColor = color;\n"
                             "\n"
                             "            return;\n"
@@ -131,6 +133,5 @@ char volume_tf_iso_frag[] = "uniform sampler3D baseTexture;\n"
                             "\n"
                             "    // we didn't find an intersection so just discard fragment\n"
                             "    discard;\n"
-                            "\n"
                             "}\n"
                             "\n";
